
/*
 * Copyright (c) 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
/********************************************************************\
*                                                                    *
*  File:      utdll16.c                                              *
*                                                                    *
*  Comments:  16-bit side of the Universal Thunk                     *
*                                                                    *
*  Functions: LibMain                                                *
*             UTInit                                                 *
*             UTProc                                                 *
*             WEP                                                    *
*                                                                    *
\********************************************************************/

/*********************  Header Files **********************/


#include "thunk.h"
#include "dafs.h"
#include "testdaf.h"

enum {
	i_GetErrorT = DAFSTART,
	i_InvertImageT,
	i_MakeARegexT,
	i_MakeURegexT,
	i_CheckARegexT,
	i_CheckURegexT,
	i_WriteRegexT,
	i_ReadRegexT,
	i_DisposeRegexT,
	i_ConcatImageT,
	i_MoveAllPropsT,
	i_CopyAllPropsT,
	i_GetImageBoundsT,
	i_GetGlyphT,
	i_SetGlyphT,
	i_GetShortT,
	i_SetShortT,
	i_SetNoDataT,
	i_GetParentPathT,
	i_GetParentOtherPathT,
	i_GetLongPropT,
	i_NewEntityT,
	i_NewEntityAfterT,
	i_NewEntityBeforeT,
	i_OrEntityT,
	i_ShareEntityT,
	i_DisposeEntityT,
	i_ReadEntityT,
	i_ReadEntityNoImT,
	i_WriteEntityT,
	i_NameToTypeT,
	i_TypeToNameT,
	i_GetEntityTypeT,
	i_SetEntityTypeT,
	i_SetLogicalTypeT,
	i_GetLogicalTypeT,
	i_MoveToFrontT,
	i_MoveToBackT,
	i_MoveAfterT,
	i_MoveBeforeT,
	i_GetNextEntityT,
	i_GetPrevEntityT,
	i_GetChildEntityT,
	i_GetLastEntityT,
	i_GetParentEntityT,
	i_CountChildrenT,
	i_CountSharesT,
	i_CountPrevT,
	i_CountNextT,
	i_FindEntityForwardT,
	i_FindEntityBackT,
	i_FindAncestorForwardT,
	i_FindAncestorBackT,
	i_SetBoolPropT,
	i_SetLongPropT,
	i_SetStringPropT,
	i_SetUStringPropT,
	i_SetUserDataPropT,
	i_SetUserPtrPropT,
	i_SetSessionBoolPropT,
	i_SetSessionLongPropT,
	i_SetSessionStringPropT,
	i_SetSessionUStringPropT,
	i_SetSessionUserDataPropT,
	i_SetSessionUserPtrPropT,
	i_MovePropT,
	i_ChangePropNameT,
	i_FindPropT,
	i_FindPropNoTypesT,
	i_FirstPropT,
	i_NextPropT,
	i_GetPropSizeT,
	i_GetPropTypeT,
	i_GetPropNameT,
	i_GetBoolPropT,
	i_GetUserPtrPropT,
	i_GetUserDataPropT,
	i_GetStringPropT,
	i_GetUStringPropT,
	i_DisposePropT,
	i_DisposeSessionPropT,
	i_DisposeAllPropsT,
	i_DisposeAllSessionPropsT,
	i_GetCommentT,
	i_IntersectBoxT,
	i_UnionBoxT,
	i_SetCommentT,
	i_GetEDataTypeT,
	i_GetTextT,
	i_SetTextT,
	i_GetUTextT,
	i_SetUTextT,
	i_GetConfT,
	i_SetConfT,
	i_GetPosSetT,
	i_SetPosSetT,
	i_AddCharsT,
	i_NewImageT,
	i_SetImageT,
	i_GetImageT,
	i_CloneEntityImageT,
	i_GetImageWidthT,
	i_GetImageHeightT,
	i_GetEnclosingBoxT,
	i_SetBoxT,
	i_GetImageResT,
	i_SetImageResT,
	i_RotateImageT,
	i_ScaleImageT,
	i_BitMap2ImageT,
	i_Image2BitMapT,
	i_ImageExtT,
	i_ReadImageT,
	i_WriteImageT,
	i_DisposeImageT,
	i_ReadUATableT,
	i_DisposeUATableT,
	i_AllocASCIITextT,
	i_FreeASCIITextT,
	i_AllocUTextT,
	i_FreeUTextT,
	i_InitLibT,
	i_SetCallBackT,
	i_DoCallBackT,
	i_ExitLibT,
	i_Read1TokenT,
	i_ReadTokenT,
	i_ReadSizeT,
	i_ReadItemT,
	i_ReadItemAllocT,
	i_ReadItemAllocHT,
	i_WriteItemT,
	i_WriteItemHT,
	i_SetImageCallBackT,
	i_GetImageTagT,
	i_SetImageTagT,
	i_AddDataToImageT,
	i_GetDataFromImageT,
	i_GetAreaT,
	i_ForceAreaTypeT,
	i_SetBoxListT,
	i_SetOrthogonListT,
	i_BoxList2OrthogonListT,
	i_Box2OrthogonListT,
	i_Box2BoxListT,
	i_OrthogonList2BoxListT,
	i_Box2OrthogonT,
	i_Orthogon2BoxListT,
	i_BoxList2BoxT,
	i_OrthogonList2BoxT,
	i_Orthogon2BoxT,
	i_NewOrthogonListT,
	i_CopyOrthogonListT,
	i_DisposeOrthogonListT,
	i_UnionOrthogonListT,
	i_IntersectOrthogonListT,
	i_SubtractOrthogonListT,
	i_NewOrthogonT,
	i_DisposeOrthogonT,
	i_CopyOrthogonT,
	i_UnionOrthogonT,
	i_IntersectOrthogonT,
	i_TranslateOrthogonT,
	i_NewBoxListT,
	i_DisposeBoxListT,
	i_CopyBoxListT,
	i_UnionBoxListT,
	i_IntersectBoxListT,
	i_SubtractBoxListT,
	i_TranslateBoxListT,

	CheckEntityT,
	rlcheckimageT,
	compare_imageT,
	i_CompareEntitysT,
	
	i_DAFSCallbackT,
	i_ImageWriteCallbackT,
	i_ImageReadCallbackT,
	i_UserPtrCallbackT,
};

#ifdef W32SUT_16

struct UserPtrData {
	void *data;
	iUserPtrFree call;
};

typedef struct UserPtrData UserPtrData,*UserPtrDataPtr;

static iCallBackProc dafs_cb;
static iImageWriteProc imagewrite_cb;
static iImageReadProc imageread_cb;
static void *gUserData=0;

DWORD
i_CSwitch(LPDWORD lp,DWORD dwFunc)
{
	iImagePtr *imp;
	char *tag;
	UserPtrDataPtr dp;

	switch (dwFunc) {
	case i_DAFSCallbackT:
		return dafs_cb((iMess)lp[0],gUserData,(iEntityPtr)lp[2]);
	case i_ImageWriteCallbackT:
		tag = (char*)UTLinearToSelectorOffset((void*)lp[0]);
		return imagewrite_cb(tag,(iImagePtr)lp[1]);
	case i_ImageReadCallbackT:
		tag = (char*)UTLinearToSelectorOffset((void*)lp[0]);
		imp = (iImagePtr*)UTLinearToSelectorOffset((void*)lp[1]);
		return imageread_cb(tag,imp);
	case i_UserPtrCallbackT:
		dp = (UserPtrDataPtr)lp[0];
		if (dp->call) dp->call(dp->data);
		GlobalFreePtr(dp);
		return 0;
	default:
		halt("bad callback function selector in dafs\n");
	}
	return 0;
}

Dll const char *  APIENTRY i_GetError(iDAFSError error)
{
	DWORD   Args[1];
	Args[0]=(DWORD)error;
	return (const char *)UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetErrorT,NULL,Args));
}

Dll iDAFSError  APIENTRY i_InvertImage(iImagePtr im)
{
	DWORD   Args[1];
	Args[0]=(DWORD)im;
	return (iDAFSError) thunkIt(i_InvertImageT,NULL,Args);
}

Dll iDAFSError  APIENTRY i_MakeARegex(iRegexPtr *rep, const char *str)
{
	DWORD   Args[2];
	Args[0]=(DWORD)rep;
	Args[1]=(DWORD)str;
	return (iDAFSError) thunkIt(i_MakeARegexT,t12,Args);
}


Dll iDAFSError  APIENTRY i_MakeURegex(iRegexPtr *rep, const iUCode *str)
{
	DWORD   Args[2];
	Args[0]=(DWORD)rep;
	Args[1]=(DWORD)str;
	return (iDAFSError) thunkIt(i_MakeURegexT,t12,Args);
}

Dll int4  APIENTRY i_CheckARegex(const iRegexPtr re, const char *str, int4 *start, int4 *end)
{
	DWORD   Args[4];
	Args[0]=(DWORD)re;
	Args[1]=(DWORD)str;
	Args[2]=(DWORD)start;
	Args[3]=(DWORD)end;
	return (int) thunkIt(i_CheckARegexT,t234,Args);
}

Dll int4  APIENTRY i_CheckURegex(const iRegexPtr re, const iUCode *str, int4 *start, int4 *end)
{
	DWORD   Args[4];
	Args[0]=(DWORD)re;
	Args[1]=(DWORD)str;
	Args[2]=(DWORD)start;
	Args[3]=(DWORD)end;
	return (int) thunkIt(i_CheckURegexT,t234,Args);
}

Dll iDAFSError  APIENTRY i_WriteRegex(const iRegexPtr re, char *filename)
{
	DWORD   Args[2];
	Args[0]=(DWORD)re;
	Args[1]=(DWORD)filename;
	return (iDAFSError) thunkIt(i_WriteRegexT,t2,Args);
}

Dll iDAFSError  APIENTRY i_ReadRegex(iRegexPtr *rep, char *filename)
{
	DWORD   Args[2];
	Args[0]=(DWORD)rep;
	Args[1]=(DWORD)filename;
	return (iDAFSError) thunkIt(i_ReadRegexT,t12,Args);
}


Dll void  APIENTRY i_DisposeRegex(iRegexPtr re)
{
	DWORD   Args[1];
	Args[0]=(DWORD)re;
	thunkIt(i_DisposeRegexT,NULL,Args);
}

Dll iDAFSError  APIENTRY i_ConcatImage(iImagePtr left,iImagePtr right)
{
	DWORD   Args[2];
	Args[0]=(DWORD)left;
	Args[1]=(DWORD)right;
	return (iDAFSError) thunkIt(i_ConcatImageT,NULL,Args);
}

Dll iDAFSError  APIENTRY i_MoveAllProps(iEntityPtr from_en, iEntityPtr to_en)
{
	DWORD   Args[2];
	Args[0]=(DWORD)from_en;
	Args[1]=(DWORD)to_en;
	return (iDAFSError) thunkIt(i_MoveAllPropsT,NULL,Args);
}

Dll iDAFSError  APIENTRY i_CopyAllProps(iEntityPtr from_en, iEntityPtr to_en)
{
	DWORD   Args[2];
	Args[0]=(DWORD)from_en;
	Args[1]=(DWORD)to_en;
	return (iDAFSError) thunkIt(i_CopyAllPropsT,NULL,Args);
}

Dll iBox  APIENTRY i_GetImageBounds(const iImagePtr image)
{
	DWORD   Args[2];
	iBox retbox;

	Args[0]=(DWORD)image;
	Args[1]=(DWORD)&retbox;
	thunkIt(i_GetImageBoundsT,t2,Args);
	return retbox;
}

Dll short  APIENTRY i_GetGlyph(const iEntityPtr entity, short* conf)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)conf;
	return (short) thunkIt(i_GetGlyphT,t2,Args);
}

Dll void  APIENTRY i_SetGlyph(iEntityPtr entity,short chr, short conf)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)chr;
	Args[2]=(DWORD)conf;
	thunkIt(i_SetGlyphT,NULL,Args);
}

Dll int2  APIENTRY i_GetShort(const iEntityPtr entity, short* conf)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)conf;
	return (int2) thunkIt(i_GetShortT,t2,Args);
}

Dll void  APIENTRY i_SetShort(iEntityPtr entity, int2 num, short conf)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)num;
	Args[2]=(DWORD)conf;
	thunkIt(i_SetShortT,NULL,Args);
}

Dll void  APIENTRY i_SetNoData(iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	thunkIt(i_SetNoDataT,NULL,Args);
}

Dll iEntityPtr  APIENTRY i_GetParentPath(iEntityPtr home, iEntityPtr target)
{
	DWORD   Args[2];
	Args[0]=(DWORD)home;
	Args[1]=(DWORD)target;
	return (iEntityPtr) thunkIt(i_GetParentPathT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_GetParentOtherPath(iEntityPtr home, iEntityPtr target)
{
	DWORD   Args[2];
	Args[0]=(DWORD)home;
	Args[1]=(DWORD)target;
	return (iEntityPtr) thunkIt(i_GetParentOtherPathT,NULL,Args);
}


Dll iDAFSError  APIENTRY i_GetLongProp(const iEntityPtr entity,const char *name,long *i)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)i;
	return (iDAFSError) thunkIt(i_GetLongPropT,t23,Args);
}


Dll iEntityPtr  APIENTRY i_NewEntity(const iTypeIndex type,iEntityPtr parent)
{
	DWORD   Args[2];
	Args[0]=(DWORD)type;
	Args[1]=(DWORD)parent;
	return (iEntityPtr) thunkIt(i_NewEntityT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_NewEntityAfter(const iTypeIndex type,iEntityPtr sibling)
{
	DWORD   Args[2];
	Args[0]=(DWORD)type;
	Args[1]=(DWORD)sibling;
	return (iEntityPtr) thunkIt(i_NewEntityAfterT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_NewEntityBefore(const iTypeIndex type,iEntityPtr sibling)
{
	DWORD   Args[2];
	Args[0]=(DWORD)type;
	Args[1]=(DWORD)sibling;
	return (iEntityPtr) thunkIt(i_NewEntityBeforeT,NULL,Args);
}

Dll iDAFSError  APIENTRY i_OrEntity(iEntityPtr *entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iDAFSError) thunkIt(i_OrEntityT,t1,Args);
}


Dll iDAFSError  APIENTRY i_ShareEntity(iEntityPtr entity,iEntityPtr parent,iEntityPtr *epp)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)parent;
	Args[2]=(DWORD)epp;
	return (iDAFSError) thunkIt(i_ShareEntityT,t3,Args);
}


Dll void  APIENTRY i_DisposeEntity(iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	thunkIt(i_DisposeEntityT,NULL,Args);
}


Dll iDAFSError  APIENTRY i_ReadEntity(const char *fileName,iEntityPtr *entityPtr)
{
	DWORD   Args[2];
	Args[0]=(DWORD)fileName;
	Args[1]=(DWORD)entityPtr;
	return (iDAFSError) thunkIt(i_ReadEntityT,t12,Args);
}


Dll iDAFSError  APIENTRY i_ReadEntityNoIm(const char *fileName,iEntityPtr *entityPtr)
{
	DWORD   Args[2];
	Args[0]=(DWORD)fileName;
	Args[1]=(DWORD)entityPtr;
	return (iDAFSError) thunkIt(i_ReadEntityNoImT,t12,Args);
}


Dll iDAFSError  APIENTRY i_WriteEntity(const char *fileName,const iEntityPtr entity, iDAFSWriteMode mode)
{
	DWORD   Args[3];
	Args[0]=(DWORD)fileName;
	Args[1]=(DWORD)entity;
	Args[2]=(DWORD)mode;
	return (iDAFSError) thunkIt(i_WriteEntityT,t1,Args);
}


Dll iDAFSError  APIENTRY i_NameToType(const char *type,iTypeIndex *val)
{
	DWORD   Args[2];
	Args[0]=(DWORD)type;
	Args[1]=(DWORD)val;
	return (iDAFSError) thunkIt(i_NameToTypeT,t12,Args);
}

Dll const char *  APIENTRY i_TypeToName(iTypeIndex type)
{
	DWORD   Args[1];
	Args[0]=(DWORD)type;
	return (const char *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_TypeToNameT,NULL,Args));
}


Dll iTypeIndex  APIENTRY i_GetEntityType(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iTypeIndex) thunkIt(i_GetEntityTypeT,NULL,Args);
}


Dll void  APIENTRY i_SetEntityType(iEntityPtr entity,iTypeIndex type)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)type;
	thunkIt(i_SetEntityTypeT,NULL,Args);
}

 
Dll void  APIENTRY i_SetLogicalType(iEntityPtr entity,iLogicalType type)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)type;
	thunkIt(i_SetLogicalTypeT,NULL,Args);
}


Dll iLogicalType  APIENTRY i_GetLogicalType(iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iLogicalType) thunkIt(i_GetLogicalTypeT,NULL,Args);
}

Dll void  APIENTRY i_MoveToFront(iEntityPtr entity, iEntityPtr new_parent)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)new_parent;
	thunkIt(i_MoveToFrontT,NULL,Args);
}

Dll void  APIENTRY i_MoveToBack(iEntityPtr entity, iEntityPtr new_parent)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)new_parent;
	thunkIt(i_MoveToBackT,NULL,Args);
}

Dll void  APIENTRY i_MoveAfter(iEntityPtr entity, iEntityPtr PrevSibling)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)PrevSibling;
	thunkIt(i_MoveAfterT,NULL,Args);
}


Dll void  APIENTRY i_MoveBefore(iEntityPtr entity, iEntityPtr NextSibling)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)NextSibling;
	thunkIt(i_MoveBeforeT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_GetNextEntity(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iEntityPtr) thunkIt(i_GetNextEntityT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_GetPrevEntity(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iEntityPtr) thunkIt(i_GetPrevEntityT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_GetChildEntity(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iEntityPtr) thunkIt(i_GetChildEntityT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_GetLastEntity(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iEntityPtr) thunkIt(i_GetLastEntityT,NULL,Args);
}


Dll iEntityPtr  APIENTRY i_GetParentEntity(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iEntityPtr) thunkIt(i_GetParentEntityT,NULL,Args);
}


Dll int4  APIENTRY i_CountChildren(iEntityPtr en)
{
	DWORD   Args[1];
	Args[0]=(DWORD)en;
	return (int4) thunkIt(i_CountChildrenT,NULL,Args);
}


Dll int4  APIENTRY i_CountShares(iEntityPtr en)
{
	DWORD   Args[1];
	Args[0]=(DWORD)en;
	return (int4) thunkIt(i_CountSharesT,NULL,Args);
}


Dll int4  APIENTRY i_CountPrev(iEntityPtr cur)
{
	DWORD   Args[1];
	Args[0]=(DWORD)cur;
	return (int4) thunkIt(i_CountPrevT,NULL,Args);
}


Dll int4  APIENTRY i_CountNext(iEntityPtr cur)
{
	DWORD   Args[1];
	Args[0]=(DWORD)cur;
	return (int4) thunkIt(i_CountNextT,NULL,Args);
}

Dll iEntityPtr  APIENTRY i_FindEntityForward(iEntityPtr home, iTypeIndex type, iSearchType method, void *name)
{
	DWORD   Args[4];
	Args[0]=(DWORD)home;
	Args[1]=(DWORD)type;
	Args[2]=(DWORD)method;
	Args[3]=(DWORD)name;
	return (iEntityPtr) thunkIt(i_FindEntityForwardT,t4,Args);
}

Dll iEntityPtr  APIENTRY i_FindEntityBack(iEntityPtr home, iTypeIndex type, iSearchType method, void *name)
{
	DWORD   Args[4];
	Args[0]=(DWORD)home;
	Args[1]=(DWORD)type;
	Args[2]=(DWORD)method;
	Args[3]=(DWORD)name;
	return (iEntityPtr) thunkIt(i_FindEntityBackT,t4,Args);
}

Dll iEntityPtr  APIENTRY i_FindAncestorForward(iEntityPtr home,iEntityPtr ans, iTypeIndex type, iSearchType method, void *name)
{
	DWORD   Args[5];
	Args[0]=(DWORD)home;
	Args[1]=(DWORD)ans;
	Args[2]=(DWORD)type;
	Args[3]=(DWORD)method;
	Args[4]=(DWORD)name;
	return (iEntityPtr) thunkIt(i_FindAncestorForwardT,t5,Args);
}

Dll iEntityPtr  APIENTRY i_FindAncestorBack(iEntityPtr home,iEntityPtr ans, iTypeIndex type, iSearchType method, void *name)
{
	DWORD   Args[5];
	Args[0]=(DWORD)home;
	Args[1]=(DWORD)ans;
	Args[2]=(DWORD)type;
	Args[3]=(DWORD)method;
	Args[4]=(DWORD)name;
	return (iEntityPtr) thunkIt(i_FindAncestorBackT,t5,Args);
}

Dll iDAFSError  APIENTRY i_SetBoolProp(iEntityPtr entity,const char *name, int data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetBoolPropT,t2,Args);
}

Dll iDAFSError  APIENTRY i_SetLongProp(iEntityPtr entity,const char *name, int4 data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetLongPropT,t2,Args);
}

Dll iDAFSError  APIENTRY i_SetStringProp(iEntityPtr entity, const char *name, char *data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetStringPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_SetUStringProp(iEntityPtr entity,const char *name, iUCode *data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetUStringPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_SetUserDataProp(iEntityPtr entity,const char *name,void *data, uint2 size)
{
	DWORD   Args[4];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	Args[3]=(DWORD)size;
	return (iDAFSError) thunkIt(i_SetUserDataPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_SetUserPtrProp(iEntityPtr entity, const char *name,
		void *data,iUserPtrFree userfree)
{
	DWORD   Args[4];
	UserPtrDataPtr dp;

	if (userfree) {
		dp = (UserPtrDataPtr)GlobalAllocPtr(GMEM_MOVEABLE,sizeof(UserPtrData));
		if (!dp) return MallocFailError;
		dp->data = data;
		dp->call = userfree;
		Args[2]=(DWORD)dp;
		Args[3]=(DWORD)1;
	} else {
		Args[2]=(DWORD)data;
		Args[3]=(DWORD)0;
	}
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	return (iDAFSError) thunkIt(i_SetUserPtrPropT,t2,Args);
}

Dll iDAFSError  APIENTRY i_SetSessionBoolProp(iTypeIndex itype,const char *name, int data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetSessionBoolPropT,t2,Args);
}

Dll iDAFSError  APIENTRY i_SetSessionLongProp(iTypeIndex itype,const char *name, int4 data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetSessionLongPropT,t2,Args);
}

Dll iDAFSError  APIENTRY i_SetSessionStringProp(iTypeIndex itype,const char *name, char *data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetSessionStringPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_SetSessionUStringProp(iTypeIndex itype,const char *name, iUCode *data)
{
	DWORD   Args[3];
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	return (iDAFSError) thunkIt(i_SetSessionUStringPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_SetSessionUserDataProp(iTypeIndex itype,const char *name,void *data, uint2 size)
{
	DWORD   Args[4];
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)data;
	Args[3]=(DWORD)size;
	return (iDAFSError) thunkIt(i_SetSessionUserDataPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_SetSessionUserPtrProp(iTypeIndex itype,const char *name,
		void *data,iUserPtrFree userfree)
{
	DWORD   Args[4];
	UserPtrDataPtr dp;

	if (userfree) {
		dp = (UserPtrDataPtr)GlobalAllocPtr(GMEM_MOVEABLE,sizeof(UserPtrData));
		if (!dp) return MallocFailError;
		dp->data = data;
		dp->call = userfree;
		Args[2]=(DWORD)dp;
		Args[3]=(DWORD)1;
	} else {
		Args[2]=(DWORD)data;
		Args[3]=(DWORD)0;
	}
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	return (iDAFSError) thunkIt(i_SetSessionUserPtrPropT,t2,Args);
}

Dll iDAFSError  APIENTRY i_MoveProp(iEntityPtr entity,iEntityPtr toen,const char *name)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)toen;
	Args[2]=(DWORD)name;
	return (iDAFSError) thunkIt(i_MovePropT,t3,Args);
}
Dll iDAFSError APIENTRY i_ChangePropName(iEntityPtr entity, const char *from_name, const char *to_name){
	DWORD Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)from_name;
	Args[2]=(DWORD)to_name;
	return (iDAFSError) thunkIt(i_ChangePropNameT,t3,Args);
}

Dll iPropPtr  APIENTRY i_FindProp(const iEntityPtr entity, const char *name)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	return (iPropPtr) thunkIt(i_FindPropT,t2,Args);
}



Dll iPropPtr  APIENTRY i_FindPropNoTypes(const iEntityPtr entity, const char *name)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	return (iPropPtr) thunkIt(i_FindPropNoTypesT,t2,Args);
}


Dll iPropPtr  APIENTRY i_FirstProp(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iPropPtr) thunkIt(i_FirstPropT,NULL,Args);
}


Dll iPropPtr  APIENTRY i_NextProp(const iEntityPtr entity, const iPropPtr prev)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)prev;
	return (iPropPtr) thunkIt(i_NextPropT,NULL,Args);
}



Dll long  APIENTRY i_GetPropSize(const iPropPtr prop)
{
	DWORD   Args[1];
	Args[0]=(DWORD)prop;
	return (long) thunkIt(i_GetPropSizeT,NULL,Args);
}


Dll iPTag  APIENTRY i_GetPropType(const iPropPtr prop)
{
	DWORD   Args[1];
	Args[0]=(DWORD)prop;
	return (iPTag) thunkIt(i_GetPropTypeT,NULL,Args);
}

Dll const char *  APIENTRY i_GetPropName(const iPropPtr prop)
{
	DWORD   Args[1];
	Args[0]=(DWORD)prop;
	return (const char *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetPropNameT,NULL,Args));
}

Dll iDAFSError  APIENTRY i_GetBoolProp(const iEntityPtr entity,const char *name,int4 *ip)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)ip;
	return (iDAFSError) thunkIt(i_GetBoolPropT,t23,Args);
}

Dll iDAFSError  APIENTRY i_GetUserPtrProp(const iEntityPtr entity,
	const char *name, void **d,iUserPtrFree *callp)
{
	DWORD   Args[4];
	iDAFSError err;
	UserPtrDataPtr dp;
	long callproc;

	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)d;
	Args[3]=(DWORD)&callproc;
	err = (iDAFSError) thunkIt(i_GetUserPtrPropT,t234,Args);
	if (!err && d && callproc == 1) {
		dp = (UserPtrDataPtr)*d;
		*d = dp->data;
		if (callp) *callp = dp->call;
	} else if (callp) *callp = 0;
	return err;
}

Dll iDAFSError  APIENTRY i_GetUserDataProp(const iEntityPtr entity,const char *name, void **d, long *size, iUserDataProc ConvertProc)
{
	DWORD   Args[5];
	iDAFSError err;
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)d;
	Args[3]=(DWORD)size;
	Args[4]=(DWORD)ConvertProc;
	
	err = (iDAFSError) thunkIt(i_GetUserDataPropT,t234,Args);
	if (d) *d = UTLinearToSelectorOffset(*d);
	return err;
}


Dll iDAFSError  APIENTRY i_GetStringProp(const iEntityPtr entity,const char *name, char **ip)
{
	DWORD   Args[3];
	iDAFSError err;
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)ip;
	
	err = (iDAFSError) thunkIt(i_GetStringPropT,t23,Args);
	if (ip) *ip = UTLinearToSelectorOffset(*ip);
	return err;	
}


Dll iDAFSError  APIENTRY i_GetUStringProp(const iEntityPtr entity,const char *name,iUCode **ip)
{
	DWORD   Args[3];
	iDAFSError err;
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	Args[2]=(DWORD)ip;
	
	err = (iDAFSError) thunkIt(i_GetUStringPropT,t23,Args);
	if (ip) *ip = (iUCode *)UTLinearToSelectorOffset((void*)*ip);
	return err;
}

Dll void  APIENTRY i_DisposeProp(const iEntityPtr entity,const char *name)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)name;
	thunkIt(i_DisposePropT,t2,Args);
}


Dll void  APIENTRY i_DisposeSessionProp(iTypeIndex itype,const char *name)
{
	DWORD   Args[2];
	Args[0]=(DWORD)itype;
	Args[1]=(DWORD)name;
	thunkIt(i_DisposeSessionPropT,t2,Args);
}


Dll void  APIENTRY i_DisposeAllProps(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	thunkIt(i_DisposeAllPropsT,NULL,Args);
}


Dll void  APIENTRY i_DisposeAllSessionProps(void)
{
	thunkIt(i_DisposeAllSessionPropsT,0,NULL);
}

Dll const char *  APIENTRY i_GetComment(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (const char *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetCommentT,NULL,Args));
}


Dll void  APIENTRY i_IntersectBox(iBox a,iBox b,iBox *c)
{
	DWORD   Args[3];
	Args[0]=(DWORD)&a;
	Args[1]=(DWORD)&b;
	Args[2]=(DWORD)c;
	thunkIt(i_IntersectBoxT,t123,Args);
}


Dll void  APIENTRY i_UnionBox(iBox b1,iBox b2, iBox *c)
{
	DWORD   Args[3];
	Args[0]=(DWORD)&b1;
	Args[1]=(DWORD)&b2;
	Args[2]=(DWORD)c;
	thunkIt(i_UnionBoxT,t123,Args);
}


Dll iDAFSError  APIENTRY i_SetComment(iEntityPtr entity,const char *comment)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)comment;
	return (iDAFSError) thunkIt(i_SetCommentT,t2,Args);
}


Dll iDTag  APIENTRY i_GetEDataType(iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iDTag) thunkIt(i_GetEDataTypeT,NULL,Args);
}


Dll const char *  APIENTRY i_GetText(const iEntityPtr entity,short *conf)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)conf;
	return (const char *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetTextT,t2,Args));
}


Dll iDAFSError  APIENTRY i_SetText(iEntityPtr entity,const char *text,short conf)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)text;
	Args[2]=(DWORD)conf;	
	return (iDAFSError) thunkIt(i_SetTextT,t2,Args);
}


Dll const iUCode *  APIENTRY i_GetUText(const iEntityPtr entity,short *conf)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)conf;
	return (const iUCode *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetUTextT,t2,Args));
}


Dll iDAFSError  APIENTRY i_SetUText(iEntityPtr entity,const iUCode *text,short conf)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)text;
	Args[2]=(DWORD)conf;
	return (iDAFSError) thunkIt(i_SetUTextT,t2,Args);
}

Dll short  APIENTRY i_GetConf(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (short) thunkIt(i_GetConfT,NULL,Args);
}

Dll void  APIENTRY i_SetConf(iEntityPtr entity,short conf)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)conf;
	thunkIt(i_SetConfT,NULL,Args);
}

Dll iPosSetPtr  APIENTRY i_GetPosSet(iEntityPtr entity, int2 *n)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)n;
	return (iPosSetPtr) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetPosSetT,t2,Args));
}

Dll iDAFSError  APIENTRY i_SetPosSet(iEntityPtr entity, int2 n, const iPosSetPtr chrs)
{
	DWORD   Args[3];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)n;
	Args[2]=(DWORD)chrs;
	return (iDAFSError) thunkIt(i_SetPosSetT,t3,Args);
}


Dll iImagePtr  APIENTRY i_NewImage(int2 w,int2 h,int2 res,iDAFSCompType comp)
{
	DWORD   Args[4];
	Args[0]=(DWORD)w;
	Args[1]=(DWORD)h;
	Args[2]=(DWORD)res;
	Args[3]=(DWORD)comp;
	return (iImagePtr) thunkIt(i_NewImageT,NULL,Args);
}


Dll void  APIENTRY i_SetImage(iEntityPtr entity,iImagePtr image)
{
	DWORD   Args[2];
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)image;
	thunkIt(i_SetImageT,NULL,Args);
}


Dll iImagePtr  APIENTRY i_GetImage(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iImagePtr) thunkIt(i_GetImageT,NULL,Args);
}


Dll iImagePtr  APIENTRY i_CloneEntityImage(const iEntityPtr entity)
{
	DWORD   Args[1];
	Args[0]=(DWORD)entity;
	return (iImagePtr) thunkIt(i_CloneEntityImageT,NULL,Args);
}


Dll int4  APIENTRY i_GetImageWidth(const iImagePtr image)
{
	DWORD   Args[1];
	Args[0]=(DWORD)image;
	return (int4) thunkIt(i_GetImageWidthT,NULL,Args);
}


Dll int4  APIENTRY i_GetImageHeight(const iImagePtr image)
{
	DWORD   Args[1];
	Args[0]=(DWORD)image;
	return (int4) thunkIt(i_GetImageHeightT,NULL,Args);
}


Dll iBox  APIENTRY i_GetEnclosingBox(const iEntityPtr entity)
{
	DWORD   Args[2];
	iBox retbox;
	
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)&retbox;
	thunkIt(i_GetEnclosingBoxT,t2,Args);
	return retbox;
}


Dll iDAFSError  APIENTRY i_SetBox(iEntityPtr entity, iBox bounds)
{
	DWORD   Args[2];
	
	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)&bounds;
	return (iDAFSError) thunkIt(i_SetBoxT,t2,Args);
}

Dll int2  APIENTRY i_GetImageRes(const iImagePtr image)
{
	DWORD   Args[1];
	Args[0]=(DWORD)image;
	return (int2) thunkIt(i_GetImageResT,NULL,Args);
}


Dll void  APIENTRY i_SetImageRes(const iImagePtr image, int2 resolution)
{
	DWORD   Args[2];
	Args[0]=(DWORD)image;
	Args[1]=(DWORD)resolution;
	thunkIt(i_SetImageResT,NULL,Args);
}


Dll iDAFSError  APIENTRY i_RotateImage(iImagePtr image, int2 rotation)
{
	DWORD   Args[2];
	Args[0]=(DWORD)image;
	Args[1]=(DWORD)rotation;
	return (iDAFSError) thunkIt(i_RotateImageT,NULL,Args);
}

Dll iDAFSError  APIENTRY i_ScaleImage(iImagePtr img, long xmag, long ymag)
{
	DWORD   Args[3];
	Args[0]=(DWORD)img;
	Args[1]=(DWORD)xmag;
	Args[2]=(DWORD)ymag;
	return (iDAFSError) thunkIt(i_ScaleImageT,NULL,Args);
}


Dll iImagePtr  APIENTRY i_BitMap2Image(iBox bounds,int2 dw, char *dp)
{
	DWORD   Args[3];
	Args[0]=(DWORD)&bounds;
	Args[1]=(DWORD)dw;
	Args[2]=(DWORD)dp;
	return (iImagePtr) thunkIt(i_BitMap2ImageT,t13,Args);
}


Dll iDAFSError  APIENTRY i_Image2BitMap(iImagePtr image,iBox bounds,long mag_exp, int2 *dataWid,char **bits)
{
	DWORD   Args[5];
	iDAFSError err;
	
	if (*bits) *bits = UTSelectorOffsetToLinear(*bits);
	Args[0]=(DWORD)image;
	Args[1]=(DWORD)&bounds;
	Args[2]=(DWORD)mag_exp;
	Args[3]=(DWORD)dataWid;
	Args[4]=(DWORD)bits;
	
	err = (iDAFSError) thunkIt(i_Image2BitMapT,t245,Args);
	*bits = UTLinearToSelectorOffset(*bits);
	return err;
}


Dll int4  APIENTRY i_ImageExt(const char *fileName)
{
	DWORD   Args[1];
	Args[0]=(DWORD)fileName;
	return (int4) thunkIt(i_ImageExtT,t1,Args);
}


Dll iDAFSError  APIENTRY i_ReadImage(const char *fileName,iImagePtr *image)
{
	DWORD   Args[2];
	Args[0]=(DWORD)fileName;
	Args[1]=(DWORD)image;
	return (iDAFSError) thunkIt(i_ReadImageT,t12,Args);
}


Dll iDAFSError  APIENTRY i_WriteImage(const char *fileName,iImagePtr image, iDAFSCompType comp)
{
	DWORD   Args[3];
	Args[0]=(DWORD)fileName;
	Args[1]=(DWORD)image;
	Args[2]=(DWORD)comp;
	return (iDAFSError) thunkIt(i_WriteImageT,t1,Args);
}


Dll void  APIENTRY i_DisposeImage(iImagePtr image)
{
	DWORD   Args[1];
	Args[0]=(DWORD)image;
	thunkIt(i_DisposeImageT,NULL,Args);
}


Dll iDAFSError  APIENTRY i_ReadUATable(iUATablePtr *table, char *filename)
{
	DWORD   Args[2];
	Args[0]=(DWORD)table;
	Args[1]=(DWORD)filename;
	return (iDAFSError) thunkIt(i_ReadUATableT,t12,Args);
}

Dll void  APIENTRY i_DisposeUATable(iUATablePtr table)
{
	DWORD   Args[1];

	Args[0]=(DWORD)table;
	thunkIt(i_DisposeUATableT,NULL,Args);
}


Dll char *  APIENTRY i_AllocASCIIText(iEntityPtr en, iUATablePtr table)
{
	DWORD   Args[2];

	Args[0]=(DWORD)en;
	Args[1]=(DWORD)table;
	return (char *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_AllocASCIITextT,NULL,Args));
}


Dll void  APIENTRY i_FreeASCIIText(char *text)
{
	DWORD   Args[1];
	Args[0]=(DWORD)text;
	thunkIt(i_FreeASCIITextT,t1,Args);
}


Dll iUCode *  APIENTRY i_AllocUText(iEntityPtr en)
{
	DWORD   Args[1];
	Args[0]=(DWORD)en;
	return (iUCode *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_AllocUTextT,NULL,Args));
}


Dll void  APIENTRY i_FreeUText(iUCode *text)
{
	DWORD   Args[1];
	Args[0]=(DWORD)text;
	thunkIt(i_FreeUTextT,t1,Args);
}


Dll iDAFSError  APIENTRY i_InitLib(void)
{
	return (iDAFSError) thunkIt(i_InitLibT,0,NULL);
}


Dll void  APIENTRY i_SetCallBack(iCallBackProc callBack,void *userData)
{
	DWORD   Args[2];
	
	dafs_cb = callBack;
	gUserData = userData;
	Args[0]=(DWORD)callBack;
	Args[1]=(DWORD)userData;
	thunkIt(i_SetCallBackT,NULL,Args);
}


Dll void  APIENTRY i_GetCallBack(iCallBackProc *callBack,void **userData)
{
	if (callBack) *callBack = dafs_cb;
	if (userData) *userData = gUserData;
}


Dll iDAFSError  APIENTRY i_DoCallBack(iMess mess,iEntityPtr entity)
{
	DWORD   Args[2];
	Args[0]=(DWORD)mess;
	Args[1]=(DWORD)entity;
	return (iDAFSError) thunkIt(i_DoCallBackT,NULL,Args);
}


Dll void  APIENTRY i_ExitLib(void)
{
	thunkIt(i_ExitLibT,NULL,NULL);
}

Dll iDAFSError  APIENTRY i_Read1Token(RafFile fo,iToken *t)
{
	DWORD   Args[2];
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	return (iDAFSError) thunkIt(i_Read1TokenT,t2,Args);
}

Dll iDAFSError  APIENTRY i_ReadToken(RafFile fo,iToken *t,int *swap)
{
	DWORD   Args[3];
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)swap;
	return (iDAFSError) thunkIt(i_ReadTokenT,t23,Args);
}


Dll iDAFSError  APIENTRY i_ReadSize(RafFile fo,iToken t,long *size, int *swap)
{
	DWORD   Args[4];
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)size;
	Args[3]=(DWORD)swap;
	return (iDAFSError) thunkIt(i_ReadSizeT,t34,Args);
}


Dll iDAFSError  APIENTRY i_ReadItem(RafFile fo,iToken t,uint2 space,uint2 *size,void *data, int *swap)
{
	DWORD   Args[6];
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)size;
	Args[3]=(DWORD)size;
	Args[4]=(DWORD)data;
	Args[5]=(DWORD)swap;
	return (iDAFSError) thunkIt(i_ReadItemT,t456,Args);
}

Dll iDAFSError  APIENTRY i_ReadItemAlloc(RafFile fo,iToken t,uint2 *size,void **data, int *swap)
{
	DWORD   Args[5];
	iDAFSError err;
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)size;
	Args[3]=(DWORD)data;
	Args[4]=(DWORD)swap;
	
	err = (iDAFSError) thunkIt(i_ReadItemAllocT,t345,Args);
	if (data ) *data = UTLinearToSelectorOffset(*data);
	return err;
}

Dll iDAFSError APIENTRY i_ReadItemAllocH(RafFile fo,iToken t,long *size,void HUGEPTR **data, int *swap)
{
	DWORD   Args[5];
	iDAFSError err;
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)size;
	Args[3]=(DWORD)data;
	Args[4]=(DWORD)swap;
	
	err = (iDAFSError) thunkIt(i_ReadItemAllocT,t345,Args);
	if (data ) *data = UTLinearToSelectorOffset(*data);
	return err;
}

Dll iDAFSError  APIENTRY i_WriteItem(RafFile fo,iToken t,uint2 size,const void *data)
{
	DWORD   Args[4];
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)size;
	Args[3]=(DWORD)data;
	return (iDAFSError) thunkIt(i_WriteItemT,t4,Args);
}

Dll iDAFSError  APIENTRY i_WriteItemH(RafFile fo,iToken t,long size,const void HUGEPTR *data)
{
	DWORD   Args[4];
	Args[0]=(DWORD)fo;
	Args[1]=(DWORD)t;
	Args[2]=(DWORD)size;
	Args[3]=(DWORD)data;
	return (iDAFSError) thunkIt(i_WriteItemHT,t4,Args);
}

Dll void APIENTRY i_SetImageCallBack(iImageWriteProc wp,iImageReadProc rp)
{
	DWORD   Args[2];
	
	imagewrite_cb = wp;
	imageread_cb = rp;
	Args[0]=(DWORD)wp;
	Args[1]=(DWORD)rp;
	thunkIt(i_SetImageCallBackT,NULL,Args);
}

Dll void APIENTRY i_GetImageCallBack(iImageWriteProc *wp,iImageReadProc *rp)
{
	if (wp) *wp = imagewrite_cb;
	if (rp) *rp = imageread_cb;
}

Dll char * APIENTRY i_GetImageTag(iImagePtr sp)
{
	DWORD   Args[1];
	Args[0]=(DWORD)sp;
	return (char *) UTLinearToSelectorOffset((LPBYTE)thunkIt(i_GetImageTagT,NULL,Args));
}

Dll iDAFSError APIENTRY i_SetImageTag(iImagePtr sp,char *tag)
{
	DWORD   Args[2];
	Args[0]=(DWORD)sp;
	Args[1]=(DWORD)tag;
	return (iDAFSError) thunkIt(i_SetImageTagT,t2,Args);
}

Dll iDAFSError APIENTRY i_AddDataToImage(iImagePtr im,char *data,int2 size)
{
	DWORD   Args[3];
	Args[0]=(DWORD)im;
	Args[1]=(DWORD)data;
	Args[2]=(DWORD)size;
	return (iDAFSError) thunkIt(i_AddDataToImageT,t2,Args);
}

Dll iDAFSError APIENTRY i_GetDataFromImage(iImagePtr im,int2 n,char **datap,int2 *sizep)
{
	DWORD   Args[4];
	iDAFSError err;
	Args[0]=(DWORD)im;
	Args[1]=(DWORD)n;
	Args[2]=(DWORD)datap;
	Args[3]=(DWORD)sizep;
	
	err = (iDAFSError) thunkIt(i_GetDataFromImageT,t34,Args);
	if (datap)
		*datap = UTLinearToSelectorOffset(*datap);
	return err;
}

Dll iDAFSError APIENTRY i_GetArea(const iEntityPtr entity,
				  void **data, iATag *tag)
{
	DWORD   Args[3];

	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)data;
	Args[2]=(DWORD)tag;
	return (iDAFSError) thunkIt(i_GetAreaT,t23,Args);
}

Dll iDAFSError APIENTRY i_ForceAreaType(const iEntityPtr entity, iATag tag)
{
	DWORD   Args[2];

	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)tag;
	return (iDAFSError) thunkIt(i_ForceAreaTypeT,NULL,Args);
}

static iBoxListPtr BoxListS2L(iBoxListPtr p)
{
	if (p) {
		p->rects = (BOX *)UTSelectorOffsetToLinear((void*)p->rects);
		p = (iBoxListPtr)UTSelectorOffsetToLinear((void*)p);
	}
	return p;
}

static void BoxListL2S(iBoxListPtr p)
{
	p->rects = (BOX *)UTLinearToSelectorOffset((void*)p->rects);
}

static iBoxListPtr BoxListPL2S(iBoxListPtr p)
{
	if (p) {
		p = (iBoxListPtr)UTLinearToSelectorOffset((void*)p);
		BoxListL2S(p);
	}
	return p;
}

Dll iDAFSError APIENTRY i_SetBoxList(iEntityPtr entity, iBoxListPtr BoxList)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)BoxListS2L(BoxList);
	e = (iDAFSError) thunkIt(i_SetBoxListT,NULL,Args);
	BoxListL2S(BoxList);
	return e;
}

static iOrthogonPtr OrthoS2L(iOrthogonPtr p)
{
	if (!p) return p;
	p->point = (iPoint *)UTSelectorOffsetToLinear((void*)p->point);
	return (iOrthogonPtr)UTSelectorOffsetToLinear((void*)p);
}

static iOrthogonListPtr OrthoListS2L(iOrthogonListPtr p)
{
	iOrthogonArrayPtr q;
	int i;
	
	if (!p) return p;
	i = p->num_poly;
	q = p->poly;
	while (i--) {
		*q = OrthoS2L(*q);
		q++;
	}
	p->poly = (iOrthogonArrayPtr)UTSelectorOffsetToLinear((void*)p->poly);
	p = (iOrthogonListPtr)UTSelectorOffsetToLinear((void*)p);
	return p;
}

static void OrthoL2S(iOrthogonPtr p)
{
	p->point = (iPoint *)UTLinearToSelectorOffset((void*)p->point);
}

static iOrthogonPtr OrthoPL2S(iOrthogonPtr p)
{
	if (!p) return p;
	p = (iOrthogonPtr)UTLinearToSelectorOffset((void*)p);
	OrthoL2S(p);
	return p;
}

static void OrthoListL2S(iOrthogonListPtr p)
{
	iOrthogonArrayPtr q;
	int i;
	
	q = p->poly = (iOrthogonArrayPtr)UTLinearToSelectorOffset((void*)p->poly);
	i = p->num_poly;
	while (i--) {
		*q = OrthoPL2S(*q);
		q++;
	}
}

static iOrthogonListPtr OrthoListPL2S(iOrthogonListPtr p)
{
	if (!p) return p;
	p = (iOrthogonListPtr)UTLinearToSelectorOffset((void*)p);
	OrthoListL2S(p);
	return p;
}

Dll iDAFSError APIENTRY i_SetOrthogonList(iEntityPtr entity,
										  iOrthogonListPtr ortho_list)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)entity;
	Args[1]=(DWORD)OrthoListS2L(ortho_list);
	e = (iDAFSError) thunkIt(i_SetOrthogonListT,NULL,Args);
	OrthoListL2S(ortho_list);
	return e;
}

/* convertors */
Dll iDAFSError APIENTRY i_BoxList2OrthogonList(iBoxListPtr reg,
											   iOrthogonListPtr *ortho_list)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)BoxListS2L(reg);
	Args[1]=(DWORD)ortho_list;
	e = (iDAFSError) thunkIt(i_BoxList2OrthogonListT,t2,Args);
	BoxListL2S(reg);
	*ortho_list = OrthoListPL2S(*ortho_list);
	return e;
}
Dll iDAFSError APIENTRY i_Box2OrthogonList(iBox box, iOrthogonListPtr *ortho_list)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)&box;
	Args[1]=(DWORD)ortho_list;
	e = (iDAFSError) thunkIt(i_Box2OrthogonListT,t12,Args);
	*ortho_list = OrthoListPL2S(*ortho_list);
	return e;
}
Dll iDAFSError APIENTRY i_Box2BoxList(iBox box, iBoxListPtr *boxes)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)&box;
	Args[1]=(DWORD)boxes;
	if (e = (iDAFSError) thunkIt(i_Box2BoxListT,t12,Args)) return e;
	*boxes = BoxListPL2S(*boxes);
	return e;
}
Dll iDAFSError APIENTRY i_OrthogonList2BoxList(iOrthogonListPtr ortho_list, iBoxListPtr *boxes)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)OrthoListS2L(ortho_list);
	Args[1]=(DWORD)boxes;
	if (e = (iDAFSError) thunkIt(i_OrthogonList2BoxListT,t2,Args)) return e;
	OrthoListL2S(ortho_list);
	*boxes = BoxListPL2S(*boxes);
	return e;
}
Dll iDAFSError APIENTRY i_Box2Orthogon(iBox box, iOrthogonPtr *ortho)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)&box;
	Args[1]=(DWORD)ortho;
	e = (iDAFSError) thunkIt(i_Box2OrthogonT,t12,Args);
	*ortho = OrthoPL2S(*ortho);
	return e;
}
Dll iDAFSError APIENTRY i_Orthogon2BoxList(iOrthogonPtr or, iBoxListPtr *boxes)
{
	DWORD   Args[2];
	iDAFSError e;

	Args[0]=(DWORD)OrthoS2L(or);
	Args[1]=(DWORD)boxes;
	if (e = (iDAFSError) thunkIt(i_Orthogon2BoxListT,t2,Args)) return e;
	OrthoL2S(or);
	*boxes = BoxListPL2S(*boxes);
	return e;
}

/* lossy convertors */
Dll void APIENTRY i_BoxList2Box(iBoxListPtr reg, iBox *box)
{
	DWORD   Args[2];

	Args[0]=(DWORD)BoxListS2L(reg);
	Args[1]=(DWORD)box;
	thunkIt(i_BoxList2BoxT,t2,Args);
	BoxListL2S(reg);
}

Dll void APIENTRY i_OrthogonList2Box(iOrthogonListPtr reg, iBox *box)
{
	DWORD   Args[2];

	Args[0]=(DWORD)OrthoListS2L(reg);
	Args[1]=(DWORD)box;
	thunkIt(i_OrthogonList2BoxT,t2,Args);
	OrthoListL2S(reg);
}

Dll void APIENTRY i_Orthogon2Box(iOrthogonPtr reg, iBox *box)
{
	DWORD   Args[2];

	Args[0]=(DWORD)OrthoS2L(reg);
	Args[1]=(DWORD)box;
	thunkIt(i_Orthogon2BoxT,t2,Args);
	OrthoL2S(reg);
}

Dll iOrthogonListPtr APIENTRY i_NewOrthogonList(void)
{
	return OrthoListPL2S((iOrthogonListPtr)thunkIt(i_NewOrthogonListT,NULL,NULL));
}
Dll iOrthogonListPtr APIENTRY i_CopyOrthogonList(iOrthogonListPtr original)
{
	DWORD   Args[1];

	Args[0]=(DWORD)OrthoListS2L(original);
	return OrthoListPL2S((iOrthogonListPtr)thunkIt(i_CopyOrthogonListT,NULL,Args));
}
Dll void APIENTRY i_DisposeOrthogonList(iOrthogonListPtr o_list)
{
	DWORD   Args[1];

	Args[0]=(DWORD)OrthoListS2L(o_list);
	thunkIt(i_DisposeOrthogonListT,NULL,Args);
}

Dll iDAFSError APIENTRY i_UnionOrthogonList(const iOrthogonListPtr o_list1,
					    const iOrthogonListPtr o_list2,
					    iOrthogonListPtr result)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)OrthoListS2L(o_list1);
	Args[1]=(DWORD)OrthoListS2L(o_list2);
	Args[2]=(DWORD)OrthoListS2L(result);
	e = (iDAFSError) thunkIt(i_UnionOrthogonListT,NULL,Args);
	OrthoListL2S(o_list1);
	OrthoListL2S(o_list2);
	OrthoListL2S(result);
	return e;
}
Dll iDAFSError APIENTRY i_IntersectOrthogonList(const iOrthogonListPtr o_list1,
					    const iOrthogonListPtr o_list2,
					    iOrthogonListPtr result)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)OrthoListS2L(o_list1);
	Args[1]=(DWORD)OrthoListS2L(o_list2);
	Args[2]=(DWORD)OrthoListS2L(result);
	e = (iDAFSError) thunkIt(i_IntersectOrthogonListT,NULL,Args);
	OrthoListL2S(o_list1);
	OrthoListL2S(o_list2);
	OrthoListL2S(result);
	return e;
}
Dll iDAFSError APIENTRY i_SubtractOrthogonList(iOrthogonListPtr o_list1,
						 iOrthogonListPtr o_list2)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)OrthoListS2L(o_list1);
	Args[1]=(DWORD)OrthoListS2L(o_list2);
	e = (iDAFSError) thunkIt(i_SubtractOrthogonListT,NULL,Args);
	OrthoListL2S(o_list1);
	OrthoListL2S(o_list2);
	return e;
}

/* orthogon functions */
Dll iOrthogonPtr APIENTRY i_NewOrthogon(int2 num)
{
	return OrthoPL2S((iOrthogonPtr) thunkIt(i_NewOrthogonT,NULL,NULL));
}
Dll void APIENTRY i_DisposeOrthogon(iOrthogonPtr ortho)
{
	DWORD   Args[1];

	Args[0]=(DWORD)OrthoS2L(ortho);
	thunkIt(i_DisposeOrthogonT,NULL,Args);
}
Dll   iOrthogonPtr APIENTRY i_CopyOrthogon(iOrthogonPtr shape)
{
	DWORD Args[1];
	iOrthogonPtr ret;

	Args[0]=(DWORD)OrthoS2L(shape);
	ret = OrthoPL2S((iOrthogonPtr)thunkIt(i_CopyOrthogonT,NULL,Args));
	OrthoL2S(shape);
	return ret;
}
Dll iDAFSError APIENTRY i_UnionOrthogon(const iOrthogonPtr o1,
					    const iOrthogonPtr o2,
					    iOrthogonListPtr result)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)OrthoS2L(o1);
	Args[1]=(DWORD)OrthoS2L(o2);
	Args[2]=(DWORD)OrthoListS2L(result);
	e = (iDAFSError) thunkIt(i_UnionOrthogonT,NULL,Args);
	OrthoL2S(o1);
	OrthoL2S(o2);
	OrthoListL2S(result);
	return e;
}
Dll iDAFSError APIENTRY i_IntersectOrthogon(const iOrthogonPtr o1,
					    const iOrthogonPtr o2,
					    iOrthogonListPtr result)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)OrthoS2L(o1);
	Args[1]=(DWORD)OrthoS2L(o2);
	Args[2]=(DWORD)OrthoListS2L(result);
	e = (iDAFSError) thunkIt(i_IntersectOrthogonT,NULL,Args);
	OrthoL2S(o1);
	OrthoL2S(o2);
	OrthoListL2S(result);
	return e;
}
Dll void APIENTRY i_TranslateOrthogon(iOrthogonPtr o1, short x, short y)
{
	DWORD   Args[3];

	Args[0]=(DWORD)OrthoS2L(o1);
	Args[1]=(DWORD)x;
	Args[2]=(DWORD)y;
	thunkIt(i_TranslateOrthogonT,NULL,Args);
	OrthoL2S(o1);
}

/* box list functions */
Dll iBoxListPtr APIENTRY i_NewBoxList(void)
{
	return BoxListPL2S((iBoxListPtr) thunkIt(i_NewBoxListT,NULL,NULL));
}
Dll void APIENTRY i_DisposeBoxList(iBoxListPtr a1)
{
	DWORD   Args[1];

	Args[0]=(DWORD)BoxListS2L(a1);
	thunkIt(i_DisposeBoxListT,NULL,Args);
}
Dll iBoxListPtr APIENTRY i_CopyBoxList(iBoxListPtr one)
{
	DWORD Args[1];

	Args[0]=(DWORD)BoxListS2L(one);
	return BoxListPL2S((iBoxListPtr) thunkIt(i_CopyBoxListT,NULL,Args));
}
Dll iDAFSError APIENTRY i_UnionBoxList(iBoxListPtr a1,
						iBoxListPtr a2, iBoxListPtr a3)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)BoxListS2L(a1);
	Args[1]=(DWORD)BoxListS2L(a2);
	Args[2]=(DWORD)BoxListS2L(a3);
	e = (iDAFSError) thunkIt(i_UnionBoxListT,NULL,Args);
	BoxListL2S(a1);
	BoxListL2S(a2);
	BoxListL2S(a3);
	return e;
}
Dll iDAFSError APIENTRY i_IntersectBoxList(iBoxListPtr a1,
						iBoxListPtr a2, iBoxListPtr a3)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)BoxListS2L(a1);
	Args[1]=(DWORD)BoxListS2L(a2);
	Args[2]=(DWORD)BoxListS2L(a3);
	e = (iDAFSError) thunkIt(i_IntersectBoxListT,NULL,Args);
	BoxListL2S(a1);
	BoxListL2S(a2);
	BoxListL2S(a3);
	return e;
}
Dll iDAFSError APIENTRY i_SubtractBoxList(iBoxListPtr a1,
						iBoxListPtr a2, iBoxListPtr a3)
{
	DWORD Args[3];
	iDAFSError e;

	Args[0]=(DWORD)BoxListS2L(a1);
	Args[1]=(DWORD)BoxListS2L(a2);
	Args[2]=(DWORD)BoxListS2L(a3);
	e = (iDAFSError) thunkIt(i_SubtractBoxListT,NULL,Args);
	BoxListL2S(a1);
	BoxListL2S(a2);
	BoxListL2S(a3);
	return e;
}
Dll iDAFSError APIENTRY i_TranslateBoxList(iBoxListPtr a1, short x, short y)
{
	DWORD   Args[3];
	iDAFSError err;

	Args[0]=(DWORD)BoxListS2L(a1);
	Args[1]=(DWORD)x;
	Args[2]=(DWORD)y;
	err = (iDAFSError) thunkIt(i_TranslateBoxListT,NULL,Args);
	BoxListL2S(a1);
	return err;
}

Dll iDAFSError APIENTRY CheckEntity(iEntityPtr entity)
{
	DWORD Args[1];

	Args[0]=(DWORD)entity;
	return (iDAFSError) thunkIt(CheckEntityT,NULL,Args);
}

Dll int4 APIENTRY rlcheckimage(iImagePtr sp)
{
	DWORD Args[1];

	Args[0]=(DWORD)sp;
	return (int4) thunkIt(rlcheckimageT,NULL,Args);
}

Dll int4 APIENTRY compare_image(iImagePtr one_image, iImagePtr two_image,
				int4 convert_flag)
{
	DWORD Args[3];

	Args[0]=(DWORD)one_image;
	Args[1]=(DWORD)two_image;
	Args[2]=(DWORD)convert_flag;
	return (iDAFSError) thunkIt(compare_imageT,NULL,Args);
}

Dll int4 APIENTRY
i_CompareEntitys(iEntityPtr zig, iEntityPtr zag,int4 convert_flag)
{
	DWORD Args[3];

	Args[0]=(DWORD)zig;
	Args[1]=(DWORD)zag;
	Args[2]=(DWORD)convert_flag;
	return (iDAFSError) thunkIt(i_CompareEntitysT,NULL,Args);
}

#endif
#ifdef W32SUT_32
iDAFSError APIENTRY
dafs_cb(iMess m, void *data, iEntityPtr en)
{
	DWORD Args[3];

	Args[0] = (DWORD) m;
	Args[1] = (DWORD) data;
	Args[2] = (DWORD) en;

	return CBThunkIt(Args, i_DAFSCallbackT);
}

iDAFSError APIENTRY
imagewrite_cb(char *tag,iImagePtr im)
{
	DWORD Args[2];

	Args[0] = (DWORD) tag;
	Args[1] = (DWORD) im;
	return CBThunkIt(Args, i_ImageWriteCallbackT);
}

iDAFSError APIENTRY
imageread_cb(char *tag,iImagePtr *im)
{
	DWORD Args[2];

	Args[0] = (DWORD) tag;
	Args[1] = (DWORD) im;
	return CBThunkIt(Args, i_ImageReadCallbackT);
}

void APIENTRY
userptrcallback(void *data)
{
	DWORD Args[1];
	
	Args[0] = (DWORD)data;
	CBThunkIt(Args,i_UserPtrCallbackT);
}

DWORD
i_Switch(LPDWORD lp, DWORD dwFunc)
{
	DWORD temp;

	switch( dwFunc ) {
	case i_GetErrorT:
		return (DWORD) i_GetError((iDAFSError)lp[0]);
	case i_InvertImageT:
		return (DWORD) i_InvertImage((iImagePtr )lp[0]);
	case i_MakeARegexT:
		return (DWORD) i_MakeARegex((iRegexPtr *)lp[0],( char *)lp[1]);
	case i_MakeURegexT:
		return (DWORD) i_MakeURegex((iRegexPtr *)lp[0],( iUCode *)lp[1]);
	case i_CheckARegexT:
		return (DWORD) i_CheckARegex((const iRegexPtr )lp[0],( const char *)lp[1],( int *)lp[2],(int *)lp[3]);
	case i_CheckURegexT:
		return (DWORD) i_CheckURegex((const iRegexPtr )lp[0],( const iUCode *)lp[1],( int *)lp[2],( int *)lp[3]);
	case i_WriteRegexT:
		return (DWORD) i_WriteRegex((const iRegexPtr )lp[0], (char *)lp[1]);
	case i_ReadRegexT:
		return (DWORD) i_ReadRegex((iRegexPtr *)lp[0],( char *)lp[1]);
	case i_DisposeRegexT:
		i_DisposeRegex((iRegexPtr) lp[0]);
		break;
	case i_ConcatImageT:
		return (DWORD) i_ConcatImage((iImagePtr )lp[0],(iImagePtr )lp[1]);
	case i_MoveAllPropsT:
		return (DWORD) i_MoveAllProps((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
	case i_CopyAllPropsT:
		return (DWORD) i_CopyAllProps((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
	case i_GetImageBoundsT:
		*(iBox *)lp[1]=i_GetImageBounds((const iImagePtr )lp[0]);
		break;
	case i_GetGlyphT:
		return (DWORD) i_GetGlyph((const iEntityPtr )lp[0],( short* )lp[1]);
	case i_SetGlyphT:
		i_SetGlyph((iEntityPtr) lp[0],(short) lp[1], (short) lp[2]);
		break;
	case i_GetShortT:
		return (DWORD) i_GetShort((const iEntityPtr)lp[0], (short*)lp[1]);
	case i_SetShortT:
		i_SetShort((iEntityPtr)lp[0], (int2)lp[1], (short)lp[2]);
		break;
	case i_SetNoDataT:
		i_SetNoData((iEntityPtr)lp[0]);
		break;
	case i_GetParentPathT:
		return (DWORD) i_GetParentPath((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
	case i_GetParentOtherPathT:
		return (DWORD) i_GetParentOtherPath((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
	case i_GetLongPropT:
		return (DWORD) i_GetLongProp((const iEntityPtr )lp[0],(const char *)lp[1],(long *)lp[2]);
	case i_NewEntityT:
		return (DWORD) i_NewEntity((const iTypeIndex )lp[0],(iEntityPtr )lp[1]);
	case i_NewEntityAfterT:
		return (DWORD) i_NewEntityAfter((const iTypeIndex )lp[0],(iEntityPtr )lp[1]);
	case i_NewEntityBeforeT:
		return (DWORD) i_NewEntityBefore((const iTypeIndex )lp[0],(iEntityPtr )lp[1]);
	case i_OrEntityT:
		return (DWORD) i_OrEntity((iEntityPtr *)lp[0]);
	case i_ShareEntityT:
		return (DWORD) i_ShareEntity((iEntityPtr )lp[0],(iEntityPtr )lp[1],(iEntityPtr *)lp[2]);
	case i_DisposeEntityT:
		i_DisposeEntity((iEntityPtr )lp[0]);
		break;
	case i_ReadEntityT:
		return (DWORD) i_ReadEntity((const char *)lp[0],(iEntityPtr *)lp[1]);
	case i_ReadEntityNoImT:
		return (DWORD) i_ReadEntityNoIm((const char *)lp[0],(iEntityPtr *)lp[1]);
	case i_WriteEntityT:
		return (DWORD) i_WriteEntity((const char *)lp[0],(const iEntityPtr )lp[1],( iDAFSWriteMode )lp[2]);
	case i_NameToTypeT:
		return (DWORD) i_NameToType((const char *)lp[0],(iTypeIndex*)lp[1]);
	case i_TypeToNameT:
		return (DWORD) i_TypeToName((iTypeIndex )lp[0]);
	case i_GetEntityTypeT:
		return (DWORD) i_GetEntityType((const iEntityPtr )lp[0]);
	case i_SetEntityTypeT:
		i_SetEntityType((iEntityPtr) lp[0],(iTypeIndex) lp[1]);
		break;
	case i_SetLogicalTypeT:
		i_SetLogicalType((iEntityPtr )lp[0],(iLogicalType )lp[1]);
		break;
	case i_GetLogicalTypeT:
		return (DWORD) i_GetLogicalType((iEntityPtr )lp[0]);
	case i_MoveToFrontT:
		i_MoveToFront((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
		break;
	case i_MoveToBackT:
		i_MoveToBack((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
		break;
	case i_MoveAfterT:
		i_MoveAfter((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
		break;
	case i_MoveBeforeT:
		i_MoveBefore((iEntityPtr )lp[0],( iEntityPtr )lp[1]);
		break;
	case i_GetNextEntityT:
		return (DWORD) i_GetNextEntity((const iEntityPtr )lp[0]);
	case i_GetPrevEntityT:
		return (DWORD) i_GetPrevEntity((const iEntityPtr )lp[0]);
	case i_GetChildEntityT:
		return (DWORD) i_GetChildEntity((const iEntityPtr )lp[0]);
	case i_GetLastEntityT:
		return (DWORD) i_GetLastEntity((const iEntityPtr )lp[0]);
	case i_GetParentEntityT:
		return (DWORD) i_GetParentEntity((const iEntityPtr )lp[0]);
	case i_CountChildrenT:
		return (DWORD) i_CountChildren((iEntityPtr )lp[0]);
	case i_CountSharesT:
		return (DWORD) i_CountShares((iEntityPtr )lp[0]);
	case i_CountPrevT:
		return (DWORD) i_CountPrev((iEntityPtr )lp[0]);
	case i_CountNextT:
		return (DWORD) i_CountNext((iEntityPtr )lp[0]);
	case i_FindEntityForwardT:
		return (DWORD) i_FindEntityForward((iEntityPtr )lp[0],( iTypeIndex )lp[1],( iSearchType )lp[2],( void *)lp[3]);
	case i_FindEntityBackT:
		return (DWORD) i_FindEntityBack((iEntityPtr )lp[0],( iTypeIndex )lp[1],( iSearchType )lp[2],( void *)lp[3]);
	case i_SetBoolPropT:
		return (DWORD) i_SetBoolProp((iEntityPtr)lp[0], (const char *)lp[1], (int)lp[2]);
	case i_SetLongPropT:
		return (DWORD) i_SetLongProp((iEntityPtr)lp[0], (const char *)lp[1], (int4)lp[2]);
	case i_SetStringPropT:
		return (DWORD) i_SetStringProp((iEntityPtr)lp[0], (const char *)lp[1], (char *)lp[2]);
	case i_SetUStringPropT:
		return (DWORD) i_SetUStringProp((iEntityPtr)lp[0], (const char *)lp[1], (iUCode *)lp[2]);
	case i_SetUserDataPropT:
		return (DWORD) i_SetUserDataProp((iEntityPtr)lp[0], (const char *)lp[1], (void *)lp[2], (uint2)lp[3]);
	case i_SetUserPtrPropT:
		if (lp[3])
			return (DWORD) i_SetUserPtrProp((iEntityPtr)lp[0], (const char *)lp[1],
					(void *)lp[2],userptrcallback);
		return (DWORD) i_SetUserPtrProp((iEntityPtr)lp[0], (const char *)lp[1],
				(void *)lp[2],0);
	case i_SetSessionBoolPropT:
		return (DWORD) i_SetSessionBoolProp((iTypeIndex)lp[0], (const char *)lp[1], (int4)lp[2]);
	case i_SetSessionLongPropT:
		return (DWORD) i_SetSessionLongProp((iTypeIndex)lp[0], (const char *)lp[1], (int4)lp[2]);
	case i_SetSessionStringPropT:
		return (DWORD) i_SetSessionStringProp((iTypeIndex)lp[0], (const char *)lp[1], (char *)lp[2]);
	case i_SetSessionUStringPropT:
		return (DWORD) i_SetSessionUStringProp((iTypeIndex)lp[0], (const char *)lp[1], (iUCode *)lp[2]);
	case i_SetSessionUserDataPropT:
		return (DWORD) i_SetSessionUserDataProp((iTypeIndex)lp[0], (const char *)lp[1], (void *)lp[2], (uint2)lp[3]);
	case i_SetSessionUserPtrPropT:
		if (lp[3])
			return (DWORD) i_SetSessionUserPtrProp((iTypeIndex)lp[0], (const char *)lp[1],
					(void *)lp[2],userptrcallback);
		return (DWORD) i_SetSessionUserPtrProp((iTypeIndex)lp[0], (const char *)lp[1],
				(void *)lp[2],0);
	case i_MovePropT:
		return (DWORD) i_MoveProp((iEntityPtr )lp[0],(iEntityPtr )lp[1],
						(const char *)lp[2]);
	case i_ChangePropNameT:
		return (DWORD) i_ChangePropName(iEntityPtr )lp[0],(const char*)lp[1],
						(const char*)lp[2]);	
	case i_FindPropT:
		return (DWORD) i_FindProp((const iEntityPtr )lp[0],( const char *)lp[1]);
	case i_FindPropNoTypesT:
		return (DWORD) i_FindPropNoTypes((const iEntityPtr )lp[0],( const char *)lp[1]);
	case i_FirstPropT:
		return (DWORD) i_FirstProp((const iEntityPtr )lp[0]);
	case i_NextPropT:
		return (DWORD) i_NextProp((const iEntityPtr )lp[0],( const iPropPtr )lp[1]);
	case i_GetPropSizeT:
		return (DWORD) i_GetPropSize((const iPropPtr )lp[0]);
	case i_GetPropTypeT:
		return (DWORD) i_GetPropType((const iPropPtr )lp[0]);
	case i_GetPropNameT:
		return (DWORD) i_GetPropName((const iPropPtr )lp[0]);
	case i_GetBoolPropT:
		return (DWORD) i_GetBoolProp((const iEntityPtr )lp[0],(const char *)lp[1],(int *)lp[2]);
	case i_GetUserPtrPropT:
		temp = (DWORD) i_GetUserPtrProp((const iEntityPtr )lp[0],(const char *)lp[1],( void **)lp[2],(iUserPtrFree*)lp[3]);
		if (*(iUserPtrFree*)lp[3] == userptrcallback)
			*(long*)lp[3] = 1;
		return temp;
	case i_GetUserDataPropT:
		return (DWORD) i_GetUserDataProp((const iEntityPtr )lp[0],(const char *)lp[1],( void **)lp[2],( long *)lp[3],( iUserDataProc )lp[4]);
	case i_GetStringPropT:
		return (DWORD) i_GetStringProp((const iEntityPtr )lp[0],(const char *)lp[1],( char **)lp[2]);
	case i_GetUStringPropT:
		return (DWORD) i_GetUStringProp((const iEntityPtr )lp[0],(const char *)lp[1],(iUCode **)lp[2]);
	case i_DisposePropT:
		i_DisposeProp((const iEntityPtr) lp[0],(const char *)lp[1]);
		break;
	case i_DisposeSessionPropT:
		i_DisposeSessionProp((iTypeIndex) lp[0],(const char *)lp[1]);
		break;
	case i_DisposeAllPropsT:
		i_DisposeAllProps((const iEntityPtr) lp[0]);
		break;
	case i_DisposeAllSessionPropsT:
		i_DisposeAllSessionProps();
		break;
	case i_GetCommentT:
		return (DWORD) i_GetComment((const iEntityPtr )lp[0]);
	case i_IntersectBoxT:
		i_IntersectBox(*(iBox *)lp[0],*(iBox *)lp[1],(iBox *)lp[2]);
		break;
	case i_UnionBoxT:
		i_UnionBox(*(iBox *)lp[0],*(iBox *)lp[1], (iBox *)lp[2]);
		break;
	case i_SetCommentT:
		return (DWORD) i_SetComment((iEntityPtr )lp[0],(const char *)lp[1]);
	case i_GetEDataTypeT:
		return (DWORD) i_GetEDataType((iEntityPtr )lp[0]);
	case i_GetTextT:
		return (DWORD) i_GetText((const iEntityPtr )lp[0],(short*)lp[1]);
	case i_SetTextT:
		return (DWORD) i_SetText((iEntityPtr )lp[0],(const char *)lp[1],(short)lp[2]);
	case i_GetUTextT:
		return (DWORD) i_GetUText((const iEntityPtr )lp[0],(short*)lp[1]);
	case i_SetUTextT:
		return (DWORD) i_SetUText((iEntityPtr )lp[0],(const iUCode *)lp[1],(short)lp[2]);
	case i_GetConfT:
		return (DWORD) i_GetConf((const iEntityPtr )lp[0]);
	case i_SetConfT:
		i_SetConf((iEntityPtr) lp[0],(short) lp[1]);
		break;
	case i_GetPosSetT:
		return (DWORD) i_GetPosSet((iEntityPtr )lp[0],(int2 *)lp[1]);
	case i_SetPosSetT:
		return (DWORD) i_SetPosSet((iEntityPtr )lp[0],(int2 )lp[1],(const iPosSetPtr )lp[2]);
	case i_NewImageT:
		return (DWORD) i_NewImage((int2)lp[0],(int2)lp[1],(int2)lp[2],(iDAFSCompType)lp[3]);
	case i_SetImageT:
		i_SetImage((iEntityPtr) lp[0],(iImagePtr) lp[1]);
		break;
	case i_GetImageT:
		return (DWORD) i_GetImage((const iEntityPtr )lp[0]);
	case i_CloneEntityImageT:
		return (DWORD) i_CloneEntityImage((const iEntityPtr )lp[0]);
	case i_GetImageWidthT:
		return (DWORD) i_GetImageWidth((const iImagePtr )lp[0]);
	case i_GetImageHeightT:
		return (DWORD) i_GetImageHeight((const iImagePtr )lp[0]);
	case i_GetEnclosingBoxT:
		*(iBox *)lp[1]=i_GetEnclosingBox((const iEntityPtr )lp[0]);
		break;
	case i_SetBoxT:
		return (DWORD) i_SetBox((iEntityPtr )lp[0],*(iBox *)lp[1]);
	case i_GetImageResT:
		return (DWORD) i_GetImageRes((const iImagePtr )lp[0]);
	case i_SetImageResT:
		i_SetImageRes((const iImagePtr) lp[0], (int2) lp[1]);
		break;
	case i_RotateImageT:
		return (DWORD) i_RotateImage((iImagePtr )lp[0],(int2 )lp[1]);
	case i_ScaleImageT:
		return (DWORD) i_ScaleImage((iImagePtr )lp[0],( long )lp[1],( long )lp[2]);
	case i_BitMap2ImageT:
		return (DWORD) i_BitMap2Image(*(iBox *)lp[0],(int2 )lp[1],( char *)lp[2]);
	case i_Image2BitMapT:
		return (DWORD) i_Image2BitMap((iImagePtr )lp[0],*(iBox *)lp[1],(long )lp[2],( int2 *)lp[3],(char **)lp[4]);
	case i_ImageExtT:
		return (DWORD) i_ImageExt((const char *)lp[0]);
	case i_ReadImageT:
		return (DWORD) i_ReadImage((const char *)lp[0],(iImagePtr *)lp[1]);
	case i_WriteImageT:
		return (DWORD) i_WriteImage((const char *)lp[0],(iImagePtr )lp[1],( iDAFSCompType )lp[2]);
	case i_DisposeImageT:
		i_DisposeImage((iImagePtr) lp[0]);
		break;
	case i_ReadUATableT:
		return (DWORD) i_ReadUATable((iUATablePtr *)lp[0],( char *)lp[1]);
	case i_DisposeUATableT:
		i_DisposeUATable((iUATablePtr) lp[0]);
		break;
	case i_AllocASCIITextT:
		return (DWORD) i_AllocASCIIText((iEntityPtr )lp[0],( iUATablePtr )lp[1]);
	case i_FreeASCIITextT:
		i_FreeASCIIText((char *)lp[0]);
		break;
	case i_AllocUTextT:
		return (DWORD) i_AllocUText((iEntityPtr )lp[0]);
	case i_FreeUTextT:
		i_FreeUText((iUCode *)lp[0]);
		break;
	case i_InitLibT:
		return (DWORD) i_InitLib();
	case i_SetCallBackT:
	    if (lp[0])
			i_SetCallBack(dafs_cb,(void *)lp[1]);
	    else
			i_SetCallBack(0,0);
	    break;
	case i_DoCallBackT:
		return i_DoCallBack((short)lp[0], (iEntityPtr)lp[1]);
	case i_ExitLibT:
		i_ExitLib();
		break;
	case i_Read1TokenT:
		return (DWORD) i_Read1Token((RafFile )lp[0],(iToken *)lp[1]);
	case i_ReadTokenT:
		return (DWORD) i_ReadToken((RafFile )lp[0],(iToken *)lp[1],(int *)lp[2]);
	case i_ReadSizeT:
		return (DWORD) i_ReadSize((RafFile )lp[0],(iToken )lp[1],(long *)lp[2],( int *)lp[3]);
	case i_ReadItemT:
		return (DWORD) i_ReadItem((RafFile )lp[0],(iToken )lp[1],(uint2 )lp[2],(uint2 *)lp[3],(void *)lp[4],( int *)lp[5]);
	case i_ReadItemAllocT:
		return (DWORD) i_ReadItemAlloc((RafFile )lp[0],(iToken)lp[1],(uint2 *)lp[2],(void **)lp[3],( int *)lp[4]);
	case i_ReadItemAllocHT:
		return (DWORD) i_ReadItemAllocH((RafFile )lp[0],(iToken)lp[1],(long *)lp[2],(void **)lp[3],( int *)lp[4]);
	case i_WriteItemT:
		return (DWORD) i_WriteItem((RafFile )lp[0],(iToken)lp[1],(uint2 )lp[2],(const void *)lp[3]);
	case i_WriteItemHT:
		return (DWORD) i_WriteItemH((RafFile )lp[0],(iToken)lp[1],(long )lp[2],(const void *)lp[3]);
	case i_SetImageCallBackT:
		i_SetImageCallBack((lp[0]?imagewrite_cb:0),(lp[1]?imageread_cb:0));
		break;
	case i_GetImageTagT:
		return (DWORD)i_GetImageTag((iImagePtr)lp[0]);
	case i_SetImageTagT:
		return (DWORD)i_SetImageTag((iImagePtr )lp[0],(char *)lp[1]);
	case i_AddDataToImageT:
		return (DWORD)i_AddDataToImage((iImagePtr )lp[0],(char *)lp[1],(int2 )lp[2]);
	case i_GetDataFromImageT:
		return (DWORD)i_GetDataFromImage((iImagePtr )lp[0],(int2 )lp[1],(char **)lp[2],(int2 *)lp[3]);
	case i_GetAreaT:
		return (DWORD)i_GetArea((const iEntityPtr)lp[0],(void **)lp[1],(iATag *)lp[2]);
	case i_ForceAreaTypeT:
		return (DWORD)i_ForceAreaType((const iEntityPtr)lp[0],(iATag)lp[1]);
	case i_SetBoxListT:
		return (DWORD)i_SetBoxList((iEntityPtr)lp[0],(iBoxListPtr)lp[1]);
	case i_SetOrthogonListT:
		return (DWORD)i_SetOrthogonList((iEntityPtr)lp[0],(iOrthogonListPtr)lp[1]);
	case i_BoxList2OrthogonListT:
		return (DWORD)i_BoxList2OrthogonList((iBoxListPtr)lp[0],(iOrthogonListPtr*)lp[1]);
	case i_Box2OrthogonListT:
		return (DWORD)i_Box2OrthogonList(*(iBox*)lp[0],(iOrthogonListPtr *)lp[1]);
	case i_Box2BoxListT:
		return (DWORD)i_Box2BoxList(*(iBox*)lp[0],(iBoxListPtr*)lp[1]);
	case i_OrthogonList2BoxListT:
		return (DWORD)i_OrthogonList2BoxList((iOrthogonListPtr)lp[0],(iBoxListPtr*)lp[1]);
	case i_Box2OrthogonT:
		return (DWORD)i_Box2Orthogon(*(iBox*)lp[0],(iOrthogonPtr*)lp[1]);
	case i_Orthogon2BoxListT:
		return (DWORD)i_Orthogon2BoxList((iOrthogonPtr)lp[0],(iBoxListPtr*)lp[1]);
	case i_BoxList2BoxT:
		i_BoxList2Box((iBoxListPtr)lp[0],(iBox*)lp[1]);
		break;
	case i_OrthogonList2BoxT:
		i_OrthogonList2Box((iOrthogonListPtr)lp[0],(iBox*)lp[1]);
		break;
	case i_Orthogon2BoxT:
		i_Orthogon2Box((iOrthogonPtr)lp[0],(iBox*)lp[1]);
		break;
	case i_NewOrthogonListT:
		return (DWORD)i_NewOrthogonList();
	case i_CopyOrthogonListT:
		return (DWORD)i_CopyOrthogonList((iOrthogonListPtr)lp[0]);
	case i_DisposeOrthogonListT:
		i_DisposeOrthogonList((iOrthogonListPtr)lp[0]);
		break;
	case i_UnionOrthogonListT:
		return (DWORD)i_UnionOrthogonList((iOrthogonListPtr)lp[0],(iOrthogonListPtr)lp[1],(iOrthogonListPtr)lp[2]);
	case i_IntersectOrthogonListT:
		return (DWORD)i_IntersectOrthogonList((iOrthogonListPtr)lp[0],(iOrthogonListPtr)lp[1],(iOrthogonListPtr)lp[2]);
	case i_SubtractOrthogonListT:
		return (DWORD)i_SubtractOrthogonList((iOrthogonListPtr)lp[0],(iOrthogonListPtr)lp[1]);
	case i_NewOrthogonT:
		return (DWORD)i_NewOrthogon((int2)lp[0]);
	case i_DisposeOrthogonT:
		i_DisposeOrthogon((iOrthogonPtr)lp[0]);
		break;
	case i_CopyOrthogonT:
		return (DWORD)i_CopyOrthogon((iOrthogonPtr)lp[0]);
	case i_UnionOrthogonT:
		return (DWORD)i_UnionOrthogon((iOrthogonPtr)lp[0],(iOrthogonPtr)lp[1],(iOrthogonListPtr)lp[2]);
	case i_IntersectOrthogonT:
		return (DWORD)i_IntersectOrthogon((iOrthogonPtr)lp[0],(iOrthogonPtr)lp[1],(iOrthogonListPtr)lp[2]);
	case i_TranslateOrthogonT:
		i_TranslateOrthogon((iOrthogonPtr)lp[0],(short)lp[1],(short)lp[2]);
		break;
	case i_NewBoxListT:
		return (DWORD)i_NewBoxList();
	case i_DisposeBoxListT:
		i_DisposeBoxList((iBoxListPtr)lp[0]);
		break;
	case i_CopyBoxListT:
		return (DWORD)i_CopyBoxList((iBoxListPtr)lp[0]);
	case i_UnionBoxListT:
		return (DWORD)i_UnionBoxList((iBoxListPtr)lp[0],(iBoxListPtr)lp[1],(iBoxListPtr)lp[2]);
	case i_IntersectBoxListT:
		return (DWORD)i_IntersectBoxList((iBoxListPtr)lp[0],(iBoxListPtr)lp[1],(iBoxListPtr)lp[2]);
	case i_SubtractBoxListT:
		return (DWORD)i_SubtractBoxList((iBoxListPtr)lp[0],(iBoxListPtr)lp[1],(iBoxListPtr)lp[2]);
	case i_TranslateBoxListT:
		return (DWORD)i_TranslateBoxList((iBoxListPtr)lp[0],(short)lp[1],(short)lp[2]);
	case CheckEntityT:
		return (DWORD)CheckEntity((iEntityPtr)lp[0]);
	case rlcheckimageT:
		return (DWORD)rlcheckimage((iImagePtr)lp[0]);
	case compare_imageT:
		return (DWORD)compare_image((iImagePtr)lp[0],(iImagePtr)lp[1], (int4)lp[2]);
	case i_CompareEntitysT:
		return (DWORD)i_CompareEntitys((iEntityPtr)lp[0],(iEntityPtr)lp[1], (int4)lp[2]);

	default:
		halt("bad function selector in dafs range\n");
	}
}
#endif
