/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "rafutil.h"
#include "str-lib.h"
#include "c-type.h"
#include "smemory.h"

#include "entity.h"
#include "dafs.h"

#ifndef NULL
#define NULL 0
#endif

/* static data */
static iCallBackProc saveCB;
/* global data */
iTypeTable Local;

void ClearID(const iEntityPtr me)
{
	iEntityPtr cur;
	
	me->data->id = 0;
	if (me->data->im)
		me->data->im->id = 0;
	for (cur = i_GetChildEntity(me);cur;cur = i_GetNextEntity(cur))
		ClearID(cur);
}

static void 
ClearIDAndCheckType(const iEntityPtr me, int4 *use)
{
	iEntityPtr cur;
	
	me->data->id = 0;
	use[i_GetEntityType(me)]=1;
	if (me->data->im)
		me->data->im->id = 0;
	for (cur = i_GetChildEntity(me);cur;cur = i_GetNextEntity(cur))
		ClearIDAndCheckType(cur, use);
}
	
static iEntityPtr 
FindEntityID(const iEntityPtr head,long id)
{
	iEntityPtr cur,en;
	
	if (head->data->id == id) return head;
	for (cur = i_GetChildEntity(head);cur;cur = i_GetNextEntity(cur))
		if (en = FindEntityID(cur,id)) return en;
	return 0;
}

static iImagePtr 
FindImageID(iGlobid *gid,long id)
{
	if (id >= gid->im_id)
		return 0;
	return gid->ims[id];
}

Dll iDAFSError APIENTRY
i_WriteItem(RafFile fo,iToken t,uint2 size,const void *data)
{
	char c;
	int4 ret,i4;
	
	if (!(t & iSMask) && size) {
		if (size < 64) t |= iOneSize;
		else if (size < 16384) t |= iTwoSize;
		else t |= iFourSize;
	}
	ret = riWrite((const char*)&t,1,1,fo);
	if (ret != 1) 
		return WriteError;
	switch (t & iSMask) {
	case iZeroSize:
		return DAFSOK;
	case iOneSize:
		/* could check to see if value is too big for char */
		c = (char) size;
		if (riWrite(&c,1,1,fo) != 1) return WriteError;
		break;
	case iTwoSize:
		/* could check to see if value is too big for short */
		if (riWrite((const char *)&size,2,1,fo) != 1) return WriteError;
		break;
	case iFourSize:
		i4 = (int4)size;
		if (riWrite((const char*)&i4,4,1,fo) != 1) return WriteError;
		break;
	}
	if (size && (riWrite(data,size,1,fo) != 1)) return WriteError;
	return DAFSOK;
}



Dll iDAFSError APIENTRY
i_WriteItemH(RafFile fo,iToken t,long size,const void HUGEPTR *data)
{
	char c;
	short s;
	int4 ret;
	
	if (!(t & iSMask) && size) {
		if (size < 64) t |= iOneSize;
		else if (size < 16384) t |= iTwoSize;
		else t |= iFourSize;
	}
	ret = riWrite((const char*)&t,1,1,fo);
	if (ret != 1) 
		return WriteError;
	switch (t & iSMask) {
	case iZeroSize:
		return DAFSOK;
	case iOneSize:
		/* could check to see if value is too big for char */
		c = (char) size;
		if (riWrite(&c,1,1,fo) != 1) return WriteError;
		break;
	case iTwoSize:
		/* could check to see if value is too big for short */
		s = (short) size;
		if (riWrite((const char *)&s,2,1,fo) != 1) return WriteError;
		break;
	case iFourSize:
		if (riWrite((const char*)&size,4,1,fo) != 1) return WriteError;
		break;
	}
	if (size && (riWrite(data,size,1,fo) != 1)) return WriteError;
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_Read1Token(RafFile fo,iToken *c)
{
	iDAFSError e;
	
	e = riRead((char*)c,1,1,fo);
	if (e != 1) return ReadError;
	return DAFSOK;
}

Dll short APIENTRY
i_SwapS(short s)
{
	char *f,*l;
	short r;
	
	l=(char *)&s;
	f=(char*)&r;
	f[0]=l[1];
	f[1]=l[0];
	return r;
}

Dll long APIENTRY
i_SwapL(long l)
{
	char *b,*e;
	long r;
	
	e = (char *)&l;
	b = (char *)&r;
	b[0]=e[3];
	b[1]=e[2];
	b[2]=e[1];
	b[3]=e[0];
	return r;
}

Dll void APIENTRY
i_SwapLArray(long *A, int4 num)
{
	while (num--) {
		*A=i_SwapL(*A);
		A++;
	}
}

Dll void APIENTRY
i_SwapSArray(short *A, int4 num)
{
	while (num--) {
		*A=i_SwapS(*A);
		A++;
	}
}

Dll iDAFSError APIENTRY
i_ReadSize(RafFile fo,iToken t,int4 *size, int *swap)
{
	char c;
	short s;
	long l;
	
	switch (t & iSMask) {
	case iZeroSize:
		*size = 0;
		break;
	case iOneSize:
		if (riRead(&c,1,1,fo) != 1) return ReadError;
		*size = c;
		break;
	case iTwoSize:
		if (riRead((char *)&s,2,1,fo) != 1) return ReadError;
		*size = s;
		if (*swap) *size = i_SwapS(s);
		break;
	case iFourSize:
		if (riRead((char*)size,4,1,fo) != 1) return ReadError;
		if (*swap) {
			l=*size;
			*size=i_SwapL(l);
		}
		break;
	}
	return DAFSOK;
}

void
i_SwapXPoints(iPoint *point,int4 n)
{
	while (n--){
		point->x = i_SwapS(point->x);
		point->y = i_SwapS(point->y);
		point++;
	}
}	

Dll void APIENTRY
i_SwapBox(iBox *b)
{
	b->x = i_SwapS(b->x);
	b->y = i_SwapS(b->y);
	b->w = i_SwapS(b->w);
	b->h = i_SwapS(b->h);
}

Dll void APIENTRY
i_SwapLBox(iLBox *b)
{
	b->x = i_SwapL(b->x);
	b->y = i_SwapL(b->y);
	b->w = i_SwapL(b->w);
	b->h = i_SwapL(b->h);
}

Dll void APIENTRY
i_SwapPos(iPosSetPtr p, int4 num) 
{
	int4 i;
	for (i=0; i<num; i++) {
		p[i].chr = i_SwapS(p[i].chr);
		p[i].conf = i_SwapS(p[i].conf);
	}
}

static void SwapIt(char t,long size,void *data)
{
	switch (t & iTMask) {
	case BeginEntity:
	case ImageID:
	case Long:
	case BeginImage:
	case BorrowEntity:
		*(long*)data = i_SwapL(*(long*)data);		
		break;
	case Box:
		i_SwapLBox(data);
		break;
	case DPosSet:
		i_SwapPos(data,size / sizeof(iPosSet));
		break;
	case D1PosSet:
		i_SwapPos(data,1);
		break;
	case Short:
		*(short*)data = i_SwapS(*(short*)data);
		break;
	case LongArray:
		i_SwapLArray(data,size / sizeof(long));
		break;
	case ShortNShort:
	case ShortArray:
		i_SwapSArray(data,size / sizeof(short));
		break;
	case Poly:
		i_SwapXPoints(data,size/sizeof(iPoint));
		break;
	case SBox:
		i_SwapBox(data);
		break;
	default:
		break;
	}
}

Dll iDAFSError APIENTRY
i_ReadItem(RafFile fo,iToken t, uint2 space, uint2 *size,void *data, int *swap)
{
	iDAFSError e;
	uint2 expected_size;
	int4 size_tmp;
	
	if (e = i_ReadSize(fo,t,&size_tmp,swap)) return e;
	if (space && size_tmp>(int4)space)
		return ReadError;

	*size=(uint2)size_tmp; /* will incorrectly report size if >65535 */
	if (!size_tmp) return DAFSOK; /* no size */
	if (!data) { /* skip it cause I didnt provide space */
		if (riSeek(fo,size_tmp,1)) return ReadError;
		return DAFSOK;
	}
	
	switch(t & iTMask){
		/* check that implicit size matches size -- */
	case BeginEntity:
	case ImageID:
	case Long:
	case BeginImage:
	case BorrowEntity:
		/* we are a long */
	case D1PosSet:
		/* we are a single pos -- short and short */
	case ShortNShort:
		/* we are a short and a short */
		expected_size = 4;
		break;
	case Box:
		/* we are a long box -- 16 */
		expected_size = 16;
		break;
	case Short:
	case ByteOrder:
		/* we are a short */
		expected_size = 2;
		break;
	case Byte:
		expected_size=1;
		break;
	case SBox:
		/* we are a box of shorts */
		expected_size = 8;
		break;
	default:
		expected_size=*size;
		break;
	}
	if(*size != expected_size)
		return ReadError;

	if (riRead(data,*size,1,fo) != 1) return ReadError;
	if ((t & AString) == AString)
		((char*)data)[*size-1] = 0;
	if (*swap) SwapIt(t,*size,data);
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_ReadItemAlloc(RafFile fo,iToken t,uint2 *size,void **data, int *swap)
{
	iDAFSError e;
	int4 size_tmp;
	
	*data=0;

	if (e = i_ReadSize(fo,t,&size_tmp,swap)) return e;
	if(size_tmp > MAX_SMALLOC)
		return ReadError;
	*size=(uint2)size_tmp;

	if (!*size) return DAFSOK;
	if (!data) {
		if (riSeek(fo,*size,1)) return ReadError;
		return DAFSOK;
	}

	*data = smalloc(*size);
	if (!*data) return MallocFailError;
	if (riRead(*data,*size,1,fo) != 1) {
		smfree(*data);
		*data = 0;
		return ReadError;
	}
	if (*swap) SwapIt(t,*size,*data);
	/* put a null at the end of strings */
	if ((t & AString) == AString)
		((char*)*data)[*size-1] = 0;
	return DAFSOK;
}

/*
// 3 cases for using this
// data == 0, we want to skip over this. it is still an error if it
// is not there
*/
Dll iDAFSError APIENTRY
i_ReadItemAllocH(RafFile fo,iToken t,long *size,void HUGEPTR **datap, int *swap)
{
	iDAFSError e=DAFSOK;
	long quan;
	void HUGEPTR *data=0;

	*datap=0;
	
	if (e = i_ReadSize(fo,t,size,swap)) goto error;
	if (!(quan = *size)) return DAFSOK;
	if (!datap) {
		if (riSeek(fo,quan,1)) return ReadError; /* huge seek should work */
		return DAFSOK;
	}
	data = smallocH(quan);
	if (!data) {e = MallocFailError; goto error; }
	if (quan && (riRead(data,quan,1,fo) != 1)) { e = ReadError; goto error; }
	if (*swap) SwapIt(t,quan,data);
	/* put a null at the end of strings */
	if ((t & AString) == AString)
		((char HUGEPTR *)data)[quan-1] = 0;
	*datap = data;
	data = 0;
error:
	if (data) smfreeH(data);
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_ReadToken(RafFile fo, iToken *c, int *swap)
{
	iDAFSError e;
	uint2 size;
	iToken t;
	short s;
	
	for (;;) {
		if (e = i_Read1Token(fo,c)) return e;
		t = *c & iTMask;
		if (t == ByteOrder) {
			if (e = i_ReadItem(fo,*c,sizeof(short),&size,&s,swap)) return e;
			if (swap) *swap = (s == 0x100);
		} else if (t >= UnknownToken1 && t <= UnknownToken2) {
			if (e = i_ReadItem(fo,*c,0,&size,0,swap)) return e;
		} else break;
	}
	return DAFSOK;
}

iDAFSError 
WriteProps(RafFile fo,iPropPtr props)
{
    iDAFSError e = DAFSOK;
    iPTag prop_type;
    const char *name;
    iPropPtr p;
    iPData *data;
	int wrote_header = FALSE;
	
    if (!props) return DAFSOK;

    p = i_FirstPropPtr(props);
    if(p)
		data = (iPData *) p->data;
    while (p) {
		prop_type = i_GetPropType(p);
		if (e) break;
		name = i_GetPropName(p);
		if (prop_type != iPUserPtr) { 
			if(name && wrote_header == FALSE) {
				if (e = i_WriteItem(fo,BeginProp,0,0)) {
					return e;
				}
				wrote_header = TRUE;
			}
			if (name && (e = i_WriteItem(fo,
										 TString,
										 strlen(name)+1,
										 name)))
				break;
		}
		switch (prop_type) {
		case iPNo:
		case iPAny:
			e = i_WriteItem(fo,NullTok,0,0);
			break;
		case iPString:
			e = i_WriteItem(fo,AString,strlen(data->val.s)+1,data->val.s);
			break;
		case iPUString:
			e = i_WriteItem(fo,ShortArray,(ucslen(data->val.us)+1)*sizeof(iUCode),data->val.us);
			break;
		case iPBool:
			e = i_WriteItem(fo,(data->val.i?BTrue:BFalse),0,NULL);
			break;
		case iPLong:
			e = i_WriteItem(fo,Long,sizeof(data->val.i),&data->val.i);
			break;
		case iPUserData:
			e = i_WriteItem(fo,Data,data->size,data->val.v);
			break;
		case iPUserIData:
			e = i_WriteItem(fo,IData,data->size,data->val.v);
			break;
		case iPUserPtr: e = DAFSOK;
			break;
		}
		if (e) break;
		p = i_NextPropPtr(props,p);
		if(p)
			data = (iPData *) p->data;
    }

	if(wrote_header && !e)
		e = i_WriteItem(fo,EndProp,0,0);

    return e;
}

#ifdef IMPLEMENT_DAFSA
static iDAFSError WriteDAFSAProps(RafFile fo,iPropPtr props)
{
	iDAFSError e = DAFSOK;
	iPTag prop_type;
	const char *name;
	iPropPtr p;
	iPData *data;
	
	if (!props) return DAFSOK;
	p = i_FirstPropertyPtr(props);
	if(p)
		data = (iPData *) p->data;
	while (p) {
		prop_type = i_GetPropertyType(p);
		if (e) break;
		name = i_GetPropertyName(p);
		if (!strcmp(name,"ATTRIBUTE"))
			riPuts(data->val.s, fo);
		else if (!strcmp(name,"ENTITY")) {
			riPuts(" ", fo);
			riPuts(data->val.s, fo);
		}
		p = i_NextPropertyPtr(props,p);
		if(p)
			data = (iPData *) p->data;
	}
	return e;
}
#endif

iDAFSError 
ReadProps(RafFile fo,iEntity *me, iGlobid *gid,iTypeIndex et)
{
	iPTag type;
	iDAFSError e;
	char *name=0,*s;
	iToken t;
	iUCode *us;
	uint2 size;
	void *v=0;
	int4 i;
	
	for (;;) {
		if (e = i_ReadToken(fo,&t, &gid->DAFSBSwap))
			goto error;
		if ((t & iTMask) == EndProp) break;
		if ((t & iTMask) != TString) {
			if (e=i_ReadItem(fo,t,0,&size,0,&gid->DAFSBSwap))
				goto error;
			continue;
		}
		if (e = i_ReadItemAlloc(fo,t,&size,(void **)&name,&gid->DAFSBSwap))
			goto error;
		if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap))
			goto error;
		switch (t & iTMask) {
		case NullTok: type = iPNo; break;
		case AString:
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&s, &gid->DAFSBSwap))
				goto error;
			v = (void*)s;
			type = iPString;
			break;
		case ShortArray:
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&us, &gid->DAFSBSwap))
				goto error;
			type = iPUString;
			v = (void*)us;
			break;
		case BTrue:
		case BFalse:
			type = iPBool;
			i = (t & iTMask) == BTrue;
			v = (void*)i;
			break;
		case Long:
			type = iPLong;
			if (e = i_ReadItem(fo,t,sizeof(long),&size,&v,&gid->DAFSBSwap))
				goto error;
			break;
		case Data:
			type = iPUserData;
			if (e = i_ReadItemAlloc(fo,t,&size,&v, &gid->DAFSBSwap))
				goto error;
			break;
		case IData:
			type = iPUserIData;
			if (e = i_ReadItemAlloc(fo,t,&size,&v,&gid->DAFSBSwap))	
				goto error;
			break;
		default:
			type = UnknownToken1;
			if (e = i_ReadItem(fo,t,0,&size,0,&gid->DAFSBSwap))
				goto error;
			break;
		}
		if (size || v) { /* don't create properties with no content */
			if (me) e= SetProperty(me,name,type,size,v,NULL,1);
#if 1
			/* reading and writing of type properties are disabled */
			else e= SetTypeProperty(et,name,type,size,v,NULL,1);
#endif
		}

error:
		if ((type == iPUserData || type == iPUserIData || type == iPString || type==iPUString) &&
			size && v){
			smfree(v);
			v = NULL;
		}

		if (name){
			smfree(name);
			name  = NULL;
		}
			
		if (e) return e;
	}
	return DAFSOK;
}

static iDAFSError 
WriteTypeTab(RafFile fo, int4 *use_list)
{
	iDAFSError e=DAFSOK;
	char *type;
	int4 i, num=1;
	
	for (i=0;i<i_GlobalType.size;i++) {
		if (use_list[i])
			if (type= i_GlobalType.table[i].name) { 
				use_list[i]=num++;
				e = i_WriteItem(fo,TString,strlen(type)+1,type);
#if 0
				if (!e && (prop = i_GlobalType.table[i].props))
					e = WriteProps(fo, prop);
#endif
				if (e)
					return e;
			}
	}
	return DAFSOK;
}

static iDAFSError  
WriteBinaryEntity(RafFile fo,const iEntityPtr me, iGlobid *gid, 
				  int4 *use_list)
{
	iDAFSError e;
	iDataPtr data = me->data;
	iEntityPtr cur;
	int2 i, n;
	char b;
	iBox box;
	BOX rect;
	long *tp;

	if (e = i_WriteItem(fo,BeginEntity,4,&gid->en_id)) return e;
	
	if (me->num) {
		if (!(tp = (long*)smalloc(sizeof(long)*me->num))) return MallocFailError;
		for (i=0;i<me->num;i++)
			tp[i] = (long)me->whichpos[i];
		if(e = i_WriteItem(fo, LongArray, sizeof(long)*me->num, tp)) return e;
		smfree(tp);
	}
	if (data->id) {
		if (e = i_WriteItem(fo,BorrowEntity,4,&data->id)) return e;
		gid->en_id++;
		e = i_WriteItem(fo,EndEntity,0,0);
		return e;
	}
	else
		data->id = gid->en_id++;
	b=(char)use_list[data->type]-1;
	if (e = i_WriteItem(fo,Byte,1,&b)) return e;
	
	switch(data->area.a_type){
	
	case iABox: 
		if (e = i_WriteItem(fo,SBox,sizeof(iBox),&data->area.a.box))
			return e;
		break;
	case iAOrthogonList:
		if (data->area.a.brdr->num_poly){
			for (i=0; i<data->area.a.brdr->num_poly; i++)
				if (n=data->area.a.brdr->poly[i]->n){
					if (e = i_WriteItem(fo,Poly,sizeof(iPoint)*n,
										data->area.a.brdr->poly[i]->point))
						return e;
				}
		}
		break;
	case iABoxList: 
		for (i=0; i<data->area.a.box_list->numRects; i++){
			rect=data->area.a.box_list->rects[i];
			box.x=rect.x1;
			box.y=rect.y1;
			box.w=rect.x2-rect.x1;
			box.h=rect.y2-rect.y1;
			if (e = i_WriteItem(fo,SBox,sizeof(iBox),&box))
				return e;
		}
		break;
	default:
		break;
	}	
	if (data->im) {
		if (data->im->id)
			e = i_WriteItem(fo,ImageID,sizeof(long),&data->im->id);
		else {
			data->im->id = gid->im_id++;
			e = WriteDAFSBImage(fo,data->im);
		}
		if (e) return e;
	}
	if (e = WriteProps(fo,me->data->props))
		return e;
	switch (data->dt) {
	case iDPosSet:
		e = i_WriteItem(fo,DPosSet,data->d.pp.len * sizeof(iPosSet),
						data->d.pp.posp);
		break;
	case iDConf:
		e = i_WriteItem(fo,Short,sizeof(short),&data->d.ps.conf);
		break;
	case iDShort:
		e = i_WriteItem(fo,ShortNShort,sizeof(struct iSData),&data->d.sd);
		break;
	case iD1PosSet:
		e = i_WriteItem(fo,D1PosSet,sizeof(iPosSet),&data->d.ps);
		break;
	case iDString:
		if (e = i_WriteItem(fo,Short,sizeof(short),&data->d.at.conf)) return e;
		e = i_WriteItem(fo,DString,data->d.at.len,data->d.at.text);
		break;
	case iDUString:
		if (e = i_WriteItem(fo,Short,sizeof(short),&data->d.ut.conf)) return e;
		e = i_WriteItem(fo,ShortArray,data->d.ut.len * sizeof(short),
						data->d.ut.unitext);
		break;
	case iDUserData:
		if (e = i_WriteItem(fo,Short,sizeof(short),&data->d.ud.conf)) return e;
		e = i_WriteItem(fo,Data,data->d.ud.len,data->d.ud.user);
		break;
	case iDNo: break;
	default: *(char*)0 = 0;
	}
	if (e) return e;
	
	if (data->child) {
		if (data->ct == iAnd) e = i_WriteItem(fo,BTrue,0,0);
		else e = i_WriteItem(fo,BFalse,0,0);
		if (e) return e;
	}
	for (cur = i_GetChildEntity(me);cur;cur = i_GetNextEntity(cur))
		if (e = WriteBinaryEntity(fo,cur,gid, use_list))
			return e;
	
	if (e = i_WriteItem(fo,EndEntity,0,0)) return e;
	return DAFSOK;
}
#if 0
static iDAFSError 
read_header(RafFile fi,char **gp,int4 *ngp,char *doctype)
{	
	char *p,buf[300],*s;

	int4 n=0;
	
	if(!riGets(buf,300,fi)) return ReadError;
	s = &buf[strlen(buf)-1];
	while (*s == '\n' || *s == '\r') *s-- = 0;
	while(buf[0]=='<' && buf[1]=='!') {
		if(strstr(&buf[2],"DOCTYPE")) {
			if(p=strchr(buf,' ')) p++;
			strcpy(doctype,p);
		} else if (strstr(&buf[2],"ENTITY")){
			if(p=strchr(buf,' ')) p++;
			s=(char*)smalloc(strlen(p)+1);
			if(!s)
				return MallocFailError;
			strcpy(s,p);
			*gp++=s;
			n++;
		}
		riGets(buf,300,fi);
		s = &buf[strlen(buf)-1];
		while (*s == '\n' || *s == '\r') *s-- = 0;
	}
	*ngp=n;
	if(buf[0] != ']' || buf[1] != '>') 
		return ReadError;
	return DAFSOK;
}
#endif
#ifdef IMPLEMENT_DAFSA
static iDAFSError ReadDAFSAProps(iEntityPtr en, char *token,char **globprops,int4 ngp)
{
	char *s,*p,*props[20],*eprops[20],**ep,**pp;
	int4 i,hit=0,minlen=3;
	iDAFSError err=DAFSOK;
	
	ep=eprops;
	pp=props;
	while(s=strchr(token,' '))
	{	
		p=strchr(token,'=');
		p++;*s++=0;
		if(strlen(p)>minlen)
			for(i=0;i<ngp;i++){
				if(strstr(globprops[i],p)) {
					*ep=s;
					ep++;
					hit=1;
					break;
				}
			}
		if(!hit)
			*pp++=s;
		token=s;
	}
	*ep=0;
	*pp=0;
	ep=eprops;pp=props;
	while(!err && *ep){
		err=i_SetStringProp(en,"ENTITY",(void*)*ep);
		ep++;
	}
	while(!err && *pp){
		err=i_SetStringProp(en,"ATTRIBUTE",(void*)*pp);
		pp++;
	}
	return err;
}
#endif
#if 0
static int4 
gettoken(RafFile fi, char *buf, int4 *pos,char *text)
{
	char *s;
	int4 ret;
	
	if(!buf[*pos]) {
		if(!riGets(buf,350,fi)) return 0;
		s = &buf[strlen(buf)-1];
		while (*s == '\n' || *s == '\r') *s-- = 0;
		*pos=0;
	} 
	if(buf[*pos]=='<')	{
		if (buf[*pos+1]=='/') {
			strcpy(text,&buf[*pos+2]);
			if(s = strchr(text,'>'))*s=0;
			(*pos)+=(strlen(text)+3);
			ret = 1;
		} else if (buf[*pos+1] == '!') {
			strcpy(text,&buf[*pos+2]);
			if(s=strchr(text,'>'))*s=0;
			*pos+=(strlen(text)+3);
			ret= 4;
		} else {
			strcpy(text,&buf[*pos+1]);
			if(s=strchr(text,'>'))*s=0;
			*pos+=(strlen(text)+2);
			if(strstr(text,"graphic")) 
				ret=5;
			else
				ret = 2;
		}
	} else {
		strcpy(text,&buf[*pos]);
		if(s=strchr(text,'<')) *s=0;
		(*pos)+=strlen(text);
		ret= 3;	
	}
	return ret;
}
#endif
#ifdef IMPLEMENT_DAFSA
/* return values :*/
/* 1 means endtoken */
/* 2 means begin token */
/* 3 means more text */
/* 4 mean comment */
/* 5 means graphic */
static iDAFSError ReadDAFSAEntity(RafFile fi,iEntityPtr *doc)
{
	iDAFSError e=DAFSOK;
	iEntityPtr cur=0,null;
	int4 i,res,pos=0,ngp,pcdata,textadded=0,addnull;
	char buf[350],*s, name[20],text[300],*globprops[100];
	char doctype[300];
	
	*buf=0;*globprops=0;
	e=read_header(fi,globprops, &ngp, doctype);
	if(e) 
		return e;
	pcdata=i_NameToType("#PCDATA");	
	while (res=gettoken(fi,buf,&pos,text))
	{	
		switch(res) {
		case 5:
			strcpy(name,text);
			if(s=(char*)strchr(name,' ')) 
				*s=0;
			cur=i_NewEntity(i_NameToType(name),cur);
			if(!cur) return MallocFailError;
			e=ReadDAFSAProps(cur,text,globprops,ngp);
			if(e) 
				goto error;
		case 1:
			if(strcmp(i_TypeToName(i_GetEntityType(cur)),text)){
				e=ReadError;
				goto error;
			}
			*doc=cur;
			cur=i_GetParentEntity(cur);
			textadded=0;
			break;
		case 2:
			if (i_GetText(cur,0)) {
				null=i_NewEntity(iNullT,cur);
				i_SetText(null,i_GetText(cur,0),0);
				i_SetText(cur,0,0);
				addnull=1;
			} else addnull=0;
			strcpy(name,text);
			if(s=(char*)strchr(name,' '))
				*s=0;
			cur=i_NewEntity(i_NameToType(name),cur);
			if(!cur) goto error;
			e=ReadDAFSAProps(cur,text,globprops,ngp);
			if(e) goto error;
			textadded=0;
			break;
		case 3:
			if(addnull){
				null=i_NewEntity(iNullT,cur);
				if(!null) e=MallocFailError;
			} else 
				null=cur;
			if(textadded)
				e=i_AddChar(null,-1,'\n');
			e=i_AddChars(null,-1,text);
			if(e) 
				goto error;
			textadded=1;
			break;
		case 4:
			cur=i_NewEntity(iCommentT,cur);
			if(!cur) goto error;
			e=i_AddChars(cur,-1,text);
			if (e)
				goto error;
			cur=i_GetParentEntity(cur);
			break;
		default:
			e=ReadError;
			goto error;
			break;
		}
	}
	e=i_SetStringProp(*doc,"DOCTYPE",(void*)doctype);
error:
	if(*globprops)
		for(i=0;i<ngp;i++)
			smfree(globprops[i]);
	if(e && *doc)
		i_DisposeEntity(*doc);	
	return e;
}
#endif

#ifdef IMPLEMENT_DAFSA
static iDAFSError WriteDAFSAEntity(RafFile fo,const iEntityPtr me,
		iGlobid *gid,iUATablePtr table)
{
	iDAFSError e;
	iDataPtr data = me->data;
	iEntityPtr cur;
	int4 wrotestuff=0,itype;
	const char *type;
	char *atext;
	char buf[2];
	
	itype=i_GetEntityType(me);
	if (itype==iCommentT) {
		if (riPuts("<!", fo) == REOF) 
			return WriteError;
		if (data->d.text && (riPuts(data->d.text, fo) == REOF)) 
			return WriteError;
		if (riPuts(">\n", fo) == REOF) 
			return WriteError;
	} else {
		type=i_TypeToName(itype);
		if(itype!=iNullT){
			if ((riPuts("<", fo) == REOF) || (riPuts(type, fo) == REOF)) 
				return WriteError;
			if (e = WriteDAFSAProps(fo,me->data->props)) return e;
			if (riPuts(">", fo) == REOF) return WriteError;
		}
		if (!(cur = i_GetChildEntity(me))) {
			switch (data->dt) {
			case iD1PosSet:
			case iDPosSet:
				wrotestuff = 1;
				buf[0] = i_GetGlyph(me,0);
				buf[1] = 0;
				if (riPuts(buf, fo) == REOF) return WriteError;
				break;
			case iDString:
				wrotestuff = 1;
				if (riPuts(data->d.at.text, fo) == REOF) return WriteError;
				break;
			case iDUString:
				wrotestuff = 1;
				atext=i_AllocASCIIText(me, table);
				if(riPuts(atext, fo) == REOF) return WriteError;
				i_FreeASCIIText(atext);
				break;
			case iDUserData:
			case iDShort:
			case iDConf:
			case iDNo: break;
			default: *(char*)0 = 0;
			}
		} else {
			wrotestuff = 1;
			for (;cur;cur = i_GetNextEntity(cur))
				if (e = WriteDAFSAEntity(fo,cur,gid,table))
					return e;
		}
		if (wrotestuff && (itype!=iNullT){
			int4 ok = 1;
			ok = ok && riPuts("</", fo) != REOF;
			ok = ok && riPuts(type, fo) != REOF;
			ok = ok && riPuts(">\n", fo) != REOF;
			if(!ok)
				return WriteError;
	}
	return DAFSOK;
}
#endif

static iDAFSError SkipImage(RafFile fo, iGlobid *gid)
{
	iDAFSError e;
	uint2 size;
	iToken t;
	
	if (e = i_ReadItem(fo,BeginImage | iOneSize,0,&size,0,&gid->DAFSBSwap)) return e;
	if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap)) return e;
	while ((t & iTMask) != EndImage) {
		if (e = i_ReadItem(fo,t,0,&size,0,&gid->DAFSBSwap)) return e;
		if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap)) return e;
	}
	return DAFSOK;
}

void 
DisposeEntityData(iEntityPtr entity)
{	
	iCallBackProc save;
	iEntityPtr cur,n;

	if (entity->data->im && entity->data->use <= 1)
		i_DisposeImage(entity->data->im);

	if (gCallBack) 
		gCallBack(mDestroy,gUserData,entity);
	i_DisposeArea(entity);
	for (cur = entity->data->child; cur; cur = n) {
		n = cur->next;
		i_DisposeEntity(cur);
	}

	if (gCallBack) save = gCallBack;
	else save = 0;
	gCallBack = 0;

	i_DisposeAllProps(entity);
	i_SetGlyph(entity,0,0);
	if (entity->data->matrs)
		smfree(entity->data->matrs);
	smfree(entity->data);
	if (save) gCallBack = save;
}

#define DISABLE_SHARE

#ifndef DISABLE_SHARE
static iDAFSError 
Share_Entity( iEntityPtr entity, iEntityPtr head)
{
	iDataPtr dp;
	iEntityPtr *temp;
	
	if(!head||!entity||!entity->data) return NotEnoughData;
	
	DisposeEntityData(head);
	dp=entity->data;
	dp->use++;
	
	temp = (iEntityPtr*)smrealloc(dp->matrs,sizeof(iEntityPtr) * dp->use);
	if (!temp)
		return MallocFailError;
	dp->matrs = temp;
	dp->matrs[dp->use-1] = head;

	head->data=dp;
	
	return DAFSOK;
}
#else
static iDAFSError 
Copy_Entity(iEntityPtr entity, iEntityPtr head)
{
  iEntityPtr new_entity;
  iDAFSError err;

  if(!head||!entity||!entity->data) return NotEnoughData;

  err = i_CopyEntity(entity, 0, &new_entity);
  if(err == DAFSOK) {
	iDataPtr dp;
	iEntityPtr *temp;
	
	DisposeEntityData(head);
	dp=new_entity->data;
	smfree(new_entity);
	
	temp = (iEntityPtr*)smrealloc(dp->matrs,sizeof(iEntityPtr) * dp->use);
	if (!temp)
		return MallocFailError;
	dp->matrs = temp;
	dp->matrs[dp->use-1] = head;

	head->data=dp;
  }
	
  return err;
}
#endif

/*
// bogus -- here is where I check for
// long and readtoken ?
*/
static iDAFSError 
ReadBinaryEntity(RafFile fo,iEntityPtr *head,iEntityPtr parent, 
				 int4 offset, iGlobid *gid)
{
	iDAFSError e;
	char *type=0;
	iToken t;
	char *type_str;
	uint2 size;
	long id,*tp=0;
	iEntityPtr me;
	iDataPtr data;
	short s;
	iBox box;
	iLBox lbox;
	iOrthogonPtr poly=0;
	int i;

	if (!*head) *head=0;
	me = i_NewEntity(iNullT,parent);
	if (!me) return MallocFailError;
	if (!*head) *head = me;
	data = me->data;
	if (e = i_ReadItem(fo,BeginEntity | iOneSize,sizeof(data->id),&size,&data->id,&gid->DAFSBSwap))
		goto error;
	
	for (t = NullTok;t != EndEntity;) {
		if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap))
			goto error;

		switch (t & iTMask) {
		default:
			if (e = i_ReadItem(fo,t,0,&size,0,&gid->DAFSBSwap))
				goto error;
			break;
		case EndEntity:
			break;
		case BTrue:
		case BFalse:
			data->ct = (t == BTrue?iAnd:iOr);
			break;
		case SBox:
			if (e = i_ReadItem(fo,t,sizeof(box),&size,&box,&gid->DAFSBSwap))
				goto error;
			if(box.w == 0)
			  break;
			if((box.w < 0) || (box.h < 0)) {
			  e = BadBoundsError;
			  goto error;
			}
			if (e=i_AppendBox(me, box)) {
			  goto error;
			}
			break;
		case Box:
			if (e = i_ReadItem(fo,t,sizeof(lbox),&size,&lbox,&gid->DAFSBSwap))
				goto error;
			box.x=(short)lbox.x;
			box.y=(short)lbox.y;
			box.w=(short)lbox.w;
			box.h=(short)lbox.h;
			if (e=i_SetBox(me, box))	
				goto error;
			break;
		case Poly:
			poly = i_NewOrthogon(0);
			if (!poly) {
				e=MallocFailError;
				goto error;
			}
			poly->point=0;
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&poly->point,&gid->DAFSBSwap))
				goto error;
			if (size%sizeof(iPoint)) {
				e=ReadError;
				goto error;
			}
			poly->n = (int2)(size/sizeof(iPoint));
			if (e=i_AppendOrthogon(me, poly))
				goto error;
			poly=0;

			break;
		case Byte:
			if (e = i_ReadItem(fo,t,sizeof(data->type),&size,&data->type,&gid->DAFSBSwap))
				goto error;

			/* convert entity type to global table */
			type_str = (char*)i_PTypeToName(&Local,me->data->type);/* when files are converted */
			if (type_str) {
				 if (e=i_NameToType(type_str,&me->data->type))
				 	goto error;
			} else
				me->data->type=0; 
			break;
		case TString:
			if (e = i_ReadItemAlloc(fo,t,&size,(void **)&type,&gid->DAFSBSwap))
				goto error;
			if (e = i_PNameToType(&Local,type,&data->type))
				goto error;
			smfree(type);
			type=0;
			
			break;
		case BeginProp:
			if (e = ReadProps(fo,me,gid,0))
				goto error;
			break;
		case ImageID:
			if (e = i_ReadItem(fo,t,sizeof(id),&size,&id,&gid->DAFSBSwap))
				goto error;	
			if (data->im && data->use == 1) 
				i_DisposeImage(data->im);
			if (gid->im_flag) {
				if (!(data->im = FindImageID(gid,id))) {
					e = ReadError;
					goto error;
				}
				if(data->use == 1) data->im->use++;
			}
			break;
		case BeginImage:
			if (gid->im_flag) {
				if (data->im && data->use == 1)
					i_DisposeImage(data->im);
				data->im = 0;
				if (e = ReadDAFSBImage(fo,&data->im, gid))
					goto error;
				if(data->use == 1) data->im->use = 1;
			} else
				if (e = SkipImage(fo, gid))
					goto error;
			break;
		case DString:
			i_FreeContents(me);
			data->dt=iDNo;
			data->d.at.conf = data->d.ps.conf;
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&data->d.at.text, &gid->DAFSBSwap))
				goto error;
			data->dt = iDString;
			data->d.at.len = size;
			break;
		case ShortArray:
			i_FreeContents(me);
			data->dt=iDNo;
			data->d.ut.conf = data->d.ps.conf;
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&data->d.ut.unitext, &gid->DAFSBSwap))
				goto error;
			data->dt = iDUString;
			data->d.ut.len = size / sizeof(short); /* data->num is short size is long */
			break;
		case Data:
			i_FreeContents(me);
			data->dt=iDNo;
			data->d.ud.conf = data->d.ps.conf;
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&data->d.ud.user,&gid->DAFSBSwap))
				goto error;
			data->dt = iDUserData;
			data->d.ud.len = size; /* bogus mix of short and long */
			break;
		case DPosSet:
			i_FreeContents(me);
			data->dt=iDNo;
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&data->d.pp.posp, &gid->DAFSBSwap))
				goto error;
			data->dt = iDPosSet;
			data->d.pp.len = size / sizeof(iPosSet);
			break;
		case D1PosSet:
			i_FreeContents(me);
			if (e = i_ReadItem(fo,t,sizeof(data->d.ps),&size,&data->d.ps,&gid->DAFSBSwap))
				goto error;
			data->dt = iD1PosSet;
			break;
		case Short:
			i_FreeContents(me);
			if (e = i_ReadItem(fo,t,sizeof(s),&size,&data->d.ps.conf,&gid->DAFSBSwap))
				goto error;
			data->dt = iDConf;
			break;
		case ShortNShort:
			i_FreeContents(me);
			if (e = i_ReadItem(fo,t,sizeof(data->d.sd),&size,&data->d.sd,&gid->DAFSBSwap))
				goto error;
			data->dt = iDShort;
			break;
		case BeginEntity:
			if (e = ReadBinaryEntity(fo,head,me, offset,gid))
				goto error;
			break;
		case BorrowEntity:
			if (e = i_ReadItem(fo,t,sizeof(id),&size,&id,&gid->DAFSBSwap))
				goto error;
#ifndef DISABLE_SHARE
			if (e = Share_Entity(FindEntityID(*head,id),me))
				goto error;
#else
			if (e = Copy_Entity(FindEntityID(*head,id),me))
				goto error;
#endif
			break;
		case LongArray:
			if (me->whichpos) {
				smfree(me->whichpos);
				me->whichpos=0;
			}
			if (e = i_ReadItemAlloc(fo,t,&size,(void**)&tp,&gid->DAFSBSwap))
				goto error;
			me->num = size/sizeof(long);
			if (!(me->whichpos = (int2*)smalloc(sizeof(int2) * me->num)))
				{ e = MallocFailError; goto error; }
			for (i=0;i<me->num;i++)
				me->whichpos[i] = (int2)tp[i];
			smfree(tp);
			tp = 0;
			break;
		}
	}
	if (saveCB)
		saveCB(mCreate,gUserData,me);

error:
	if (poly) i_DisposeOrthogon(poly);
	if (type) smfree(type);
	if (tp) smfree(tp);
	return e;
}

static iDAFSError 
WriteENTITY(RafFile fo,iEntityPtr doc)
{
	iEntityPtr cur;
	char *str;
	iDAFSError e = DAFSOK;
	char *temp;
	int4 i;
	
	if (!i_GetStringProp(doc,"ENTITY",&str)) {
		while (*str && (*str != '=')) str++;
		if (*str) {
			while (*str && (*str != '"')) str++;
			if (*str) {
				str++;
				i = strlen(str);
				temp = smalloc(i+1);
				if (!temp) return MallocFailError;
				strcpy(temp,str);
				if (temp[i-1] == '"')
					temp[i-1] = 0;
				{
					int4 ok = 1;
					ok = ok && riPuts("<!ENTITY ", fo) != REOF;
					ok = ok && riPuts(temp, fo) != REOF;
					ok = ok && riPuts(" SYSTEM \"", fo) != REOF;
					ok = ok && riPuts(temp, fo) != REOF;
					ok = ok && riPuts("\" NDATA TIF>\n", fo) != REOF;
					if(!ok)
						e = WriteError;
				}
				smfree(temp);
				if (e) return e;
			}
		}
	}
	for (cur=i_GetChildEntity(doc);cur;cur = i_GetNextEntity(cur))
		if (e = WriteENTITY(fo,cur))
			return e;
	return DAFSOK;
}

iDAFSError 
WriteHeader(RafFile fo,iDAFSWriteMode mode,iEntityPtr doc)
{
	iDAFSError e = DAFSOK;
	short s = 1;
	
	switch (mode) {
	case DAFSB:
		if (e = i_WriteItem(fo,NullTok,0,0))
			return e;
		if (e = i_WriteItem(fo,NullTok,0,0))
			return e;
		if (e = i_WriteItem(fo,ByteOrder,2,&s))
			return e;
		break;
	case DAFSU:
		break;
	case DAFSA:
#ifdef IMPLEMENT_DAFSA
		if (!i_GetStringProp(doc,"DOCTYPE",&str)) {
			{
				int4 ok = 1;
				ok = ok && riPuts("<!DOCTYPE ", fo) != REOF;
				ok = ok && riPuts(str, fo) != REOF;
				ok = ok && riPuts("[\n", fo) != REOF;
				if(!ok)
					return WriteError;
			}
			if (e = WriteENTITY(fo,doc)) return e;
			if (riPuts("]>\n", fo) == REOF) return WriteError;			
		}
#else
		halt("dafsa not implemented\n");
#endif
		break;
	default:
		e = DAFSWriteModeError;
		break;
	}
	return e;
}

iDAFSError 
_WriteEntity(RafFile fo, const iEntityPtr entity, 
			 iDAFSWriteMode mode)
{
	iDAFSError e;
	iGlobid gid={0,0,0,0,0};
	int4 *use_list;
	int4 i;
	
	if (e = WriteHeader(fo,mode,entity)) {
		riClose(fo);
		return e;
	}
	use_list = (int4*)smalloc(sizeof(int4)*i_GlobalType.size);
	for(i=0; i<i_GlobalType.size; i++)
		use_list[i]=0;
	ClearIDAndCheckType(entity, use_list);
	gid.en_id = 1;
	gid.im_id = 1;
	switch (mode) {
	case DAFSB:
		if (e = WriteTypeTab(fo, use_list)) return e;
		e = WriteBinaryEntity(fo,entity,&gid, use_list);
		break;
	case DAFSU:
		break;
	case DAFSA:
#ifdef IMPLEMENT_DAFSA
		if (i_GetUserPtrProp(entity,"Uni2ASCII",(void**)&table))
			table = 0;
		e = WriteDAFSAEntity(fo,entity,&gid,table);
#else
		e = !DAFSOK;
		halt("dafsa not implemented.\n");
#endif
		break;
	}
	smfree(use_list);
	ClearID(entity);
	return e;
}

Dll iDAFSError APIENTRY
i_WriteEntity(const char *fileName,const iEntityPtr entity,
			  iDAFSWriteMode mode)
{	iDAFSError err;
	RafFile fo;
	
	fo=riOpen(fileName,"wb");
	if (!fo) return WriteLockedFileError;
	err = _WriteEntity(fo, entity, mode);
	riClose(fo);
	return err;
}

Dll iDAFSError APIENTRY
i_WriteEntityF(RafFile fo,const iEntityPtr entity,
			  iDAFSWriteMode mode)
{	iDAFSError err;

	err = _WriteEntity(fo, entity, mode);
	return err;
}

iDAFSError
ReadHeader(RafFile fo,iDAFSWriteMode *mode)
{
	char b[2];
	
	if (riRead(b,1,2,fo) != 2)
		return ReadError;
	if (!b[0] && !b[1]) {
		*mode = DAFSB;
		return DAFSOK;
	}
	if (b[0]=='<' && (isascii(b[1]))){
#ifdef IMPLEMENT_DAFSA
		*mode = DAFSA;
		riSeek(fo,0,0);
#else
		halt("dafsa not implemented.\n");
#endif
		return DAFSOK;
	}
	riSeek(fo,0,0);
	return ReadError;
}


static iDAFSError 
_ReadEntity(RafFile fo,iEntityPtr *entityPtr, iGlobid *gid)
{
	iDAFSError e;
	iDAFSWriteMode mode;
	char *type=0;
	iToken t;
	uint2 size;
	long offset=0;
	iTypeIndex et;
	
	Local.size=0;
	saveCB = gCallBack;
	gCallBack = 0;
	if (e = ReadHeader(fo,&mode))
		goto error;

	*entityPtr = 0;
	switch (mode) {
	case DAFSB:
		if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap))
			goto error;
		while ((t & iTMask) != BeginEntity) {
			if ((t & iTMask) == TString){
				if (e = i_ReadItemAlloc(fo,t,&size,(void**)&type,&gid->DAFSBSwap))
					goto error;
				if (e=i_PNameToType(&Local,type,0))
					goto error;
				if (e = i_NameToType(type,&et))
					goto error;
				if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap))
					goto error;
				if ((t & iTMask) == BeginProp) {
					if (e = ReadProps(fo, 0, gid,et))
						goto error;
					if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap))
						goto error;
				}
				smfree(type);
				type=0;
			} else {
				if (e = i_ReadItem(fo,t,0,&size,0,&gid->DAFSBSwap))
					goto error;
				if (e = i_ReadToken(fo,&t,&gid->DAFSBSwap))
					goto error;
			}
		}
		if (e = ReadBinaryEntity(fo,entityPtr,0, offset, gid))
			goto error;

		break;
	case DAFSU:
		break;
	case DAFSA:
#ifdef IMPLEMENT_DAFSA
		e = ReadDAFSAEntity(fo,entityPtr);
#else
		halt("dafsa not implemented.\n");
#endif
		break;
	default:
		e = DAFSWriteModeError;
		break;
	}
	i_DisposeTypeTab(&Local);
  error:
	if (type) smfree(type);
	if (e && *entityPtr) {
		i_DisposeEntity(*entityPtr);
		*entityPtr=0;
	}
	gCallBack = saveCB;
	return e;
}

static void 
ChangeTypeNumber(iEntityPtr ent, int4 old, int4 new)
{	
	iEntityPtr cur;
	if (ent->data->type==old)
		ent->data->type=(unsigned char)new;
	for(cur = i_GetChildEntity(ent);cur;cur = i_GetNextEntity(cur))
		ChangeTypeNumber(cur, old, new);
}

Dll iDAFSError APIENTRY
i_ReadEntity(const char *fileName,iEntityPtr *entityPtr)
{
	iGlobid gid={0,0,0,0,0};
	iDAFSError e;
	RafFile fo;
	
	gid.im_flag=1;
	fo = riOpen(fileName,"rb");
	if (!fo) {
		return NonExistentFileError;
	}
	e = _ReadEntity(fo,entityPtr,&gid);
	riClose(fo);
	if (gid.ims) smfree(gid.ims);
	return e;
}

Dll iDAFSError APIENTRY
i_ReadEntityF(RafFile fo,iEntityPtr *entityPtr)
{
	iGlobid gid={0,0,0,0,0};
	iDAFSError e;
	
	gid.im_flag=1;
	e = _ReadEntity(fo,entityPtr,&gid);
	if (gid.ims) smfree(gid.ims);
	return e;
}

Dll iDAFSError APIENTRY
i_ReadEntityNoIm(const char *fileName,iEntityPtr *entityPtr)
{
	iDAFSError err;
	iGlobid gid={0,0,0,0,0};
	RafFile fo;

	fo = riOpen(fileName,"rb");
	if (!fo) {
		return NonExistentFileError;
	}
	err = _ReadEntity(fo,entityPtr,&gid);
	riClose(fo);
	return err;
}


