/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"
#include "rafutil.h"
#include "c-lib.h"
#include "str-lib.h"

#include "entity.h"
#include "rlimage.h"
#include "dafs.h"
#include "comp.h"
#include "smemory.h"

/* it won't matter if this value of CHUNK is different from others */
#define CHUNK 8192

#if 0
static void
dumpraw(iImagePtr sp)
{
	int4 i;
	RafFile fo;
	
	if (sp->type != Raw) { halt("this isn't a raw guy\n"); return; }
	fo = riOpen("fred.raw","w");
	for (i=0;i<sp->d.rw.nchunks;i++) {
		riWrite(sp->d.rw.chunks[i].data,1,sp->d.rw.chunks[i].len,fo);
	}
	riClose(fo);
}
#endif

Dll iDAFSError APIENTRY
i_CompressImage(iImagePtr sp,iDAFSCompType comp)
{
	iImagePtr np=0;
	int amt,len,trip;
	int4 i,cur;
	EncodeStatePtr es=0;
	short *lcode;
	iDAFSError e = DAFSOK;
	iImage temp;

	if (comp == CCITT32D) return CantWrite32D;
	if (sp->type == Raw) {
		if (sp->d.rw.comp == comp) return DAFSOK;
		if (e = i_UncompressImage(sp))
			goto error;
	}
	np = newrawimage(sp->width, sp->height,comp);
	if (!np) { e = MallocFailError; goto error; }

	len = CHUNK;
	trip = sp->width * 2;
	if (trip < MINCOMP) trip = MINCOMP;
	if (len < trip) len = trip;
	
	if (e = newchunk(np,len))
		goto error;
	es = EncodeAlloc();
	if (!es) { e = MallocFailError; goto error; }
	EncodeInit(es,sp,comp);

	cur = 0;
	EncodeSetData(es,np->d.rw.chunks[cur].data,len);
	lcode = 0;
	for (i = 0; i < sp->height; i++) {
		Encode(es, (short *)sp->d.im.rl[i].code, lcode);
		amt = EncodeGetAmt(es);
		if ((amt + trip) > len) {
			np->d.rw.chunks[cur].len = amt;
			if (e = newchunk(np,len))
				goto error;
			cur++;
			EncodeSetData(es,np->d.rw.chunks[cur].data,len);
		}
		lcode = (short *)sp->d.im.rl[i].code;
	}
	EncodeFlush(es,sp,1);
	amt = EncodeGetAmt(es);
	np->d.rw.chunks[cur].len = amt;
	temp = *np;
	*np = *sp;
	*sp = temp;

	sp->id = np->id;
	sp->dpi = np->dpi;
	sp->use = np->use;
	freeimage(np);
error:
	if (es) EncodeFree(es);
	if (e && np) freeimage(np);
	return e;
}

Dll iDAFSError APIENTRY
i_UncompressImage(iImagePtr image)
{
	return i_uncompressimage(image,0);
}

iDAFSError
i_uncompressimage(iImagePtr image,int4 rowsper)
{
	short *code=0,*lcode, index=0;
	int4 row, cur=0;
	int2 n,w,h,alloc;
	iImagePtr np=0;
	iImage temp;
	RlLinePtr l;
	void *sp=0;
	iDAFSError e = DAFSOK;
	
	if (image->type == Image) return DAFSOK;
	h = image->height;
	w = image->width;

	if (!image->d.rw.nchunks) {
		np = rlnewimage(w, h, 2);
		if (!np) { e = MallocFailError; goto error; }
		l = np->d.im.rl;
		for (row=0;row < h;l++,row++) {
			l->run[0].up = w;
			l->run[0].dn = w;
			l->run[1].up = -1;
			l->size = 2;
		}
	} else {
		np = rlnewimage(w, h, 0);
		if (!np) { e = MallocFailError; goto error; }
		code = (short *) smalloc(sizeof(short) * (w + 4));
		if (!code) { e = MallocFailError; goto error; }

		sp = DecodeAlloc();
		if (!sp) { e = MallocFailError; goto error; }

		DecodeInit(sp,image, image->d.rw.comp);
		DecodeSetData(sp,image->d.rw.chunks[cur].data,image->d.rw.chunks[cur].len);
		cur++;
		
		lcode = 0;
		
		l = np->d.im.rl;
		for (row = 0; row < h; l++,row++) {
			if (rowsper && (++index > rowsper)) {
				index=1;
				lcode = 0;
				
				DecodeInit(sp, image,image->d.rw.comp);
				DecodeSetData(sp,image->d.rw.chunks[cur].data,
							image->d.rw.chunks[cur].len);
				cur++;
			}
			DecodeSetCodeRef(sp,code,lcode);
			while ((n = Decode(sp)) == COMPMORE) {
				DecodeSetData(sp,image->d.rw.chunks[cur].data,
							image->d.rw.chunks[cur].len);
				cur++;
			}
			if (n < 0)  { e = WrongImageFormat; goto error; }
			n >>= 1;
			alloc = n+image->d.rw.invert;
			l->code = (short *) smalloc(SL + (ST * alloc));
			if (!l->code) { e = MallocFailError; goto error; }
			l->run = (transition*)&l->code[1];
			l->code[0] = 0;
			memcpy((char *) l->code, (char *) code, (size_t)n * ST);
			l->size = n;
			l->asize = alloc;
			l->last = 0;
			lcode = l->code;
		}
		DecodeFinish(sp,np);
	}
	if (image->d.rw.invert && (e = i_InvertImage(np)))
		goto error;
	temp = *np;
	*np = *image;
	*image = temp;

	image->id = np->id;
	image->dpi = np->dpi;
	image->use = np->use;
	freeimage(np);
	np = 0;
error:
	if (sp) DecodeFree(sp);
	if (code) smfree(code);
	if (np) freeimage(np);
	return e;
}
