/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/* 
// seems to have global type stuff
// callbacks
// and lots of the conversions between different types of borders
*/


#include "rafport.h"
#include "rafutil.h"

#include "str-lib.h"
#include "smemory.h"
#include "entity.h"
#include "dafs.h"


/* global data */
iCallBackProc gCallBack;
void *gUserData;
iImageWriteProc gWriteImageProc;
iImageReadProc gReadImageProc;

const char *i_Types[] = {"",iDocument, iParagraph, 
						 iLine, iWord, iGlyph,
						 iZone, iRegion, iSpace, iNewLine, iComment};


iDAFSError
i_InitTab(iTypeTablePtr tab)
{
	int4 i;
	
	tab->size = iNTypesT;
	tab->table = (iTypesPtr) smalloc(sizeof(iTypes) * tab->size);
	if (!tab->table)
		return (MallocFailError);
	for (i=0;i<tab->size;i++) {
		tab->table[i].name = (char*)smalloc(strlen(i_Types[i])+1);
		if (!tab->table[i].name)
			return (MallocFailError);	
		strcpy(tab->table[i].name, i_Types[i]);
		tab->table[i].props = 0;
	}
	return DAFSOK;	
}

Dll iDAFSError APIENTRY
i_InitTypeProp(void)
{	
  return DAFSOK;
}

void
i_DisposeTypeTab(iTypeTablePtr tab)
{
	int4 i;
	
	if (tab->table ) {
		for (i=0;i<tab->size;i++) {
			if (tab->table[i].props) {
				i_DisposeAllPropsPtr(&tab->table[i].props);
				tab->table[i].props = 0;
			}
			if (tab->table[i].name) smfree(tab->table[i].name);
			tab->table[i].name = 0;
		}
		smfree(tab->table);
		tab->table = 0;
	}
	tab->size = 0;
}

Dll iDAFSError APIENTRY
i_InitLib(void)
{	iDAFSError err=DAFSOK;

	gCallBack = 0;
	if (err=i_InitTab(&i_GlobalType))
		return err;
	if (err=i_InitTypeProp())
		return err;
	i_InitRasNRun();
	i_InitComp();
	return err;
}

iDAFSError 
i_PNameToType(iTypeTablePtr tab,const char *type, iTypeIndex *val)
{
	iTypeIndex ind;
	iTypesPtr temp;
	
	for (ind = 0; ind < tab->size; ind++)
		if (!strcmp(type, tab->table[ind].name)) {
			if (val) *val=ind;
			return DAFSOK;
		}
	
	if (!ind) temp = (iTypesPtr) smalloc(sizeof(iTypes));
	else temp = (iTypesPtr) smrealloc(tab->table, sizeof(iTypes) * (ind+1));
	if (!temp)
		return MallocFailError;
	tab->table = temp;
	
	tab->table[ind].name =  (char *) smalloc(strlen(type)+1);
	if (!tab->table[ind].name)
		return MallocFailError;

	tab->size++;
	strcpy(tab->table[ind].name, type);
	tab->table[ind].props = 0;
	
	if (val) *val=ind;
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_NameToType(const char *type, iTypeIndex *val)
{
	return i_PNameToType(&i_GlobalType,type, val);
}

const char *
i_PTypeToName(iTypeTablePtr tab,iTypeIndex type)
{
	if (type<tab->size)
		return tab->table[type].name;
	return 0;
}

Dll void APIENTRY 
i_SetImageCallBack(iImageWriteProc wp,iImageReadProc rp)
{
	gWriteImageProc = wp;
	gReadImageProc = rp;
}

Dll void APIENTRY 
i_GetImageCallBack(iImageWriteProc *wp,iImageReadProc *rp)
{
	if (wp) *wp = gWriteImageProc;
	if (rp) *rp = gReadImageProc;
}

Dll const char * APIENTRY
i_TypeToName(iTypeIndex type)
{
	return i_PTypeToName(&i_GlobalType,type);
}

Dll void APIENTRY
i_SetCallBack(iCallBackProc callBack, void *userData)
{
	gCallBack = callBack;
	gUserData = userData;
}

Dll void APIENTRY
i_GetCallBack(iCallBackProc * callBack, void **userData)
{
	if (callBack)
		*callBack = gCallBack;
	if (userData)
		*userData = gUserData;
}

Dll iDAFSError APIENTRY
i_DoCallBack(iMess mess, iEntityPtr entity)
{
	if (gCallBack)
		return gCallBack(mess, gUserData, entity);
	return DAFSOK;
}


Dll void APIENTRY
i_ExitLib(void)
{
	i_DisposeTypeTab(&i_GlobalType);
}
