/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/* $XConsortium: PolyReg.c,v 11.22 94/04/17 20:20:27 rws Exp $ */
/************************************************************************

Copyright (c) 1987  X Consortium

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
X CONSORTIUM BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

Except as contained in this notice, the name of the X Consortium shall not be
used in advertising or otherwise to promote the sale, use or other dealings
in this Software without prior written authorization from the X Consortium.


Copyright 1987 by Digital Equipment Corporation, Maynard, Massachusetts.

                        All Rights Reserved

Permission to use, copy, modify, and distribute this software and its 
documentation for any purpose and without fee is hereby granted, 
provided that the above copyright notice appear in all copies and that
both that copyright notice and this permission notice appear in 
supporting documentation, and that the name of Digital not be
used in advertising or publicity pertaining to distribution of the
software without specific, written prior permission.  

DIGITAL DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING
ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL
DIGITAL BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR
ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION,
ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
SOFTWARE.

************************************************************************/


#include "rafport.h"

#include "rafutil.h"
#include "entity.h"
#include "dafs.h"
#include "smemory.h"

#define LARGE_COORDINATE 32767
#define SMALL_COORDINATE -LARGE_COORDINATE

#include "region.h"
#include "poly.h"

/*
 *     InsertEdgeInET
 *
 *     Insert the given edge into the edge table.
 *     First we must find the correct bucket in the
 *     Edge table, then find the right slot in the
 *     bucket.  Finally, we can insert it.
 *
 */
static iDAFSError
InsertEdgeInET(
    iEdgeTable *ET,
    iEdgeTableEntry *ETE,
    int4 scanline,
    iScanLineListBlock **SLLBlock,
    int4 *iSLLBlock)
{
	iEdgeTableEntry *start, *prev;
	iScanLineList *pSLL, *pPrevSLL;
	iScanLineListBlock *tmpSLLBlock;
	iDAFSError e=DAFSOK;

	/*
	 * find the right bucket to put the edge into
	 */
	pPrevSLL = &ET->scanlines;
	pSLL = pPrevSLL->next;
	while (pSLL && (pSLL->scanline < scanline)){
        pPrevSLL = pSLL;
        pSLL = pSLL->next;
    }

    /*
     * reassign pSLL (pointer to ScanLineList) if necessary
     */
    if ((!pSLL) || (pSLL->scanline > scanline)) {
        if (*iSLLBlock > SLLSPERBLOCK-1) {
            tmpSLLBlock = (iScanLineListBlock *)smalloc(sizeof(iScanLineListBlock));
            if (!tmpSLLBlock) {
            	e=MallocFailError;
            	goto error;
			}
            (*SLLBlock)->next = tmpSLLBlock;
            tmpSLLBlock->next = (iScanLineListBlock *)NULL;
            *SLLBlock = tmpSLLBlock;
            *iSLLBlock = 0;
        }
        pSLL = &((*SLLBlock)->SLLs[(*iSLLBlock)++]);

        pSLL->next = pPrevSLL->next;
        pSLL->edgelist = (iEdgeTableEntry *)NULL;
        pPrevSLL->next = pSLL;
    }
	pSLL->scanline = scanline;

	/*
	 * now insert the edge in the right bucket
	 */
	prev = (iEdgeTableEntry *)NULL;
	start = pSLL->edgelist;
	while (start && (start->bres.minor_axis < ETE->bres.minor_axis)) {
        prev = start;
        start = start->next;
    }
	ETE->next = start;

	if (prev)
		prev->next = ETE;
	else
		pSLL->edgelist = ETE;

error:
	return e;
}

/*
 *     CreateEdgeTable
 *
 *     This routine creates the edge table for
 *     scan converting polygons. 
 *     The Edge Table (ET) looks like:
 *
 *    EdgeTable
 *     --------
 *    |  ymax  |        ScanLineLists
 *    |scanline|-->------------>-------------->...
 *     --------   |scanline|   |scanline|
 *                |edgelist|   |edgelist|
 *                ---------    ---------
 *                    |             |
 *                    |             |
 *                    V             V
 *              list of ETEs   list of ETEs
 *
 *     where ETE is an EdgeTableEntry data structure,
 *     and there is one ScanLineList per scanline at
 *     which an edge is initially entered.
 *
 */

static iDAFSError
CreateETandAET(
    int4 count,
    iPoint *pts,
    iEdgeTable *ET,
    iEdgeTableEntry *AET,
    iEdgeTableEntry *pETEs,
    iScanLineListBlock   *pSLLBlock)
{
    iPoint *top, *bottom;
    iPoint *PrevPt, *CurrPt;
    int4 iSLLBlock = 0;
    int2 dy;
    iScanLineListBlock *cur,*next;
    iDAFSError e;

    if (count < 2)  return DAFSOK;

    /*
     *  initialize the Active Edge Table
     */
    AET->next = (iEdgeTableEntry *)NULL;
    AET->back = (iEdgeTableEntry *)NULL;
    AET->nextWETE = (iEdgeTableEntry *)NULL;
    AET->bres.minor_axis = SMALL_COORDINATE;

    /*
     *  initialize the Edge Table.
     */
    ET->scanlines.next = (iScanLineList *)NULL;
    ET->ymax = SMALL_COORDINATE;
    ET->ymin = LARGE_COORDINATE;
    pSLLBlock->next = (iScanLineListBlock *)NULL;

    PrevPt = &pts[count-1];

    /*
     *  for each vertex in the array of points.
     *  In this loop we are dealing with two vertices at
     *  a time -- these make up one edge of the polygon.
     */
    while (count--) {
		CurrPt = pts++;

        /*
         *  find out which point is above and which is below.
         */
		if (PrevPt->y > CurrPt->y) {
		    bottom = PrevPt, top = CurrPt;
		    pETEs->ClockWise = 0;
		} else {
		    bottom = CurrPt, top = PrevPt;
		    pETEs->ClockWise = 1;
		}

		/*
		 * don't add horizontal edges to the Edge table.
		 */
		if (bottom->y != top->y) {
			pETEs->ymax = bottom->y-1;  /* -1 so we don't get last scanline */

			/*
			 *  initialize integer edge algorithm
			 */
			dy = bottom->y - top->y;
			iBRESINITPGONSTRUCT(dy, top->x, bottom->x, pETEs->bres);

			if (e=InsertEdgeInET(ET, pETEs, top->y, &pSLLBlock, &iSLLBlock))
				goto error;

			if (PrevPt->y > ET->ymax)
				ET->ymax = PrevPt->y;
			if (PrevPt->y < ET->ymin)
				ET->ymin = PrevPt->y;
			pETEs++;
        }
        PrevPt = CurrPt;
    }
error:
	if (e) {
		for (cur=pSLLBlock; cur; cur=next) {
			next=cur->next;
			smfree(cur);
		}
	}
	return e;
}

/*
 *     loadAET
 *
 *     This routine moves EdgeTableEntries from the
 *     EdgeTable into the Active Edge Table,
 *     leaving them sorted by smaller x coordinate.
 *
 */

static void
loadAET(iEdgeTableEntry *AET,  iEdgeTableEntry *ETEs)
{
    iEdgeTableEntry *pPrevAET;
    iEdgeTableEntry *tmp;

    pPrevAET = AET;
    AET = AET->next;
    while (ETEs) 
    {
        while (AET && (AET->bres.minor_axis < ETEs->bres.minor_axis)) 
        {
            pPrevAET = AET;
            AET = AET->next;
        }
        tmp = ETEs->next;
        ETEs->next = AET;
        if (AET)
            AET->back = ETEs;
        ETEs->back = pPrevAET;
        pPrevAET->next = ETEs;
        pPrevAET = ETEs;

        ETEs = tmp;
    }
}

/*
 *     computeWAET
 *
 *     This routine links the AET by the
 *     nextWETE (winding iEdgeTableEntry) link for
 *     use by the winding number rule.  The final 
 *     Active Edge Table (AET) might look something
 *     like:
 *
 *     AET
 *     ----------  ---------   ---------
 *     |ymax    |  |ymax    |  |ymax    | 
 *     | ...    |  |...     |  |...     |
 *     |next    |->|next    |->|next    |->...
 *     |nextWETE|  |nextWETE|  |nextWETE|
 *     ---------   ---------   ^--------
 *         |                   |       |
 *         V------------------->       V---> ...
 *
 
static void
computeWAET(AET)
    iEdgeTableEntry *AET;
{
    iEdgeTableEntry *pWETE;
    int4 inside = 1;
    int4 isInside = 0;

    AET->nextWETE = (iEdgeTableEntry *)NULL;
    pWETE = AET;
    AET = AET->next;
    while (AET) 
    {
        if (AET->ClockWise)
            isInside++;
        else
            isInside--;

        if ((!inside && !isInside) ||
            ( inside &&  isInside)) 
        {
            pWETE->nextWETE = AET;
            pWETE = AET;
            inside = !inside;
        }
        AET = AET->next;
    }
    pWETE->nextWETE = (iEdgeTableEntry *)NULL;
}
*/
/*
 *     InsertionSort
 *
 *     Just a simple insertion sort using
 *     pointers and back pointers to sort the Active
 *     Edge Table.
 *
 */

static void
InsertionSort(iEdgeTableEntry *AET)
{
    iEdgeTableEntry *pETEchase;
    iEdgeTableEntry *pETEinsert;
    iEdgeTableEntry *pETEchaseBackTMP;

    AET = AET->next;
    while (AET) 
    {
        pETEinsert = AET;
        pETEchase = AET;
        while (pETEchase->back->bres.minor_axis > AET->bres.minor_axis)
            pETEchase = pETEchase->back;

        AET = AET->next;
        if (pETEchase != pETEinsert) 
        {
            pETEchaseBackTMP = pETEchase->back;
            pETEinsert->back->next = AET;
            if (AET)
                AET->back = pETEinsert->back;
            pETEinsert->next = pETEchase;
            pETEchase->back->next = pETEinsert;
            pETEchase->back = pETEinsert;
            pETEinsert->back = pETEchaseBackTMP;
        }
    }
}

/*
 *     Clean up our act.
 */
static void
FreeStorage(iScanLineListBlock   *pSLLBlock)
{
    iScanLineListBlock   *tmpSLLBlock;

    while (pSLLBlock) 
    {
        tmpSLLBlock = pSLLBlock->next;
        smfree((char *)pSLLBlock);
        pSLLBlock = tmpSLLBlock;
    }
}

/*
 *     Create an array of rectangles from a list of points.
 *     If indeed these things (POINTS, RECTS) are the same,
 *     then this proc is still needed, because it allocates
 *     storage for the array, which was allocated on the
 *     stack by the calling procedure.
 *
 */
static iDAFSError PtsToRegion(
    int2  numFullPtBlocks, int2 iCurPtBlock,
    POINTBLOCK *FirstPtBlock,
    iBoxListPtr reg)
{
    BOX  *rects, *temp;
    iPoint *pts;
    POINTBLOCK *CurPtBlock;
    int4 i;
    BOX *extents;
    int2 numRects;
 
    extents = &reg->extents;
 
    numRects = ((numFullPtBlocks * NUMPTSTOBUFFER) + iCurPtBlock) >> 1;
 
    if (!(temp = (BOX *)smrealloc((char *)reg->rects, 
	    (unsigned) (sizeof(BOX) * numRects))))
		return MallocFailError;
	reg->rects=temp;

	reg->size = numRects;
	CurPtBlock = FirstPtBlock;
	rects = reg->rects - 1;
	numRects = 0;
	extents->x1 = 32767,  extents->x2 = -32768;
 
    for ( ; numFullPtBlocks >= 0; numFullPtBlocks--) {
		/* the loop uses 2 points per iteration */
		i = NUMPTSTOBUFFER >> 1;
		if (!numFullPtBlocks)
		    i = iCurPtBlock >> 1;
		for (pts = CurPtBlock->pts; i--; pts += 2) {
			if (pts->x == pts[1].x)
				continue;
		    if (numRects && pts->x == rects->x1 && pts->y == rects->y2 &&
				pts[1].x == rects->x2 &&
				(numRects == 1 || rects[-1].y1 != rects->y1) &&
				(!i || pts[2].y > pts[1].y)) {
					rects->y2 = pts[1].y + 1;
					continue;
			}
		    numRects++;
		    rects++;
		    rects->x1 = pts->x;  rects->y1 = pts->y;
		    rects->x2 = pts[1].x;  rects->y2 = pts[1].y + 1;
		    if (rects->x1 < extents->x1)
			extents->x1 = rects->x1;
		    if (rects->x2 > extents->x2)
			extents->x2 = rects->x2;
	    }
		CurPtBlock = CurPtBlock->next;
    }

    if (numRects) {
		extents->y1 = reg->rects->y1;
		extents->y2 = rects->y2;
    } else {
		extents->x1 = 0;
		extents->y1 = 0;
		extents->x2 = 0;
		extents->y2 = 0;
    }
    reg->numRects = numRects;
 
    return DAFSOK;
}

/*
 *     polytoregion
 *
 *     Scan converts a polygon by returning a run-length
 *     encoding of the resultant bitmap -- the run-length
 *     encoding is in the form of an array of rectangles.
 */
Dll iDAFSError APIENTRY
i_Orthogon2BoxList(iOrthogonPtr orthogon, iBoxListPtr *boxes)
{
	int4 Count = orthogon->n;
	iPoint *Pts = orthogon->point;
/*     int4	rule=1;			     winding rule is ignored */
	iDAFSError e=DAFSOK;
    iBoxListPtr box_list;
    iEdgeTableEntry *pAET;   /* Active Edge Table       */
    int2 y;                  /* current scanline        */
    int2 iPts = 0;           /* number of pts in buffer */
    iScanLineList *pSLL;     /* current scanLineList    */
    iPoint *pts;             /* output buffer           */
    iEdgeTableEntry *pPrevAET;        /* ptr to previous AET     */
    iEdgeTable ET;                    /* header node for ET      */
    iEdgeTableEntry AET;              /* header node for AET     */
    iEdgeTableEntry *pETEs;           /* EdgeTableEntries pool   */
    iScanLineListBlock SLLBlock;      /* header for scanlinelist */
    POINTBLOCK FirstPtBlock, *curPtBlock; /* PtBlock buffers    */
    POINTBLOCK *tmpPtBlock;
    int2 numFullPtBlocks = 0;

	*boxes = NULL;
 
    if (! (box_list = i_NewBoxList())) return MallocFailError;

    /* special case a rectangle */
    pts = Pts;
    if (((Count == 4) ||
	 ((Count == 5) && (pts[4].x == pts[0].x) && (pts[4].y == pts[0].y))) &&
	(((pts[0].y == pts[1].y) &&
	  (pts[1].x == pts[2].x) &&
	  (pts[2].y == pts[3].y) &&
	  (pts[3].x == pts[0].x)) ||
	 ((pts[0].x == pts[1].x) &&
	  (pts[1].y == pts[2].y) &&
	  (pts[2].x == pts[3].x) &&
	  (pts[3].y == pts[0].y)))) {
		box_list->extents.x1 = MIN(pts[0].x, pts[2].x);
		box_list->extents.y1 = MIN(pts[0].y, pts[2].y);
		box_list->extents.x2 = MAX(pts[0].x, pts[2].x);
		box_list->extents.y2 = MAX(pts[0].y, pts[2].y);
		if ((box_list->extents.x1 != box_list->extents.x2) &&
		    (box_list->extents.y1 != box_list->extents.y2)) {
		    box_list->numRects = 1;
		    *(box_list->rects) = box_list->extents;
		}

		*boxes = box_list;
		return DAFSOK;
    }

	pts = FirstPtBlock.pts;
	FirstPtBlock.next=NULL;

	pETEs = (iEdgeTableEntry *)smalloc((unsigned) (sizeof(iEdgeTableEntry) * Count));
    if (!pETEs) {
    	e=MallocFailError;
    	goto error;
    }

	if (e=CreateETandAET(Count, Pts, &ET, &AET, pETEs, &SLLBlock))
		goto error;

	pSLL = ET.scanlines.next;
	curPtBlock = &FirstPtBlock;
 
/* if (rule == EvenOddRule) { */
     /*
      *  for each scanline
      */
     for (y = ET.ymin; y < ET.ymax; y++) {
         /*
          *  Add a new edge to the active edge table when we
          *  get to the next edge.
          */
         if (pSLL != NULL && y == pSLL->scanline) {
             loadAET(&AET, pSLL->edgelist);
             pSLL = pSLL->next;
         }
         pPrevAET = &AET;
         pAET = AET.next;

         /*
          *  for each active edge
          */
         while (pAET) {
             pts->x = pAET->bres.minor_axis;
             pts->y = y;
             pts++, iPts++;

             /*
              *  send out the buffer
              */
             if (iPts == NUMPTSTOBUFFER) {
                 tmpPtBlock = (POINTBLOCK *)smalloc(sizeof(POINTBLOCK));
                 if (!tmpPtBlock) {
                 	e=MallocFailError;
                 	goto error;
                 }
                 tmpPtBlock->next=NULL;
                 curPtBlock->next = tmpPtBlock;
                 curPtBlock = tmpPtBlock;
                 pts = curPtBlock->pts;
                 numFullPtBlocks++;
                 iPts = 0;
             }
             iEVALUATEEDGEEVENODD(pAET, pPrevAET, y);
         }
         InsertionSort(&AET);
     }
 

	if (e=PtsToRegion(numFullPtBlocks, iPts, &FirstPtBlock, box_list))
		goto error;

error:
	FreeStorage(SLLBlock.next);	

	for (curPtBlock = FirstPtBlock.next; curPtBlock; curPtBlock = tmpPtBlock) {
		tmpPtBlock = curPtBlock->next;
		smfree((char *)curPtBlock);
    }

	if (e && box_list)
		i_DisposeBoxList(box_list);
	else
		*boxes = box_list;
	
	if (pETEs) smfree((char *)pETEs);
	
	return e;
}
