/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"
#include "rafutil.h"

#include <stdio.h>
#include <stdlib.h>
#include "entity.h"
#include "rlimage.h"
#include "dafs.h"
#include "smemory.h"

#if !DO_INLINE
#define inline
#endif
#define INITSIZE 30

#ifdef ECHECK
#include "debug.h"
#endif

static iImagePtr newimage(int2 w,int2 h)
{
	iImagePtr np;
	np = (iImagePtr ) smalloc(sizeof(iImage));
	if (!np)
		return 0;
	np->width = w;
	np->height = h;
	np->xOff = 0;
	np->yOff = 0;
	np->tag = 0;

	np->use = 0;
	np->dpi = 0;
	np->id = 0;
	return np;
}

iImagePtr
newrawimage(int2 w, int2 h,int2 comp)
{
	iImagePtr np = newimage(w,h);
	
	if (!np) return np;
	np->type = Raw;
	np->d.rw.nchunks = 0;
	np->d.rw.invert = 0;
	np->d.rw.chunks = 0;
	np->d.rw.comp = comp;
	
	return np;
}

iImagePtr 
rlnewimage(int2 w, int2 h, int2 asize)
{
	iImagePtr np = newimage(w,h);
	RlLinePtr cr;
	int4 i;
	int4 temp_size;
	
	if (!np)
		return 0;

	temp_size = sizeof(RlLine) * h;
	np->d.im.rl = (RlLinePtr) smalloc(temp_size);
	if (!np->d.im.rl) {
		smfree(np);
		return 0;
	}

	np->type = Image;
	np->d.im.next = 0;
	np->d.im.prev = 0;
	np->d.im.pixcnt = 0;
	np->d.im.trncnt = 0;
	np->d.im.rot = 0;
	np->d.im.xmul = iFP_ONE;
	np->d.im.ymul = iFP_ONE;
	np->d.im.x = w;
	np->d.im.y = h;
	np->d.im.w = -w;
	np->d.im.h = -h;

	for (i = 0, cr=np->d.im.rl; i < h; i++,cr++) {
		if (asize) {
			temp_size = (ST * asize) + sizeof(short);
			cr->code = (short *) smalloc(temp_size);
			if (!cr->code) {
				while (i)
					smfree(np->d.im.rl[--i].code);
				smfree(np->d.im.rl);
				smfree(np);
				return 0;
			}
			cr->code[0] = 0;
			cr->run = (transition*)&cr->code[1];
		} else {
			cr->code = 0;
			cr->run = 0;
		}
		cr->last = -1;
		cr->asize = asize;
		cr->size = 0;
	}
	return np;
}

Dll iImagePtr APIENTRY
i_NewImage(int2 w,int2 h,int2 res,iDAFSCompType comp)
{
	iImagePtr im;
	RlLinePtr cr;
	transition tr;
	int i;
	
	if (comp == NoComp) {
		tr.up = w;
		tr.dn = w;
		im = rlnewimage(w,h,10);
		if (!im) return 0;
		for (i=0,cr = im->d.im.rl;i<h;i++,cr++) {
			cr->run[0] = tr;
			cr->run[1].up = -1;
			cr->size = 2;
		}
	} else {
		im = newrawimage(w,h,comp);
		if (!im) return 0;
	}
	im->dpi = res;
	im->use++;
	return im;
}

Dll iDAFSError APIENTRY
i_AddDataToImage(iImagePtr im,char *data,int2 size)
{
	iDAFSError e;
	
	if (e = i_CompressImage(im,CCITT4)) return e;
	if (e = newchunk(im,size)) return e;
	memcpy(im->d.rw.chunks[im->d.rw.nchunks-1].data,data,size);
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_GetDataFromImage(iImagePtr im,int2 n,char **data,int2 *size)
{
	iDAFSError e;
	
	if (e = i_CompressImage(im,CCITT4)) return e;
	if (n < 0 || n >= im->d.rw.nchunks) return NoMoreData;
	if (data) *data = im->d.rw.chunks[n].data;
	if (size) *size = im->d.rw.chunks[n].len;
	return DAFSOK;
}

Dll iBox APIENTRY
i_GetMinBounds(iImagePtr im)
{
	iBox b={0,0,0,0};
	
	if (i_UncompressImage(im)) return b;
	b.x = im->d.im.x;
	b.y = im->d.im.y;
	b.w = im->d.im.w;
	b.h = im->d.im.h;
	return b;
}

Dll char * APIENTRY
i_GetImageTag(iImagePtr sp)
{
	return sp->tag;
}

Dll iDAFSError APIENTRY
i_SetImageTag(iImagePtr sp,char *tag)
{
	if (sp->tag) smfree(sp->tag);
	if (tag) {
		sp->tag = smalloc(strlen(tag) + 1);
		if (!sp->tag) return MallocFailError;
		strcpy(sp->tag,tag);
	} else sp->tag = 0;
	return DAFSOK;
}

static void rlfree(iImagePtr sp)
{
	int4 i;
	RlLinePtr cr;
	extern void freeimage(iImagePtr);
	
	cr = sp->d.im.rl;
	for (i = 0; i < sp->height; i++,cr++) {
		if (cr->code)
			smfree(cr->code);
	}
	smfree(sp->d.im.rl);
	if (sp->d.im.next)
		freeimage(sp->d.im.next);
}

static void rawfree(iImagePtr sp)
{
	int4 i;
	
	if (sp->d.rw.chunks) {
		for (i=0;i<sp->d.rw.nchunks;i++) {
			if (sp->d.rw.chunks[i].data)
				smfree(sp->d.rw.chunks[i].data);
		}
		smfree(sp->d.rw.chunks);
	}
}

void freeimage(iImagePtr ip)
{
	if (ip->tag) smfree(ip->tag);
	switch(ip->type){
	case Raw:
		rawfree(ip);
		break;      
		
	case Image:
		rlfree(ip);
		break;
	}
	smfree(ip);
}

Dll iImagePtr APIENTRY rotate(iImagePtr sp, short rot)
{
	iImagePtr np;
	iDAFSError e;
	
	if (rot == 180)
		return 0;
	np = rlnewimage(sp->height, sp->width, INITSIZE);
	if (!np)
		return 0;
	np->d.im.rot = sp->d.im.rot + rot;
	np->d.im.xmul = sp->d.im.ymul;
	np->d.im.ymul = sp->d.im.xmul;
	if (e = rotimg(sp, np, (rot < 0 ? 1 : -1),INITSIZE)) {
		freeimage(np);
		return 0;
	}
	np->d.im.pixcnt = sp->d.im.pixcnt;
	np->d.im.trncnt = sp->d.im.trncnt;
	np->dpi = sp->dpi;
	return np;
}

Dll transition * APIENTRY rlseekx(RlLinePtr cl, int2 x)
{
#if 1
	transition *s,*lrp,*hrp;
	int i;

	if (x == cl->run[cl->size-2].dn) return &cl->run[cl->size-2];
	if (cl->last >= 0) {
		s = &cl->run[cl->last];
		i=2;
		if (s->dn <= x) {
			while (s->dn <= x && i--)
				s++;
		} else {
			while (s->dn > x && i--)
				s--;
			s++;
		}
		if (i >= 0) return s;
	}
	lrp = cl->run;
	hrp = &cl->run[cl->size-1];
	do {
		s = lrp + ((hrp - lrp) / 2);
		if (s->dn > x)
			hrp = s;
		else if (s->dn <= x)
			lrp = s+1;
	} while (hrp != lrp);
	return lrp;
#else
	transition *s;

	if (x == cl->run[cl->size-2].dn) return &cl->run[cl->size-2];
	if (cl->last < 0) {
		s = cl->run;
		while (s->dn <= x)	/* crank out to where we care */
			s++;
	} else {
		s = &cl->run[cl->last];
		if (s->dn <= x) {
			while (s->dn <= x)	/* crank out to where we care */
				s++;
		} else {
			while (s->dn > x)	/* don't have to worry about
								 * s < cl->run */
				s--;
			s++;
		}
	}
	return s;
#endif
}

Dll iDAFSError APIENTRY
rlxscale(iImagePtr sp, iFixedPoint xmul, iImagePtr *result)
{
	iImagePtr tp;
	RlLinePtr fl, tl;
	short *t, *f;
	int4 i,temp,power,n;

	*result=0;
	tp = newimage(0,0);
	if (!tp)
		return MallocFailError;
	*tp = *sp;
	tp->type = Image;
	tp->d.im.next = 0;
	tp->d.im.prev = 0;
	tp->d.im.pixcnt = 0;
	tp->d.im.trncnt = 0;

	if ((long)tp->width*(long)i_FP_TO_INT2(xmul)+ i_FP_TO_INT2(i_FP_FRAC(xmul)*(long)tp->width)>32767L)
		return BadScaleError;

	tp->width = i_FP_TO_INT2(tp->width * xmul);
	tp->d.im.rl = (RlLinePtr) smalloc(sizeof(RlLine) * sp->height);
	if (!tp->d.im.rl) {
		smfree(tp);
		return MallocFailError;
	}
	fl = sp->d.im.rl;
	tl = tp->d.im.rl;

	/* check and see if ymul is power of two */
	for (temp=xmul,power=0; !(temp&1); temp>>=1, power++)
		;
	if (temp!=1)
		power=0;
	else
		power-=16;
		
	for (i = 0; i < sp->height; i++, tl++, fl++) {
		tl->code = (short *) smalloc(SL + (ST * fl->size));
		if (!tl->code) {
			while (i--)
				smfree((--tl)->code);
			smfree(tp->d.im.rl);
			smfree(tp);
			return MallocFailError;
		}
		tl->last = -1;
		tl->asize = fl->size;
		tl->size = fl->size;
		tl->code[0] = 0;
		tl->run = (transition*)&tl->code[1];
		t = (short *) tl->run;
		f = (short *) fl->run;
		if (power>0)
			while (*f != -1)
				*t++ = (*f++) << power;
		else if (power==0) {
			if (xmul>=iFP_ONE)
				while (*f != -1)
					*t++ = i_FP_TO_INT2(*f++ * xmul);
			else {
				/* this is to make sure t-1 points to something valid in while loop */
				if (*f!=-1)
					*t++ = i_FP_TO_INT2(*f++ * xmul);

				while (*f != -1) {
					*t = i_FP_TO_INT2(*f++ * xmul);
					if (*t!=*(t-1))
						t++;
					else if (*t==tp->width) {
						n=t-((short *)tl->run);
						if (n&1)
							t++;
						tl->size=(transition *)t - tl->run + 1;
						break;
					} else {
						t--;
						tl->size--;
						if (t==(short *)tl->run)
							if (*f!=-1)
								*t++ = i_FP_TO_INT2(*f++ * xmul);
					}
				}
			}
		} else {
			/* this is to make sure t-1 points to something valid in while loop */
			if (*f!=-1)
				*t++=(*f++)>>(-power);

			while (*f != -1) {
				*t = (*f++)>>(-power);
				if (*t!=*(t-1))
					t++;
				else if (*t==tp->width) {
					n=t-((short *)tl->run);
					if (n&1)
						t++;
					tl->size=(transition *)t - tl->run + 1;
					break;
				} else {
					t--;
					tl->size--;
					if (t==(short *)tl->run)
						if (*f!=-1)
							*t++=(*f++)>>(-power);
				}
			}
		}
		*t=-1;
	}
	tp->d.im.xmul = xmul;

	*result=tp;
	return DAFSOK;
}

Dll iDAFSError APIENTRY
rlyscale(iImagePtr sp, iFixedPoint ymul, iImagePtr *result)
{
	int newim;

	short rot;
	iImagePtr rp,rsp;
	iDAFSError e=DAFSOK;
	
	*result=0;
	/* should check if integer multiplication and do faster scale */

	for (rp=sp->d.im.next; rp; rp=rp->d.im.next)
		if (rp->d.im.rot==90 || rp->d.im.rot==-90)
			break;
	if (rp) {
		newim=FALSE;
	} else {
		rp=rotate(sp, 90);
		if (!rp) return MallocFailError;
		newim=TRUE;
	}
	rot=rp->d.im.rot;

	e=rlxscale(rp, ymul,&rsp);
	if (newim)
		freeimage(rp);
	if (e) return e;

	rp = rotate(rsp, -rot);
	freeimage(rsp);
	if (!rp)
		return MallocFailError;
	*result=rp;
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_ScaleImage(iImagePtr img, iFixedPoint xmag, iFixedPoint ymag)
{
	iImagePtr new_image=0, result=0;
	iImage temp;
	iDAFSError e;
	
	if (e = i_UncompressImage(img)) return e;

	if (xmag==0 || ymag==0)
		return BadScaleError;

	if (xmag==iFP_ONE && ymag==iFP_ONE)
		return DAFSOK;

	
	if (ymag!=iFP_ONE) {
		if (e=rlyscale(img,ymag,&new_image))
			return e;
	}
	if (img->d.im.next)
		freeimage(img->d.im.next);
	img->d.im.next=0;
	
	if (xmag!=iFP_ONE) {
		if (new_image) {
			if (e=rlxscale(new_image,xmag,&result)) {
				freeimage(new_image);
				return e;
			}
		} else {
			if (e=rlxscale(img,xmag, &result))
				return e;
		}
	} else {
		result=new_image;
		new_image=0;
	}
	if (new_image)
		freeimage(new_image);

	temp=*img;
	*img=*result;
	*result=temp;
	
	freeimage(result);
	img->d.im.w=i_FP_TO_INT2(img->d.im.w*xmag);
	img->d.im.h=i_FP_TO_INT2(img->d.im.h*ymag);
	return DAFSOK;
}

Dll iDAFSError APIENTRY
i_ConcatImage(iImagePtr left,iImagePtr right)
{
	iDAFSError e;
	RlLinePtr sr,cr,fr;
	int2 h,asize;
	transition *t,*f;
	short *temp,w,lw;
	
	if (e = i_UncompressImage(left)) return e;
	if (e = i_UncompressImage(right)) return e;
	if (left->type != Image) return WrongImageFormat;
	if (right->type != Image) return WrongImageFormat;
	if (left->height < right->height) {
		sr = (RlLinePtr) smrealloc(left->d.im.rl,sizeof(RlLine) * right->height);
		if (!sr) return MallocFailError;
		left->d.im.rl = sr;
		for (cr=&sr[left->height],h = left->height;h<right->height;h++,cr++) {
			asize = right->d.im.rl[h].size;
			cr->code = (short*)smalloc(SL + (ST * asize));
			if (!cr->code) {
				left->height = h;
				return MallocFailError;
			}
			cr->size = 2;
			cr->asize = asize;
			cr->run = (transition*)&cr->code[1];
			cr->code[0] = 0;
			cr->last = -1;
			cr->run[0].up = left->width;
			cr->run[0].dn = left->width;
			cr->run[1].up = -1;
		}
		left->height = h;
	} else
		sr = left->d.im.rl;
	lw = left->width;
	w = lw + right->width;
	for (cr=sr,fr = right->d.im.rl,h=0;h<right->height;fr++,cr++,h++) {
		asize = cr->size + fr->size - 1;
		if (cr->asize < asize) {
			temp = (short*)smrealloc(cr->code,SL + (ST * asize));
			if (!temp) return MallocFailError;
			cr->code=temp;
			cr->run = (transition*)&cr->code[1];
			cr->asize = asize;
			cr->code[0] = 0;
		}
		asize = fr->size-1;
		t = &cr->run[cr->size-2];
		f = fr->run;
		cr->size += fr->size - 2;
		if (t->up != lw) {
			if (!f->up) {
				t->dn = lw + f->dn;
				f++;
				asize--;
			} else
				cr->size++;
			t++;
		}
		while (asize--) {
			t->up = lw + f->up;
			t->dn = lw + f->dn;
			t++;
			f++;
		}
		t->up = -1;
	}
	for (;h<left->height;cr++,h++) {
		t = &cr->run[cr->size-2];
		t->dn = w;
		if (t->up == lw)
			t->up = w;
	}
	left->width = w;
#ifdef ECHECK
	rlcheckimage(left);
#endif
	return DAFSOK;
}

iImagePtr rlgetzone(iImagePtr sp, iBox b)
{
	iImagePtr np;
	RlLinePtr cr, sr;
	transition *s1, *s2;
	int4 i, j;
	np = newimage(b.w,b.h);
	if (!np)
		return 0;
	np->type = Image;

	np->d.im.next = 0;
	np->d.im.prev = 0;
	np->d.im.pixcnt = 0;
	np->d.im.trncnt = 0;
	np->d.im.rot = sp->d.im.rot;
	np->d.im.xmul = sp->d.im.xmul;
	np->d.im.ymul = sp->d.im.ymul;
	np->d.im.rl = (RlLinePtr) smalloc(sizeof(RlLine) * np->height);
	if (!np->d.im.rl) {
		smfree(np);
		return 0;
	}
	sr = &sp->d.im.rl[b.y];
	cr = np->d.im.rl;
	for (i = 0; i < np->height; i++, cr++,sr++) {
		s1 = rlseekx(sr, b.x);
		s2 = rlseekx(sr, b.x + b.w);
		cr->code = (short*) smalloc(SL + (ST * (s2 - s1 + 2)));
		if (!cr->code) {
			while (i)
				smfree(np->d.im.rl[--i].code);
			smfree(np->d.im.rl);
			smfree(np);
			return 0;
		}
		cr->run = (transition*)&cr->code[1];
		cr->code[0] = 0;
		cr->size = s2 - s1 + 2;
		cr->asize = cr->size;
		cr->last = -1;
		for (j = 0; s1 < s2; j++, s1++) {
			cr->run[j].up = s1->up - b.x;
			cr->run[j].dn = s1->dn - b.x;
			np->d.im.pixcnt += s1->dn - s1->up;
		}
		if (s2->up < (b.x + b.w)) {
			cr->run[j].up = s2->up - b.x;
			if (cr->run[j].up < 0)
				cr->run[j].up = 0;
			np->d.im.pixcnt += b.w - cr->run[j].up;
		} else {
			if (j && cr->run[j - 1].dn == b.w) {
				j--;
				cr->size--;
			} else
				cr->run[j].up = b.w;
		}
		cr->run[j].dn = b.w;
		cr->run[j + 1].up = -1;
		if (cr->run[0].up < 0) {
			np->d.im.pixcnt += cr->run[0].up;
			cr->run[0].up = 0;
		}
	}
	return np;
}

Dll void APIENTRY
i_DisposeImage(iImagePtr ip)
{
	if (!ip) return;
	ip->use--;
	if(ip->use <= 0){
		if (gCallBack) 
			gCallBack(mDestImg,gUserData,(iEntityPtr)ip);
		freeimage(ip);
	}
}

int4 
i_CountMem(iImagePtr ip)
{
	int4 i,tot;
	RlLinePtr cr;
	
	tot = s_getsize(ip);
	switch(ip->type){
	case Raw:
		if (ip->d.rw.chunks) {
			for (i=0;i<ip->d.rw.nchunks;i++) {
				if (ip->d.rw.chunks[i].data)
					tot += s_getsize(ip->d.rw.chunks[i].data);
			}
			tot += s_getsize(ip->d.rw.chunks);
		}
		break;      
		
	case Image:
		cr = ip->d.im.rl;
		for (i = ip->height; i--;cr++) {
			if (cr->code)
				tot += s_getsize(cr->code);
		}
		tot += s_getsize(ip->d.im.rl);
		if (ip->d.im.next)
			tot += i_CountMem(ip->d.im.next);
		break;
	}
	return tot;
}

