
#include "rafport.h"
#include "rafutil.h"
#include "dafstype.h"
#include "smemory.h"
#include "rlimage.h"
#include "entity.h"
#include "dafs.h"

static int4 
rlcheckline(RlLinePtr t, int4 width)
{
	int4 j;
	
	if(t->last>t->size-2) return 1;
	if (t->code[0]) return 1;
	if (t->run[0].up < 0) return 1;
	for (j = 0; j < (t->size - 2); j++) {
		if (t->run[j].up >= t->run[j].dn)
			return 1;
		if (j && (t->run[j].up <= t->run[j - 1].dn))
			return 1;
	}
	if (t->run[j].up > t->run[j].dn) return 1;
	if (t->run[j++].dn != width) return 1;
	if (t->run[j].up != -1)
		return 1;
	return 0;
}

Dll int4 APIENTRY
rlcheckimage(iImagePtr sp)
{
	int4 i;
	int4 result=0,badline;
	iDAFSError e;
	
	if (sp->type == Raw)
		if (e=i_UncompressImage(sp))
			return e;
	for (i = 0; i < sp->height; i++) {
		badline=rlcheckline(&sp->d.im.rl[i], sp->width);
		result|=badline;
		if (badline)
			debugf("bad image on line %ld\n",i);
	}
	return result;
}

static iDAFSError checkimage(iImagePtr sp)
{
	int4 i;
	iDAFSError err = DAFSOK;
	
	if (!sp->id) {
		sp->id = 1;
		switch(sp->type){
		case Image:	if (checkptr(sp->d.im.rl)) err = -11;
		else {
			for (i = 0; i < sp->height && !err; i++) {
				if (rlcheckline(&sp->d.im.rl[i], sp->width))
					err = -1;
			}
		}
		break;
		
		case Raw:
			if (checkptr(sp->d.rw.chunks)) err=-12;
			for (i = 0;i<sp->d.rw.nchunks && !err;i++) {
				if (checkptr(sp->d.rw.chunks[i].data))
					err=-12;
			}
			break;
		
		default:		break;
		}
	}
	return err;       
}

static iDAFSError check_a_entity(iEntityPtr ent) {
	
	iDAFSError err = DAFSOK;
	iData *data;
	iPropPtr prop;
	int4 index;
	iPData *pdata;

	if (checkptr(ent)) err = -1;
	if (!err && ent) {
		if (checkptr(ent->next)) err = -2;
		if (checkptr(ent->prev)) err = -3;
		if (checkptr(ent->prdata)) err = -4;
		if (checkptr(ent->data)) err = -5;
		data = ent->data;
		if (!err && data){ 
			if (checkptr(data->child)) err = -5;
			if (checkptr(data->last)) err = -7;
			if (checkptr(data->im)) err = -8;
			if (checkptr(data->userPtr)) err = -9;
			if (checkptr(data->props)) err = -10;
			if (data->im && !err) {
				err = checkimage(data->im);
			}
			if (data->props && !err) {
				for (prop=i_FirstProp(ent); (prop&&!err); 
                                     prop=i_NextProp(ent, prop)) {
					pdata = (iPData *) prop->data;

					switch(pdata->type) {
					case iPString:	if (checkptr(pdata->val.s)) err=-23;
						break;
					case iPUserData: if (checkptr(pdata->val.v)) err=-24;
					case iPUserIData:
						break;
					default:	break;
					}
				}
			}
			if (!err) {
				switch(data->dt) {
				case iDString: if (checkptr(data->d.at.text)) err=-25;
					break;
				case iDPosSet: if (checkptr(data->d.pp.posp)) err=-26;
				else {
					for (index=0;index<ent->num;index++)
						if (ent->whichpos[index] >= data->d.pp.len)
							err = -27;
				}
					break;
				case iD1PosSet: if (ent->num > 1) err = -28;
					break;
				case iDUserData: if (checkptr(data->d.ud.user)) err=-29;
				default:				break;
				}
			}
		}
	}
	if (err)
		halt("Bad Pointer In Entity");
	return err;
}
static iDAFSError checkentity(iEntityPtr me) {		
	iEntityPtr cur;
	
	if (check_a_entity(me)) return -1;
	for (cur = i_GetChildEntity(me);cur;cur = i_GetNextEntity(cur)) {
		if (checkentity(cur)) return -1;
	}
	return DAFSOK;
}

Dll iDAFSError APIENTRY
CheckEntity(iEntityPtr entity)
{
	ClearID(entity);
	return checkentity(entity);
}

static int4 compare_RLimage(iImagePtr image1, iImagePtr image2, int4 skip)
{
	int4 e=0, index=0;
	iRl RLimage1, RLimage2;
	
	RLimage1 = image1->d.im;
	RLimage2 = image2->d.im;
	
	if (RLimage1.xmul   != RLimage2.xmul)   e=1;
	if (RLimage1.ymul   != RLimage2.ymul)   e=1;
	if (RLimage1.rot    != RLimage2.rot)    e=1;
	if (RLimage1.ymul   != RLimage2.ymul)   e=1;
	if (RLimage1.x   != RLimage2.x)   e=1;
	if (RLimage1.y   != RLimage2.y)   e=1;
	if (RLimage1.w   != RLimage2.w)   e=1;
	if (RLimage1.h   != RLimage2.h)   e=1;
	
	if (skip) {
		if (((!RLimage1.next) && (RLimage2.next)) ||
			((RLimage1.next) && (!RLimage2.next))) e = 1;
		if (((!RLimage1.prev) && (RLimage2.prev)) ||
			((RLimage1.prev) && (!RLimage2.prev))) e = 1;
	}	
	for (index=0;!e && index<image1->height; index++){
		if (RLimage1.rl[index].size != RLimage2.rl[index].size) {
			e=1;
			break;
		}
		if (memcmp(	RLimage1.rl[index].run,
						RLimage2.rl[index].run,
						(RLimage1.rl[index].size * sizeof(transition)) -
						sizeof(short))) {
			e=1;
			break;
		}
	}	
	return e;
}

Dll int4 APIENTRY
compare_image(iImagePtr one_image, iImagePtr two_image, int4 convert_flag)
{	
	int4 e=0;
	
	if (i_UncompressImage(one_image)) halt("hi there2345");
	if (i_UncompressImage(two_image)) halt("Ho tawe98896");

	if(convert_flag) {
		if(one_image->type==Raw)
			i_UncompressImage(one_image);
		else if(two_image->type==Raw)
			i_UncompressImage(two_image);
	}
	if (one_image->id) return 0; 
	one_image->id=1;
	two_image->id=1; 
	if ((one_image->type    != two_image->type)) 	e=1;
	if ((one_image->use     != two_image->use)) 		e=1;
	if ((one_image->height  != two_image->height)) 	e=1;
	if ((one_image->width   != two_image->width)) 	e=1;
	if ((one_image->dpi     != two_image->dpi)) 		e=1;

	if (compare_RLimage(one_image, two_image, 0)) e=1;

	return e;
}
static int4  count, trigger;

static int4 compare_rect(BOX a, BOX b)
{
	int4 e=0;
	if (a.x1!=b.x1)
		e=1;
	else
		if(a.y1!=b.y1)
			e=2;
		else
			if(a.x2!=b.x2)
				e=3;
			else
				if(a.y2!=b.y2)
					e=4;
	return e;
}
static int4 compare_box(iBox a, iBox b)
{
	int4 e=0;
	if (a.x!=b.x)
		e=1;
	else
		if(a.y!=b.y)
			e=2;
		else
			if(a.w!=b.w)
				e=3;
			else
				if(a.h!=b.h)
					e=4;
	return e;
}
static int4 compare_bounds(iEntityPtr one, iEntityPtr two)
{
	int4 e=0, i,j;
	iOrthogonListPtr this, that;
	iBox box1, box2;
	iATag tag1, tag2;
	iBoxListPtr tweeter, woofer;
	
	tag1=one->data->area.a_type;
	tag2=two->data->area.a_type;
	if (tag1!=tag2)
		e=-1;
	else{
		switch(tag1){
		
		case iABox:
			box1=one->data->area.a.box;
			box2=two->data->area.a.box;
			if (compare_box(box1,box2))
				e=2;
			break;
		case iAOrthogonList:	
			this = one->data->area.a.brdr;
			that = two->data->area.a.brdr;
			if ((!this && that)||(!that && this))
				e=1;
			else
			 if (this){
				if (this->num_poly != that->num_poly)
					e=1;
				else 
					for (i=0; i<this->num_poly; i++)
						for (j=0; j<this->poly[i]->n; j++){
							if (this->poly[i]->point[j].x != that->poly[i]->point[j].x)
								e=1;
							if (this->poly[i]->point[j].y != that->poly[i]->point[j].y)
								e=1;
						}
				

				}
			break;
		case iABoxList:
			tweeter=one->data->area.a.box_list;
			woofer=one->data->area.a.box_list;
			if (tweeter->numRects!=woofer->numRects)
				e=3;
			else
				if (compare_rect(tweeter->extents,woofer->extents))
					e=4;
				else
					for(i=0; (i<tweeter->numRects)&&!e; i++)
						if (compare_rect(tweeter->rects[i],woofer->rects[i]))
							e=10+i;
			break;
		default:
			e=5;
			break;

		}
	}
	return e;
}

static int4 compare_prop(iEntityPtr one, iEntityPtr two)
{
	int4 e=0;
	iPropPtr one_props, two_props;
	iPData *one_pdata, *two_pdata;
	iPDataPtr one_data, two_data;
	
	if ((one->data->props) && (two->data->props)) {
		one_props = i_FirstProp(one);
		two_props = i_FirstProp(two);
		one_pdata = (iPDataPtr) one_props->data;
		two_pdata = (iPDataPtr) two_props->data;
		if (one_props && two_props) {	
			one_data = one_props->data;
			two_data = two_props->data;
			if (one_data->type != two_data->type) 
				e=1;
			if (one_data->size != two_data->size) 
				e=1;
			
			switch (one_pdata->type) {
			case iPUserPtr:
			case iPBool:
			case iPLong:	
				if (one_data->val.i != two_data->val.i) 
					e=1;
				break;
				
			case iPString: 
				if (strcmp(one_data->val.s, two_data->val.s))
					e=1;
				break;
			case iPUserData:	
				if (memcmp(one_data->val.v, two_data->val.v, 
						   one_data->size))
					e=1;
				break;
					
			default:		
				break;
			}
		} else {
			e=1;
		}
	}
	else
		if ((one->data->props) || (two->data->props))
		e=1;
	return e;
}

static int4 compare_posset(iDataPtr one_data, iDataPtr two_data)
{

	int4 e=0, num, i;
	if (one_data->dt!=two_data->dt)
		return 1;

	switch(one_data->dt) {
		case iDString:	if (strcmp(one_data->d.at.text, two_data->d.at.text)) e=1;
							break;
		case iDPosSet:	if (num = one_data->d.pp.len) {
								for (i=0;i<num;i++){
									if (one_data->d.pp.posp[i].chr != two_data->d.pp.posp[i].chr) e=1;
									if (one_data->d.pp.posp[i].conf != two_data->d.pp.posp[i].conf) e=1;
								}
							}
							break;
		
		case iDUserData:if (num = one_data->d.ud.len) {
								for (i=0;i<num;i++)
									/*if (one_data->d.ud.user[i] != two_data->d.ud.user[i]) e=1*/;
							}
							break;
		
		case iD1PosSet: if ((one_data->d.ps.chr != two_data->d.ps.chr) ||
								(one_data->d.ps.conf != two_data->d.ps.conf)) e=1;
							break;
		default:	break;
	}
	return e;
}
							
		
static int4 compare_a_entity(iEntityPtr one, iEntityPtr two,int4 convert_flag)
{
	int4 e=0, index;
	
	if ((!(one->next) && (two->next)) || ((one->next) && !(two->next))) e=1;
	if ((!(one->prev) && (two->prev)) || ((one->prev) && !(two->prev))) e=2;
	if ((!(one->prdata) && (two->prdata)) || ((one->prdata) && !(two->prdata))) e=3;
	if (one->num != two->num) e=4;
	else
		for (index=0;index<one->num;index++)
			if (one->whichpos[index] != two->whichpos[index]) e=4;

	if ((one->data) && (two->data)){		
		if (one->data->ct != two->data->ct) 
			e=5;
		if (compare_bounds(one, two)) e=6;
		if ((!(one->data->child) && (two->data->child)) ||
			((one->data->child) && !(two->data->child))) e=7;
			
		if (one->data->use != two->data->use) e=8;
		if (one->data->im && two->data->im) {
			if (compare_image(one->data->im, two->data->im, convert_flag)) e=9;
		} else if (one->data->im || two->data->im)
			e=9;
		if (compare_prop(one, two)) 
				e=10;
		
		if ((one->data->d.at.len != two->data->d.at.len)) e=11;
		if (strcmp(i_TypeToName(i_GetEntityType(one)),
			i_TypeToName( i_GetEntityType(two)))) 
			e=12;
		else
			if (compare_posset(one->data, two->data)) 
				e=13;
	}
	return e;
}

static int4 CompareEntitys(iEntityPtr zig, iEntityPtr zag,int4 convert_flag)
{
	iEntityPtr zigger, zagger;	

	count++;
	trigger = compare_a_entity(zig, zag,convert_flag);
	zigger = i_GetChildEntity(zig);
	zagger = i_GetChildEntity(zag);
	
	while (zigger && zagger && !trigger) {
		trigger = CompareEntitys(zigger, zagger,convert_flag);
		zigger = i_GetNextEntity(zigger);
		zagger = i_GetNextEntity(zagger);
	}
	return trigger;

}

static int4 rlcmpline(RlLinePtr t, RlLinePtr f,int4 width)
{
	int4 j;
	
	if (t->size != f->size)
		return 1;
	for (j = 0; j < (t->size - 1); j++) {
		if (t->run[j].up != f->run[j].up)
			return 1;
		if (t->run[j].dn != f->run[j].dn)
			return 1;
	}
	if (t->run[j].up != f->run[j].up)
		return 1;
	return 0;
}

int4 
rlcmpimage(iImagePtr sp,iImagePtr np)
{
	int4 i;
	if (sp->height != np->height)
		*(char *) 0 = 0;
	if (sp->width != np->width)
		*(char *) 0 = 0;
	for (i = 0; i < sp->height; i++)
		if (rlcmpline(&sp->d.im.rl[i], &np->d.im.rl[i],sp->width))
			*(char *) 0 = 0;
	return 0;
}

Dll int4 APIENTRY
i_CompareEntitys(iEntityPtr zig, iEntityPtr zag,int4 convert_flag)
{
	int4 e;
	
	count=0;
	trigger=0;
	ClearID(zig);
	ClearID(zag);
	e = CompareEntitys( zig, zag,convert_flag);
	ClearID(zig);
	ClearID(zag);
	if (e) e = count;  
	return e;
}
	

