/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
/*
 * $XConsortium: atowc.c,v 1.6 91/07/01 11:00:05 xguest Exp $
 */

/*
 * Copyright 1990, 1991 by OMRON Corporation, NTT Software Corporation,
 *                      and Nippon Telegraph and Telephone Corporation
 * Copyright 1991 by the Massachusetts Institute of Technology
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the names of OMRON, NTT Software, NTT, and M.I.T.
 * not be used in advertising or publicity pertaining to distribution of the
 * software without specific, written prior permission. OMRON, NTT Software,
 * NTT, and M.I.T. make no representations about the suitability of this
 * software for any purpose.  It is provided "as is" without express or
 * implied warranty.
 *
 * OMRON, NTT SOFTWARE, NTT, AND M.I.T. DISCLAIM ALL WARRANTIES WITH REGARD
 * TO THIS SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS, IN NO EVENT SHALL OMRON, NTT SOFTWARE, NTT, OR M.I.T. BE
 * LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES 
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 *	Author: Li Yuhong	OMRON Corporation
 *
 */
 
#include "rafport.h"
#include "rafutil.h"

#include "smemory.h"
#include "ucs.h"
#include "c-type.h"


#ifndef NULL
#define	NULL	0
#endif

/*
 * true if uc is DIS 10646's alphabet or digit, i.e,
 *	code from 0x20202030 to 0x20202039
 *	code from 0x20202041 to 0x2020205a
 *	code from 0x20202061 to 0x2020207a
 */

Dll int4 APIENTRY
isualnum( iUCode uc)
{
    return _isualnum(uc);
}

/*
 * true if uc is DIS 10646's alphabet, i.e,
 *	code from 0x20202041 to 0x2020205a
 *	code from 0x20202061 to 0x2020207a
 */

Dll int4 APIENTRY
isualpha( iUCode uc)
{
    return _isualpha(uc);
}

/*
 * true if uc is DIS 10646's ASCII, i.e, Latin-1 script, code 0x2020207F(DEL),
 * and control set(C0 PAD PAD PAD) plus UNULL-terminator UNULL.
 */
#ifdef DEL
#undef DEL
#endif

#define DEL	0x7f

Dll int4 APIENTRY
isuascii(iUCode uc)
{
	return(((uc&CONTROLSCRIPT) == CLPADPADPAD && iscntrl(wgetgroup(uc))) ||
		   ((uc&SCRIPT1) == LATINSCRIPT && (isprint(wgetcell(uc)) ||
											wgetcell(uc) == DEL)) || (uc == UNULL));	
}

#undef DEL


/*
 * true if uc is DIS 10646's digit, i.e,
 *	code from 0x20202030 to 0x20202039.
 */

Dll int4 APIENTRY
isudigit( iUCode uc)
{
    return _isudigit(uc);
}



/*
 * true if uc is DIS 10646's latin1, 
 * It's includes C0/C1/DEL(is it OK?) i.e,
 *	code from 0x20202000 to 0x2020207f
 */

Dll int4 APIENTRY
isulatin1( iUCode uc)
{
    return _isulatin1(uc);
}


/*
 * true if uc is DIS 10646's lower, i.e,
 *	code from 0x20202061 to 0x2020207a.
 */

Dll int4 APIENTRY
isulower( iUCode uc)
{
    return _isulower(uc);
}
/*
 * true if uc is DIS 10646's punctual.
 */

Dll int4 APIENTRY
isupunct( iUCode uc)
{
    return _isupunct(uc);
}


/*
 * true if uc is a whitespace.
 * where the whitespace is defined one of the following:
 *   . blank same as function isublank() or
 *   . control spaces that are defined same as ANSI C library, i.e.,
 *     any character for which isspace() returns true.
 *
 */

Dll int4 APIENTRY
isuspace( iUCode uc)
{
    return _isuspace(uc);
}

/*
 * true if uc is DIS 10646's upper, i.e,
 *	code from 0x20202041 to 0x2020205a.
 */

Dll int4 APIENTRY
isuupper( iUCode uc)
{
    return _isuupper(uc);
}

/*
 * true if uc is DIS 10646's xdigit, i.e,
 *	code from 0x20202030 to 0x20202039.
 *	code from 0x20202041 to 0x20202046.
 *	code from 0x20202061 to 0x20202066.
 */

Dll int4 APIENTRY
isuxdigit( iUCode uc)
{
    return _isuxdigit(uc);
}


/*
 * Copy uc string ws2 to ws1.  ws1 must be large enough.
 * return ws1
 */

Dll iUCode * APIENTRY
ucscpy(iUCode *ws1,
	   const iUCode *ws2)
{
	 iUCode *ows1;
	
	ows1 = ws1;
	while ((*ws1 = *ws2++) != UNULL)
		ws1++;
	return(ows1);
}
/*
 * Copy ws2 to ws1, truncating or null-padding to always copy n bytes
 * return ws1
 */

Dll iUCode * APIENTRY
ucsncpy(iUCode *ws1,
		const iUCode *ws2,
		int2 n)
{
	 int4 i;
	 iUCode *ows1;
	
	ows1 = ws1;
	for (i = 0; i < n; i++)
		if ((*ws1++ = *ws2++) == UNULL) {
			while (++i < n)
				*ws1++ = UNULL;
			return(ows1);
		}
	return(ows1);
}
/*
 * Concatenate ws2 on the end of ws1.  Ws1's space must be large enough.
 * Return ws1.
 */

Dll iUCode * APIENTRY
ucscat(iUCode *ws1,
	   const iUCode *ws2)
{
	 iUCode *ows1;
	
	ows1 = ws1;
	while (*ws1++ != UNULL)
		;
	--ws1;
	while ((*ws1++ = *ws2++) != UNULL)
		;
	return(ows1);
}
/*
 * Concatenate ws2 on the end of ws1.  Ws1's space must be large enough.
 * At most n characters are moved.
 * Return ws1.
 */

Dll iUCode * APIENTRY
ucsncat(iUCode *ws1,
		const iUCode *ws2,
		int2 n)
{
	 iUCode *ows1;
	
	ows1 = ws1;
	while (*ws1++ != UNULL)
		;
	--ws1;
	while ((*ws1++ = *ws2++) != UNULL)
		if (n-- == 0) {
			*--ws1 = UNULL;
			break;
		}
	return(ows1);
}

#if 0
/*
 * Duplicate ws1 and return it.  
 */

Dll iUCode * APIENTRY
ucsdup( iUCode *ws1)
{
	iUCode *ws2;
	 iUCode *ows1;
	
	ows1 = (iUCode *)smalloc((ucslen(ws1)+1)*sizeof(iUCode));
	ws2=ows1;
	while ((*ows1++ = *ws1++) != UNULL)
		;
	*ows1=0;
	return(ws2);
}
#endif
/*
 * Compare uc strings:  ws1>ws2: >0  ws1==ws2: 0  ws1<ws2: <0
 */

Dll int APIENTRY
ucscmp(const iUCode *ws1,
	   const iUCode *ws2)
{
	
	while (*ws1 == *ws2++)
		if (*ws1++ == UNULL)
			return(0);
	return(*ws1 - *--ws2);
}

/*
 * Compare uc strings (at most n bytes):  ws1>ws2: >0  ws1==ws2: 0  ws1<ws2: <0
 */

Dll int APIENTRY
ucsncmp(const iUCode *ws1,
		const iUCode *ws2,
		int2 n)
{
	while (n-- != 0 && *ws1 == *ws2++)
		if (*ws1++ == UNULL)
			return(0);
	return(n+1==0 ? 0 : *ws1 - *--ws2);
}
/*
 * Search first occurance of character uc in iUCode string; return the
 * position pointer. 
 * The UNULL is treated as a character occurance.
 */

Dll const iUCode * APIENTRY
ucschr( const iUCode *ws,  iUCode uc)
{
	while ((*ws != UNULL) && (*ws != uc)) ws++;
	if (*ws == uc)
		return(ws);   /* accept UNULL as character if uc is UNULL.*/
	else
		return(UNULL);
}

/*
 * Search last occurance of character uc in iUCode string; return the
 * position pointer.
 * The UNULL is treated as a character occurance.
 */

Dll const iUCode * APIENTRY
ucsrchr(const iUCode *ws,
		iUCode uc)
{
	 const iUCode *oldws = ws;
	
	while (*ws++ != UNULL);
	while ((ws != oldws) && (*ws != uc)) ws--;
	if (*ws == uc)
		return(ws);   /* accept UNULL as character if uc is UNULL.*/
	else
		return(UNULL);
}

/*
 * Returns the number of
 * non-NULL iUCode in iUCode string argument.
 */

Dll unsigned APIENTRY
ucslen(const iUCode *ws)
{
    unsigned n;
	
    n = 0;
    if (ws)
        while (*ws++ != UNULL)
            n++;
    return(n);
}

Dll void APIENTRY
ucsreverse(iUCode ucs[])
{
    int4 i, j;
    iUCode uc;
    for (i = 0, j = ucslen(ucs) - 1; i < j; i++, j--) {
		uc = ucs[i];
		ucs[i] = ucs[j];
		ucs[j] = uc;
    }
}



#if !defined(SYS_OS_AIX)
/*
 * convert character from DIS 10646's lower to upper, i.e,
 *	FROM code from 0x20202061 to 0x2020207a.
 *	TO   code from 0x20202041 to 0x2020205a.
 */

Dll iUCode APIENTRY
touupper(iUCode uc)
{
    return _towupper(uc);
}

/*
 * convert character DIS 10646's upper to lower, i.e,
 *	FROM code from 0x20202041 to 0x2020205a.
 *	TO   code from 0x20202061 to 0x2020207a.
 */

Dll iUCode APIENTRY
toulower(iUCode uc)
{
    return _towlower(uc);
}

#endif

Dll unsigned long APIENTRY
ucsspn(ws1,ws2)
	 iUCode *ws1,*ws2;
{
	 unsigned long n = 0;
	 iUCode *wp;
	 iUCode uc;
	
	while (uc = *ws1++) {
		for (wp = ws2; *wp!=UNULL; wp++)
			if (uc == *wp)
				break;
		if (!*wp) 
			return(n);
		n++;
	}
	return(n);
}


Dll unsigned long APIENTRY
ucscspn(ws1,ws2)
	 iUCode *ws1,*ws2;
{
	 unsigned long n = 0;
	 iUCode *wp;
	 iUCode uc;
	
	while (uc = *ws1++) {
		for (wp = ws2; *wp!=UNULL; wp++)
			if (uc == *wp)
				break;
		if (*wp) 
			return(n);
		n++;
	}
	return(n);
}


Dll iUCode * APIENTRY
ucspbrk(ws1, ws2)
	 iUCode *ws1, *ws2;
{
	 iUCode *wp;
	 iUCode uc;
	
	while ( uc = *ws1) {
		for (wp = ws2; *wp; wp++)
			if (uc == *wp) 
				return(ws1);
		ws1++;
	}
	return(NULL);
}

/*
 * Here's private function declarations
 */
static int4 checkbase(iUCode**,int4);	/* checks base			*/


/* uc2i(uc) -- convert iUCode to integer value
 *   IN:
 *	uc:	iUCode to be computed
 *   RETURN VALUE:
 *	-1:	not a digit
 *	other:	integer value
 */
static int4
uc2i( iUCode uc)
{
    if ( _isudigit(uc) ) return (_uctoa(uc) - '0');
    if ( _isuupper(uc) ) return (_uctoa(uc) - 'A' + 10 );
    if ( _isulower(uc) ) return (_uctoa(uc) - 'a' + 10 );
    return -1;
}

/* ucstol(ucs, ptr, base) -- convert wstring to integer
 *   IN:
 *	ucs:	string to be scanned as number
 *	base:	base of the number
 *   OUT:
 *	ptr:	next character of the number scanned, if ptr is not null.
 *   RETURN VALUE:
 *		value of the number
 *   ON ERROR:
 *		return value is 0 and *ptr==ucs.
 */
Dll long APIENTRY
ucstol(iUCode *ucs, iUCode **ptr, int4 base)
{
    iUCode *cur;	/* pointer where we are looking at	*/
    iUCode *start;	/* starting point of the number		*/
    long value;		/* value of iUCode string		*/
    int4  sign;		/* positive value or negative value	*/
    int4  temp;		/* value of current iUCode		*/
	
    /* first of all, looking at first character of ucs */
    cur = ucs;
	
    /* skipping spaces */
    while (_isuspace(*cur)) cur++;
	
    /* checking base */
    base = checkbase(&cur, base);
    if (base == 0) {
		/* invalid base value */
		if (ptr != NULL) *ptr = ucs;
		return 0;
    }
	
    /* get positive value or negative value */
    if (*cur == _atouc('-')) {
		sign = -1;
		cur ++;
    } else {
		sign = 1;
    }
	
    start = cur;
    value = 0;
    for (; *cur != UNULL; cur++) {
		temp = uc2i(*cur);
		if ( temp < 0 || temp >= base ) break;
		value = value * base + temp;
    }
    if (start == cur) {
		/* there's no number */
		if (ptr != NULL) *ptr = ucs;
		return 0;
    }
    if (ptr != NULL) *ptr = cur;
    return (sign>0) ? value : -value;
}


/* basevalue(ucs) -- returns the base expected
 *   IN:
 *	ucs:	iUCode string to be scanned
 *   RETURN VALUE:
 *	16: starting with "0x" or "0X" followed by hex-digit
 *	 8: starting with "0"
 *	10: starting with "1"-"9" and "-"
 */
static int4
basevalue( iUCode *ucs)
{
    /* first character should be digit or "-" */
    if (_isudigit(ucs[0])) {
		if (ucs[0] == _atouc('0')) {
			if ((ucs[1] == _atouc('x') || ucs[1] == _atouc('X'))
				&& _isuxdigit(ucs[2])) {
				return 16;	/* "0xH" or "0XH" (hexdecimal) */
			} else {
				return 8;	/* "0" or "0D" (octal) */
			}
		} else {
			return 10;		/* "1" - "9" (decimal) */
		}
    } else if (ucs[0] == _atouc('-')) {
		return 10;		/* "-" (perhaps decimal) */
    } else {
		/* The iUCode string is not a number */
		return -1;
    }
}

/* checkbase(ptr,base) -- checks whether base is valid or not.
 *   IN:
 *	ptr:	pointer to string to be scanned.
 *	base:	copy of the argument of ucstol()
 *   OUT:
 *	ptr:	next character where scanning should be started at
 *   RETURN VALUE:
 *	0:	invalid base or string
 *	other:	valid base value ( 0 < ret <= 36 )
 */
static int4
checkbase(iUCode** ptr,int4 base)
{
     iUCode *cur;	/* pointer where we are looking at	*/
	
    cur = *ptr;
    switch (base) {
	case 16:
		/* leading "0x" should be skipped */
		if (basevalue(cur)==16) *ptr = &cur[2];	/* skipping leading "0x" */
		return base;
	case 0:
		/* base should be computed from ucs */
		base = basevalue(cur);
		if (base < 0) return 0;		/* the str is not a number */
		if (base == 16) *ptr = &cur[2];	/* skipping leading "0x" */
		return base;
	default:
		/* check the range of base */
		if (base < 0 || base > 36) return 0;	/* invalid base range */
		return base;
    }
}

/*
// searches for one ucs inside on another ? 
*/
Dll iUCode  * APIENTRY
ucsucs(ws1,ws2)
	iUCode *ws1,*ws2;
{
	 iUCode *pos, *wp1 = ws1, *wp2 = ws2;
	 iUCode uc; 
	int4	 len2;
	
	len2 = ucslen(ws2);
	if (ws1 == NULL || ws2 == NULL || *ws2 == UNULL 
		|| *ws1 == UNULL || (len2 > ucslen(ws1)))
		return(NULL); 
	
	uc = *ws2;
	while (*ws1 != UNULL) {
		if ((wp1 = pos = (iUCode*)ucschr(ws1, uc)) == NULL)
			return(NULL);
		if (len2 > ucslen(wp1))
			return(NULL);
		while ( *wp2 != UNULL && *wp1 == *wp2){
			wp1++;
			wp2++;
		}
		if (*wp2 == UNULL) 
			return(pos);
		ws1 = pos + 1; /* forward ws1 for the next occurence of uc */
		wp2 = ws2; /* Restore wp2 to the start of ws2 */
	}
	return(NULL);
}




#ifdef	DEBUG
void
dump_mb(mb)
    unsigned char *mb;
{
    int4 len, i, j = 0;
	
    while(*mb) {
		len = mblen(mb, strlen(mb));
		if(len <= 0)	break;
		for (i = 0; i < len; i++) {
			debugf("%02x", *mb);
			mb++;
		}
		debugf(" ", *mb);
		j++;
    }
    debugf("\nmb len = %d\n", j);
}

void
dump_uc(uc)
    iUCode *uc;
{
    int4 len;
	
    len = ucslen(uc);
	
    while(*uc) {
		debugf("%08x ", *uc);
		uc++;
    }
    debugf("\n");
    
    debugf("uc len = %d\n", len);
	
}

#endif	/* DEBUG */
