
#include <stdio.h>
#define DEBUG
#include "rafport.h"
#include "rafutil.h"
#include "dafs.h"

#include "ifc.h"
#include "smemory.h"
#include "str-lib.h"
#include "c-lib.h"
#include "testdaf.h"

#define die() halt("Fix this\n")


typedef iDAFSError (*TestFunction)(void);	

struct Test {
	char* function;
	TestFunction test_function;
	};

typedef struct Test Test;


static int4 start, end,memory;
static ifc myargs[] = {
	{(char **) &start, -1,-1, 100, IFC_INT, "s", 0},
	{(char **) &end, -1,-1, 100, IFC_INT, "e", 0},
	{(char **) &memory,0, 3, 9, IFC_INT, "m", 0},
	{0, 0,0, 1, IFC_NA, 0, 0}
};

static 
int usage(char *s)
{
	printf( " -s XX  Starting or single test #.\n");
	printf( " -e XX  Ending test #.\n\n");
	printf( " If no starting point is provided\n");
	printf( " all tests will be perfomed.\n");
	printf( " -m for memory checking options\n");
	return 0;
}

int userdata;
iMess lastmess,create,update;

static iDAFSError APIENTRY
mycall(iMess mess,void *ud,iEntityPtr en)
{
#if defined(_WINDOWS) && !defined(WIN32)
__asm push ds
__asm push ax
__asm mov ax,ss
__asm mov ds,ax
__asm pop ax
#endif
	if ((int*)ud != &userdata)
		printf("callback didn't give me my user data\n");
	switch (mess) {
		case mUpdate:
		case mWillUpdate:
		case mWillMove:
		case mMove:
			lastmess = mess;
			update++;
			break;
		case mDestroy:
			lastmess = mess;
			break;
		case mCreate:
			lastmess = mess;
			create++;
			break;
		case mDestImg:
			break;
		default:
			printf("bad message\n");
	}
#if defined(_WINDOWS) && !defined(WIN32)
__asm pop ds
#endif
	return DAFSOK;
}

static iDAFSError 
movepoly(void)
{	
	iDAFSError err=DAFSOK;
	iOrthogonPtr poly;
	iBox box;
	
	box.x = 0;
	box.y = 0;
	box.h = 5;
	box.w = 5;
	if(i_Box2Orthogon(box, &poly))
		die();
	if (!poly) die();
	i_TranslateOrthogon(poly, 3, 4);
	if (poly->n==2){
		if ((poly->point[0].x != 3)||
			(poly->point[0].y != 4) ||
			(poly->point[1].x != 5) ||
			(poly->point[1].y != 5)) err = 1;
	}
	else{
		if ((poly->point[0].x != 3)||
			(poly->point[0].y != 4) ||
			(poly->point[2].x != 8) ||
			(poly->point[2].y != 9)) err = 1;
	}
	i_DisposeOrthogon(poly);
	return err;
}
			
static iDAFSError 
newentitytype(void) 
{
	iEntityPtr new_entity=0, refrence_entity=0, child_entity=0;
	iDAFSError e = DAFSOK;
	
	new_entity = i_NewEntity(0, 0);
	if (!new_entity) die();
	if (e = i_ReadEntity("refnew.dr", &refrence_entity)) die();
	if (!refrence_entity) die();
	if (e = i_CompareEntitys(refrence_entity, new_entity,0)) die();
	child_entity = i_NewEntity(55, new_entity);
	if (!child_entity) die();
	if ((i_GetParentEntity(child_entity)!=new_entity) ||  
		(i_GetChildEntity(new_entity)!=child_entity))
		die();
	if(i_GetEntityType(child_entity)!=55)
		die();
	i_DisposeEntity(new_entity);
	i_DisposeEntity(refrence_entity);
	return e;
}

static iEntityPtr 
str2en(char *word)
{
	iEntityPtr en, child;

	en = i_NewEntity(iWordT, NULL);
	if(!en)
		return 0;

	while(*word){
		child = i_NewEntity(iGlyphT, en);
		if(!child)
			return 0;

		i_SetGlyph(child, *word, 255);

		word++;
	}

	return en;
}

static iDAFSError 
orentity(void) {

	iEntityPtr parent_entity, child_entity, new;
	iDAFSError e;
	
	parent_entity = i_NewEntity(iNullT,0);
	if (!parent_entity) die();
	i_SetLogicalType(parent_entity, iAnd);
	child_entity = i_NewEntity(iNullT,parent_entity);
	if (!child_entity) die();
	if (e = i_OrEntity(&parent_entity)) die();
	if (i_GetLogicalType(parent_entity)!= iOr)
		die();
	new = i_GetChildEntity(parent_entity);
	if (i_GetParentEntity(child_entity)!= new)
		die();
	if ((i_GetParentEntity(new)!=parent_entity) ||
		( i_GetChildEntity(new)!= child_entity))
		die();
	i_DisposeEntity(parent_entity);
	return e;
}

static iDAFSError 
union_intersect(void)
{
	iBox box1={0,1,5,5}, box2={1,0,5,5}, result; 
	iOrthogonListPtr border1;
	iBoxListPtr region1, region2, reg;
	BOX rect;
	iOrthogonPtr poly;
	/*  boxes  */
	
	i_UnionBox(box1, box2, &result);
	if (result.x!=0 ||result.y!=0 || result.h!=6 || result.w !=6)
		die();
	i_IntersectBox(box1, box2, &result);
	if (result.x!=1 ||result.y!=1 || result.h!=4 || result.w !=4)
		die();
	if(i_Box2BoxList(box1, &region1))
		halt("failed test\n");
	if(i_Box2BoxList(box2, &region2))
		halt("failed test\n");
	i_BoxList2Box(region1, &result);
	if (!region1 || !region2) die();
	if (result.x!=box1.x ||result.y!=box1.y ||result.h!=box1.h ||result.w!=box1.w)
		die();
	reg=i_NewBoxList();
	if (i_UnionBoxList(region1, region2, reg))
		die();
	if (reg->numRects!=3)
		die();
	rect=reg->rects[0];
	if (rect.x1!=1 || rect.y1!=0 || rect.x2!=6 || rect.y2!=1)
		die();
	rect=reg->rects[1];
	if (rect.x1!=0 || rect.y1!=1 || rect.x2!=6 || rect.y2!=5)
		die();
	rect=reg->rects[2];
	if (rect.x1!=0 || rect.y1!=5 || rect.x2!=5 || rect.y2!=6)
		die();
	
	if (i_BoxList2OrthogonList(reg, &border1))
		die();
	if (border1->num_poly!=1)
		die();
	poly=border1->poly[0];
	if (poly->point[3].x!=6||poly->point[3].y!=5)
		die();

	i_DisposeBoxList(region1);
	i_DisposeBoxList(region2);
	i_DisposeBoxList(reg);
	i_DisposeOrthogonList(border1);

	return DAFSOK;
}

static iDAFSError 
shareentity(void) 
{

	iEntityPtr one, two, head;
	iDAFSError e = DAFSOK;
	
	head = i_NewEntity(iNullT, 0);
	one = i_NewEntity(iNullT, head);
	if (e = i_ShareEntity(one, head, &two)) die();
	if (!head || !one || !two) die();
	i_SetGlyph(one,'9',0);
	if (i_GetGlyph(two,0) != '9') die();
	i_DisposeEntity(head);
	return e;
}
int dummy(){
prall();
return 0;
}
static char imagebits[] = {
	0x12,0x34,0x56,0x78,
	0x9a,0xbc,0xde,0xf0,
	0xaa,0xaa,0xaa,0xaa,
	0x55,0x55,0x55,0x55,
	0x55,0x55,0x55,0x55,
	0x00,0x00,0x00,0x00,
	0x00,0x00,0x00,0x00,
	0x96,0x5d,0x12,0x48,
};

static char rotimagebits[] = {
0x86,
0x18,/* This was generated by the rotate and image2bit so it can't be testing */
0x04,/* the version that I used to create it */
0x9b,
0x06,
0x98,
0x87,
0x18,
0x06,
0x98,
0x07,
0x9b,
0x86,
0x9b,
0x04,
0x98,
0x06,
0x1b,
0x04,
0x9b,
0x06,
0x1b,
0x87,
0x18,
0x06,
0x9b,
0x07,
0x1b,
0x85,
0x18,
0x04,
0x18
};
 
static int hitit;
static iDAFSError APIENTRY writeproc(char *tag,iImagePtr im)
{
#if defined(_WINDOWS) && !defined(WIN32)
__asm push ds
__asm push ax
__asm mov ax,ss
__asm mov ds,ax
__asm pop ax
#endif
	if (!tag) die();
	if (strcmp(tag,"ATag")) die();
	hitit = 1;
	
#if defined(_WINDOWS) && !defined(WIN32)
__asm pop ds
#endif
	return DAFSOK;
}

static iDAFSError APIENTRY readproc(char *tag,iImagePtr *imp)
{
	iImagePtr im;

#if defined(_WINDOWS) && !defined(WIN32)
__asm push ds
__asm push ax
__asm mov ax,ss
__asm mov ds,ax
__asm pop ax
#endif
	if (!tag) die();
	if (strcmp(tag,"ATag")) die();
	hitit = 1;
	if (!imp) die();
	if (!(im = i_NewImage(48,32,300,NoComp))) die();
	*imp = im;
#if defined(_WINDOWS) && !defined(WIN32)
__asm pop ds
#endif
	return DAFSOK;
}

static iDAFSError imagecallback(void)
{
	iImagePtr im,im2;
	iEntityPtr en;
	char *tag;

	if (!(en = i_NewEntity(0,0))) die();
	if (!(im = i_NewImage(32,32,300,NoComp))) die();
	i_SetImage(en,im);
	im2 = i_GetImage(en);
	if (im2 != im) die();
	i_SetImageTag(im,"ATag");
	hitit = 0;
	if (i_WriteEntity("temp.dr",en,DAFSB)) die();
	if (hitit) die();
	i_DisposeEntity(en);
	i_DisposeImage(im);
	if (i_ReadEntity("temp.dr",&en)) die();
	im = i_GetImage(en);
	if (!im) die();
	tag = i_GetImageTag(im);
	if (!tag) die();
	if (strcmp(tag,"ATag")) die();
	
	i_SetImageCallBack(writeproc,readproc);
	if (i_WriteEntity("temp.dr",en,DAFSB)) die();
	if (!hitit) die();
	i_DisposeEntity(en);
	if (i_ReadEntity("temp.dr",&en)) die();
	im = i_GetImage(en);
	if (!im) die();
	tag = i_GetImageTag(im);
	if (!tag) die();
	if (strcmp(tag,"ATag")) die();
	i_SetImageCallBack(0,0);
	i_DisposeEntity(en);
	return DAFSOK;
}

static iDAFSError 
readwrite(void) 
{
	iEntityPtr original = 0;
	iEntityPtr copy = 0;
	iDAFSError err;
	iImagePtr im,rot;
	iBox b;
	char *bim,*data;
	int2 w,size;
	int i;
	
	if (!(im = i_NewImage(32,32,300,NoComp))) die();
	b=i_GetImageBounds(im);

	size = ((b.w+31)/32)*sizeof(long)*b.h;
	bim=(char *)smalloc(size+sizeof(long));
	if (!bim) die();
	if (err = i_Image2BitMap(im,b,i_INT2_TO_FP(1),&w,&bim)) die();
	for (i=0;i<size;i++) if (bim[i]) die();
	smfree(bim);
	i_DisposeImage(im);
	
	b.x=0;b.y=0;b.w=32;b.h=8;
	if (!(im = i_BitMap2Image(b,4,imagebits))) die();

	size = ((b.w+31)/32)*sizeof(long)*b.h;
	bim=(char *)smalloc(size+sizeof(long));
	if (i_Image2BitMap(im,b,i_INT2_TO_FP(1),&w,&bim)) die();
	for (i=0;i<sizeof(imagebits);i++) if (bim[i]!=imagebits[i]) die();
	smfree(bim);

	if(i_RotateImage(im, 90)) die(); /* test rotation some what */
	b = i_GetImageBounds(im);
	if (b.x || b.y || b.w != 8 || b.h != 32) die();
	size = ((b.w+31)/32)*sizeof(long)*b.h;
	bim=(char *)smalloc(size+sizeof(long));
	if (i_Image2BitMap(im,b,i_INT2_TO_FP(1),&w,&bim)) die();
	if (w != 4) die();
	data = bim;
	for (i=0;i<b.h;i++) {
		if (*data != rotimagebits[i]) die();
		data += 4;
	}
	smfree(bim);
	if(i_RotateImage(im, -90)) die();

	b = i_GetImageBounds(im);
	if (b.x || b.y || b.w != 32 || b.h != 8) die();
	
	if (i_GetDataFromImage(im,0,&bim,&size)) die();
	if (i_GetDataFromImage(im,1,0,0) != NoMoreData) die();
	if (!(data = smalloc(size))) die();
	memcpy(data,bim,size);
	i_DisposeImage(im);
	if (!(im = i_NewImage(32,8,300,CCITT4))) die();
	if (i_AddDataToImage(im,data,size)) die();
	smfree(data);
	b=i_GetImageBounds(im);

	size = ((b.w+31)/32)*sizeof(long)*b.h;
	bim=(char *)smalloc(size+sizeof(long));
	if (i_Image2BitMap(im,b,i_INT2_TO_FP(1),&w,&bim)) die();
	for (i=0;i<sizeof(imagebits);i++) if (bim[i]!=imagebits[i]) die();
	smfree(bim);
	i_DisposeImage(im);

	if (err = i_ReadEntity("read.dr",&original)) die();
	if (!original) die();
	if (err = i_WriteEntity("daftest.dr", original, DAFSB)) die();
	if (err = i_ReadEntity("daftest.dr",&copy)) die();
	if (!original) die();
	if (err = i_CompareEntitys(original, copy,0)) die();
	if (err = CheckEntity(original)) die();
	unlink("daftest.dr");
	i_DisposeEntity(original);
	i_DisposeEntity(copy);
	return err;
}                               

static iDAFSError 
readentitynoimage(void) 
{

	iEntityPtr one; /* buckle shoe */
	iDAFSError err;
	
	if (err = i_ReadEntityNoIm("read.dr", &one)) die();
	if (!one) die();
	if (i_GetImage(one)) die();
	if (one) i_DisposeEntity(one);
	return err;
}

static iDAFSError 
types(void)
{
	iTypeIndex i,t;
	iDAFSError err = DAFSOK;

	for (i=0; i< 10; i++){
		
		err=i_NameToType(i_TypeToName(i),&t);
		if (err) halt("error in i_NameToType %s\n",i_GetError(err));

		if (i != t)
			halt("Bad result in i_NameToType\n") ;
	}
	
	if (err = i_NameToType("Even Braver new type",&i))
		halt("error in i_NameToType %s\n",i_GetError(err));
	if (strcmp(i_TypeToName(i), "Even Braver new type"))
		halt("Bad result in i_NameToType\n") ;
	return err;
}
		
static iDAFSError 
getnextentity(void) 
{

	iEntityPtr new_entity;
	iDAFSError e = DAFSOK;
	
	new_entity = i_NewEntity(iNullT, 0);
	if (!new_entity) die();
	if (i_GetNextEntity(new_entity))
		die();
	return e;
}

static iDAFSError 
getsettype(void)
{

	iEntityPtr entity, entity2;
	iDAFSError err = DAFSOK;
	iTypeIndex t;
	
	entity = i_NewEntity(iNullT,0);
	entity2 = i_NewEntity(iNullT,0);
	if (strcmp(i_TypeToName(i_GetEntityType(entity)),"")) err = -1;
	if (i_GetEntityType(entity) != 0) err =  -1;
	i_SetEntityType(entity, 65);
	if (i_GetEntityType(entity) != 65) err =  -1;
	
	if (err=i_NameToType("All types great and small",&t))
		halt("error in i_NameToType %s\n",i_GetError(err));
	i_SetEntityType(entity, t);

	if (err=i_NameToType("All types wise and wonderful",&t))
		halt("error in i_NameToType %s\n",i_GetError(err));
	i_SetEntityType(entity2, t);

	if (strcmp(i_TypeToName(i_GetEntityType(entity)), "All types great and small")) err = -1;
	if (strcmp(i_TypeToName(i_GetEntityType(entity2)),"All types wise and wonderful")) err = -1;
	
	i_DisposeEntity(entity);
	i_DisposeEntity(entity2);
	return err;
}

static iDAFSError 
childtype(void) 
{

	iEntityPtr entity;
	iDAFSError err = DAFSOK;
	
	entity = i_NewEntity(iNullT,0);
	i_SetLogicalType(entity, iOr);
	if (i_GetLogicalType(entity) != iOr) err = -1;
	i_SetLogicalType(entity, iAnd);
	if (i_GetLogicalType(entity) != iAnd) err = -1;
	i_DisposeEntity(entity);
	return err;
}

static iDAFSError 
whichpos(void) 
{
    iDAFSError err = DAFSOK;        
	
#if 0
	iEntityPtr entity;
	
    entity = i_NewEntity(iNullT,0);
	if (i_GetWhichPos(entity) != -1) err = -1;
	i_SetWhichPos(entity, 86);
	if (i_GetWhichPos(entity) != 86) err = -1;
	i_DisposeEntity(entity);
#endif
    return err;
}

static iDAFSError 
movefrontback(void) 
{

	iEntityPtr parent, child1, child2, child3, grkid1, grkid3, reference=0;
	iDAFSError err = DAFSOK;
	iTypeIndex t;

	parent = i_NewEntity(iNullT,0);


	if (err=i_NameToType("child1",&t))
		halt("foo\n");
	child1 = i_NewEntity(t, parent);
	if (!child1)
		halt("bar\n");

	if (err=i_NameToType("child2",&t))
		halt("foo\n");
	child2 = i_NewEntity(t, parent);
	if (!child2)
		halt("bar\n");

	if (err=i_NameToType("child3",&t))
		halt("foo\n");
	child3 = i_NewEntity(t, parent);
	if (!child3)
		halt("bar\n");

	if (err=i_NameToType("grkid1",&t))
		halt("foo\n");
	grkid1 = i_NewEntity(t, child1);
	if (!grkid1)
		halt("bar\n");

	if (err=i_NameToType("grkid3",&t))
		halt("foo\n");
	grkid3 = i_NewEntity(t, child3);
	if (!grkid3)
		halt("bar\n");


	if (err = i_WriteEntity("move.dr", parent, DAFSB)) {
		i_DisposeEntity(parent);
		return err;
	}
	i_MoveToFront(child3, parent);
	i_MoveToBack(child1, parent);
	if ((i_GetNextEntity(child3)!=child2)||(i_GetPrevEntity(child1)!= child2)) halt("movefronttoback");
	if (i_GetPrevEntity(child3) || i_GetNextEntity(child1)) halt("movefronttoback");
	if (i_GetNextEntity(child2)!=child1) halt("movefronttoback");
	if (i_GetPrevEntity(child2)!=child3) halt("movefronttoback");
	if (i_GetChildEntity(parent)!=child3) halt("movefronttoback");
	if (i_GetLastEntity(parent)!=child1) halt("movefronttoback");
	if (i_GetChildEntity(child1)!= grkid1) halt("movefronttoback");
	if (i_GetChildEntity(child3)!=grkid3) halt("movefronttoback");
	if (i_GetParentEntity(child1) != parent) halt("movefronttoback");
	if (i_GetParentEntity(grkid1) != child1) halt("movefronttoback");
	i_MoveToFront(child1, parent);
	i_MoveToBack(child3, parent);

	if (!err)
		if (i_ReadEntity("move.dr", &reference)) err = 2;
	if(!err) 
		err = i_CompareEntitys(parent, reference,0);
	if (!err){
		i_MoveAfter(grkid1, child2);
		if (i_GetParentEntity(grkid1) != parent) err = -1;
		if (i_GetNextEntity(child2) != grkid1) err = -1;
		if (i_GetPrevEntity(grkid1) != child2) err = -1;
		if (i_GetNextEntity(grkid1) != child3) err = -1;
		if (i_GetPrevEntity(child3) != grkid1) err = -1;

		i_MoveBefore(grkid3, child2);
		if (i_GetParentEntity(grkid1) != parent) err = -1;
		if (i_GetNextEntity(child1) != grkid3) err = -1;
		if (i_GetPrevEntity(grkid3) != child1) err = -1;
		if (i_GetNextEntity(grkid3) != child2) err = -1;
		if (i_GetPrevEntity(child2) != grkid3) err = -1;
	}       
	if (!err){
		i_MoveAfter(grkid1, child3);
		if (i_GetParentEntity(grkid1) != parent) err = -1;
		if (i_GetNextEntity(child2) != child3) err = -1;
		if (i_GetPrevEntity(child3) != child2) err = -1;
		if (i_GetNextEntity(grkid1) != 0) err = -1;
		if (i_GetPrevEntity(grkid1) != child3) err = -1;
		if (i_GetLastEntity(parent) != grkid1) err = -1;

		i_MoveBefore(grkid3, child1);
		if (i_GetParentEntity(grkid3) != parent) err = -1;
		if (i_GetNextEntity(child1) != child2) err = -1;
		if (i_GetPrevEntity(grkid3) != 0) err = -1;
		if (i_GetNextEntity(grkid3) != child1) err = -1;
		if (i_GetPrevEntity(child2) != child1) err = -1;
		if (i_GetChildEntity(parent) != grkid3) err = -1;
	}       
	if (err!=2&&reference)
		i_DisposeEntity(reference);
	i_DisposeEntity(parent);
	return err;
}


static iDAFSError 
countchild(void) 
{

	iEntityPtr parent, child1, child2, child3, grkid1, grkid3;
	iDAFSError err = DAFSOK;
	iTypeIndex t;
		
	parent = i_NewEntity(iNullT,0);

	if (err=i_NameToType("child1",&t))
		halt("foo\n");
	child1 = i_NewEntity(t, parent);
	if (!child1)
		halt("bar\n");

	if (err=i_NameToType("child2",&t))
		halt("foo\n");
	child2 = i_NewEntity(t, parent);
	if (!child2)
		halt("bar\n");

	if (err=i_NameToType("child3",&t))
		halt("foo\n");
	child3 = i_NewEntity(t, parent);
	if (!child3)
		halt("bar\n");

	if (err=i_NameToType("grkid1",&t))
		halt("foo\n");
	grkid1 = i_NewEntity(t, child1);
	if (!grkid1)
		halt("bar\n");

	if (err=i_NameToType("grkid3",&t))
		halt("foo\n");
	grkid3 = i_NewEntity(t, child3);
	if (!grkid3)
		halt("bar\n");

	
	if (i_CountChildren(grkid1) != 0 ) err = -1;
	if (i_CountChildren(parent) != 3 ) err = -1;
	if (i_CountPrev(child1) != 0) err = -1;
	if (i_CountNext(child1) != 2) err = -1;
	if (i_CountPrev(child3) != 2) err = -1;
	if (i_CountNext(child3) != 0) err = -1;
	i_DisposeEntity(parent);
	return err;
}

/* used in properties test */
static void *data_ptr;

static void APIENTRY
freeCallback(void *ptr)
{
#if defined(_WINDOWS) && !defined(WIN32)
__asm push ds
__asm push ax
__asm mov ax,ss
__asm mov ds,ax
__asm pop ax
#endif
	if(ptr != data_ptr)
		halt("wrong pointer passed back\n");
	smfree(ptr);
#if defined(_WINDOWS) && !defined(WIN32)
__asm pop ds
#endif
}

static iDAFSError 
propertys(void) {
	
	iEntityPtr entity, duplicate;
	long Number;
	iTypeIndex t;
	char *message;
	iDAFSError err = DAFSOK;
	iUserPtrFree cbp;
	void *data_ptr2;
	
	if (err=i_NameToType("Type",&t))
		halt("foo\n");

	entity = i_NewEntity(t, 0);
	data_ptr = smalloc(10);
	if (i_SetUserPtrProp(entity, "test", data_ptr, freeCallback)) die();
	if (i_GetUserPtrProp(entity, "test", &data_ptr2,&cbp)) die();
	if (data_ptr != data_ptr2) die();
	if (cbp != freeCallback) die();
	if (i_GetUserPtrProp(entity, "test", &data_ptr2,0)) die();
	i_DisposeEntity(entity);
	/* data_ptr should be freed */

	entity = i_NewEntity(t, 0);
	data_ptr = i_NewEntity(iNullT,0);
	Number = 42;
	i_SetLongProp(entity, "test", Number);
	if (i_GetLongProp(entity, "test", &Number)) die();
	if (Number != 42) die();

	Number = 1;
	i_SetBoolProp(entity, "test", Number);
	if (i_GetBoolProp(entity, "test", (int4 *)&Number)) die();
	if (Number != 1) die();

	i_SetStringProp(entity, "test", "Hi There!!");
	if (i_GetStringProp(entity, "test", &message)) die();
	if (strcmp(message, "Hi There!!")) die();
	
	i_SetUserDataProp(entity, "test", "Hi There!!", 11);
	if (i_GetUserDataProp(entity, "test", (void *)&message, 0, 0)) die();
	if (strcmp(message, "Hi There!!")) die();

	i_SetUserPtrProp(entity, "test2", data_ptr, NULL);
	if (i_GetUserPtrProp(entity, "test2", &data_ptr2,&cbp)) die();
	if (data_ptr != data_ptr2) die();
	if (cbp) die();
	

	/* now test that error is returned */	
	if (!i_GetLongProp(entity, "nill", &Number)) die();
	if (!i_GetBoolProp(entity, "nill", (int4 *)&Number)) die();  
	if (!i_GetStringProp(entity, "nill", &message)) die();
	if (!i_GetUserDataProp(entity, "nill", (void *)&message, 0, 0)) die();
	if (!i_GetUserPtrProp(entity, "nill", &data_ptr2,0)) die();
	if (err)
		return err;
		
	/* test i_MoveAllProps  */
	duplicate = i_NewEntity(iNullT,entity);
	if (i_MoveAllProps(entity, duplicate)) die();
	if (!i_GetUserDataProp(entity, "test", (void *)&message, 0, 0)) die();
	if (!i_GetUserPtrProp(entity, "test2", &data_ptr2,0)) die();

	if (i_GetUserDataProp(duplicate, "test", (void *)&message, 0, 0)) die();
	if (strcmp(message, "Hi There!!")) die();

	if (i_GetUserPtrProp(duplicate, "test2", &data_ptr2,0)) die();
	if (data_ptr != data_ptr2) die();
	
	i_DisposeEntity(entity);
	i_DisposeEntity(data_ptr);
	return err;
}

static iDAFSError 
findprop(void) 
{

	iEntityPtr top, middle, bottom;
	iTypeIndex t;

	if (i_NameToType("",&t))
		halt("foo\n");
	
	top = i_NewEntity(t, 0);
	middle = i_NewEntity(t, top);
	bottom = i_NewEntity(t, middle);
	
	i_SetStringProp(top, "test", "Hi There!!");
	
	if (!i_FindProp(top, "test")) die();
	if (i_FindProp(middle, "test")) die();
	i_DisposeProp(top, "test");
	if (i_FindProp(top, "test")) die();
	i_DisposeEntity(top);
	
	return 0;
}

static iDAFSError 
typeprop(void) 
{
	long Number;
	iEntityPtr entity1, entity2, data;
	char *message;
	iDAFSError err = DAFSOK;
	iTypeIndex t;	
	void *data_ptr2;
	iUserPtrFree cbp;

	entity1 = i_NewEntity(iWordT,0);

	if (err=i_NameToType("new_type",&t))
		halt("foo\n");
	entity2 = i_NewEntity(t, 0);

	data_ptr = smalloc(1024);
	i_SetSessionUserPtrProp(t, "free-me", data_ptr, freeCallback);
	if (i_GetUserPtrProp(entity2, "free-me", &data_ptr2,&cbp)) die();
	if (data_ptr != data_ptr2) die();
	if (cbp != freeCallback) die();
	if (i_GetUserPtrProp(entity2, "free-me", &data_ptr2,0)) die();
	i_DisposeSessionProp(t, "free-me");
	/* data_ptr should be freed */

	Number = 42;
	i_SetSessionLongProp(t, "test", Number);
	Number = 0;
	if (i_GetLongProp(entity2, "test", &Number)) die();
	if (Number != 42) die();

	i_SetSessionLongProp(iWordT, "test", Number);
	Number = 0;
	if (i_GetLongProp(entity1, "test", &Number)) die();
	if (Number != 42) die();

	Number = 1;
	i_SetSessionBoolProp(t, "test", Number);
	Number = 0;
	if (i_GetBoolProp(entity2, "test", (int4 *)&Number)) die();
	if (Number != 1) die();

	i_SetSessionStringProp(t, "test", "Hi There!!");
	if (i_GetStringProp(entity2, "test", &message)) die();
	if (strcmp(message, "Hi There!!")) die();
	
	i_SetSessionUserDataProp(t, "test", "Hi There!!",11);
	if (i_GetUserDataProp(entity2, "test", (void *)&message, 0, 0)) die();
	if (strcmp(message, "Hi There!!")) die();

	i_SetSessionUserPtrProp(t, "test", entity1, NULL);
	if (i_GetUserPtrProp(entity2, "test", (void*)&data,0)) die();
	if (entity1 != data) die();
	
	i_DisposeEntity(entity2);
	i_DisposeEntity(entity1);
	return err;
}

iDAFSError comment(void) {

	iEntityPtr entity;
	iDAFSError err = DAFSOK;
	
	entity = i_NewEntity(iNullT,0);
	i_SetComment(entity, "Frodo lives");
	if (strcmp(i_GetComment(entity), "Frodo lives")) err = -1;
	i_DisposeEntity(entity); 
	return err;
}

static iDAFSError 
boxes(void) 
{

	iEntityPtr entity;
	iDAFSError err = DAFSOK;
	iBox one, two, inter, join;
	
	one.x = 1;
	one.y = 0;
	one.w = 4;
	one.h = 2;
	two.x = 0;
	two.y = 1;
	two.w = 2;
	two.h = 4;
	entity = i_NewEntity(iNullT,0);
	i_UnionBox(one, two, &join);
	i_IntersectBox(one, two, &inter);
	i_SetBox(entity, join);
	join = i_GetEnclosingBox(entity);
	if ((join.x!=0)||(join.y!=0)||(join.h!=5)||(join.w!=5)) err = -1;
	if ((inter.x!=1)||(inter.y!=1)||(inter.h!=1)||(inter.w!=1)) err = -1;
	i_DisposeEntity(entity);
	return err;
}

static iDAFSError 
gstext(void)
{

	iEntityPtr top,entity;
	iTypeIndex t;
	short conf,buf[2];
	
	if (i_NameToType("Parent",&t)) die();
	
	entity = top = i_NewEntity(t, 0);
	if (i_SetText(entity, "TeXt",34)) die();
	if (i_GetEDataType(entity)!=iDString) die();
	if (strcmp(i_GetText(entity,&conf), "TeXt")) die();
	if (conf != 34) die();
	i_GetText(entity,0);

	entity = i_NewEntity(t, entity);
	buf[0] = '9';
	buf[1] = 0;
	if (i_SetUText(entity, buf,34)) die();
	if (i_GetEDataType(entity)!=iDUString) die();
	if (ucscmp(i_GetUText(entity,&conf),buf)) die();
	if (conf != 34) die();
	i_GetUText(entity,0);
	
	if (i_WriteEntity("fred.dr",top,DAFSB)) die();
	i_DisposeEntity(top);
	if (i_ReadEntity("fred.dr",&top)) die();
	entity = top;
	if (i_GetEDataType(entity)!=iDString) die();
	if (strcmp(i_GetText(entity,&conf), "TeXt")) die();
	if (conf != 34) die();
	entity = i_GetChildEntity(entity);
	if (i_GetEDataType(entity)!=iDUString) die();
	if (ucscmp(i_GetUText(entity,&conf),buf)) die();
	if (conf != 34) die();
		
	i_DisposeEntity(top);
	return 0;
}

static iDAFSError
uatable(void)
{
	iEntityPtr en;
	iUATablePtr ua_table;
	iDAFSError e;
	char *text;

	en = str2en("this is a test@");
	if(!en) die();

	if(e = i_ReadUATable(&ua_table, "ua.txt"))
		die();
	
	text = i_AllocASCIIText(en, ua_table);
	if(!text) die();

	if (e = strcmp(text, "This is a TesT[Ampersand]")) die();
	smfree(text);

	i_DisposeUATable(ua_table);
	i_DisposeEntity(en);
	return DAFSOK; 
}

static iDAFSError 
gsglyph(void)
{

	iEntityPtr entity, entity2,child;
	iDAFSError err = DAFSOK;

	iTypeIndex t;
	
	if (err=i_NameToType("Parent",&t))
		halt("foo\n");

	entity = i_NewEntity(t, 0);

	if (err=i_NameToType("Child",&t))
		halt("foo\n");
	child = i_NewEntity(t, entity);

	err = i_ShareEntity(child, entity, &entity2);
	if (err)
		return err;
	i_SetGlyph(child, 'Q', 0);
	if (i_GetEDataType(child)!=iD1PosSet) err = -1;
	if (!err) 
		if (i_GetGlyph(child,0) != 'Q') err = -1;
	if (!err) 
		if (i_GetEDataType(entity2)!=iD1PosSet) err = -1;
	if (!err) 
		if (i_GetGlyph(entity2,0) != 'Q') err = -1;
	i_DisposeEntity(entity);

	return err;
}

static iDAFSError 
gsconf(void)
{

	iEntityPtr entity;
	iDAFSError err = DAFSOK;
	iTypeIndex t;
	
	if (err=i_NameToType("Parent",&t))
		halt("foo\n");
	
	entity = i_NewEntity(t, 0);
	i_SetConf(entity, 255);
	if (i_GetEDataType(entity)!=iDConf) err = -1;
	if (!err) 
		if (i_GetConf(entity) != 255) err = -1;
	i_DisposeEntity(entity);
	return err;
}

static iDAFSError 
gsposset(void)
{

	iEntityPtr entity;
	iDAFSError err = DAFSOK;
	iPosSet posset[2];
	iPosSetPtr posptr;
	int2 num=2;
	
	iTypeIndex t;
	
	if (err=i_NameToType("Parent",&t))
		halt("foo\n");

	posset[0].chr = 'Q';
	posset[0].conf = 3;
	posset[1].chr = 'O';
	posset[1].conf = 4;
	entity = i_NewEntity(t, 0);
	if (i_SetPosSet(entity,num, (void*)&posset)) err = -1;
	if (i_GetEDataType(entity)!=iDPosSet) err = -1;
	if (!err) {
		posptr = i_GetPosSet(entity, &num);
		if (posptr[0].chr != posset[0].chr) err = -1;
		if (posptr[0].conf != posset[0].conf) err = -1;
		if (posptr[1].chr != posset[1].chr) err = -1;
		if (posptr[1].conf != posset[1].conf) err = -1;
	}       
	i_DisposeEntity(entity); 
	return err;
}

static iDAFSError 
cloneimage(void) 
{
	iEntityPtr entity1 = 0;
	iImagePtr im,ci;
	iDAFSError err = DAFSOK;
	iBox box;

	if (!(im = i_NewImage(50,50,300,NoComp))) die();
	if (!(entity1 = i_NewEntity(iNullT, 0))) die();
	i_SetImage(entity1, im);

	box.x = 1;
	box.y = 3;
	box.w = 10;
	box.h = 5;
	err = i_SetBox(entity1, box);		
	if (!(ci = i_CloneEntityImage(entity1))) die();
	if (i_GetImageWidth(ci) != 10) die();
	if (i_GetImageHeight(ci) != 5) die();
	i_DisposeImage(ci);
	
	i_DisposeImage(im);
	i_DisposeEntity(entity1);
	return err;
}

static iDAFSError 
rwimage(void)
{
	iImagePtr image1=0, image2=0, image3=0, ref_image=0, image0=0;
	iDAFSError err;
	
	err = i_ReadImage("m_strips.tif", &ref_image);
	if (!err)
		if (i_WriteImage("temp_orig.tif", ref_image, CCITT4)) err = -1;
	if (!err)
		err = i_ReadImage("temp_orig.tif", &image0);
	if (!err)
		err = compare_image(ref_image, image0, 0);
	if (!err)
		err = rlcheckimage(ref_image);
	if (!err) {
		i_SetImageRes(ref_image, 300);
		if (i_WriteImage("temp0.tif", ref_image, NoComp)) err = -1;
		if (i_WriteImage("temp4.tif", ref_image, CCITT4)) err = -1;
		if (!err) {
			if (i_ReadImage("temp0.tif", &image1)) err = -1;
			err = rlcheckimage(image1);
		}
		if (!err) {
			if (i_ReadImage("temp4.tif", &image2)) err = -1;
			err = rlcheckimage(image2);
		}
		err = rlcheckimage(image2);
		if (!err)
			err = compare_image(ref_image, image1, 0);
		if (!err)
			err = compare_image(ref_image, image2, 0);
	}
	if (!err) {
		unlink("temp_orig.tif");
		unlink("temp0.tif");
		unlink("temp4.tif");
	}
	if (image0) 
		i_DisposeImage(image0); 
	if (image1)
		i_DisposeImage(image1);
	if (image2)
		i_DisposeImage(image2);
	if (image3)
		i_DisposeImage(image3);
	if (!err) {
		i_SetImageRes(ref_image, 300);
		if (i_WriteImage("temp0.pda", ref_image, NoComp)) err = -1;
		if (i_WriteImage("temp4.pda", ref_image, CCITT4)) err = -1;
		if (!err)
			if (i_ReadImage("temp0.pda", &image1)) err = -1;
		if (!err)
			if (i_ReadImage("temp4.pda", &image2)) err = -1;
		if (!err)
			err = compare_image(ref_image, image1, 0);
		if (!err)
			err = compare_image(ref_image, image2, 0);
	}
	if (!err) {
		unlink("temp0.pda");
		unlink("temp4.pda");
	}
	if (image1)
		i_DisposeImage(image1);
	if (image2)
		i_DisposeImage(image2);
	if (ref_image)
		i_DisposeImage(ref_image);
	return err;
}
	
static iDAFSError 
imageext(void)
{
	iDAFSError err = DAFSOK;

	if (!i_ImageExt("a.pda")) err = -1;
	if (!i_ImageExt("a.PDA")) err = -1;
	if (!i_ImageExt("a.tif")) err = -1;
	if (!i_ImageExt("a.TIF")) err = -1;
	if (!i_ImageExt("a.raw")) err = -1;
	if (!i_ImageExt("a.RAW")) err = -1;
	if (!i_ImageExt("a.tiff")) err = -1;
	if (!i_ImageExt("a.TIFF")) err = -1;
	if (i_ImageExt("theirs.pict")) err = -1;
	if (i_ImageExt("tif")) err = -1;
	
	return err;
}

static iDAFSError 
imagewh(void)
{
	iDAFSError err = DAFSOK;
	iImagePtr image;
	iEntityPtr entity;

	if (i_ReadEntity("test.dr", &entity))
		halt("failed to read test.dr\n");

	image = i_GetImage(entity);

	if (i_GetImageWidth(image) != 2560) err = -1;
	if (i_GetImageHeight(image) != 3328) err = -1;
	
	i_DisposeEntity(entity);

	return err;
}

static iDAFSError 
poslimit(void) {
#if 0
	iEntityPtr ent, ent2;
	int n;
	int2 pos_n;
	short info,thresh;
	iPosSet pos[5];
	iPosSetPtr pp;
	iDAFSError err;

	iTypeIndex t;
	
	if (err=i_NameToType("type",&t))
		halt("foo\n");

	ent = i_NewEntity(t,0);
	for (n=0;n<5;n++) {
		pos[n].chr = n;
		pos[n].conf = n;
	}
	i_SetPosSet(ent,n,pos);
	i_AddPosLimit(ent, 2);
	i_WriteEntity("temp.dr", ent, DAFSB);
	i_ReadEntity("temp.dr", &ent2);
	if (err = i_CompareEntitys(ent, ent2,0))
		return err;
	i_AddPosLimit(ent, 0);
	info = i_GetLimitGlyph(ent);
	if (info != 0)
		return -1;

	thresh=0;
	pp = i_GetTrimPosSet(ent, pos, &pos_n, thresh);
	if (pp != pos)
		halt("bitterly complain\n");
	if (pos_n != 2)
		return -1;
	if (pos[0].chr !=0 )
		return -1;
	if (pos[1].chr != 2)
		return -1;
	i_TrimPosLimit(ent);
	if (ent->data->d.posp[1].chr != 2)
		return -1;
	i_AddPosLimit(ent, 48);
	if (ent->whichpos[ent->num-1] != 48)
		return -1;
	i_NoPosLimit(ent);
	if (ent->num)
		return -1;

	i_DisposeEntity(ent);
	i_DisposeEntity(ent2);
	return err;
#else
	return 0;
#endif
}

static iDAFSError 
memtest(void)
{
	iDAFSError err=DAFSOK;
	char **ptr;
	int i;
	
	s_SetMemFlags(3);
	ptr=(char**)smalloc(sizeof(char*)*100);
	if (!ptr)
		err=MallocFailError;
	for(i=0;(i<100)&&!err;i++) {
		ptr[i]=smalloc(i);
		if(!ptr && !(i%3) )
			err=DAFSOK;
		else if(!ptr[i])
			err=MallocFailError;
	}
	if (!err){
		for(i=0; i<100; i++)
			smfree(ptr[i]);
		smfree(ptr);
	}
	return err;

}

static iDAFSError 
byteswap(void)
{
	iDAFSError err;
	iEntityPtr sun_ent=0, pc_ent=0;	
	err = i_ReadEntity("pcdata.dr", &pc_ent);
	if (!err)
		err = i_ReadEntity("sundata.dr", &sun_ent);
	if (!err)
		err = i_CompareEntitys(pc_ent, sun_ent,0);
	if (pc_ent)
		i_DisposeEntity(pc_ent);
	if (sun_ent)
		i_DisposeEntity(sun_ent);
	return err;
}

static iDAFSError 
search(void)
{
	iDAFSError err;
	iEntityPtr parent, child1, child2, child3, grkid1, grkid2,ent;
	
	parent = i_NewEntity(iNullT, 0);
	
	if(!parent) return MallocFailError;
	child1 = i_NewEntity(iNullT, parent);
	if(!child1) return MallocFailError;
	if (err = i_ShareEntity(child1, parent, &child2))
		return err;
	child3 = i_NewEntity(iNullT, parent);
	if(!child3) return MallocFailError;

	grkid1 = i_NewEntity(iNullT, child1);
	if(!grkid1) return MallocFailError;

	if (err=i_ShareEntity(child3, child1, &grkid2))
		return err;
	ent = i_FindEntityForward(parent, 0, ALL, 0);
	if (ent!=child1)
		return -1;
	ent = i_FindEntityForward(ent,  0, ALL, 0);
	if (ent!=grkid1)
		return -1;
	ent = i_FindEntityForward(ent,  0, ALL, 0);
	if (ent!=grkid2)
		return -1;
	ent = i_FindEntityForward(ent,  0, ALL, 0);
	if (ent!=child2)
		return -1;
	ent = i_FindEntityForward(ent,  0, ALL, 0);
	if (ent!=grkid1)
		return -1;
	ent = i_FindEntityForward(ent,  0, ALL, 0);
	if (ent!=grkid2)
		return -1;
	ent = i_FindEntityForward(ent,  0, ALL, 0);
	if (ent!=child3)
		return -1;
	ent = i_FindEntityForward(ent, 0, ALL, 0);
	if (ent)
		return -1;

	ent = i_FindEntityBack(child3, 0, ALL, 0);
	if (ent!=grkid2)
		return -1;
	ent = i_FindEntityBack(ent,  0, ALL, 0);
	if (ent!=grkid1)
		return -1;
	ent = i_FindEntityBack(ent, 0, ALL, 0);
	if (ent!=child2)
		return -1;
	ent = i_FindEntityBack(ent, 0, ALL, 0);
	if (ent!=grkid2)
		return -1;
	ent = i_FindEntityBack(ent, 0, ALL, 0);
	if (ent!=grkid1)
		return -1;
	ent = i_FindEntityBack(ent, 0, ALL, 0);
	if (ent!=child1)
		return -1;
	ent = i_FindEntityBack(ent, 0, ALL, 0);
	if (ent!=parent)
		return -1;	
		
	i_DisposeEntity(parent);
	return DAFSOK;
}

Test test_array[] = {
	{"Union Intersect ", union_intersect},
	{"i_NewEntity ",  newentitytype},
	{"i_OrEntity ", orentity},
	{"i_ShareEntity ", shareentity},
	{"Read/Write Entity ", readwrite},
	{"SetImageCallBack",imagecallback},
	{"Read W/o image ",readentitynoimage},
	{"Typetable referencing ",types},
	{"Get/Set types ",getsettype},
	{"Move Polygon ", movepoly},
	{"Get/Set child type ",childtype},
	{"Get/Set Which Pos ", whichpos},
	{"Entity Navagation ", movefrontback},
	{"Search F/B       ", search},
	{"Children count ", countchild},
	{"Set/Get Property ", propertys},
	{"Find Property    ", findprop},
	{"Get/Set Comments ", comment},
	{"Boxes          ", boxes},
	{"Get/Set Text   ", gstext},
	{"Read UA Table ", uatable},
	{"Get/Set Glyph  ", gsglyph},
	{"Get/Set Conf   ", gsconf},
	{"Get/Set Pos Set ", gsposset},
	{"Read write image ", rwimage},
	{"Image Extension ", imageext}, 
	{"Get Image H & W ", imagewh},  
	{"Get/Set Type Props ", typeprop},
	{"Pos Limit Routines ", poslimit},
	{"Memory Testing ", memtest},
	{"Clone Ent. Im. ", cloneimage}, 
	{"Data Byte Swapping ", byteswap},
	{0,0}
};

static iDAFSError 
run_tests(int s, int e) 
{
	int i, tot_err = 0;
	iDAFSError err;
	
	printf("\n");
	
	s_InitLib(memory, 0, 0);
	i_InitLib();
	i_SetCallBack(mycall,(void*)&userdata);
	for (i=s; i!=e && test_array[i].test_function; i++) {
		printf( "%2u %s",i, test_array[i].function);
		if (err=test_array[i].test_function()){
			printf(" - FAILURE - %s\n",i_GetError(err));
			tot_err++;
		}
		else {
			printf(" - ok\n");
		}
	}
	i_ExitLib();
	s_ExitLib();
	printf("-----------\nTotal failures: %d\n",tot_err);
	return DAFSOK;
}

int
main(int c, char *v[])
{       
	if (processargs(c, v, myargs))
		exit(usage(v[0]));
	if (start < 0) {
		start = 0;
		if(end < 0)
			end = 100;
    } else if (end < 0) 
		end = start+1;
	run_tests(start, end);

	return 0;
}
