
#include <stdio.h>
#include "rafport.h"

#include "rafutil.h"

#include "ifc.h"
#include "dafs.h"
#include "smemory.h"

static char *filename, *bunchfile;
static int verbose;
int debug=1;

static ifc myargs[] = {
	{&filename, 0,(int) "bunch.db", 1, IFC_STR, "f", 0},
	{&bunchfile, 0,0, 1, IFC_STR, "r", 0},
	{(char **) &verbose, 0,0, 1, IFC_FLAG, "v", 0},
	{0,0, 0, 0, IFC_NA, 0, 0}
};

static void
usage(char *p)
{
	fprintf(stderr, "usage: %s [-f <outputfile>] [-r <bunchfile>] [-v] file\n", p);
	fprintf(stderr, "-f\t changes the name of the output file.  Default is bunch.db\n");
	fprintf(stderr, "-r\t reverses the process, and unbunches the bunchfile.\n");
	fprintf(stderr, "-v\t prints out a message for every file concatenated.\n");
	exit(-1);
}

int
main(int c, char *v[])
{
  
  char *name;
	int i;
	iDAFSError error;
	iEntityPtr fp, Total, doc;
	i_InitLib();
	if (processargs(c, v, myargs))
		usage(v[0]);

	if (!bunchfile) {
		iTypeIndex bunch_index;
		error = i_NameToType("iBunch", &bunch_index);
		if(error)
			halt("couldn't create bunch type.\n");
														 
		Total = i_NewEntity(bunch_index, NULL);
		for (i = 1; i < c; i++) {
			if (!v[i])
				continue;
			if (verbose)
				printf("concatenating file %s\n", v[i]);
			error = i_ReadEntity(v[i], &fp);
			if (error) {
			  fprintf(stderr, "couldn't open %s\n", v[i]);
			  continue;
			}
			i_SetComment(fp, v[i]);
			i_MoveToBack(fp, Total);
			if (error) {
			  fprintf(stderr, "couldn't transfer %s\n", v[i]);
			  continue;
			}

		}
		error = i_WriteEntity((char *) filename, Total, DAFSB);
		if (error) 
		  fprintf(stderr, "couldn't write %s\n", filename);

		i_DisposeEntity(Total);

	} else {
		error = i_ReadEntity(bunchfile, &fp);
		if (error) halt("Error reading %s\n",bunchfile);
		if (verbose)
			printf("using bunchfile %s\n", bunchfile);
		for (doc = i_GetChildEntity(fp);doc;doc = i_GetNextEntity(doc)) {
			filename = i_GetComment(doc);
		        name=(char*)smalloc(strlen(filename)+1);
			strcpy(name,filename);
			
			if (!filename) printf("This file has no name associated\n");
			i_SetComment(doc, "");
			if (verbose)
				printf("un-concatenating file %s\n", name);
			error = i_WriteEntity((char *) name, doc, DAFSB);
			if (error)
				halt("Error writing %s\n",name);
			smfree(name);
		}
	}
	i_ExitLib();
	return 0;
}
