
/*
 * Copyright (c) 1994, 1995  RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
#include "illum.h"

#include "rafutil.h"

#include "text/text.h"
#include "text/TextElement.h"
#include "TypeFrame.h"
#include "EntityAux.h"
#include "View.h"
#include "ViewCanvas.h"
#include "scroll.h"

static void redrawWindow(TypeFrame *frame);

void refreshFlagParent(TextElement *element);

#if 0
#define MESSAGE_UPDATING
#endif

int
chainedPositionCalculate(TextElement *element, TextElement *sib, TextElement *parent)
{
	int new_rx;
	int new_ry;
	
	element->position_valid = TRUE;
	
	new_rx = 0;
	new_ry = 0;
	
	if(parent == 0){
		// you are the root
		// 00 fine
		return 0;
	} else {
		// your relative xy is some function of your sibs dimensions
		// maybe next maybe prev depending on direction
		if(sib) {
			// bogus I know that I want relative to parent
			new_rx = sib->rx;
			new_ry = sib->ry;
			short sib_w;
			short sib_h;
			sib->getDimensions(sib_w, sib_h);
			
			if((sib->pack->flags & pfReset) && sib->pack->orientation == pfVert){
				new_ry += sib_h;
				new_rx = 0;
			} else {
				switch(sib->pack->orientation){
				case pfHoriz:
					new_rx += sib_w;
					break;
				case pfVert:
					new_ry += sib_h;
					break;
				default:
					halt("Illegal orientation.\n");
				}
			}
		}
	}
	if(new_rx == element->rx && new_ry == element->ry){
		return 0;
	} else {
		element->rx = new_rx;
		element->ry = new_ry;
		return 1;
	}
}


void
changeAndCalculatePosition(TextElement *element)
{
	TextElement *parent = element->getParentElement();
	int do_continue;
	
	if(parent){
		// loop over sibs in relevant direction
		// ogus -- handle reverse direction case
		Ref r;
		TextElement *sib;
		TextElement *prev_sib;
		r = parent->seek(element);
		if(r) {
			do_continue = 1;
			for(prev_sib = element, parent->next(&r); do_continue && r; prev_sib = sib, parent->next(&r)){
				sib = (TextElement *) parent->contents(r);
				do_continue = chainedPositionCalculate(sib, prev_sib, parent);
				//do_continue = 1;
				//sib->position_valid = FALSE;
			}
		}
		changeAndCalculatePosition(parent);
	}
}

void
changeAndCalculate(TextElement *element, int im_deleted)
{
	TextElement *parent = element->getParentElement();
	
	if(im_deleted == FALSE) {
		element->dimension_valid = FALSE;
		element->calculateDimensions();
	} else {
		element->w = 0;
		element->h = 0;
	}
	
	int do_continue;
	
	if(parent){
		// loop over sibs in relevant direction
		// ogus -- handle reverse direction case
		Ref r;
		TextElement *sib;
		TextElement *prev_sib;
		r = parent->seek(element);
		if(r) {
			do_continue = 1;
			for(prev_sib = element, parent->next(&r); do_continue && r; prev_sib = sib, parent->next(&r)){
				sib = (TextElement *) parent->contents(r);
				do_continue = chainedPositionCalculate(sib, prev_sib, parent);
				//do_continue = 1;
				//sib->position_valid = FALSE;
			}
		}
		changeAndCalculate(parent, FALSE);
	}
}


#if 0
void
refreshFlagParent(TextElement *element)
{
	EntityFace *flag_parent;
	
	do {
		if(flag_parent = eaGetFace(element->entity, vtFLAG))
			// bogus
			flag_parent->select(element->entity);
	} while (element = element->getParentElement());
}
#endif

/* 
// should blank
*/
void 
TextElement::willUpdate()
{
//   this->willUpdate();
}

void 
TextElement::willMove()
{
	this->willUpdate();
}

iBox
findBox(TypeFrame *frame, TextElement *element)
{
	iBox box;
	box.w = 0;
	short box_x;
	short box_y;
	short box_w;
	short box_h;
	
	element->getPosition(frame->td, 0, box_x, box_y);
	element->getDimensions(box_w, box_h);
	
	box.x = box_x;
	box.y = box_y;
	box.w = box_w;
	box.h = box_h;
	
	
	if(element->type == iNewLineT){
		box.w = box.x + 1;
		box.x = 0;
	}
	
	return box;
}

#if 0
// I don't understand this guy
void
betterChangeAndCalculate(TextElement *element)
{
	element->dimension_valid = FALSE;
	element->calculateDimensions();
	TextElement *parent = element->getParentElement();
	int do_continue;
	
	
	if(parent && parent->pack->orientation == pfHoriz){
		// loop over sibs in relevant direction
		// ogus -- handle reverse direction case
		Ref r;
		TextElement *sib;
		TextElement *prev_sib;
		r = parent->seek(element);
		if(r) {
			do_continue = 1;
			for(prev_sib = element, parent->next(&r); do_continue && r; prev_sib = sibparent->next(&r)){
				sib = (TextElement *) parent->contents(r);
				do_continue = chainedPositionCalculate(sib, prev_sib, parent);
			}
		}
		changeAndCalculate(parent, FALSE);
	} else {
		parent->dimension_valid = FALSE;
	}
}
#endif

#if 0
void
betterChangeAndCalculate(TextElement *element, int im_deleted)
{
	element->dimension_valid = FALSE;
	TextElement *parent = element->getParentElement();
	
	if(parent) {
		if(parent->pack->orientation == pfHoriz){
			// loop over sibs in relevant direction
			// ogus -- handle reverse direction case
			Ref r;
			TextElement *sib;
			r = parent->seek(element);
			if(r) {
				for(parent->next(&r); r; parent->next(&r)){
					sib = (TextElement *) parent->contents(r);
					sib->position_valid = FALSE;
				}
			}
			changeAndCalculate(parent, FALSE);
		} else {
			parent->dimension_valid = FALSE;
		}
	}
}
#endif


void
redrawToEndOfLine(TextElement *element)
{
	TypeFrame *frame = (TypeFrame *) element->frame;
	iBox box;
	
	if(frame->managed) {
		Widget sw = frame->view->scrolled_window;
		iBox visible_box;
		
		visible_box = visibleRectangle(sw);
		
		box = findBox(frame, element);
		box.x -= charWidth('M');
		// box.w = visible_box.w;
		if(box.w && box.h)
			frame->drawRegion(box);
	}
}

void
redrawToEndOfPage(TextElement *element)
{
	TypeFrame *frame = (TypeFrame *) element->frame;
	iBox box;
	
	if(frame->managed) {
		Widget sw = frame->view->scrolled_window;
		iBox visible_box;
		
		box = findBox(frame, element);
		visible_box = visibleRectangle(sw);
		box.w = visible_box.w;
		box.h = visible_box.h;
		box.y -= charHeight('M');
		box.x = visible_box.x;
		if(box.w && box.h)
			frame->drawRegion(box);
	}
}

#if 0
void
invalidateChildrenPositions(TextElement *parent)
{
	Ref r;
	TextElement *child;
	
	for(r = parent->first(); r; parent->next(&r)){
		child = (TextElement *) parent->contents(r);
		child->position_valid = FALSE;
	}
}


// opps - I have changed my mind again
void
invalidateChildPositions(TextElement *element)
{
	Ref r;
	TextElement *child;
	
	for(r = element->first(); r; element->next(&r)){
		child = (TextElement *) element->contents(r);
		child->position_valid = FALSE;
	}
}
#endif


static void redrawElement(TextElement *);

/*
// bogus -- steal that old code that limits the
// range of drawing to that in the 
// visible area
*/
void
redrawWindow(TypeFrame *frame)
{
	
	if(frame->managed == FALSE)
		return;
	
	redrawElement(frame->td);
}


static void
redrawElement(TextElement *element)
{
	TypeFrame *frame = (TypeFrame *) element->frame;
	iBox box;
	
	if(frame->managed) {
		box = findBox(frame, element);
		if(box.w && box.h)
			frame->drawRegion(box);
	}
}


void
TextElement::create()
{
#if 0
	TypeFrame *frame;
	
	frame = (TypeFrame *) this->user_data;
	
#ifdef MESSAGE_UPDATING
	if(this->type == iNewLineT) {
		changeAndCalculate(this, FALSE);
	} else if (this->type != iGlyphT) {
		betterChangeAndCalculate(this, FALSE);
	}
	
	invalidateChildrenPositions(this);
#endif
	
	refreshFlagParent(this);
#endif
}



/* 
// should blank
// bogus only redraw lower part if I am a line
*/
void 
TextElement::destroy()
{
#if 0
#ifdef MESSAGE_UPDATING
	changeAndCalculate(this, TRUE);
#endif
	refreshFlagParent(this);
#endif
	this->deleteAndUpdate();
}

/* 
// should draw directly
*/

void 
TextElement::update()
{
	this->initializeText();
}

void 
TextElement::move()
{
	this->update();
}


/*
// should draw directly
*/
void 
TextElement::select()
{
	int from;
	int to;
	
	getEntitySelectionRange(entity, from, to);
	
	if(from == -1 && to == -1)
		return;
	
	// defend again item selection
	if(from == -1 || to == -1) {
		TypeFrame *frame;
		frame = (TypeFrame *) this->frame;
		// extends over more than one element
		redrawWindow(frame);
	} else {
		// is inside me only
		redrawElement(this);
	}
}

/*
// should draw directly
*/
void 
TextElement::unSelect()
{
	int from;
	int to;
	
	getEntitySelectionRange(entity, from, to);
	if(from == -1 || to == -1) {
		TypeFrame *frame;
		frame = (TypeFrame *) this->frame;
		// extends over more than one element
		redrawWindow(frame);
	} else {
		redrawElement(this);
	}
}


void 
TextElement::recognize()
{
#if 0
	this->update();
#endif
}

MainWindow *
TextElement::getMainWindow()
{
	TypeFrame *type_frame = (TypeFrame *)this->frame;
	return type_frame->view->main_window;
}

EntityFace *
getTypeViewer(iEntityPtr entity)
{
	return eaGetFace(entity, vtTYPE);
}

void
setTypeViewer(iEntityPtr entity, EntityFace *face)
{
	eaSetFace(entity, vtTYPE, face);
	if(face && face->frame)
		face->frame->addFace(face);
}


/*
// font hackery
*/
static int char_width = 0;
static int char_height = 0;

int values_initialized = FALSE;


void
setFontValues(int input_char_width, int input_char_height)
{
	values_initialized = TRUE;
	
	char_width = input_char_width;
	char_height = input_char_height;
}

void
getFontValues(int & output_char_width, int & output_char_height)
{
	output_char_width=char_width;
	output_char_height=char_height;
}

int charHeight(iUCode c)
{
	if(values_initialized == FALSE)
		halt("Didnt set font dims yet.\n");
	
	return char_height;
}

int charWidth(iUCode c)
{
	if(values_initialized == FALSE)
		halt("Didnt set font dims yet.\n");
	
	return char_width;
}

#if 0
aString
suckText(TextElement *element)
{
	aString as;
	Ref r;
	
	if(element->text!=0)
		as=element->text;
	
	for(r= element->first(); r; element->next(&r)){
		TextElement *child;
		child = (TextElement *) element->contents(r);
		as += suckText(child);
	}
	
	return as;
}
#endif


void
TextElement::deleteAndUpdate()
{
	this->frame->removeFace(this);

	setTypeViewer(this->entity, 0);
	
	delete this;
}
