
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
#include "rafport.h"

#include "illum.h"
#include "rafutil.h"

#include "ucs.h"

#include "MainWindow.h"

#include "OOCPage.h"
#include "OOCMode.h"


#include "Mode.h"
#include "EntityDialog.h"
#include "ImageMode.h"
#include "Document.h"

#include "EntityAux.h"


#include "SelectionManager.h"


static iCallBackProc the_cb = 0;
static void *the_data = 0;

int disable_level = 0;


void enableCallbacks()
{
	if(--disable_level == 0)
		i_SetCallBack(the_cb, the_data);
}

void disableCallbacks()
{
	if(disable_level++ == 0){
		i_GetCallBack(&the_cb, &the_data);
		i_SetCallBack(0, 0);
	}
}



EntityAux::EntityAux(iEntityPtr input_back_pointer)
	: back_pointer(input_back_pointer), selection_class(scNOT),
	  from(-1),
	  to(-1),
	  show(1)
{
	int i;
	
	disableCallbacks();
	i_SetUserPtrProp(this->back_pointer, ILLUM_PROP, this, NULL); 
	enableCallbacks();
	
	this->init();
	
	for(i = 0; i < NUM_MODE_TYPES; i++){
		this->faces[i] = 0;
	}
}

/* bogus limit partitions to first letter of posset */
uString
getFirstOfPosset(const iEntityPtr entity)
{
	iUCode ts[2] = {0,0};
	uString result;
	iUCode *us;
	char *as;
	aString atemp;
	
	switch(i_GetEDataType(entity)) {
    case iDPosSet:
    case iD1PosSet:
		ts[0] = i_GetGlyph(entity,0);
		result = ts;
		break;
    case iDString:
		as = (char*)i_GetText(entity,0);
		atemp = as;
		result.getUnicode(as);
		break;
	case iDUString:
		us = (iUCode*)i_GetUText(entity,0);
		result = us;
		break;
    default:
		ts[0]=0;
		result = ts;
		break;
	}

	return result;
}


ConfClass eaConfClassify(iEntityPtr entity)
{
	int2 n;
	iPosSetPtr ps;
	int mask;
	
	ps = i_GetPosSet(entity, &n);
	
	switch(n) {
    case 0:
		mask = agNONREC;
		break;
    case 1:
		mask = 0;
		break;
    case 2:
		if(
			((ps[0].chr == 'O') && (ps[1].chr == '0')) ||
			((ps[0].chr == '0') && (ps[1].chr == 'O')) 
			)
			mask = agOZERO;
		else 
			mask = agMULTIREC;
		break;
    default:
		mask = agMULTIREC;
		break;
	}
	
	if ((n >= 1) && (ps[0].conf < 245))
		mask |= agLOWREC;
	
	mask = mask ? mask : agOTHER;
	
	return (ConfClass)mask;
}


void
EntityAux::init()
{
	long f = 0;
	iDAFSError ok;
	ok = i_GetLongProp(this->back_pointer, FLAG_PROP, &f);
	this->flagged = ok == DAFSOK;
	
	
	this->glyph_class = eaConfClassify(this->back_pointer);
}

#ifdef RAF_INTERNAL

ConfClass
eaGetGlyphClass(const iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	
	return ea->glyph_class;
}

#endif

EntityAux::~EntityAux()
{
}

EntityAux *
getController(iEntityPtr entity)
{
	EntityAux *ea;
	void *usr_ptr;
	iDAFSError err;
	
	err = i_GetUserPtrProp(entity, ILLUM_PROP, &usr_ptr,0);
	if (err) 
		usr_ptr=0;
	ea = (EntityAux *) usr_ptr;    
	ea = ea ? ea : new EntityAux(entity);
	return ea;
}

void
removeController(iEntityPtr entity)
{
	disableCallbacks();
	i_DisposeProp(entity, ILLUM_PROP);
	enableCallbacks();
}

EntityAux *
getControllerMaybe(iEntityPtr entity)
{
	EntityAux *ea;
	void *usr_ptr;
	iDAFSError err;
	
	err = i_GetUserPtrProp(entity, ILLUM_PROP, &usr_ptr,0);
	if (err) 
		usr_ptr=0;
	ea = (EntityAux *) usr_ptr;    
	ea = ea ? ea : 0;
	return ea;
}

void
setEntitySelection(iEntityPtr entity, SelectionClass selection_class, int from, int to)
{
	EntityAux *ea = getController(entity);
	
	ea->selection_class = selection_class;
	ea->from = from;
	ea->to = to;
}

void
setEntitySelection(iEntityPtr entity)
{
	// make the range nonsense
	setEntitySelection(entity, scITEM, -1, -1);
}

void
setEntitySelection(iEntityPtr entity, int from, int to)
{
	setEntitySelection(entity, scRANGE, from, to);
}

void
clearEntitySelection(iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	
	ea->selection_class = scNOT;
	// don't clear the range
}

SelectionClass
getEntitySelection(const iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	
	return ea->selection_class;
}

void
getEntitySelectionRange(const iEntityPtr entity, int& from, int& to)
{
	EntityAux *ea = getController(entity);
	
	from = ea->from;
	to = ea->to;
}


void
toggleEntitySelection(iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	
	ea->selection_class = !ea->selection_class;
}

void
eaSetFace(iEntityPtr e, ModeType vt, EntityFace *face)
{
	EntityAux *ea = getController(e);
	ea->faces[vt] = face;
}

EntityFace *
eaGetFace(const iEntityPtr e, ModeType vt)
{
	EntityAux *ea = getController(e);
	
	return ea->faces[vt];
}


/*
================================= Callbacks ===========================
*/

static void
entityRecognize(iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->recognize();
}

static void
entitySelect(iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->select();
}

static void 
entityWillMove(iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->willMove();
}

static void 
entityMove(MainWindow *mw, iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;

	// bogus why?
	if(mw->document->modes[vtRTE] != 0){
	  mw->document->setDirty();
	}
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->move();
	
	
	if(mw) mw->updateMode(entity);
}

/*
// bogus -- make sure the range hasn't been erased by the time this is
// called
*/

static void
entityUnSelect(iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->unSelect();
}

static void
entityCreate(MainWindow *main_window, iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;

	if(main_window->document->modes[vtRTE] != 0){
	  main_window->document->setDirty();
	}
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->create();
}

static void
entityDestroy(MainWindow *mw, iEntityPtr entity)
{
	EntityAux *ea = getController(entity);
	int i;

	// bogus -- why
	if(mw->document->modes[vtRTE] != 0){
	  mw->document->setDirty();
	}
	
	// may delete itself
	// tricky -- might delete the OOCPage
	for(i = 0; i < NUM_MODE_TYPES; i++) {
		EntityFace *face = ea->faces[i];
		// bogus order dependence sucks
		if(i != vtIMAGE)
			ea->faces[i] = 0;
		if(face)
			face->destroy();
		if(i == vtIMAGE)
			ea->faces[i] = 0;
	}
	
	mw->clearMode(entity);

	// and finally remove the property as we may be being faked out
	// by Bruce's edit stuff
	// definitely disable callbacks
	disableCallbacks();
	i_DisposeProp(entity, ILLUM_PROP);
	enableCallbacks();

	

	delete ea;
}

static void
entityUpdate(MainWindow *mw, iEntityPtr entity)
{
	EntityAux *ea;
	int i;

	// bogus -- why
	if(mw->document->modes[vtRTE] != 0){
	  mw->document->setDirty();
	}

	ea = (EntityAux *) getController(entity);
	ea->init();


	
	for(i = 0; i < NUM_MODE_TYPES; i++) {
		EntityFace *face = ea->faces[i];
		if(face)
			face->update();
	}
	
	if(mw) mw->updateMode(entity);
}


static void
entityWillUpdate(iEntityPtr entity)
{
	int i;
	EntityAux *ea = getController(entity);
	
	for(i = 0; i < NUM_MODE_TYPES; i++)
		if(ea->faces[i])
			ea->faces[i]->willUpdate();
}


void
imageDestroy(MainWindow *mw, iImagePtr image)
{
	int i;
	for(i = 0; i < NUM_MODE_TYPES; i++) {
		Mode *mode=mw->document->modes[i];
		if (mode)
			mode->imageDeleted(image);
	}
}



int
entityFlaggedP(iEntityPtr entity)

{
	long flagged_value;
	iDAFSError ok;
	ok = i_GetLongProp(entity, FLAG_PROP, &flagged_value);
	long flagged = DAFSOK == ok;
	return flagged;
}


void
entitySetFlag(MainWindow *mw, iEntityPtr entity)
{
	if (entityFlaggedP(entity)) return;
	EntityAux *ea = getController(entity);
	mw->document->setDirty();
	int flagged_value = TRUE;
	
	i_SetLongProp(entity,FLAG_PROP,flagged_value);
	if(ea->faces[vtFLAG]) {
		OOCMode *ooc_mode;
		ooc_mode=(OOCMode *)mw->document->modes[vtFLAG];
		if (ooc_mode)
			ooc_mode->addNewEntity(entity);
	}
	ImageMode *image_mode;
	image_mode=(ImageMode *)mw->document->modes[vtIMAGE];
	if (image_mode)
		image_mode->reconsider(entity);
}

void
entityClearFlag(MainWindow *mw, iEntityPtr entity)
{
	if (!entityFlaggedP(entity)) 
		return;
	
	mw->document->setDirty();
	i_DisposeProp(entity,FLAG_PROP);
#if 0
	if(ea->faces[vtFLAG]) {
//    ea->faces[vtFLAG]->update(entity);
		ea->faces[vtFLAG]=0;
	}
#endif
	
	ImageMode *image_mode;
	image_mode=(ImageMode *)mw->document->modes[vtIMAGE];
	if (image_mode)
		image_mode->reconsider(entity);
}

void
entityToggleFlag(MainWindow *mw, iEntityPtr entity)
{
	long flagged_value;
	iDAFSError ok;
	ok = i_GetLongProp(entity, FLAG_PROP, &flagged_value);
	long flagged=ok == DAFSOK;

	if (flagged) 
		entityClearFlag(mw,entity);
	else 
		entitySetFlag(mw,entity);
}







iDAFSError
entityCB(iMess m, void *user, iEntityPtr entity)
{
	MainWindow *mw = (MainWindow *) user;
	
	switch(m) {
    case mCreate:
		entityCreate(mw, entity);
		break;
    case mDestroy:
		entityDestroy(mw, entity);
		break;
    case mUpdate:
		entityUpdate(mw, entity);
		break;
    case mWillUpdate:
		entityWillUpdate(entity);
		break;
    case mDestImg:
		imageDestroy(mw,(iImagePtr)entity);
		break;
    case mMove:
		entityMove(mw, entity);
		break;
    case mWillMove:
		entityWillMove(entity);
		break;
    case mRecognize:
		entityRecognize(entity);
		break;
    case mSelect:
		entitySelect(entity);
		break;
    case mUnSelect:
		entityUnSelect(entity);
		break;
    default:
		break;
	}
	
	return DAFSOK;
}
