
/*
 * Copyright (c) 1994, 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include <stdio.h>
#include <ctype.h>

#include "smemory.h"


#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>

#include "illum.h"

#include "ucs.h"
#include "rafutil.h"

#include "misc/Unicode.h"
#include "misc/color.h"
#include "misc/XArg.h"

#include "scroll.h"
#include "bb.h"
#include "mag.h"

#include "QuadTree.h"
#include "EntityAux.h"
#include "View.h"
#include "MainWindow.h"
#include "ViewCanvas.h"
#include "Document.h"
#include "ChooseDialog.h"
#include "SelectionManager.h"

#include "ImageFrame.h"
#include "fudge.h"
#include "IE.h"
#include "entity.h"

#ifdef RAF_INTERNAL
#include "block.h"
#include "form.h"
#endif


#ifdef STEVE
extern int sps_display;
#endif

void
ImageFrame::drawBoxes(Display *display, Drawable drawable, 
					  int x_off, int y_off,
					  DLList *list, 
					  int selected)
{	
	GC gc;

	Ref r;
	iEntityPtr child;
	int mag = 1 << (16 + this->getMagExp());
	int mag_exp = this->getMagExp();
#ifdef RAF_INTERNAL
	XRectangle *pbb_rects = 0;
	int prects = 0;
#endif
	iPolyPtr poly, newpoly, grownpoly;
	iBorderPtr border;
	int maxpoints=100, i, j;
	XRectangle rect;
	void *data;
	iBTag tag;
	iBox box;
	newpoly = i_NewPolyPtr(maxpoints+1);
	grownpoly = i_NewPolyPtr(maxpoints+1);
#ifdef RAF_INTERNAL
	int maxrects;
	
	maxrects = dllistLength(list);
	pbb_rects = (XRectangle *) smalloc(maxrects * sizeof(XRectangle));
#endif
	
	for(r = dllistFirst(list); r ; dllistNext(list, &r)){
		iDAFSError e;
#ifdef RAF_INTERNAL
		iBox pbox;
		iBox *pbp;
		void *vp;
#endif
		child = (iEntityPtr) dllistContents(list, r);

		aString color;
		
		color = 
		this->view->main_window->getTypeColor(i_TypeToName(i_GetEntityType(child)));

		if(selected)
			gc = this->getCanvas()->thickGC(color.string());
		else
			gc = this->getCanvas()->GC(color.string());

		if (!i_GetBorderPtr(child, &data, &tag)){
			switch (tag){
			
			case iBRegion:  /* change to iBBorder and fall through  */
					border=i_NewBorderPtr();
					if (i_Region2Border(child->data->b.region, border))
						break;
					i_SetBorder(child, border);
					if (i_GetBorderPtr(child, &data, &tag))
						break;
			case iBBorder:
			border=(iBorderPtr)data;
			if (border->num_poly){
				for(i=0; i<border->num_poly; i++){
					poly=border->poly[i];
						if(poly->n){
							if (poly->n > maxpoints){
								maxpoints = grownpoly->n = poly->n;
								newpoly->point = 
									(iPoint*)smrealloc(newpoly->point,
														sizeof(iPoint)*
														(poly->n+1));
								grownpoly->point = 
									(iPoint*)smrealloc(grownpoly->point,
														sizeof(iPoint)*
														(poly->n+1));
							}
							for (j=0; j<poly->n; j++) {
								newpoly->point[j].x = 
									(short)(magUpInt(poly->point[j].x, mag_exp) - x_off);
								newpoly->point[j].y = 
									(short)(magUpInt(poly->point[j].y, mag_exp) - y_off);
							}
							newpoly->n = poly->n;
							i_GrowPoly(newpoly, grownpoly, BOX_FUDGE);
							grownpoly->point[grownpoly->n].x = 
								grownpoly->point[0].x;
							grownpoly->point[grownpoly->n++].y = 
								grownpoly->point[0].y;
							
							XDrawLines(display, drawable, gc, 
										(XPoint*)grownpoly->point, 
										grownpoly->n, CoordModeOrigin);
						}
					}
			}
			break;
			
			case iBBox:
					box=*(iBox*)data;
					rect.x = magUpInt(box.x, mag_exp)
						- x_off - BOX_FUDGE;
					rect.y = magUpInt(box.y, mag_exp)
						- y_off - BOX_FUDGE;
					rect.width = magUpInt(box.w, mag_exp)
						+ BOX_FUDGE;
					rect.height = magUpInt(box.h, mag_exp)
						+ BOX_FUDGE;
					XDrawRectangles(display, drawable, gc, &rect, 1);
					break;
			}
		}				
#ifdef RAF_INTERNAL
		box = i_GetBounds(child);
		if (!i_GetUserProp(child, "PBB", &vp , 0, i_SwapUBox)) {
			pbp=(iBox*)vp;
			pbox.x = (((pbp->x + box.x) * mag) >> 16);
			pbox.y = (((pbp->y + box.y) * mag) >> 16);
			pbox.w = (pbp->w * mag) >> 16;
			pbox.h = (pbp->h * mag) >> 16;
			pbb_rects[prects].x = pbox.x - x_off - BOX_FUDGE;
			pbb_rects[prects].y = pbox.y - y_off - BOX_FUDGE;
			pbb_rects[prects].width = pbox.w + BOX_FUDGE;
			pbb_rects[prects].height = pbox.h + BOX_FUDGE;
			prects++;
		}
#endif
		
	}
	newpoly->n=grownpoly->n=1; /*force points to be freed even if n was zero */
	i_DisposePolyPtr(newpoly);
	i_DisposePolyPtr(grownpoly);
#ifdef RAF_INTERNAL
	XDrawRectangles(display, drawable, this->getCanvas()->redGC(), 
					pbb_rects, prects);

	smfree(pbb_rects);
#endif
}

#define KNOB_BASE 1

void
ImageFrame::drawKnobs(Display *display, Drawable drawable, 
					  int x_off, int y_off,
					  iEntityPtr sole_item,
					  int selected)
{ 
	iBox box;
	XRectangle rects[4];
	GC gc;

	aString color;
	color = 
	this->view->main_window->getTypeColor(i_TypeToName(i_GetEntityType(sole_item)));

	if(selected)
		gc = this->getCanvas()->thickGC(color.string());
	else
		gc = this->getCanvas()->GC(color.string());
	
	box = i_GetBounds(sole_item);
	if(bbNullP(box) == FALSE){
		bbScaleUp(box, this->getMagExp());
		box.w -= 1;
		box.h -= 1;
		
		rects[0].x      = box.x - x_off - 2 * KNOB_BASE;
		rects[0].y      = box.y - y_off - 2 * KNOB_BASE; 
		rects[0].width  = KNOB_BASE * 4;
		rects[0].height = KNOB_BASE * 4;
		
		rects[1].x      = box.x - x_off - 2 * KNOB_BASE + box.w;
		rects[1].y      = box.y - y_off - 2 * KNOB_BASE;
		rects[1].width  = KNOB_BASE * 4;
		rects[1].height = KNOB_BASE * 4;
		
		rects[2].x      = box.x - x_off - 2 * KNOB_BASE;
		rects[2].y      = box.y - y_off - 2 * KNOB_BASE + box.h; 
		rects[2].width  = KNOB_BASE * 4;
		rects[2].height = KNOB_BASE * 4;
		
		rects[3].x      = box.x - x_off - 2 * KNOB_BASE + box.w;
		rects[3].y      = box.y - y_off - 2 * KNOB_BASE + box.h;
		rects[3].width  = KNOB_BASE * 4;
		rects[3].height = KNOB_BASE * 4;
		
		XDrawRectangles(display, drawable, gc, 
						rects, 4);
	}
}
void
ImageFrame::drawText(Display *display, Drawable drawable, 
					 int x_off, int y_off,
					 DLList *list, 
					 int selected)
{
	Ref r;
	iEntityPtr child;
	int mag = 1 << (16 + this->getMagExp());
	IE *ie;
	const iUCode *text;
	GC gc;
	GC bgc;
	GC blank_gc;
	aString color;
	

	// bogus
	// accumulate the rectangles and blast them all at once
	// bogus -- dont be constantly creating new gcs
	for(r = dllistFirst(list); r ; dllistNext(list, &r)){
		iBox box;
		
		child = (iEntityPtr) dllistContents(list, r);
		const int being_edited = this->editing(child);

		color = 
		this->view->main_window->getTypeColor(i_TypeToName(i_GetEntityType(child)));

		if(selected && !being_edited) 
			gc = this->getCanvas()->inverseGC(color.string());
		else 
			gc = this->getCanvas()->GC(color.string());

		blank_gc = this->getCanvas()->blankGC();

		if(being_edited) {
			const aString rcolor = "black";
			bgc = this->getCanvas()->GC(rcolor.string());
		}

		box = i_GetBounds(child);
		if(bbNullP(box) == FALSE){
			box.x = (box.x * mag) >> 16;
			box.y = (box.y * mag) >> 16;
			box.w = (box.w * mag) >> 16;
			box.h = (box.h * mag) >> 16;
			
			ie = (IE *) eaGetFace(child, vtIMAGE);
			text = ie->real_text;

			iUCode buf[2] = {' ', '\0'};
			if((!text || !text[0]) && being_edited && selected)
				text = buf;

			if(text && text[0]){
				int tx, ty, tw, th;
				int ascent;
				int ch;
				
				this->getCanvas()->measureText(text, 1, &tw, &ch, &ascent);
				this->getCanvas()->measureText(text, ucslen(text), &tw, &th, &ascent);
				tx = box.x + box.w/2 - tw/2;
				ty = box.h + box.y + ch + MAX_BOX_THICKNESS; 
					
				XFillRectangle(display, drawable, blank_gc,
							   tx - x_off - BLANK_FUDGE, 
							   ty - y_off - ascent - BLANK_FUDGE,
							   tw + 2 * BLANK_FUDGE, 
							   th + 2 * BLANK_FUDGE);
					
				drawUnicode(display,drawable,gc,
							tx - x_off, ty - y_off,
							text, ucslen(text));

				if (selected && being_edited) 
					XDrawRectangle(display, drawable, bgc,
							   	tx - x_off - RECT_GAP, 
							   	ty - y_off - ascent - RECT_GAP,
							   	tw + 2 * RECT_GAP, 
							   	th + 2 * RECT_GAP);
			}
		}
	}
}
