
/*
 * Copyright (c) 1994, 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"
#include "illum.h"

#include "c-type.h"

#include "smemory.h"
#include "ucs.h"
#include "rafutil.h"

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>

#include "misc/Unicode.h"
#include "misc/color.h"
#include "misc/XArg.h"

#include "misc/scroll.h"
#include "misc/bb.h"
#include "mag.h"

#include "MainWindow.h"
#include "ViewCanvas.h"
#include "SelectionManager.h"

#include "ImagePage.h"
#include "fudge.h"
#include "IE.h"

#ifdef RAF_INTERNAL
#include "form.h"
#include "formint.h"
#endif


#ifdef STEVE
extern int sps_display;
#endif

void
drawLinesPitch(ImagePage *page,
			   Display *display, Drawable drawable,
			   iEntityPtr e, iBox b, int mag,
			   int x_off, int y_off)
{
#ifdef RAF_INTERNAL
	void *vp;
	pandp_t *pp;
	
#ifdef STEVE
	struct dashed_line_type {
		long pitch;
		long phase;
		long height; /* height of each dash */
		long width;  /* width of each dash */
		long x,y; /* x not useful? just use phase ? */
		long slope;  
		/* slope of the line (with respect to orientation) */
		int orientation; /* 0 = horiz, 1=vertical */
	};
	typedef struct dashed_line_type dashed_line_type ;
	
	dashed_line_type *dash;
	if (!i_GetUserDataProp( e, "dashed-line",(void **)&dash,0,0)) {
		GC box_gc;
		box_gc = page->getCanvas()->blankGC();
		
		XFillRectangle(display, drawable, box_gc,
					   ((b.x*mag)>>16)-x_off, 
					   (((b.y+(dash->y>>16))*mag)>>16)-y_off, 
					   (b.w*mag)>>16,
					   ((dash->height>>16)*mag)>>16);
	}
#endif
	/* bogus -- possible to have a pandp prop that is null! */
	if (!i_GetUserDataProp( e, "pandp", &vp , 0, f_SwapPandP) && vp &&
		((pp = (pandp_t *)vp)->pitch >> 16)) {
		GC box_gc;
		box_gc = page->getCanvas()->redGC();
		
		long spot;
		
		// bogus - should be removed once there aren''t
		// any "old" pitch and phases left around
		if (pp->phase<= (pp->pitch>>16)) {
			// printf("old style pitch and phase???\n");
			spot = pp->phase << 16;
		} else 
			spot=pp->phase;
		
		while ((spot >> 16) < (b.w)) {
			int x;
			x = spot >> 16;
			iBox bar;
			bar = b;
			bar.x +=x;
			
			bar.x = (bar.x * mag) >> 16;
			bar.y = (bar.y * mag) >> 16;
			bar.w = (bar.w * mag) >> 16;
			bar.h = (bar.h * mag) >> 16;
			
			XDrawRectangle(display, drawable, box_gc,
						   bar.x - x_off, 
						   bar.y - y_off, 
						   1, bar.h);
			spot += pp->pitch;
		}
	}
	if (!i_GetUserDataProp( e, "LPBB", &vp , 0, i_SwapUBox)) {
		iBox *bp,bar;
		bp = (iBox *) vp;
		GC pbb_gc;
		
		pbb_gc = page->getCanvas()->greenGC();
		bar = *bp;
		bar.x = (bar.x * mag) >> 16;
		bar.y = (bar.y * mag) >> 16;
		bar.w = (bar.w * mag) >> 16;
		bar.h = (bar.h * mag) >> 16;
		
		// I turned off this subline related code because I
		// thought either it or the data was wrong
#ifdef RAF_INTERNAL
		if (!i_GetUserDataProp( e, "subl", &vp , 0, i_SwapUBox)) {
			iBox *sp = (iBox *)vp;
			bar.y = (sp->y * mag) >> 16;
			bar.h = ((sp->x - sp->y) * mag) >> 16;
			XDrawRectangle(display, drawable, pbb_gc,
						   bar.x - x_off, 
						   bar.y - y_off, 
						   bar.w, bar.h);
			bar.y = (sp->w * mag) >> 16;
			bar.h = ((sp->h - sp->w) * mag) >> 16;
			XDrawRectangle(display, drawable, pbb_gc,
						   bar.x - x_off, 
						   bar.y - y_off, 
						   bar.w, bar.h);
		} else {
#endif
			XDrawRectangle(display, drawable, pbb_gc,
						   bar.x - x_off, 
						   bar.y - y_off, 
						   bar.w, bar.h);
#ifdef RAF_INTERNAL
		}
#endif
	}
#endif
}

void
drawPhonesPitch(ImagePage *page, 
				Display *display, Drawable drawable, 
				iEntityPtr e, iBox b, int mag,
				int x_off, int y_off)
{
#ifdef RAF_INTERNAL
	void *vp;
	pandp_t *pp;
	
	if (!i_GetUserDataProp( e, "Ppandp", &vp , 0, f_SwapPandP) && 
		((pp = (pandp_t *)vp)->pitch >> 16)) {
		GC box_gc;
		box_gc = page->getCanvas()->redGC();
		
		long spot;
		
		spot=pp->phase;
		
		while ((spot >> 16) < (b.h)) {
			int y;
			y = spot >> 16;
			iBox bar;
			bar = b;
			bar.y +=y;
			
			bar.x = (bar.x * mag) >> 16;
			bar.y = (bar.y * mag) >> 16;
			bar.w = (bar.w * mag) >> 16;
			bar.h = (bar.h * mag) >> 16;
			
			XDrawRectangle(display, drawable, box_gc,
						   bar.x - x_off, 
						   bar.y - y_off, 
						   bar.w, 1);
			spot += pp->pitch;
		}
	}
#endif
}

void
ImagePage::drawPitch(Display *display, Drawable drawable, 
					  int x_off, int y_off,
					  DLList *list)
{
	Ref r;
	iEntityPtr e;
	
	int mag = 1 << (16 + this->getMagExp());
		
	// bogus
	// accumulate the lines and blast them all at once
	for(r = dllistFirst(list); r ; dllistNext(list, &r)){
		iBox b;
		e = (iEntityPtr) dllistContents(list, r);
		
		if(bbNullP(b = i_GetEnclosingBox(e)) == FALSE) {
			int e_type;
			const char *e_type_s;
			
			// these are strcmps because you can affect 
			// the type table when you write out otherwise
			// bogus should make type table only have the types you
			// use
			if(!(strcmp(e_type_s = i_TypeToName(i_GetEntityType(e)), "LineSeg")) || (e_type=i_GetEntityType(e)==iLineT)){
				drawLinesPitch(this,
							   display, drawable,
							   e, b, mag,
							   x_off, y_off);
			} else if (strcmp(e_type_s, "PhonePage") == 0) {
				drawPhonesPitch(this,
								display, drawable,
								e, b, mag,
								x_off, y_off);
			}
		}
	}
}
