
/*
 * Copyright (c) 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "illum.h"

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/cursorfont.h>

#include <Xm/ScrolledW.h>

#include "rafutil.h"
#include "smemory.h"

#include "misc/Unicode.h"
#include "misc/XArg.h"
#include "misc/misc.h" 
#include "scroll.h"
#include "font.h"
#include "bb.h"
#include "MainWindow.h"

#include "EntityAux.h"
#include "MainWindow.h"
#include "Document.h"
#include "SelectionManager.h"

#include "LTEMode.h"
#include "LTEPage.h"

#include "IllEditor.h"

LTEPage::LTEPage(Mode *input_mode, 
				 const aString& label, 
				 iEntityPtr entity)
	: Page(label, input_mode)
{
  LTEMode *lte_mode = (LTEMode *) this->mode;

  this->ue = new IllEditor(lte_mode->main_window, lte_mode->sw, TRUE);
  XtManageChild(this->ue->getWidget());
  XmScrolledWindowSetAreas(lte_mode->sw,
						   0, 0,
						   this->ue->getWidget());
  this->ue->setEntity(entity);
}

#if 0
void 
LTEPage::addMember(iEntityPtr member)
{
}
#endif

LTEPage::~LTEPage()
{
  delete this->ue;
}

int
LTEPage::resizeNecessaryP()
{
	return 0;
}

void 
LTEPage::conditionallyInitialiseCanvas()
{
}

void 
LTEPage::redisplay(XmDrawingAreaCallbackStruct *call_data)
{
}

void
LTEPage::drawRegionNow(iLBox refresh_box)
{
}

void 
LTEPage::addNewEntity(iEntityPtr entity)
{
}

void
centerIfNecessary(LTEPage *page, iEntityPtr entity)
{
}

void
LTEPage::findNextFlagEntity()
{	
	halt("got next flag entity in LTEPage\n");
}

iEntityPtr
LTEPage::findEnclosingEntity(int x, int y)
{
	halt("got findEnclosingEntity\n");
	return NULL;
}

void
LTEPage::nextEntity()
{
  this->ue->cursorRight();
}

void
LTEPage::prevEntity()
{
  this->ue->cursorLeft();
}


void
LTEPage::upEntity()
{
  this->ue->cursorUp();
}

void
LTEPage::downEntity()
{
  this->ue->cursorDown();
}



void
LTEPage::centerOnSelected(iEntityPtr entity)
{
}


int 
LTEPage::pressKey(iUCode key_value)
{
  return this->ue->pressKey(key_value);
}

void
LTEPage::pressButton(XEvent *event)
{
}

void
LTEPage::releaseButton(XEvent *event)
{
}

void
LTEPage::moveButton(int state, int x, int y)
{
}

void
LTEPage::manage()
{
  short x, y;
  iEntityPtr from_entity, to_entity;
  int from_index, to_index;

  int from_line, from_col;
  int to_line, to_col;

  this->getSM()->convertToRange();

	
  this->getSM()->getRange(from_entity, from_index,
						  to_entity, to_index);
  
  if(from_entity) {
	this->ue->convertDAFS2Line(from_entity, from_index,
							   &from_line, &from_col);

	this->ue->convertDAFS2Line(to_entity, to_index,
							   &to_line, &to_col);
	// override our own setSelection, which would try
	// send an unnecssary message to the SM
	this->ue->UniEditor::setSelection(from_line, from_col,
									  to_line, to_col);
  }

  //XmProcessTraversal(this->getCanvas()->canvas, 0);
  //this->getCanvas()->setCursor(XC_xterm);
}

void
LTEPage::unmanage()
{
  // we either have legacy or our IllEditor has set things properly
  this->getSM()->convertToItem();

  // we are doomed but we might as well do this
  //this->clearPopup();
}


/* these useless magnification functions are virtual requirements */
void 
LTEPage::setMag(int x, int ox)
{
}

void 
LTEPage::_setMag(int x, int ox)
{
}

void
LTEPage::configure()
{
}

void
LTEPage::processDelete(void)
{
  this->ue->processDelete();
}

#if 0
/*
// This is a concept of John's
// that hasn't really been implemented
*/
int 
TextPage::isVisible(TextElement *element)
{
	short x,y;
	Widget sw = this->mode->scrolled_window;
	iBox inner_bounds;
	iBox outer_bounds;
	iBox view_bounds;

	short w, h;
	int visible;
	
	element->getPosition(this->td, 0, x, y);
	element->getDimensions(w, h);

	visible = visiblePositionP(sw, x, y) && visiblePositionP(sw, x+w, y+h);
	if(this->display_popup){
		// would the popup be visible too?
		this->decideImageBounds(element->entity, 
								&inner_bounds,
								&outer_bounds);

		view_bounds =
			this->decideViewBounds(element->entity, 
								   outer_bounds);
		visible = (visible && 
				   visiblePositionP(sw, view_bounds.x, view_bounds.y) && 
				   visiblePositionP(sw, 
									view_bounds.x + view_bounds.w, 
									view_bounds.y + view_bounds.h));
	}

	return visible;
}
	

static int 
entityVisible(iEntityPtr entity)
{
	int is_visible = FALSE;
	TextElement *element = (TextElement *) getTextViewer(entity);
	if(element) {
		TextPage *page = (TextPage *) element->page;
		is_visible = page->isVisible(element);
	}
	return is_visible;
}

/*
// this is function is necessary because the selection could be at
// the beginning of a word or a paragraph that is selected
*/
iEntityPtr 
findStartEntity(iEntityPtr from_entity)
{
	iEntityPtr start_entity;
	iEntityPtr f, e;
	int fi, ei;
	iEntityPtr curr;


	for(start_entity = from_entity, 
			curr = from_entity ? i_GetParentEntity(from_entity) : 0; 
		curr; 
		curr = i_GetParentEntity(curr)){

		i_Object2Range(curr, 
					   &f, (int4 *) &fi, 
					   &e, (int4 *) &ei);
		if(f == from_entity)
			start_entity = curr;

	}

	return start_entity;
}

void
TextPage::findNextFlagEntitySub()
{
	// is there already an entity stored in the popup
	iEntityPtr from_entity;
	iEntityPtr to_entity;
	int from_index, to_index;
	int object_from_index, object_to_index;
	iEntityPtr flagged_entity = 0;
	int allow_off_page;
	
	iEntityPtr begin_doc_entity;
	iEntityPtr start_entity;
	int had_popup_before = FALSE;

	this->getSM()->getRange(from_entity, from_index, to_entity, to_index);	

	// find the existing popup and clear it
	if(this->flag_entity && from_entity) {
		// so we must compare the bbox of elements
		// and we can't extract from the flag entity until
		// we are sure it hasn't been deleted
		int ancestorP(iEntityPtr child, iEntityPtr would_be_ancestor);
		if(from_entity == this->flag_entity || ancestorP(from_entity, this->flag_entity)){
			iEntityPtr ffe, fte;
			int ffi, fti;
			i_Object2Range(this->flag_entity, 
						   &ffe, (int4 *) &ffi, 
						   &fte, (int4 *) &fti);
			if(ffe == from_entity &&
			   ffi == from_index &&
			   fte == to_entity &&
			   fti == to_index) {
				// before this, the flag was selected using ctrl-g
				// and wasn't edited
				// therefore, we clear this flag and continue from here
				
				// must happen before we clear the popup
				iEntityPtr holdit = this->flag_entity;
				this->clearPopup();
				entityClearFlag(this->mode->main_window, holdit);
				had_popup_before = TRUE;
			}
		}
	}
	begin_doc_entity = this->td->entity;

	// we start our search in the nearest parent
	// who would have the same from_entity
	start_entity = findStartEntity(from_entity);
	// now, if we have no from entity, start at root

	if(start_entity == 0)
		// this is ok because we won't be needing
		// it in the reset last flagged guy
		// at the end
		from_entity = begin_doc_entity;

	for(int i = 0; flagged_entity == 0 && i < 4; i++) {
		// 4 times through
		iEntityPtr loop;
		
		loop = start_entity;

		if(start_entity == begin_doc_entity && (i == 0 || i == 2))
			continue; // the succeeding loop will do this anyway

		if(i == 1 || i == 3) // odd times -- start from the beginning
			loop = begin_doc_entity;

		allow_off_page = (i == 2 || i == 3); // last 2 times allow off page

		
		// now, if the current start_entity is still a flagged entity
		// we select
		if(loop && entityFlaggedP(loop) && (entityVisible(loop) || allow_off_page)) {
			flagged_entity = loop;
		} else { // do an iEntityFindForward
			while(loop) {
				loop = 
					i_FindEntityForward(loop,0, PROPERTY, (void*)FLAG_PROP);
				if(loop && (entityVisible(loop) || allow_off_page))
					break;
			}
			flagged_entity = loop; // loop may be null
		}
	}

	this->flag_entity = flagged_entity;

	if(flagged_entity){
		i_Object2Range(flagged_entity, 
					   &start_entity, (int4 *) &from_index,
					   &to_entity, (int4 *) &to_index);
		from_index = object_from_index;
		to_index = object_to_index;
		this->getSM()->setRangeAdjacent(start_entity, from_index, 
								to_entity, to_index);
		if(entityVisible(flagged_entity) == FALSE) {
		   TextElement *element = (TextElement *) getTextViewer(flagged_entity);
		   if(element)
			   this->centerCursor(element);		
		}
	}

	if(flagged_entity == 0 && had_popup_before){
		// we just cleared the last flag, and don't want anything
		// to be selected now
		// heinous - need to reset selection so it will get refreshed when cleared
		this->getSM()->setRange(from_entity, from_index, to_entity, to_index);
		this->getSM()->clear();
	}
}


void
TextPage::findNextFlagEntity()
{	
	this->findNextFlagEntitySub();
	this->setPopup(this->flag_entity);
}

#define DEFAULT_POPUP -1 // popup is shown as default

	  display_popup(DEFAULT_POPUP),
	  popup_image(0)
	this->clearPopup();
	this->setPopup(this->flag_entity);


/*
// popup stuff
*/

void 
TextPage::clearPopup()
{
	// find the popup
	// order the area to redraw
	if(this->display_popup && this->bounds_on_view.w){
		this->addRedrawRegion(&this->bounds_on_view);
	}
	this->bounds_on_view.w = 0;
	this->outer_image_bounds.w = 0;
	this->inner_image_bounds.w = 0;
	this->popup_image = 0;
	this->flag_entity = 0;
}

void
TextPage::setPopup(iEntityPtr entity)
{
	// if there is a popup area already, clear it
	// set the popup
	// order the area to redraw
	this->clearPopup();
	if(entity){
		this->flag_entity = entity;
		this->decideImageBounds(entity, 
								&this->inner_image_bounds,
								&this->outer_image_bounds);

		this->bounds_on_view = 
			promoteBox(this->decideViewBounds(entity, 
								   this->outer_image_bounds));
		this->popup_image = i_GetImage(entity);
		if(this->display_popup && this->bounds_on_view.w && this->popup_image){
			this->addRedrawRegion(&this->bounds_on_view);
		}
	}
}

void
TextPage::refreshPopup(Display *display, Pixmap pixmap, int off_x, int off_y)
{
	ViewCanvas *vc = this->getCanvas();
	Widget widget = vc->canvas;

	char *bits1 = 0;
	int2 dw1;
	XImage *x_image1 = 0;

	char *bits2 = 0;
	int2 dw2;
	XImage *x_image2 = 0;

	GC gc;

	iBox ibox = this->inner_image_bounds;
	iBox obox = this->outer_image_bounds;

	// actually draw

	int normal_image_mag_exp = 1 << 16;
	if (i_Image2BitMap(this->popup_image, obox,
					   normal_image_mag_exp, &dw1, &bits1)) 
		goto clean_images;
	if (i_Image2BitMap(this->popup_image, ibox,
					   normal_image_mag_exp, &dw2, &bits2))
		goto clean_images;

	// create the image to be the size of the bitmap
	x_image1 = XCreateImage(display, 0, 1, XYBitmap, 
							0, // tricky -- ignore initial bits
							bits1,
							obox.w, 
							obox.h, 
							8, // quanta of scan line
							dw1);

	x_image2 = XCreateImage(display, 0, 1, XYBitmap, 
							0, // tricky -- ignore initial bits
							bits2,
							ibox.w, 
							ibox.h, 
							8, // quanta of scan line
							dw2);

	x_image1->byte_order = MSBFirst;
	x_image1->bitmap_bit_order = MSBFirst;
	x_image2->byte_order = MSBFirst;
	x_image2->bitmap_bit_order = MSBFirst;
	_XInitImageFuncPtrs(x_image1);
	_XInitImageFuncPtrs(x_image2);

	gc = vc->darkgrayGC();

	XPutImage(display, pixmap,
			  gc, x_image1, 
			  0,0, 
			  off_x, off_y,
			  obox.w, obox.h);

	gc = vc->blackGC();

	XPutImage(display, pixmap,
			  gc, x_image2, 
			  0,0, 
			  off_x + ibox.x - obox.x, off_y + ibox.y - obox.y,
			  ibox.w, ibox.h);


	// this is the border
	XDrawLine(display, pixmap, vc->blackGC(),
			  off_x + 0,         off_y + obox.h-1,
			  off_x + obox.w-1,  off_y + obox.h-1);
	XDrawLine(display, pixmap, vc->blackGC(),
			  off_x + obox.w-1, off_y + 0, 
			  off_x + obox.w-1, off_y + obox.h-1);

	XDrawRectangle(display, pixmap, 
				   vc->blueGC(),
				   off_x + 0, off_y + 0, 
				   obox.w-2, obox.h-2); 

clean_images :
		if(bits1){
			smfree(bits1);
		}
	if(x_image1) {
		x_image1->data=0;
		XDestroyImage(x_image1);
	}

	if(bits2){
		smfree(bits2);
	}
	if(x_image2) {
		x_image2->data=0;
		XDestroyImage(x_image2);
	}
}

void
TextPage::decideImageBounds(iEntityPtr entity, 
							 iBox *inner_bounds, iBox *outer_bounds)
{
	iBox ibox = i_GetEnclosingBox(entity);
	iBox obox = ibox;
	iImagePtr image = i_GetImage(entity);

	if(image == 0) {
		obox.w = 0;
		ibox.w = 0;
	} else {
		int iw, ih;
		// no scaling because we always display in mag 1
		iw = i_GetImageWidth(image);
		ih = i_GetImageHeight(image);
		if (obox.x>IMAGE_FRINGE){
			obox.x-=IMAGE_FRINGE;
		} 	else{
			obox.x=0;
		}

		if (obox.y>IMAGE_FRINGE){
			obox.y-=IMAGE_FRINGE;
		} else {
			obox.y = 0;
		}
		obox.w = MIN(iw-obox.x, obox.w+2*IMAGE_FRINGE);
		obox.h = MIN(ih-obox.y, obox.h+2*IMAGE_FRINGE);
	}

	*inner_bounds = ibox;
	*outer_bounds = obox;
}

iBox 
TextPage::decideViewBounds(iEntityPtr entity, iBox image_bounds)
{
	short x, x1, x2, y;
	short w, h;
	iBox b;

	iEntityPtr beg_ent, end_ent;
	int beg_ind, end_ind;

	i_Object2Range(entity, 
				   &beg_ent, (int4 *) &beg_ind, 
				   &end_ent, (int4 *) &end_ind);
	TextElement *begin = (TextElement*)getTextViewer(beg_ent);
	// cause sometime object to range return 0 in
	// that second slot -- bogus
	end_ent = end_ent ? end_ent : beg_ent;
	TextElement *end = (TextElement*)getTextViewer(end_ent);
	begin->getPosition(this->td, beg_ind, x1, y);
	end->getPosition(this->td, end_ind, x2, y);
	begin->getDimensions(w, h);

	w = image_bounds.w;
	x = x1 + ((x2-x1) / 2) - (w/2);
	y += h;
	h = image_bounds.h;

	// hmm should check if both are being violated at once
	if(x < DISTANCE_FROM_EDGE)
		x = DISTANCE_FROM_EDGE;
	if(x + w > this->width - DISTANCE_FROM_EDGE)
		x = this->width - w - DISTANCE_FROM_EDGE;

	b.x = x;
	b.y = y;
	b.w = w;
	b.h = h;

	return b;
}


void 
TextPage::togglePopup(void)
{	
	XArg args;
	iBox box;
	MainWindow *mw=this->mode->main_window;

	this->display_popup = !this->display_popup;
	if (this->flag_entity){
		args.reset();
		XtVaGetValues(mw->scroll_window, 
			XmNheight, &box.h, XmNwidth, &box.w, NULL);

		/* refresh screen  */
		// bogus -- we know what to redraw!
		this->drawRegion(this->bounds_on_view);
	}
}

#endif
