
/*
 * Copyright (c) 1994, 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "rafutil.h" // for MIN and MAX
#include "c-type.h"

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrolledWP.h>

#include "illum.h"

#include "misc/XArg.h"

#include "Mode.h"
#include "MainWindow.h"
#include "Application.h"
#include "ViewCanvas.h"
#include "Document.h"
#include "EntityAux.h"
#include "SelectionManager.h"
#include "misc/bb.h"
#include "misc/scroll.h"

#include "Page.h"
#include "text/EntityFace.h"

#ifndef min
#define min(x,y)           (((x) > (y)) ? (y) : (x) )
#endif

Page::Page(const aString& input_name, Mode *input_mode)
	: name(input_name), mode(input_mode),
	  managed(0),
	  width(0), height(0),
	  im_deleted(FALSE),
	  should_delete(FALSE),
	  redraw_procid(0)
{
	this->redraw_region.w = 0;
	dllistCtor(&this->face_list);
	this->picture.w = 0;
	if (this->getCanvas()->canvas) {	
		this->getCanvas()->initializeScrollBarValues();
		this->getCanvas()->pix_hoffset = 0;
		this->getCanvas()->pix_voffset = 0;
	}
}

Page::~Page()
{
	this->flushRedrawRegion();
	if(this->managed)
		this->unmanage();
	
	Ref r;
	EntityFace *member;
	DLList *list = &this->face_list;
	
	for(r = dllistFirst(list); r; dllistNext(list, &r)){
		member = (EntityFace *) dllistContents(list, r);
		member->deleteCheap();
	}
	
	dllistDtor(&this->face_list);
}

void
Page::purge()
{
	if(this->managed)
		this->unmanage();
	
	Ref r;
	EntityFace *member;
	DLList *list = &this->face_list;
	
	for(r = dllistFirst(list); r; dllistNext(list, &r)){
		member = (EntityFace *) dllistContents(list, r);
		eaSetFace(member->entity, this->mode->getMode(), 0);
		member->deleteCheap();
	}
	
	dllistClear(&this->face_list);
	delete this;
}

int
Page::getMagExp()
{
	return this->mode->main_window->mag_exp;
}

/*
// bogus little function required because the canvas doesn't seem 
// to stick to its original size
*/
void
Page::syncSize()
{
	if(this->getCanvas()) {
		int h;
		int w;
		XArg args;
		Widget hsb,vsb;
		iBox view_port;
		ViewCanvas *vc = this->getCanvas();

		
		view_port = this->getViewPort();
		
		if (this->should_delete) {
			// well I'd rather be unmanaged
			w=11; // why 11? well why not!
			h=11;
		} else {
			w = this->width;
			h = this->height;
			w = w < 10 ? view_port.w - 1 : w;
			h = h < 10 ? view_port.h - 1 : h;
		}
		// If ther is no picture reset the scrollbar values to 0

		args.reset();
		args.setArg(XmNhorizontalScrollBar,&hsb);
		args.setArg(XmNverticalScrollBar,&vsb);
		args.getValues(XtParent(vc->canvas));
		args.reset();
		args.setArg(XmNvalue, &w);
		args.getValues(hsb);
		args.reset();
		args.setArg(XmNvalue,&h);
		args.getValues(vsb);
		
		if(!this->picture.w)
			vc->initializeScrollBarValues();			
#if 0
		vc->setSize(w, h);
#else
#endif

		if(this->picture.w) {
			int maximum = MAX(view_port.h,picture.h);
			int slider_size = MAX(view_port.h,1);
			args.reset();
			args.setArg(XmNmaximum,maximum);
			args.setArg(XmNsliderSize, slider_size );
			if ((maximum - slider_size) <= 0) {
				vc->pix_voffset = 0;
				args.setArg(XmNvalue, 0);
			} else if (h > (maximum - slider_size)) {
				vc->pix_voffset = maximum - slider_size;
				args.setArg(XmNvalue, vc->pix_voffset);
			}
			args.setArg(XmNpageIncrement, MAX(view_port.h/2,10));
			XtSetValues(vsb, args.getArgs(),args.getSize());

			maximum = MAX(view_port.w,picture.w);
			slider_size = MAX(view_port.w,1);
			args.reset();
			args.setArg(XmNmaximum,maximum);
			args.setArg(XmNsliderSize, slider_size);
			if ((maximum - slider_size) <= 0) {
				vc->pix_hoffset = 0;
				args.setArg(XmNvalue, 0);
			} else if (w > (maximum - slider_size)) {
				vc->pix_hoffset = maximum - slider_size;
				args.setArg(XmNvalue, vc->pix_hoffset);
			} 
			args.setArg(XmNpageIncrement, MAX(view_port.w/2,10));
			XtSetValues(hsb, args.getArgs(),args.getSize());
		}	
		vc->setSize(view_port.w, view_port.h);
#if 0
		// hmm here?
		if(this->should_delete)
			vc->unmanage();
#endif
		
	}
}


int
Page::managedP()
{
	return this->managed;
	return this->getCanvas()->managedP();
}

void
Page::manage()
{
	if (this->managed)
		return;
	
	this->managed = TRUE;

	// bogus -- why do this
	this->_setMag(this->mode->main_window->mag_exp, 
				  this->mode->main_window->mag_exp);

	if(this->getCanvas()){
		this->getCanvas()->setClient(this);
		// bogus I took this out
		//this->configureCanvas(this, 10, 10);
		this->getCanvas()->manage();
		this->syncSize();
	}
}

void
Page::unmanage()
{
	if (!this->managed)
		return;
	
	this->managed = FALSE;
	if(this->getCanvas()){
		this->getCanvas()->setClient(0);
		// bogus I took this out
		//this->configureCanvas(0, 10, 10);
		this->getCanvas()->unmanage();
		this->syncSize();
	}
}

void
Page::pressButton(XEvent *event)
{
    iEntityPtr entity;
	
    switch(event->xbutton.button) {
	case Button1:
		entity = this->findEnclosingEntity(event->xbutton.x, event->xbutton.y);
		if(entity != 0) {
			if(event->xbutton.state & ShiftMask){
				this->toggleItem(entity);
			} else {
				this->setItem(entity);
			}
		} else {
			this->clear();
		}
		break;
	case Button2:
		break;
    }
}


void
Page::releaseButton(XEvent *event)
{
}

void
Page::moveButton(int state, int x, int y)
{
#if 0
	printf("pages move button.\n");
#endif
}

void
Page::reassignValues(const DLList *list, iUCode key_value)
{
	Ref r;
	iEntityPtr entity;
	Mode *v;
	
	int changed=FALSE;
	
	v = this->mode;
	for(r = dllistFirst(list); r; dllistNext(list, &r)){
		entity = (iEntityPtr) dllistContents(list, r);
		if(iGlyphT == i_GetEntityType(entity)) {
			i_SetGlyph(entity, key_value,CERTCONF);
			i_DisposeProp(entity, FLAG_PROP);
			changed=TRUE;
		}
	}
	// we may have been deleted from underneath
	if (changed) 
		v->main_window->document->setDirty();
}
iLBox
Page::getPicture()
{
	return this->picture;
}
void
Page::setPicture(iLBox box)
{
	this->picture = box;
}
void 
Page::setViewPort(iBox box)
{
	
	this->getCanvas()->view_port = box;
}

iBox 
Page::getViewPort()
{
	iBox view_port={0,0,0,0};
	Widget ww = this->getCanvas()->canvas;
	int w,h;
	
	getWidgetSize(ww,&w, &h);
	
	view_port.w = w;
	view_port.h = h;

	return view_port;
}

void
Page::deleteSelection()
{
}
void
Page::resize(XmDrawingAreaCallbackStruct *call_data)
{
	Dimension old_w, old_h;
	int clear = 0;
	ViewCanvas *vc = this->getCanvas();
	iBox new_port={0,0,0,0};
	iLBox picture=this->getPicture();
	iLBox visible;
	Widget hsb,vsb;
	XArg args;
	
	args.reset();
	args.setArg(XmNhorizontalScrollBar,&hsb);
	args.setArg(XmNverticalScrollBar,&vsb);
	args.getValues(XtParent(vc->canvas));
	
	new_port = this->getViewPort();
	old_h = vc->view_port.h;
	old_w = vc->view_port.w;
	if ((old_w == new_port.w) && (old_h == new_port.h))
		return;
	if ((new_port.h >= picture.h)||(old_h > picture.h)
				||(new_port.w >=picture.w)||(old_w > picture.w))
		clear = 1;
		
	if((int)new_port.h>=picture.h) {
		vc->pix_voffset = 0;
		clear = 1;
	} else {
		if (vc->pix_voffset) vc->pix_voffset += (old_h - new_port.h)/2;
		vc->pix_voffset = MIN(vc->pix_voffset,(picture.h-new_port.h));
		if(old_h > picture.h)
			clear = 1;
	}
	if((int)new_port.w >= picture.w) {
		clear = 1;
		vc->pix_hoffset = 0;
	} else {
		if (vc->pix_hoffset)vc->pix_hoffset += (old_w - new_port.w)/2;
		vc->pix_hoffset = MIN(vc->pix_hoffset, (picture.w - new_port.w));
		if(old_w > picture.w)
			clear = 1;
	}
	this->setViewPort(new_port);
	args.reset();
	args.setArg(XmNmaximum,MAX(new_port.w,picture.w));
	args.setArg(XmNsliderSize, MAX(new_port.w,1));
	args.setArg(XmNvalue,	(int)vc->pix_hoffset);
	XtSetValues(hsb, args.getArgs(),args.getSize());

	args.reset();
	args.setArg(XmNmaximum,MAX(new_port.h,picture.h));
	args.setArg(XmNsliderSize, MAX(new_port.h,1));
	args.setArg(XmNvalue,	(int)vc->pix_voffset);
	XtSetValues(vsb,args.getArgs(),args.getSize());

	visible = visibleRectangle(XtParent(vc->canvas));

	this->drawRegionNow(visible);
}

void
Page::track(Focus focus)
{
	iEntityPtr sole;
	iBox bounds;
	int visible, w, h, ent_w, ent_h;
	
	Mode *mode = this->mode;
	MainWindow *mw=mode->main_window;


	getWidgetSize(mw->view_canvas->canvas, &w, &h);

	sole = this->getSM()->getSoleItem();
	if (sole) {
		bounds = mode->getBounds(sole);
		if(bbNullP(bounds) == FALSE) {
			visible = visiblePositionP(mw->scroll_window, 
									   bounds.x, 
									   bounds.y);
			visible &= visiblePositionP(mw->scroll_window, 
										bounds.x+bounds.w-1, 
										bounds.y+bounds.h-1);
			if(!visible){
				ent_w = bounds.w;
				ent_h = bounds.h;

				switch (focus) {
				case TOP:
					this->scrollToTop(sole);
					break;
				case RIGHT:
					this->scrollToRight(sole);
					break;
				case BOTTOM:
					this->scrollToBottom(sole);
					break;
				case LEFT:
					this->scrollToLeft(sole);
					break;
				case CENTER:
					if (ent_w < w || ent_h < h)
						this->centerOnSelected(sole);
					else 
						this->scrollToUpperLeft(sole);
					break;
				}
			}
		}
	}
}


int
Page::pressKey(iUCode key_value)
{
	if (key_value == '\b') {
		this->mode->main_window->deleteSelection(FALSE);
	} else if(key_value == 20) {
		killContentsForSelection();
   	} else {
		if(key_value < ' ' || this->getSM()->getState() == scNOT) {
			ringTheBell();
			return 0;
		}
		
		// bogus why does this make sense here?
		this->track(CENTER);
		this->reassignValues(this->getItems(), key_value);
	}
	
	return 1;
}

ViewCanvas *
Page::getCanvas()
{
	return this->mode->main_window->view_canvas;
}

void
Page::configureCanvas(Page *x, int w, int h)
{
	ViewCanvas *vc = this->mode->main_window->view_canvas;
	vc->setSize(w, h);
	

}

void
Page::removeFace(EntityFace *face)
{
	Ref r;
	DLList *l = &this->face_list;
	
	r = dllistSeek(l, face);
	//  bogus -- an error condition if I don't find it?
	if(r)
		dllistRemove(l, r);
	eaSetFace(face->entity, this->mode->getMode(), 0);
}

void
Page::addFace(EntityFace *face)
{
	DLList *l = &this->face_list;
	
	dllistAppend(l, face);
}

SelectionManager *
Page::getSM()
{
	return this->mode->getSM();
}

void
Page::setItem(iEntityPtr entity)
{
	this->getSM()->setItem(entity);
}

void
Page::toggleItem(iEntityPtr entity)
{
	this->getSM()->toggleItem(entity);
}

void
Page::clear()
{
	this->getSM()->clear();
}

iEntityPtr
Page::getSoleItem()
{
	return this->getSM()->getSoleItem();
}

const DLList *
Page::getItems()
{
	return this->getSM()->getItems();
}



char 
executeRedrawRegion(Page *page)
{
	if(page->resizeNecessaryP()){
		// potentially dangerous loop here
		page->conditionallyInitialiseCanvas();
		// this says -- call this workproc again because
		// we didnt do our work -- we resized
		// the canvas so that things would fit
		// and we want to come back here when the xwindows
		// stuff is done
		return 0;
	} else {
		if(page->managedP()){
			// only draw in the visible area
			Widget sw = page->mode->scrolled_window;
			iLBox truncated_region, window_region = visibleRectangle(sw);
		
			i_IntersectLBox(window_region, page->redraw_region, &truncated_region);
			if (truncated_region.w > 0 && truncated_region.h > 0)
				page->drawRegionNow(truncated_region);
		}
	}
	
	page->redraw_region.w = 0;
	return 1; /* says not to call this workproc again */
}

void
Page::addRedrawRegion(iLBox *b)
{
	if (this->redraw_region.w) {
		i_UnionLBox(*b, this->redraw_region, &this->redraw_region);
	} else {
		this->redraw_region = *b;
		this->redraw_procid = 
			XtAppAddWorkProc(this->mode->main_window->app->app_context,
							 executeRedrawRegion, this);
	}
}

void 
Page::flushRedrawRegion()
{
	if (this->redraw_region.w == 0) 
		return;
	XtRemoveWorkProc(this->redraw_procid);
	this->redraw_region.w = 0;
}


void
Page::drawRegion(iLBox box)
{
	ViewCanvas *view_canvas = this->getCanvas();
	if (view_canvas->follow)
		view_canvas->client->drawRegionNow(box);
	else
		this->addRedrawRegion(&box);
}

aString
Page::getName()
{
	return this->name;
}
