
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include <X11/Intrinsic.h>
#include "Xm/ScrolledW.h"
#include <Xm/RowColumn.h>
#include <Xm/Frame.h>
#include <Xm/MainW.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>

#include "illum.h"
#include "misc/XArg.h"
#include "svpmap.h"
#include "dafs.h"
#include "dafstype.h"


#include "MainWindow.h"
#include "RTEPage.h"
#include "RTElement.h"
#include "ViewCanvas.h"
#include "Document.h"
#include "SelectionManager.h"

#include "config.h"

#include "RTEMode.h"

RTEMode::RTEMode(MainWindow *input_mw, iEntityPtr input_entity)
	: Mode(input_mw)
{
	XArg args;
	Widget new_swindow,pw;
	this->old_page = this->main_window->current_frame;
	pw = XtParent(this->old_page);
	XtUnmanageChild(this->old_page);
	
	args.reset();
	args.setArg(XmNmarginWidth, 2); 
	args.setArg(XmNmarginHeight, 2); 
	args.setArg(XmNshadowThickness, 1); 
	args.setArg(XmNshadowType, XmSHADOW_OUT); 
	this->page = XmCreateFrame (pw, "rte_frame", 
				    args.getArgs(), args.getSize());
	
	args.reset();
	args.setArg(XmNscrollBarDisplayPolicy, XmAS_NEEDED); 
	args.setArg(XmNscrollingPolicy, XmAUTOMATIC); 
	new_swindow = XmCreateScrolledWindow (this->page, "rte_sw", 
					      args.getArgs(), args.getSize());
		

	this->swindow = new_swindow;
	this->createPages(input_entity)	;
	XtManageChild(new_swindow);
}

RTEMode::~RTEMode()
{
	XtUnmanageChild(this->page);
	XtManageChild(old_page);
	this->main_window->current_frame = old_page;
	XtDestroyWidget(this->page);
	XtDestroyWidget(this->swindow);
	
}

void
RTEMode::createPages(iEntityPtr entity)
{	
DLList page_entity_list;

	dllistCtor(&page_entity_list);
	
	this->findPageEntities(entity, &page_entity_list);
	this->createPages(&page_entity_list);
	dllistDtor(&page_entity_list);
}


void
RTEMode::createPages(DLList *page_entity_list)
{
	int page_count = 0;
	Ref r;
	iEntityPtr entity;
	char page_name[24];

	RTEPage *page;

	for(r = dllistFirst(page_entity_list); r; 
		dllistNext(page_entity_list, &r)){
	  entity = (iEntityPtr) dllistContents(page_entity_list, r);
	  sprintf(page_name, "%d", ++page_count);
	  page = new RTEPage(this,page_name,entity);
	  page->createEditors();
	  this->addPage(page);
	}
	
	for(r = dllistFirst(&this->page_list); r; 
		dllistNext(&this->page_list, &r)){
	  page = (RTEPage *) dllistContents(&this->page_list, r);
	  page->expandEditors(); 
	  page->resetExpansionFactor(); // prevent from expanding twice
	}
}

void
RTEMode::findPageEntitiesSub(iEntityPtr entity, 
							 DLList *page_entity_list)
{
  Ref r;
  iEntityPtr child;

  if(getPage(entity)) {
	r = dllistSeek(page_entity_list, entity);
	// avoid entering anything twice
	if(r == 0)
	  dllistAppend(page_entity_list, entity);
  } else {
	for(child = i_GetChildEntity(entity); child; child = i_GetNextEntity(child)){
	  this->findPageEntitiesSub(child, page_entity_list);
	}
  }
}

void
RTEMode::findPageEntities(iEntityPtr root_entity, 
						  DLList *page_entity_list)
{
  this->findPageEntitiesSub(root_entity, page_entity_list);
  if(dllistEmpty(page_entity_list))
	dllistAppend(page_entity_list, root_entity);
}


void
RTEMode::configure()
{
}

aString
RTEMode::getName()
{
	return "RTE";
}

ModeType
RTEMode::getMode()
{
	return vtRTE;
}

void
RTEMode::addNewEntity(iEntityPtr entity)
{
	RTEPage *page = (RTEPage *)dllistContents(&this->page_list,this->current_page_ref);
	page->addNewEntity(entity);
}

void 
RTEMode::choose(int buttons_changed)
{
	Ref r;
	RTEPage *page;
	RTEPage *current_page;
	int managed;
	iEntityPtr entity;
	
	current_page = (RTEPage*)this->getCurrentPage();
	managed = current_page && current_page->managedP();
	
	if ((current_page || buttons_changed) == FALSE) 
		return;
	if (managed)  {
		entity = current_page->root_entity;
		current_page->unmanage();
	} else {
		entity = 0;
	}
	
	this->current_page_ref = 0;
	
	for (r = dllistFirst(&this->page_list);r;dllistNext(&this->page_list,&r)) {
		page = (RTEPage*) dllistContents(&this->page_list,r);
		page->purge();
	}
	
	dllistClear(&this->page_list);
	iEntityPtr root_entity = this->main_window->document->root_entity;
	this->createPages(root_entity);
	r = 0;
	
	if (managed) {
		for (r = dllistFirst(&this->page_list);r;dllistNext(&this->page_list,&r)){
			page = (RTEPage*)dllistContents(&this->page_list,r);
			if (entity = page->root_entity)
				break;
		}
	}
	if (managed) {
		if (r)
			this->setCurrentPage(r);
		else 
			this->start();
	}
	
	// go through the list of entities selected
	// and unselect the ones that no longer have TreeElement's
	const DLList *selected;
	DLList copy;
	dllistCtor(&copy);
	selected = this->getSM()->getItems();
	// have to have a copy because we will modify the
	// list in the loop
	dllistAssign(&copy, selected);

	for(r = dllistFirst(&copy); r; dllistNext(&copy, &r)){
		iEntityPtr entity;
		RTElement *re;
		entity = (iEntityPtr) dllistContents(&copy, r);
		re = (RTElement *) eaGetFace(entity, vtRTE);
		if(re == 0)
			this->getSM()->toggleItem(entity);
	}
	
	dllistDtor(&copy);
	// bogus do I have to do this
	this->main_window->refresh_dialogs();
	
}
static RTElement * 
getElement(iEntityPtr entity) {

	RTElement *elem=0;
	
	while (entity && !(elem = (RTElement*)eaGetFace(entity,vtRTE)))
		entity = i_GetParentEntity(entity);
		
	return elem;
}

void
RTEMode::centerOnSelected(iEntityPtr entity)
{
	RTElement *re = (RTElement*)getElement(entity);
	
	DLList *l = &this->page_list;
	Ref r;
	if (!re) return;
	r = dllistSeek(l, re->page);
	if(r){
		setCurrentPage(r);
		re->page->centerOnSelected(entity);
	}      
	

}

void
RTEMode::processDelete(void)
{
  RTEPage *page = (RTEPage *) this->getCurrentPage();
  if(page){
	// bogus -- should handle case where the stuff isn't on the current
	// page but a non-visible page
	page->processDelete();
  }
}
