
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "illum.h"

#include <Xm/ArrowBG.h>

#include "smemory.h"
#include "rafutil.h" // for MIN and MAX
#include "misc/Unicode.h"
#include "misc/scroll.h"
#include "misc/XArg.h"

#include "MainWindow.h"
#include "Mode.h"
#include "ViewCanvas.h"
#include "EntityAux.h"
#include "TreePage.h"
#include "SelectionManager.h"
#include "config.h"

#include "TreeElement.h"

TreeElement::TreeElement(Page *input_page,iEntityPtr input_entity,
						int xpos,int ypos,int showval,int lev)
	: EntityFace(input_page, input_entity),
	x(xpos),
	y(ypos),
	text(NULL),
	show(showval),
	lev(lev),
	kids(0)
{
	or_path = 0;
	prev_or_path = 0;
}

TreeElement::~TreeElement()
{
	if (text) smfree(text);
}

void
TreeElement::redraw()
{
	TreePage *page = (TreePage *) this->page;
	ViewCanvas *vc = page->getCanvas();
	Widget widget = vc->canvas;
	iLBox box;

	if (page->stale) 
		return;
	box = visibleRectangle(XtParent(widget));
	page->stale = 1;

	page->drawRegion(box);

}


void 
TreeElement::deleteCheap()
{
	delete this;
}

void TreeElement::deleteAndUpdate()
{
}

void TreeElement::create()
{
}

void 
TreeElement::willUpdate()
{
}

extern "C" int prall();
void myfoo() 
{
	prall();
}

void TreeElement::drawBox(Display *display, Pixmap pixmap, GC gc, iLBox *bp)
{
	XPoint xp[8];
	int off;
	
	off = 5;
	xp[0].x = bp->x; 			xp[0].y = bp->y + off;
	xp[1].x = bp->x;			xp[1].y = bp->y - off + bp->h;
	
	xp[2].x = bp->x + off;		xp[2].y = bp->y + bp->h;
	xp[3].x = bp->x - off + bp->w;	xp[3].y = bp->y + bp->h;
	
	xp[4].x = bp->x + bp->w;	xp[4].y = bp->y + bp->h - off;
	xp[5].x = bp->x + bp->w;	xp[5].y = bp->y + off;
	
	xp[6].x = bp->x + bp->w - off;	xp[6].y = bp->y;
	xp[7].x = bp->x + off;		xp[7].y = bp->y;
	XDrawLines(display, pixmap,gc,xp,8,CoordModeOrigin);
	XDrawLine(display,pixmap,gc,bp->x+off,bp->y,bp->x,bp->y+off);

}

void TreeElement::drawTri(Display *display, Pixmap pixmap, GC gc)
{
	XPoint xp[6];
	TreePage *page = (TreePage*)this->page;

	int x = this->x - page->getCanvas()->pix_hoffset;
	int y = this->y - page->getCanvas()->pix_voffset + page->ch/2;
	
	if (show) {
		xp[0].x = x;				xp[0].y = y;
		xp[1].x = x - 10;			xp[1].y = y;
		xp[2].x = x - 10;			xp[2].y = y;
		xp[3].x = x - 5;			xp[3].y = y + 10; 
		xp[4].x = x - 5;			xp[4].y = y + 10;
		xp[5].x = x;				xp[5].y = y;
	} else {
		xp[0].x = x;				xp[0].y = y;
		xp[1].x = x - 10;			xp[1].y = y + 5;
		xp[2].x = x - 10;			xp[2].y = y + 5;
		xp[3].x = x - 10;			xp[3].y = y - 5;
		xp[4].x = x - 10;			xp[4].y = y - 5;
		xp[5].x = x;				xp[5].y = y;
	}	
	XDrawLines(display, pixmap,gc,xp,6,CoordModeOrigin);
	XFillPolygon(display, pixmap,gc,xp,6,Convex,CoordModeOrigin);

}
iUATablePtr allocTable()
{
	iUATablePtr table = (iUATablePtr)smalloc(sizeof(iUATable));
	iUAEntry *entries = (iUAEntry*)smalloc(sizeof(iUAEntry));

	table->entries = entries;
	table->num = 1;
	table->entries->index = '\n'; 
	table->entries->string = "\\n";
	return table;
}
void freeTable(iUATablePtr table)
{
	smfree(table->entries);
	smfree(table);
}

void
TreeElement::drawTheString(Display *display, Pixmap pixmap, int xp, int yp,iEntityPtr entity)
{
	char *str;
	iLBox visible = visibleRectangle(this->page->mode->scrolled_window);
	TreePage *page = (TreePage*)this->page;
	ViewCanvas *vc = page->getCanvas();
	iUATablePtr table = allocTable();
	int i= (visible.w - xp)/page->cw;
	char *display_str = (char*)smalloc(i*sizeof(char));

	str = i_AllocASCIIText(entity,table);
	strncpy(display_str,str,i);

	// if there is a null before this, we won't see this. fine
	display_str[i-4]='.';display_str[i-3]='.';display_str[i-2]='.';
	display_str[i-1]=0;

	XDrawString(display, pixmap, vc->colorGC("grey40"), xp, yp, display_str, strlen(display_str));
	smfree(str);
	smfree(display_str);

	freeTable(table);
} 

void TreeElement::drawRectangle(Display *display, Pixmap pixmap, GC gc,int x, int y)
{	
	TreePage *page = (TreePage*)this->page;
	ViewCanvas *vc = page->getCanvas();
	XFillRectangle(display,pixmap,vc->blankGC(),x-10,y-5,20,10);
	XDrawRectangle(display,pixmap,gc,x-10,y-5,20,10);
	XDrawRectangle(display,pixmap,gc,x,y,4,2);
}
void fixnewline(iUCode *buf,int len){
	iUCode *p,*u,*new_buf = (iUCode *)smalloc((len+1) * sizeof(iUCode));
	ucsncpy(new_buf,buf,len);
	p = (iUCode*)ucschr(buf,'\n');
	*p = 0;
	int used = ucslen(buf);
	*p++ = '\\'; *p++ = 'n';
	u = (iUCode*)ucschr(new_buf,'\n');
	ucsncpy(p,++u,len-used);
	smfree(new_buf);

}
void 
TreeElement::getString(iEntityPtr entity, iUCode *buf, int *return_len)
{
	aString as;
	uString us;
	int len = 0,bigger = 0;
	as = i_GetText(entity,0);
	if (strlen(as.string()) > MAXCHARS)
		bigger = 1;
	us.getUnicode(as);
	len = MIN(MAXCHARS,strlen(as.string()));
	len += 2;
	*buf = '\"';
	ucsncpy(buf+1,us.string(),len);
	if (bigger){
		int j;
		for (j=MAXCHARS-2;j<MAXCHARS+1;j++)
			buf[j] = '.';
	}
	buf[len-1] = '\"';
	if (ucschr(buf,'\n')) {
		fixnewline(buf,len);
		len ++;
	}
	*return_len = len;
}

void
TreeElement::getUString(iEntityPtr entity, iUCode *buf, int *return_len)
{
	uString us;
	int len = 0,bigger = 0;
	us  = i_GetUText(entity,0);
	len = MIN(MAXCHARS,ucslen(us.string()));
	len += 2;
	if (ucslen(us.string()) > MAXCHARS)
		bigger = 1;
	*buf = '\"';
	ucsncpy(buf+1,us.string(),len);
	if (bigger){
		int j;
		for (j=MAXCHARS-2;j<MAXCHARS+1;j++)
			buf[j] = '.';
	}
	buf[len-1] = '\"';
	if (ucschr(buf,'\n')){
		fixnewline(buf,len);
		len ++;
	}
	*return_len = len;
}

void
TreeElement::getPosSet(iEntityPtr entity,iUCode *buf, int *return_len)
{
	int i, len = 0;
	int bigger=0;
	short valid;
	*buf = '{';
	iPosSetPtr ps = i_GetPosSet(entity,&valid);
	if (valid > MAXCHARS)
		bigger = 1;
	valid = MIN(MAXCHARS,valid);
	i = 1;
	while (valid--) {
		buf[i++] = ps->chr;
		ps++;
	}
	if (bigger){
		int j;
		for (j=MAXCHARS-1;j<MAXCHARS+1;j++)
			buf[j] = '.';
	}
	buf[i++] = '}';
	buf[i] = 0;
	len = ucslen(buf);
	if (ucschr(buf,'\n')){
		len ++;
		fixnewline(buf,len);
	}
	*return_len = len;
}
void
TreeElement::getBoxTextandWidth()
{
	TreePage *page = (TreePage *) this->page;
	int len=0,type_length,type_len;
	int content_length;
	iUCode buf[MAXBOXCHARS+3];
	char string[MAXBOXWIDTH];
	aString as = (char*)i_TypeToName(i_GetEntityType(entity));
	type_len = strlen(as.string());
	if (this->text) smfree(this->text);
	iUCode *box_text = (iUCode*)smalloc((type_len + 1)*sizeof(iUCode));
	uString us;
	us.getUnicode(as);
	type_length = page->cw * type_len + 1;
	ucsncpy(box_text,us.string(),type_len);
	box_text[type_len] = ' ';
	
	len = 0;
	switch(i_GetEDataType(entity)){
	case iDPosSet:
	case iD1PosSet:
		this->getPosSet(entity, buf, &len);
		break;
	case iDString:
		this->getString(entity, buf, &len);
		break;
	case iDUString:
		this->getUString(entity, buf, &len);
		break;		
	case iDShort:
		sprintf(string,"%d",i_GetShort(entity,0));
		as = string;
		us.getUnicode(as);
		len = MIN(MAXCHARS,ucslen(us.string()));
		ucsncpy(buf,us.string(),len);
		break;
	case iDConf:
		sprintf(string,"%d",i_GetConf(entity));
		as = string;
		us.getUnicode(as);
		len = MIN(MAXCHARS,ucslen(us.string()));
		ucsncpy(buf,us.string(),len);
		break;
	case iDNo:
	case iDUserData:
	default:
		*buf = 0;
		break;
	}
	content_length = measureContentWidth(buf,len);

	iUCode *new_box_text = (iUCode*)smrealloc(box_text,sizeof(iUCode)*(len+type_len+1));
	ucsncpy(&new_box_text[type_len+1],buf,len);
	this->numchars = len + type_len + 1;
	this->text = new_box_text;
	this->width = content_length + type_length + 3*SPACER;
}

int 
TreeElement::measureContentWidth(iUCode *buf,int chars)
{
	int i, char_width, dummy, str_len = 0;
	TreePage *page = (TreePage*)this->page;
	ViewCanvas *vc = this->page->getCanvas();
	
	for (i = 0; i < chars;i++){
		vc->measureText(&buf[i], 1, &char_width, &dummy, &dummy);
		str_len += char_width;
	}
	if (!str_len)
		str_len = chars * page->cw;
	return str_len;
}

int 
TreeElement::drawAnEntity(Display *display,Pixmap pixmap)
{
	iEntityPtr parent;
	TreePage *page = (TreePage*)this->page;
	ViewCanvas *vc = page->getCanvas();
	TreeElement *par_element;
	GC gc = vc->colorGC("grey40");
	aString color = getColor(entity);
	iLBox b,draw_box;
	
	b.y = this->y - vc->pix_voffset;
	b.x = this->x - vc->pix_hoffset;
	b.w = this->width;
	b.h = this->height;
	parent = i_GetParentEntity(entity); 
#ifndef IGNORE_ORS
	if(i_GetLogicalType(parent)==iOr)
		parent = i_GetParentEntity(parent);
#endif
	if (parent) par_element = (TreeElement*)eaGetFace(parent,vtTREE);
	else par_element = 0;
	int par_y = (par_element ? par_element->y:0) - vc->pix_voffset;
	int par_x =(par_element ? par_element->x:0) - vc->pix_hoffset;
	if (!par_element) par_x -= page->hspacer;
	if (parent && i_GetLogicalType(parent) == iOr){
		 XGCValues values;
		 values.line_style = LineDoubleDash;
		 XChangeGC(display, gc, GCLineStyle, &values);
	}
	if (b.y > vc->view_port.h) {
		if((par_y > vc->view_port.h) || !par_element) return 0;
		XDrawLine(display,pixmap, gc, 
					par_x, MAX(par_y+page->cw/2,0),
					par_x , vc->view_port.h+page->cw);
		return 1;	
	}
	// need to do a little jiggleing of the coordinates to get
	// the lines to the slanted corners of the boxes
	if((par_y < vc->view_port.h) && par_element){
		XDrawLine(display, pixmap, gc, 
					par_x, MAX(par_y+page->ch/2,0), 
					par_x, MIN(vc->view_port.h+page->ch,(b.y+page->ch/2)));
		XDrawLine(display, pixmap,  gc,
					par_x, b.y+page->ch/2,
					b.x, b.y+page->ch/2);
	}
	if (parent && i_GetLogicalType(parent) == iOr){
		 XGCValues values;
		 values.line_style = LineSolid;
		 XChangeGC(display, gc, GCLineStyle, &values);
	}
	if (i_GetChildEntity(entity))
		drawTri(display, pixmap, gc);
#ifndef IGNORE_ORS
	if ((parent = i_GetParentEntity(entity)) && i_GetLogicalType(parent)==iOr)
	  drawRectangle(display,pixmap, gc, par_x, b.y+page->ch/2);
#endif
	
	draw_box = b;
	b.x += SPACER;
	b.y += (this->height - 2);
	if (this->text && *this->text) 
		drawUnicode(display, pixmap, gc, b.x, b.y, this->text, this->numchars);
	int selected = (getEntitySelection(entity) == scITEM);
	if(selected)
		gc = vc->thickGC(color.string());
	else
		gc = vc->colorGC(color.string());
	drawBox(display, pixmap,gc,&draw_box);
	if(!this->show && i_GetChildEntity(entity)){
		int xp = draw_box.x + draw_box.w + page->vspacer;
		int yp = draw_box.y + page->ch;
		drawTheString(display,pixmap,xp,yp,entity);
	}

	return 1;



}

void 
TreeElement::willMove()
{
}

void 
TreeElement::destroy()
{
	this->redraw();
}

void 
TreeElement::update()
{
	TreePage *page = (TreePage *) this->page;
	this->getBoxTextandWidth();
	page->refreshSel(entity);
}

void 
TreeElement::move()
{
}

int TreeElement::inBox(int xb,int yb)
{
	int here = this->x;
	if ((xb > here) && (xb < (here + this->width))
			&& (yb <= (this->y + this->height)) && (yb >= this->y))
			return 1;
	return  0;
}
int TreeElement::inTri(int xb,int yb)
{
	int here = this->x;
	if  ((xb < here) && (xb > (here - 10))
			&& (yb <= (this->y + this->height)) && (yb >= this->y))
		return 1;
	return 0;
}
int TreeElement::inRectangle(int xb,int yb)
{
	TreePage *page = (TreePage *) this->page;
	int here = this->x-page->hspacer;
	int yhere = this->y + this->height/2;
	if ((xb < (here+5)) && (xb > (here - 5))
		&& (yb <= (yhere + 5)) && (yb >= (yhere - 5)))
		return 1;
	return  0;
}

void 
TreeElement::select()
{
	TreePage *page = (TreePage *) this->page;
	
	page->refreshSel(entity);
}

void 
TreeElement::unSelect()
{
	TreePage *page = (TreePage *) this->page;
	
	page->refreshSel(entity);
}

void 
TreeElement::recognize()
{
	TreePage *page = (TreePage *) this->page;
	ViewCanvas *vc = page->getCanvas();
	Widget widget = vc->canvas;
	iLBox box;

	box = visibleRectangle(XtParent(widget));
	page->stale = 1;

	page->drawRegionNow(box);
}

MainWindow *
TreeElement::getMainWindow()
{
	return this->page->mode->main_window;
}

