
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"
#include "illum.h"

#include <Xm/Label.h>
#include <Xm/SelectioB.h>
#include <Xm/PushB.h>
#include <Xm/PushBG.h>
#include <Xm/Form.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/Text.h>

#include "UniEdit.h"

#include "misc/XArg.h"
#include "misc/misc.h"
#include "smemory.h"
#include "misc/Unicode.h"

static void 
stuffData(MenuItem *menus, void *data)
{
	int i;
	for(i = 0; menus[i].label != NULL; i++) {
		menus[i].callback_data = data;
	}
}


UniEdit::UniEdit(MainWindow *input_mw,
				 Widget input_app_shell, 
				 KeyBoardHandler input_key_h,
				 void *input_user_data)
	: Dialog(input_mw, input_app_shell),
	  key_h(input_key_h),
	  user_data(input_user_data),
	  glyph_display(0),
	  text_input(0),
	  ok_button(0),
	  apply_button(0)
{
}

UniEdit::~UniEdit()
{
}

static void do_applyCB(Widget w, void *ue, void *call)
{
	((UniEdit *)ue)->applyCB((XmFileSelectionBoxCallbackStruct *)call);
}

static void do_okCB(Widget w, void *ue, void *call)
{
	((UniEdit *)ue)->okCB((XmFileSelectionBoxCallbackStruct *)call);
}

static void do_closeCB(Widget w, void *ue, void *call)
{
	((UniEdit *)ue)->closeCB((XmFileSelectionBoxCallbackStruct *)call);
}
static void do_returnCB(Widget w, void *ue, void *call)
{
	((UniEdit *)ue)->okCB((XmFileSelectionBoxCallbackStruct *)call);
}
static MenuItem DummyMenu[] = {
	{ 0, "OK", &xmPushButtonGadgetClass, 0,
	  "<Key>Return",0, do_returnCB, 0, NULL },
	NULL_MENU,
};

static void addButton(Widget w,char *str,XtCallbackProc cb, UniEdit *ue)
{
	Widget n;
	XmString xm_string;
	XArg args;

	args.reset();
	xm_string = XmStringCreateLocalized(str);
	args.setArg(XmNlabelString, xm_string);
	args.setArg(XmNalignment, XmALIGNMENT_CENTER);
/*	args.setArg(XmNdefaultButtonShadowThickness,1);*/
	n = XmCreatePushButton(w, str,args.getArgs(), args.getSize());
	XmStringFree(xm_string);
	XtManageChild(n);

	if (cb)
		XtAddCallback(n, XmNactivateCallback, cb, ue);

}

static void addCellLabel(Widget w,Widget *np,char *str,int align)
{
	Widget n;
	XmString xm_string;
	XArg args;

	args.reset();
	xm_string = XmStringCreateLocalized(str);
	args.setArg(XmNlabelString, xm_string);
	args.setArg(XmNalignment, align);
	n = XmCreateLabel(w, str,args.getArgs(), args.getSize());
	XmStringFree(xm_string);
	XtManageChild(n);
	if (np) *np = n;
}


static iUCode get_value(Widget w)
{
	iUCode value,temp;
	int i,len;
	char *s,c;

	s = XmTextFieldGetString(w);
	if (s) {
		len=strlen(s);
		if (len==0 || len>4)
			value=0;
		else {
			value=0;
			for (i=0; i<len; i++) {
				c=s[i];
				if (c<='9')
					temp=c-'0';
				else
					temp=c-'a'+10;
				value=(value<<4)+temp;
			}
		}
		XtFree(s);
	} else
		value=0;
	return value;
}


static void value_changed(Widget w, UniEdit *ue, XmTextVerifyCallbackStruct *cbs)
{
	ue->setGlyph(get_value(w));
}


static void verify_input(Widget w, XtPointer unused, XmTextVerifyCallbackStruct *cbs)
{
	int i,len,ok;
	char c;

	if (cbs->text->ptr==NULL) /* backspace */
		return;

	len=cbs->text->length;
	ok=TRUE;
	for (i=0; i<len; i++) {
		c=cbs->text->ptr[i];
		if (c>='A' && c<='Z') {
			c=c-'A'+'a';
			cbs->text->ptr[i]=c;
		}
		ok = ok && ((c>='0' && c<='9') || (c>='a' && c<='f'));
	}
	if (!ok)
		cbs->doit=FALSE;
}

void UniEdit::setGlyph(iUCode glyph)
{
	XmString str;
	XmFontList font_list;
	XArg args;
	
	if (glyph==0)
		glyph=' ';

	str=makeCompoundString(XtDisplay(this->glyph_display),&glyph,1,&font_list);
	
	args.setArg(XmNlabelString, str);
	if(font_list)
		args.setArg(XmNfontList, font_list);
	args.setValues(this->glyph_display);
	XmStringFree(str);
}


void UniEdit::closeCB(XmFileSelectionBoxCallbackStruct *call)
{
	this->hide();
}

extern void 	whereIsInputFocus();
void UniEdit::applyCB(XmFileSelectionBoxCallbackStruct *call)
{
	iUCode value=get_value(this->text_input);
#ifdef DEBUG
	whereIsInputFocus();
#endif
	if(!key_h(this->user_data, 0, value))
		ringTheBell();
}

void UniEdit::okCB(XmFileSelectionBoxCallbackStruct *call)
{
	iUCode value=get_value(this->text_input);
	if(key_h(this->user_data, 0, value))
		this->hide();
	else
		ringTheBell();
}



#define MIN_DIALOG_WIDTH 150
#define MIN_DIALOG_HEIGHT 150
void 
UniEdit::create()
{
	XArg args;
	Widget parent = this->app_shell;
	XmString xm_string;
	Widget w,button_form,holder,b_rowcol,i_rowcol,input_w;

	args.reset();
	args.setArg(XmNtitle, "Input Unicode");
	stuffData(DummyMenu, this);
	this->dialog =
		XmCreateFormDialog(parent,"UniEdit",args.getArgs(),args.getSize());
	this->dummy_menu = 
		BuildMenu(this->dialog, XmMENU_POPUP, "Buttons", 'F', DummyMenu);

#if 0
	args.reset();
	args.setArg(XmNleftAttachment,XmATTACH_FORM);
	args.setArg(XmNrightAttachment,XmATTACH_FORM);
	args.setArg(XmNbottomAttachment,XmATTACH_FORM);
	/*args.setArg(XmNwidth, MIN_DIALOG_WIDTH);
	args.setArg(XmNheight, MIN_DIALOG_HEIGHT);*/
	holder = XmCreateForm(this->dialog,"holder",
										args.getArgs(), args.getSize());
	XtManageChild(holder);
#endif

	args.reset();
	args.setArg(XmNleftAttachment,XmATTACH_FORM);
	args.setArg(XmNrightAttachment,XmATTACH_FORM);

	args.setArg(XmNshadowType, XmSHADOW_ETCHED_IN);
	/*args.setArg(XmNleftAttachment,XmATTACH_FORM);*/
	args.setArg(XmNbottomAttachment,XmATTACH_FORM);
	button_form = XmCreateFrame(this->dialog,"frame",args.getArgs(),args.getSize());
	XtManageChild(button_form);

	args.reset();
	args.setArg(XmNpacking,XmPACK_COLUMN);
	args.setArg(XmNnumColumns,3);
	args.setArg(XmNorientation,XmVERTICAL);
	args.setArg(XmNisAligned,FALSE);
	b_rowcol = XmCreateRowColumn(button_form,"Buttons",
				args.getArgs(),args.getSize());
	XtManageChild(b_rowcol);

	addButton(b_rowcol,"OK",do_okCB,this);
	addButton(b_rowcol,"Apply",do_applyCB,this);
	addButton(b_rowcol,"Close",do_closeCB,this);

	args.reset();
	args.setArg(XmNshadowType, XmSHADOW_ETCHED_IN);
	args.setArg(XmNtopAttachment,XmATTACH_FORM);
	args.setArg(XmNbottomAttachment,XmATTACH_WIDGET);
	args.setArg(XmNbottomWidget,button_form);
	args.setArg(XmNleftAttachment,XmATTACH_FORM);
	args.setArg(XmNrightAttachment,XmATTACH_FORM);
	input_w = XmCreateFrame(this->dialog,"input",args.getArgs(),args.getSize());
	XtManageChild(input_w);

	args.reset();
	args.setArg(XmNpacking,XmPACK_COLUMN);
	args.setArg(XmNnumColumns,2);
	args.setArg(XmNorientation,XmVERTICAL);
	args.setArg(XmNisAligned,FALSE);
	i_rowcol = XmCreateRowColumn(input_w,"Buttons",
				args.getArgs(),args.getSize());
	XtManageChild(i_rowcol);

	addCellLabel(i_rowcol,0,"Unicode Value",XmALIGNMENT_CENTER);

	args.reset();
	/*args.setArg(XmNleftAttachment,XmATTACH_FORM);
	args.setArg(XmNrightAttachment,XmATTACH_FORM);
	args.setArg(XmNbottomAttachment,XmATTACH_FORM);*/
	args.setArg(XmNalignment, XmALIGNMENT_CENTER);
	holder = XmCreateForm(i_rowcol,"input_holder",
										args.getArgs(), args.getSize());
	XtManageChild(holder);

	args.reset();
	xm_string = XmStringCreateLocalized("U+");
	args.setArg(XmNlabelString, xm_string);
	args.setArg(XmNleftAttachment,XmATTACH_FORM);
	args.setArg(XmNalignment, XmALIGNMENT_CENTER);
	w = XmCreateLabel(holder,"U+",args.getArgs(), args.getSize());
	XmStringFree(xm_string);
	XtManageChild(w);

	args.reset();
	/*args.setArg(XmNrightAttachment,XmATTACH_FORM);*/
	args.setArg(XmNleftAttachment,XmATTACH_WIDGET);
	args.setArg(XmNleftWidget,w);
	args.setArg(XmNmaxLength,4);
	args.setArg(XmNcolumns,4);
	args.setArg(XmNleftWidget,w);
	args.setArg(XmNmarginHeight,2);
	args.setArg(XmNmarginWidth,2);
	this->text_input=XmCreateTextField(holder,"text_input",args.getArgs(),args.getSize());
	XtAddCallback(this->text_input,XmNmodifyVerifyCallback,verify_input,0);
	XtAddCallback(this->text_input,XmNvalueChangedCallback,value_changed,this);
	XtManageChild(this->text_input);

	addCellLabel(i_rowcol,0,"Glyph",XmALIGNMENT_CENTER);
	addCellLabel(i_rowcol,&this->glyph_display," ",XmALIGNMENT_CENTER);
}


void UniEdit::show()
{
	if(this->text_input)
		XmTextSetString(this->text_input,"");
	this->Dialog::show();
}


