
/*
 * Copyright (c) 1994, 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/PushB.h>

#include <stdio.h>

#include "illum.h"

#include "View.h"
#include "Frame.h"
#include "MainWindow.h"
#include "ViewCanvas.h"
#include "smemory.h"


View::View(MainWindow *input_main_window)
	: current_frame_ref(0),
	  scrolled_window(0), main_window(input_main_window)
{
	dllistCtor(&this->frame_list);
}


View::~View()
{
	Frame *frame;
	DLList *fl = &this->frame_list;
	Ref r;
	
	for(r = dllistFirst(fl); r; dllistNext(fl, &r)){
		frame = (Frame *) dllistContents(fl, r);
		if(frame->managed) {
			if(this->main_window->view_canvas)
				this->main_window->view_canvas->unmanage();
		}
		delete frame;
	}
	dllistDtor(fl);
}

void
View::purge()
{
	Frame *frame;
	DLList *fl = &this->frame_list;
	Ref r;
	
	for(r = dllistFirst(fl); r; dllistNext(fl, &r)){
		frame = (Frame *) dllistContents(fl, r);
		if(frame->managed) {
			if(this->main_window->view_canvas)
				this->main_window->view_canvas->unmanage();
		}
		frame->purge();
	}
	dllistClear(fl);

	delete this;
}

iBox 
View::getBounds(iEntityPtr entity)
{ 
	iBox box;
	box.w = 0;
	return box;
}


Frame *
View::getCurrentFrame ()
{
	Ref r;
	Frame *frame = 0;
	r = this->current_frame_ref;
	if (r)
		frame = (Frame *) dllistContents(&this->frame_list, r);
	return frame;
}


void
View::show()
{
	Frame *frame;
	DLList *fl = &this->frame_list;
	
	if(current_frame_ref != 0){
		frame = (Frame *) dllistContents(fl, this->current_frame_ref);
		frame->manage();
	}
}

void 
View::makeFrameList(XmStringTable *list, int *list_size)
{
	Ref r;
	int i;
	Frame *frame;
	
	*list_size=0;
	DLList *l = &this->frame_list;
	
	for(r = dllistFirst(l), i = 0; r; dllistNext(l, &r))
		i++;
	*list = (XmStringTable)smalloc(i * sizeof(XmString *));
	for(i = 0, r = dllistFirst(l); r; dllistNext(l, &r)) {
		frame = (Frame *) dllistContents(l, r);
		if (!frame->should_delete)
			(*list)[i++] = XmStringCreateLocalized((char *) frame->name.string());
	}
	*list_size=i;
}

void
View::hide()
{
	Frame *frame;
	DLList *fl = &this->frame_list;
	
	if(current_frame_ref != 0){
		frame = (Frame *) dllistContents(fl, this->current_frame_ref);
		frame->unmanage();
	}
}

void
View::emptied(Frame *frame)
{
	Ref r,new_r=0;
	
	DLList *fl = &this->frame_list;
	r=dllistSeek(fl,frame);
	
	if (r) {
		Ref r1=r;
		Ref r2=r;
		dllistNext(fl,&r1);
		new_r=r1;
		if (!r1) {
			dllistPrev(fl,&r2);
			new_r=r2;
		}
	}
	if (frame->managedP()) {
		this->setCurrentFrame(new_r);
	} else {
		if(r == current_frame_ref)
			current_frame_ref = 0;
	}
	if (r) 
		dllistRemove(fl,r);
	if (frame) {
		//delete frame;
	}
	
	
}

void
View::setCurrentFrame(Ref r)
{
	Frame *frame;
	DLList *fl = &this->frame_list;
	
	if(current_frame_ref != 0){
		frame = (Frame *) dllistContents(fl, this->current_frame_ref);
		frame->unmanage();
	}
	this->current_frame_ref = r;
	if(this->current_frame_ref != 0) {
		frame = (Frame *) dllistContents(fl, this->current_frame_ref);
		frame->conditionallyInitialiseCanvas();
		frame->manage();
		this->main_window->standardTitle();
	}
}

void
View::start()
{
	Frame *frame;
	DLList *fl = &this->frame_list;
	Ref r;
	
	for(r = dllistFirst(fl); 
		r && (frame = (Frame *) dllistContents(fl, r))->should_delete;
		dllistNext(fl, &r));

	this->setCurrentFrame(r);
}

int
View::countFrames()
{
	int i;
	Ref r;
	DLList *fl=&this->frame_list;
	
	for (i=0, r=dllistFirst(fl); r; dllistNext(fl,&r))
		if (!((Frame *) dllistContents(fl, r))->should_delete) i++;
	return i;
//   return dllistLength(&this->frame_list);
}

Ref
View::forwardFrameRef()
{
	DLList *fl = &this->frame_list;
	Ref r;
	Frame *frame;
	
	if((r = this->current_frame_ref))
		dllistNext(fl, &r);
	
	for(;
       r && (frame = (Frame *) dllistContents(fl, r))->should_delete;
       dllistNext(fl, &r));
	
	return r;
}

Ref
View::backwardFrameRef()
{
	DLList *fl = &this->frame_list;
	Ref r;
	Frame *frame;
	
	if((r = this->current_frame_ref))
		dllistPrev(fl, &r);
	
	for(;
       r && (frame = (Frame *) dllistContents(fl, r))->should_delete;
       dllistPrev(fl, &r));
	
	return r;
}


int
View::forward()
{
	Ref r;
	
	r=this->forwardFrameRef();
	if(r) {
		this->setCurrentFrame(r);
		return TRUE;
	} else
		return FALSE;
}

int
View::back()
{
	Ref r;
	
	r=this->backwardFrameRef();
	if(r) {
		this->setCurrentFrame(r);
		return TRUE;
	} else
		return FALSE;
}

void
View::setParentWidget (Widget parent)
{
	this->scrolled_window = parent;
}

void
View::setMag(int mag_exp, int old_mag)
{
	Frame *frame;

	frame = this->getCurrentFrame();
	if (frame) {
		frame->setMag(mag_exp, old_mag);
	}
}

int 
View::managedP() 
{
	Frame *current_frame = this->getCurrentFrame();
	if(current_frame == 0)
		return 0;
	return current_frame->managedP();
}

SelectionManager *View::getSM()
{
	return this->main_window->sm;
}

void
View::addFrame(Frame *frame)
{
	Ref r = this->current_frame_ref;
	Frame *current_frame = this->getCurrentFrame();
	dllistAppend(&this->frame_list, frame);
	this->current_frame_ref = dllistSeek(&this->frame_list, current_frame);
}

Ref
View::addFrameBefore(Ref p_r, Frame *frame)
{
	Ref r = this->current_frame_ref;
	Frame *current_frame = this->getCurrentFrame();

	p_r = dllistInsertBefore(&this->frame_list, p_r, frame);

	this->current_frame_ref = dllistSeek(&this->frame_list, current_frame);

	return p_r;
}
