
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include <X11/cursorfont.h>
#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrollBar.h>

#include "misc/color.h"
#include "misc/XArg.h"

#include "rafport.h"
#include "rafutil.h"
#include "illum.h"
#include "mag.h"
#include "Page.h"
#include "Mode.h"
#include "Application.h"
#include "MainWindow.h"
#include "MainWindow.cb.h"
#include "keyboard/KeyBoard.h"
#include "keyboard/KeyMap.h"

#include "ViewCanvas.h"

#include "misc/scroll.h"
#include "misc/Unicode.h"

#define FOLLOW_INCREMENT 30
#ifndef max
#define max(a,b) ( (a)>(b) ? (a):(b))
#endif

static char
follow(ViewCanvas *vc)
{
	Window root;
	Window child;
	int root_x, root_y, px, py;
	int4 h_value,v_value;
	unsigned int mask;

	/* make a call to xquerypointer and feed your client*/
	XQueryPointer(XtDisplay(vc->canvas), XtWindow(vc->canvas),
				  &root, &child,
				  &root_x, &root_y,
				  &px, &py,
				  &mask);
	
	if (mask&Button1Mask || mask&Button3Mask || mask&Button2Mask) {
		//Widget ww = XtParent(vc->canvas);
		Widget sw = XtParent(vc->canvas);
		iLBox vr;
		
		vr = visibleRectangle(sw);
		getScrollBarValues(sw,&h_value,&v_value);
		px += h_value;
		py += v_value;
		int4 c_x;
		int4 c_y;
		getScrollPositionCenter(sw, &c_x, &c_y);
		if(px < vr.x) {
			c_x-=FOLLOW_INCREMENT;
			px = vr.x;
		} else if(px > (vr.x + vr.w)) {
			c_x+=FOLLOW_INCREMENT;
			px = vr.x + vr.w;
		}
		
		if(py < vr.y){
			c_y-=FOLLOW_INCREMENT;
			py = vr.y;
		} else if(py > (vr.y + vr.h)) {
			c_y+=FOLLOW_INCREMENT;
			py = vr.y + vr.h;
		}
		
		vc->client->moveButton(mask, px, py);	 
		vc->scrollToPosition(sw, c_x, c_y);
		}
	return 0;
}
	
void
ViewCanvas::scrollToPosition(Widget sw, int4 x, int4 y)
{
	// bogus double check identity of scrolled window
	XArg args;
	Widget work_window;
	Widget hsb;
	Widget vsb;
	Dimension height;
	Dimension width;
	int minimum;
	int value;
	int maximum;
	int slider_size;
	int page_increment;
	int increment;;
	
	args.reset();
	args.setArg(XmNworkWindow, &work_window);
	args.setArg(XmNhorizontalScrollBar, &hsb);
	args.setArg(XmNverticalScrollBar, &vsb);
	args.getValues(sw);
	
	if(work_window) {
		args.reset();		
		
		args.reset();
		args.setArg(XmNheight, &height);
		args.setArg(XmNwidth,  &width);
		args.getValues(work_window);

		// bogus check if hsb is managed
		args.reset();
		args.setArg(XmNminimum, &minimum);
		args.setArg(XmNmaximum, &maximum);
		args.setArg(XmNvalue, &value);
		args.setArg(XmNsliderSize, &slider_size);
		args.setArg(XmNincrement, &increment);
		args.setArg(XmNpageIncrement, &page_increment);
		args.getValues(hsb);

		
		int x_prime;
		x_prime = MAX(0, x - width/2);

		value = x_prime ;

		value = MAX(value, minimum);
		value = MIN(value, maximum - slider_size);
		XmScrollBarSetValues(hsb, value, slider_size, increment, page_increment, TRUE);
		this->pix_hoffset = value;
		
		// bogus check if vsb is managed
		args.reset();
		args.setArg(XmNminimum, &minimum);
		args.setArg(XmNmaximum, &maximum);
		args.setArg(XmNvalue, &value);
		args.setArg(XmNincrement, &increment);
		args.setArg(XmNpageIncrement, &page_increment);
		args.setArg(XmNsliderSize, &slider_size);
		args.getValues(vsb);
		
		int y_prime;
		y_prime = MAX(0, y - height/2);
		value = y_prime;
		
		value = MAX(value, minimum);
		value = MIN(value, maximum - slider_size);
		XmScrollBarSetValues(vsb, value, slider_size, increment, page_increment, TRUE);
		this->pix_voffset = value;
	
	}
}
static void
activateFollow(ViewCanvas *vc)
{
	vc->follow = 1;
	if(vc->proc_id)
		return;
	vc->proc_id = 
		XtAppAddWorkProc(vc->client->mode->main_window->app->app_context,
						 follow, vc);
}

static void
deactivateFollow(ViewCanvas *vc)
{
	vc->follow = 0;
	if(vc->proc_id == 0)
		return;
	XtRemoveWorkProc(vc->proc_id);
	vc->proc_id = 0;
	
}


void redisplayCB (Widget         w,
				  ViewCanvas *vc,
				  XmDrawingAreaCallbackStruct *call_data)
{
	if(vc->client && !vc->client->should_delete)
		vc->client->redisplay(call_data);
}
void resizeCB (Widget         w,
				  ViewCanvas *vc,
				  XmDrawingAreaCallbackStruct *call_data)
{
	if(vc->client && !vc->client->should_delete)
		vc->client->resize(call_data);
}

void pressButtonEH(Widget          w,
				   ViewCanvas      *vc,
				   XEvent         *event)
{
	vc->release_expected = TRUE;
	if(vc->client && !vc->client->should_delete)
		vc->client->pressButton(event);
}

void releaseButtonEH(Widget          w,
					 ViewCanvas      *vc,
					 XEvent         *event)
{
	if(vc->release_expected == FALSE)
		return;
	if(vc->client && !vc->client->should_delete)
		vc->client->releaseButton(event);
	deactivateFollow(vc);
	vc->release_expected = FALSE;
}

void moveButtonEH(Widget          w,
				  ViewCanvas      *vc,
				  XEvent         *event)
{
	if(vc->release_expected == FALSE)
		return;
	Widget sw = XtParent(vc->canvas);
	iLBox vr;
	int4 h_value,v_value;
	if(!vc->client)
		return;
	if (vc->client->should_delete)
		return;
	

	vr = visibleRectangle(sw);
	int px = event->xmotion.x + vc->pix_hoffset;
	int py = event->xmotion.y + vc->pix_voffset;
	getScrollBarValues(sw,&h_value,&v_value);
	if((px >= vr.x && px <= (vr.x + vr.w)) &&
	   (py >= vr.y && py <= (vr.y + vr.h))) {
		deactivateFollow(vc);
		vc->client->moveButton(event->xmotion.state, px, py);

	} else {
		int4 c_x;
		int4 c_y;
		getScrollPositionCenter(sw, &c_x, &c_y);
		if(px < vr.x) {
			c_x-=FOLLOW_INCREMENT;
			px = vr.x;
		} else if(px > (vr.x + vr.w)) {
			c_x+=FOLLOW_INCREMENT;
			//c_x++;
			px = vr.x + vr.w;
		}
		
		
		if(py < vr.y){
			c_y-=FOLLOW_INCREMENT;
			//c_y--;
			py = vr.y;
		} else if(py > vr.y + vr.h) {
			c_y+=FOLLOW_INCREMENT;
			py = vr.y + vr.h;
		}
		
		vc->client->moveButton(event->xmotion.state, px, py);	 
		vc->scrollToPosition(sw, c_x, c_y);
		 
		activateFollow(vc);
		//printf("outside\n");
	}		

	
}

void pressKeyEH(Widget          w,
				ViewCanvas      *vc,
				XEvent         *event)
{
	char key_value;
	iUCode char_value;
	int n;
	KeySym ks;
	int state;
	int alt_on;
	MainWindow *mw;
	
	if(!vc->client)
		return;
	if(vc->client->should_delete)
		return;
	
	
	state = event->xkey.state;
	alt_on = state & Mod1Mask;

	if(state & ControlMask)
	   return;

	n = XLookupString(&event->xkey,
					  &key_value,
					  1,
					  &ks,
					  0);
	mw=vc->client->mode->main_window;

	if (ks == XK_BackSpace || ks == XK_Delete)
		char_value='\b';
	else
		if(alt_on)
			char_value = mw->kb->key_map->lookupControlValue(ks);
		else
			char_value = mw->kb->key_map->lookupValue(ks);
	
	// this is a bit gross, but I need to make sure that the deleteCB
	// does exactly the same thing as hitting back space
	if (char_value=='\b')
		deleteCB(w,vc->client->mode->main_window,event);
	else if (char_value)
		vc->client->pressKey(char_value);
}


static void
calcFontDims(Widget canvas, int *char_width, int *char_height, int *ascent)
{
	iUCode M='M';

	measureUnicode(XtDisplay(canvas), &M, 1, char_width, char_height, ascent);
}

#include "SelectionManager.h"
#include "Document.h"

ViewCanvas::ViewCanvas(Widget parent, Page *input_client, int width, int height)
	: proc_id(0),
	  client(input_client),
	  blank_gc(0),
	  blue_gc(0),
	  inverse_blue_gc(0),
	  thick_blue_gc(0),
	  grot_gc(0),
	  inverse_grot_gc(0),
	  thick_grot_gc(0),
	  flag_gc(0),
	  inverse_flag_gc(0),
	  black_gc(0),
	  inverse_black_gc(0),
	  red_gc(0),
	  thick_red_gc(0),
	  green_gc(0),
	  gray_gc(0),
	  dark_gray_gc(0),
	  xor_gc(0),
	  copy_gc(0),
	  saveshape(-1),
	  char_width(0),
	  char_height(0),
	  ascent(0),
	  release_expected(0),
	  gc_ptr(0),
	  inverse_gc_ptr(0),
	  thick_gc_ptr(0),
	  medium_gc_ptr(0),
	  follow(0)
{
	XArg args;
	pix_voffset = 0;
	pix_hoffset = 0;
	
	// bogus
	args.setArg(XmNbackground, getColorByName(parent, GRAY_COLOR));
	this->canvas = XtCreateWidget("canvas", 
								  xmDrawingAreaWidgetClass, 
								  parent, args.getArgs(), args.getSize());
	args.reset();
	args.setArg(XmNworkWindow,this->canvas);
	args.setValues(parent);
	
	iBox b;
	b.x = b.y = 0;
	b.w = width;
	b.h = height;
	this->view_port=b;
	this->font_set=0;
	this->font_struct=0;
	XtAddCallback(this->canvas, 
				XmNexposeCallback, 
					redisplayCB, 
						(XtPointer)this); 
	XtAddCallback(this->canvas, 
				XmNresizeCallback, 
					resizeCB, 
						(XtPointer)this); 
	
	XtAddEventHandler(this->canvas, ButtonPressMask, FALSE,
					  (XtEventHandler) pressButtonEH, this);
	XtAddEventHandler(this->canvas, ButtonReleaseMask, FALSE,
					  (XtEventHandler) releaseButtonEH, this);
	XtAddEventHandler(this->canvas, ButtonMotionMask, FALSE,
					  (XtEventHandler) moveButtonEH, this);
	
	XtAddEventHandler(this->canvas, KeyPressMask, FALSE, 
					  (XtEventHandler) pressKeyEH, this); 
	
	if (saveshape >= 0)
		setCursor(saveshape);
	
	calcFontDims(this->canvas, &this->char_width, &this->char_height, &this->ascent);
}


	
void
ViewCanvas::activateGrab()
{
#if 0
	XGrabButton(XtDisplay(this->canvas), Button2, AnyModifier, 
				XtWindow(this->canvas), TRUE, 
				ButtonPressMask | ButtonMotionMask | 
				ButtonReleaseMask,
				GrabModeAsync, GrabModeAsync,
				XtWindow(this->canvas), 
				XCreateFontCursor(XtDisplay(this->canvas),
								  XC_crosshair));
	
	XGrabButton(XtDisplay(this->canvas), Button3, AnyModifier, 
				XtWindow(this->canvas), TRUE, 
				ButtonPressMask | ButtonMotionMask | 
				ButtonReleaseMask,
				GrabModeAsync, GrabModeAsync,
				XtWindow(this->canvas), 
				XCreateFontCursor(XtDisplay(this->canvas),
								  XC_fleur));
#endif
}

void
ViewCanvas::deactivateGrab()
{
#if 0
	XUngrabButton(XtDisplay(this->canvas), Button2, AnyModifier, 
				  XtWindow(this->canvas));
	XUngrabButton(XtDisplay(this->canvas), Button3, AnyModifier, 
				  XtWindow(this->canvas));
#endif
}


ViewCanvas::~ViewCanvas()
{
	this->blank_gc ? XFreeGC(XtDisplay(this->canvas), this->blank_gc) : 0;
	this->blue_gc ? XFreeGC(XtDisplay(this->canvas), this->blue_gc) : 0;
	this->inverse_blue_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->inverse_blue_gc) : 0;
	this->thick_blue_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->thick_blue_gc) : 0;
	this->grot_gc ? XFreeGC(XtDisplay(this->canvas), this->grot_gc) : 0;
	this->inverse_grot_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->inverse_grot_gc) : 0;
	this->thick_grot_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->thick_grot_gc) : 0;
	this->black_gc ? XFreeGC(XtDisplay(this->canvas), this->black_gc) : 0;
	this->inverse_black_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->inverse_black_gc) : 0;
	this->flag_gc ? XFreeGC(XtDisplay(this->canvas), this->flag_gc) : 0;
	this->inverse_flag_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->inverse_flag_gc) : 0;
	this->red_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->red_gc) : 0;
	this->green_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->green_gc) : 0;
	this->gray_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->gray_gc) : 0;
	this->dark_gray_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->dark_gray_gc) : 0;
	this->xor_gc ? 
		XFreeGC(XtDisplay(this->canvas), this->xor_gc) : 0;
	
	if(this->copy_gc)
		XtReleaseGC(this->canvas, this->copy_gc);

	if(this->inverse_gc_ptr)
		XtReleaseGC(this->canvas, this->inverse_gc_ptr);

	if(this->gc_ptr) 
		XtReleaseGC(this->canvas, this->gc_ptr);

	if(this->thick_gc_ptr)
		XtReleaseGC(this->canvas, this->thick_gc_ptr);
	
	if(this->medium_gc_ptr)
		XtReleaseGC(this->canvas, this->medium_gc_ptr);
	
	XtDestroyWidget(this->canvas);
}
void
ViewCanvas::setFont(XFontStruct *font)
{
	this->font_struct = font;
}
void
ViewCanvas::releaseFont()
{
	this->font_struct = 0;
}

void
ViewCanvas::initializeScrollBarValues()
{
	Widget hsb,vsb,sw;
	XArg args;
	
	sw = XtParent(this->canvas);
	args.reset();
	args.setArg(XmNhorizontalScrollBar,&hsb);
	args.setArg(XmNverticalScrollBar,&vsb);
	args.getValues(sw);
	args.reset();
	args.setArg(XmNvalue,0);
	args.setValues(hsb);
	args.reset();
	args.setArg(XmNvalue,0);
	args.setValues(vsb);

}

// bogus -- check the definition of syncSize
void
ViewCanvas::setSize(int w, int h)
{
	iBox new_port={0,0,0,0};
	if(!this->client)
		return;
	iBox view_port = this->client->getViewPort();
	
	// only set new size if a change has been made
	if(view_port.w != w || view_port.h != h){
		new_port.w = w;
		new_port.h = h;
				
		this->client->setViewPort(new_port);

	}
}

int 
ViewCanvas::managedP() 
{ 
	return XtIsManaged(this->canvas); 
}

void
ViewCanvas::manage()
{
	Widget swindow;
	XArg args;
	Widget hb, vb;
	
	XtManageChild(this->canvas);
	
	swindow = XtParent(this->canvas);
	
	args.setArg(XmNhorizontalScrollBar,&hb);
	args.setArg(XmNverticalScrollBar,&vb);
	args.getValues(swindow);


	XmScrolledWindowSetAreas(swindow, hb, vb, this->canvas);
	this->activateGrab();
	//this->syncSize();
}

void
ViewCanvas::unmanage()
{
	this->deactivateGrab();
	XtUnmanageChild(this->canvas);
}

GC
ViewCanvas::blankGC()
{
	if(this->blank_gc == 0) {
		XGCValues values;
		int mask;
		values.foreground = getColorByName(this->canvas, "white");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->blank_gc = XCreateGC(XtDisplay(this->canvas), 
								   XtWindow(this->canvas),
								   mask,
								   &values);
	}
	
	return this->blank_gc;
}


GC
ViewCanvas::blueGC()
{
	if(this->blue_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "blue");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->blue_gc = XCreateGC(XtDisplay(this->canvas), 
								  XtWindow(this->canvas),
								  mask,
								  &values);
	}
	
	return this->blue_gc;
}

GC
ViewCanvas::inverseBlueGC()
{
	if(this->inverse_blue_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "white");
		values.background = getColorByName(this->canvas, "blue");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->inverse_blue_gc = XCreateGC(XtDisplay(this->canvas), 
										  XtWindow(this->canvas),
										  mask,
										  &values);
	}
	
	return this->inverse_blue_gc;
}


GC
ViewCanvas::thickBlueGC()
{
	if(this->thick_blue_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "blue");
		values.background = getColorByName(this->canvas, "white");
		values.line_width = 3;
		mask =  GCForeground | GCBackground | GCLineWidth;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->thick_blue_gc = 
			XCreateGC(XtDisplay(this->canvas), XtWindow(this->canvas),
					  mask,
					  &values);
	}
	
	return this->thick_blue_gc;
}

GC
ViewCanvas::grotGC()
{
	if(this->grot_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "RosyBrown");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->grot_gc = XCreateGC(XtDisplay(this->canvas), 
								  XtWindow(this->canvas),
								  mask,
								  &values);
	}
	
	return this->grot_gc;
}

GC
ViewCanvas::inverseGrotGC()
{
	if(this->inverse_grot_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "white");
		values.background = getColorByName(this->canvas, "RosyBrown");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->inverse_grot_gc = XCreateGC(XtDisplay(this->canvas), 
										  XtWindow(this->canvas),
										  mask,
										  &values);
	}
	
	return this->inverse_grot_gc;
}


GC
ViewCanvas::thickGrotGC()
{
	if(this->thick_grot_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "RosyBrown");
		values.background = getColorByName(this->canvas, "white");
		values.line_width = 3;
		mask =  GCForeground | GCBackground | GCLineWidth;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->thick_grot_gc = 
			XCreateGC(XtDisplay(this->canvas), XtWindow(this->canvas),
					  mask,
					  &values);
	}
	
	return this->thick_grot_gc;
}


GC
ViewCanvas::blackGC()
{
	if(this->black_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "black");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		
		this->black_gc = XCreateGC(XtDisplay(this->canvas), XtWindow(this->canvas),
								   mask,
								   &values);
	}
	
	return this->black_gc;
}


GC
ViewCanvas::inverseBlackGC()
{
	if(this->inverse_black_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "white");
		values.background = getColorByName(this->canvas, "black");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->inverse_black_gc = XCreateGC(XtDisplay(this->canvas), 
										   XtWindow(this->canvas),
										   mask,
										   &values);
	}
	
	return this->inverse_black_gc;
}
GC
ViewCanvas::FlagGC()
{
	if(this->flag_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "white");
		values.background = getColorByName(this->canvas, "DodgerBlue");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->flag_gc = XCreateGC(XtDisplay(this->canvas), 
										   XtWindow(this->canvas),
										   mask,
										   &values);
	}
	
	return this->flag_gc;
}
GC
ViewCanvas::inverseFlagGC()
{
	if(this->inverse_flag_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "white");
		values.background = getColorByName(this->canvas, "blue");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->inverse_flag_gc = XCreateGC(XtDisplay(this->canvas), 
										   XtWindow(this->canvas),
										   mask,
										   &values);
	}
	
	return this->inverse_flag_gc;
}


GC
ViewCanvas::redGC()
{
	if(this->red_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "red");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->red_gc = XCreateGC(XtDisplay(this->canvas), 
								 XtWindow(this->canvas),
								 mask,
								 &values);
	}
	
	return this->red_gc;
}

GC
ViewCanvas::thickRedGC()
{
	if(this->thick_red_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "red");
		values.background = getColorByName(this->canvas, "white");
		values.line_width = 3;
		mask =  GCForeground | GCBackground | GCLineWidth;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->thick_red_gc = 
			XCreateGC(XtDisplay(this->canvas), XtWindow(this->canvas),
					  mask,
					  &values);
	}
	
	return this->thick_red_gc;
}


GC
ViewCanvas::greenGC()
{
	if(this->green_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "maroon");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->green_gc = XCreateGC(XtDisplay(this->canvas), 
								   XtWindow(this->canvas),
								   mask,
								   &values);
	}
	
	return this->green_gc;
}

GC
ViewCanvas::grayGC()
{
	if(this->gray_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, GRAY_COLOR);
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->gray_gc = XCreateGC(XtDisplay(this->canvas), 
								  XtWindow(this->canvas),
								  mask,
								  &values);
	}
	
	return this->gray_gc;
}

GC
ViewCanvas::darkgrayGC()
{
	if(this->dark_gray_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "gray40");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->dark_gray_gc = XCreateGC(XtDisplay(this->canvas), 
								  XtWindow(this->canvas),
								  mask,
								  &values);
	}
	
	return this->dark_gray_gc;
}
GC
ViewCanvas::xorGC()
{
	if(this->xor_gc == 0) {
		XGCValues values;
		int mask;
		
		values.function = GXinvert;
		mask = GCFunction;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->xor_gc = XCreateGC(XtDisplay(this->canvas), 
								 XtWindow(this->canvas),
								 mask,
								 &values);
	}
	
	return this->xor_gc;
}



GC
ViewCanvas::copyGC()
{
	if(this->copy_gc == 0) {
		XGCValues values;
		int mask;
		
		values.foreground = getColorByName(this->canvas, "black");
		values.background = getColorByName(this->canvas, "white");
		mask = GCForeground | GCBackground;
		if(this->font_struct) {
			values.font = this->font_struct->fid;
			mask = mask | GCFont;
		}
		this->copy_gc = XtGetGC(this->canvas,
								mask,
								&values);
	}
	
	return this->copy_gc;
}

GContext ViewCanvas::getFont()
{
	return XGContextFromGC(this->copyGC());
}


#if 0
void enterCallback(Widget w,
				   ViewCanvas *vx,
				   XEvent *event)
{
	vc->activateGrab();
}

void leaveCallback(Widget w,
				   ViewCanvas *vx,
				   XEvent *event)
{
	vc->deactivateGrab();
}
#endif

void
ViewCanvas::setClient(Page *input_client)
{
	this->client = input_client;
}

void
ViewCanvas::setCursor(int shape)
{
	if (this->canvas) {
		Window window = XtWindow(this->canvas);
		Display *display = XtDisplay(this->canvas);
		Cursor cursor;
		
		
		cursor =  XCreateFontCursor(display, shape);
		
		if(window)
			XDefineCursor(display, window, cursor);
	}
	saveshape = shape;
}

void
ViewCanvas::unsetCursor()
{
	if (this->canvas) {
		Window window = XtWindow(this->canvas);
		Display *display = XtDisplay(this->canvas);
		
		XUndefineCursor(display, window);
	}
	saveshape = -1;
}


void
ViewCanvas::measureText(const iUCode *s, int l, int *tw, int *th, int *ascent)
{
	measureUnicode(XtDisplay(this->canvas),s,l,tw,th,ascent);
}






GC
ViewCanvas::colorGC(const char *color)
{
	XGCValues values;
	int mask;

	values.foreground = getColorByName(this->canvas, color);
	values.background = getColorByName(this->canvas, "white");
	mask = GCForeground | GCBackground;
	if(this->font_struct) {
		values.font = this->font_struct->fid;
		mask = mask | GCFont;
	}

	if(this->gc_ptr == 0) {
		this->gc_ptr = XCreateGC(XtDisplay(this->canvas), 
								 XtWindow(this->canvas),
								 mask,
								 &values);
	} else {
		XChangeGC(XtDisplay(this->canvas),
				  this->gc_ptr,
				  mask,
				  &values);
	}
	
	return this->gc_ptr;
}

GC
ViewCanvas::inverseGC(const char *color)
{
	XGCValues values;
	int mask;

	values.foreground = getColorByName(this->canvas, "white");
	values.background = getColorByName(this->canvas, color);
	mask = GCForeground | GCBackground;
	if(this->font_struct) {
		values.font = this->font_struct->fid;
		mask = mask | GCFont;
	}

	if(this->inverse_gc_ptr == 0) {
		this->inverse_gc_ptr = XCreateGC(XtDisplay(this->canvas), 
										 XtWindow(this->canvas),
										 mask,
										 &values);
	} else {
		XChangeGC(XtDisplay(this->canvas),
				  this->inverse_gc_ptr,
				  mask,
				  &values);
	}
	
	return this->inverse_gc_ptr;
}


GC
ViewCanvas::thickGC(const char *color)
{
	XGCValues values;
	int mask;
		
	values.foreground = getColorByName(this->canvas, color);
	values.background = getColorByName(this->canvas, "white");
	values.line_width = 3;
	mask =  GCForeground | GCBackground | GCLineWidth;
	if(this->font_struct) {
		values.font = this->font_struct->fid;
		mask = mask | GCFont;
	}

	if(this->thick_gc_ptr == 0) {
		this->thick_gc_ptr = 
			XCreateGC(XtDisplay(this->canvas), XtWindow(this->canvas),
					  mask,
					  &values);
	} else {
		XChangeGC(XtDisplay(this->canvas),
				  this->thick_gc_ptr,
				  mask,
				  &values);
	}
	
	return this->thick_gc_ptr;
}
GC
ViewCanvas::mediumGC(const char *color)
{
	XGCValues values;
	int mask;
		
	values.foreground = getColorByName(this->canvas, color);
	values.background = getColorByName(this->canvas, "white");
	values.line_width = 2;
	mask =  GCForeground | GCBackground | GCLineWidth;
	if(this->font_struct) {
		values.font = this->font_struct->fid;
		mask = mask | GCFont;
	}

	if(this->medium_gc_ptr == 0) {
		this->medium_gc_ptr = 
			XCreateGC(XtDisplay(this->canvas), XtWindow(this->canvas),
					  mask,
					  &values);
	} else {
		XChangeGC(XtDisplay(this->canvas),
				  this->medium_gc_ptr,
				  mask,
				  &values);
	}
	
	return this->medium_gc_ptr;
}

