
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "dafs.h"
#include "scroll.h"
#include "misc/XArg.h"

#ifndef MAX
#define MAX(x,y) (((x) > (y)) ? (x) : (y))
#endif
#ifndef MIN
#define MIN(x,y) (((x) < (y)) ? (x) : (y))
#endif

/*
// This has functions for figuring out and setting the visible
// area of a scrolled window
// these are not illum specific and should be move to
// misc
*/


void
getScrollPositionCenter(Widget sw, int4 *x_ptr, int4 *y_ptr)
{
	// bogus double check identity of scrolled window
	XArg args;
	Widget work_window=0;
	Widget hsb;
	Widget vsb;
	Dimension height;
	Dimension width;
	int minimum;
	int value;
	int maximum;
	int slider_size;
	int4 x, y;
	int wi,he;
	
	args.reset();
	args.setArg(XmNworkWindow, &work_window);
	args.setArg(XmNhorizontalScrollBar, &hsb);
	args.setArg(XmNverticalScrollBar, &vsb);
	args.getValues(sw);
	
	if(work_window) {
		
		//try widgetsize  MO
		getWidgetSize(work_window,&wi,&he);
		height = (Dimension)he;
		width = (Dimension)wi;

		args.reset();
		args.setArg(XmNminimum, &minimum);
		args.setArg(XmNmaximum, &maximum);
		args.setArg(XmNvalue, &value);
		args.setArg(XmNsliderSize, &slider_size);
		args.getValues(vsb);
		// bogus check if hsb is managed

		Position y_prime;
		if((maximum - slider_size - minimum) != 0) {
			y = value + height/ 2;
		} else 
			y = height/2;

		args.reset();
		args.setArg(XmNminimum, &minimum);
		args.setArg(XmNmaximum, &maximum);
		args.setArg(XmNvalue, &value);
		args.setArg(XmNsliderSize, &slider_size);
		args.getValues(hsb);
		
		Position x_prime;
		if((maximum - slider_size - minimum) != 0) {
			x = value + width/ 2;
		} else {
			x = width / 2;
		}
		
	} else {
		x = 0; y = 0;
	}
	
	*x_ptr = x;
	*y_ptr = y;
}
void
getScrollBarValues(Widget sw, int4 *x_ptr, int4 *y_ptr)
{
	XArg args;
	Widget hsb;
	Widget vsb;
	int h_value,v_value;
	
	args.reset();
	args.setArg(XmNhorizontalScrollBar, &hsb);
	args.setArg(XmNverticalScrollBar, &vsb);
	args.getValues(sw);
	
		

	args.reset();
	args.setArg(XmNvalue, &h_value);
	args.getValues(hsb);

	args.reset();
	args.setArg(XmNvalue, &v_value);
	args.getValues(vsb);
		
	
	*x_ptr = h_value;
	*y_ptr = v_value;
}

iLBox 
visibleRectangle(Widget sw)
{
	int wi, he;
	iLBox window_region;
	Widget ww,vsb,hsb;
	XArg args;


	int4 s_x;
	int4 s_y;
	args.reset();
	args.setArg(XmNworkWindow, &ww);
	args.getValues(sw);
	getWidgetSize(ww, &wi, &he);
	
	getScrollPositionCenter(sw, &s_x, &s_y);
	
	window_region.x = s_x - wi/2;
	window_region.y = s_y - he/2;
	window_region.x = MAX(window_region.x, 0);
	window_region.y = MAX(window_region.y, 0);
	window_region.w = wi;
	window_region.h = he;
	
	return window_region;
}
/*
// I wrote a new version that shares more code 
*/

int
visiblePositionP(Widget sw, int4 input_x, int4 input_y)
{
	XArg args;
	Widget work_window;
	
	args.reset();
	args.setArg(XmNworkWindow, &work_window);
	args.getValues(sw);
	
	iLBox vr = visibleRectangle(sw);
	
	int4 ulx = vr.x;
	int4 uly = vr.y;
	int4 lrx = vr.x + vr.w;
	int4 lry = vr.y + vr.h;
	
	return ( 
		input_x >= ulx && 
		input_x <= lrx && 
		input_y >= uly && 
		input_y <= lry);
}



void
getWidgetSize(Widget wt, int *w, int *h)
{
	Dimension width;
	Dimension height;
	XArg args;
	Widget hsb,vsb;
	Dimension wi,he;
	
	args.setArg(XmNwidth, &width);
	args.setArg(XmNheight, &height);
	XtGetValues(wt, args.getArgs(), args.getSize());

	*w = width;   
	*h = height;
}
void
getViewableSize(Widget ww, int *w, int *h)
{
	int width;
	int height;
	XArg args;
	Widget hsb,vsb;
	Dimension wi,he;
	
	args.reset();
	args.setArg(XmNhorizontalScrollBar, &hsb);
	args.setArg(XmNverticalScrollBar, &vsb);
	args.getValues(XtParent(ww));
	
	XtVaGetValues(hsb,
			XmNheight,&he,
			NULL);
	XtVaGetValues(vsb,
			XmNwidth,&wi,
			NULL);

	getWidgetSize(ww,&width,&height);

	width -= wi;
	height -= he;
	*w = width;   
	*h = height;
}

void
getScrollPositionUL(Widget sw, Position *tlx, Position *tly)
{
	int wi, he;
	iBox window_region;
	Widget ww;
	XArg args;

	args.reset();
	args.setArg(XmNworkWindow, &ww);
	args.getValues(sw);
	
	int4 s_x;
	int4 s_y;
	getWidgetSize(ww, &wi, &he);
	getScrollPositionCenter(sw, &s_x, &s_y);
	
	*tlx = (Position)MAX(s_x - wi/2,0);
	*tly = (Position)MAX(s_y - he/2,0);
	
	return;
}	
