
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#ifndef _KeyBoard_h_ 
#define _KeyBoard_h_

typedef unsigned short iUCode;

/*
// this is a keyboard display class
// it creates an independent top level window
// in which one can 
// display the key map
//
// It has a map from KeySym's to buttons
//
*/

class KeyMap;

#define SCALE_FACTOR 5
#define KEY_H        10
#define NORMAL_KEY_W 10
#define ROW_W        150

#include <X11/X.h>
#include <X11/Intrinsic.h>

//#include "aString.h"
#include "svpmap.h"

struct KeySymPair {
	KeySym lower;
	KeySym upper;
};

typedef struct KeySymPair KeySymPair;

typedef int (*KeyBoardHandler)(void *user_data, KeySym sym, iUCode result);

class KeyBoard {
public:
	KeyMap *key_map;
	KeyBoardHandler handler;
	void *user_data;
	Widget bb;
	Widget sub_bb;
	stringVoidPtrMap button_map;
	
	int shift_on;
	int control_on;
	XmFontList font_list;
public:
	KeyBoard(Widget parent, 
			 KeyMap *key_map,
			 KeyBoardHandler handler, 
			 void *user_data);
	~KeyBoard();
	
	void setKeyMap(KeyMap *input_key_map);
	
	void toggleShift();
	void toggleControl();
	
	Widget findButton(KeySym);
	void setButton(KeySym, Widget);
	
	void initialiseModKeys();
	
	void addKey(Widget parent,
				int x,
				int y,
				int w,
				int h,
				KeySymPair *pair);
	
	void createRow(Widget Parent, int x, int y, KeySymPair *list, int n);
	
	void createKeyBoard(Widget parent);
	
	void setKeyNames();
	
	void show();
	void hide();
	
	void value(Widget w);
};

#endif
