
/*
 * Copyright (c) 1994, 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */


#ifndef _Canvas_h_
#define _Canvas_h_

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>

#include <Xm/DrawingA.h>
#include "dafstype.h" // for iBox

class CanvasClient;
class KeyMap;
#include "ScrollClient.h" // we are subclassed

class Canvas : public ScrollClient {
public:
  XtAppContext app_context;

  int follow_procid;    // bogus -- what is this proc_id for
  int follow;    // refers to dragging out of the window
  // because x events are stopped, we will need
  // to draw as we go
  // eventually, I want to vary the follow increment

  int release_expected; // bogus -- I think this
  // might date back to the time when I intepreted drag
  // events incorrectly

  // delayed drawing workproc
  iLBox redraw_region;
  int redraw_procid;
	
  // for delayed drawing
  void addRedrawRegion(iLBox *b);
  void flushRedrawRegion();
  void drawRegion(iLBox box);

  CanvasClient *client;
  Widget canvas_widget;
	
  void setClient(CanvasClient *client);
  Canvas(ScrollWindow *sw, XtAppContext app_context, Widget parent, 
		 CanvasClient *client, int w, int h);
  virtual ~Canvas();
	

	
  int managedP();
  void manage();
  void unmanage();

  friend void redisplayCB (Widget w,
						   Canvas *vc,
						   XmDrawingAreaCallbackStruct *call_data);
  friend void resizeCB (Widget w,
						Canvas *vc,
						XmDrawingAreaCallbackStruct *call_data);
  friend void pressButtonEH(Widget w,
							Canvas *vc,
							XEvent *event);
  friend void releaseButtonEH(Widget w,
							  Canvas *vc,
							  XEvent *event);
  friend void moveButtonEH(Widget w,
						   Canvas *vc,
						   XEvent *event);
  friend void pressKeyEH(Widget w,
						 Canvas *vc,
						 XEvent *event);

  virtual KeyMap *getKM() = 0;
  
  iLBox getVisibleArea(void); // for now returns widget area
  // later on returns area visible due to scrolling
  /*virtual*/ void setSize(int width, int height);
};


#endif
