
/*
 * Copyright (c) 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */


#ifndef _rafio_h_
#define _rafio_h_


/*
// This module allows the developer to override the standard file io
// and substitute his own. The format of the functions is based on the
// interface to the stream (fopen, etc) calls, but we already have
// interfaces to the file no calls (open, etc)
//
// So by default, you get the stream library
*/


#include "rafport.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef void *RafFile; /* usually a FILE * */
#define REOF -1

/* equiv of fread */
Dll int4 APIENTRY riRead(char *ptr, int4 size, int4 nitems, RafFile);  
typedef int4 (*ReadFunc)(char *, int4, int4, RafFile);

/* equiv of fwrite */
Dll int4 APIENTRY riWrite(const char *ptr, int4 size, int4 nitems, RafFile); 
typedef int4 (*WriteFunc)(const char *, int4, int4, RafFile);


/* equiv of fopen */
Dll RafFile APIENTRY riOpen(const char *filename, const char *type);        
typedef RafFile (*OpenFunc)(const char *, const char *);

/* equiv of fclose */
Dll int4 APIENTRY riClose(RafFile);                              
typedef int4 (*CloseFunc)(RafFile);


/* fseek  */
Dll int4 APIENTRY riSeek(RafFile handle, long offset, int4 whence); 
typedef int4 (*SeekFunc) (RafFile, long, int4);

/* bogus want to fold this into seek */ /* ftell  */
Dll long APIENTRY riTell(RafFile handle); 
typedef long (*TellFunc)(RafFile);


/* want to kill all these */

/* fgetc */
Dll int4 APIENTRY riGetc(RafFile handle);
typedef int4 (*GetcFunc) (RafFile);

/* fgets */
Dll char * APIENTRY riGets(char *s, int4 size, RafFile handle); 
typedef char *(*GetsFunc) (char *, int4, RafFile);

/* fputs */
Dll int4 APIENTRY riPuts(char *s, RafFile handle); 
typedef int4 (*PutsFunc) (char *, RafFile);

/* feof */
Dll int4 APIENTRY riEof(RafFile handle); 
typedef int4 (*EofFunc)(RafFile);

struct IoFunctionTable {
	ReadFunc read;
	WriteFunc write;
	OpenFunc open;
	CloseFunc close;
	SeekFunc seek;
	TellFunc tell;
	GetcFunc getcf;
	GetsFunc getsclone;
	PutsFunc putsf;
	EofFunc eof;
};

typedef struct IoFunctionTable IoFunctionTable;

Dll void APIENTRY riInit(IoFunctionTable *table);

/* not implemented on purpose */

/* no fprintf */
/* no vfprintf */
/* no fscanf */
/* no fputc */
/* no fflush */

/* no ferror */
/* no freopen */
/* no fdopen */
/* no rewind */
/* no getc -- is a potentially dangerous macro */
/* no clearerr */
/* no fileno */


#ifdef __cplusplus
};
#endif

#endif
