
#ifndef _PackedRectangle_h_
#define _PackedRectangle_h_

#include "Rectangle.h"

class PackedRectangle : public ParentRectangle 
{
	friend class RTECursor;
private:

protected:
	Coord spacing; // e.g. line height for LtoR, TtoB
	Direction primary_writing_direction;
	Direction secondary_writing_direction;
	void positionOnTopOfChild(ChildRectangle *child, Rectangle *rect);
	void moveForward(Coord *, Coord *, Coord);
	void moveForward(Rectangle *, Coord);
	int needToRepack(Coord dw, Coord dh);
	int containsParents();

public:
	PackedRectangle();
	PackedRectangle(Direction, Direction, Coord); // prim. dir, secondary dir, spacing
	PackedRectangle(Coord, Coord, Coord, Coord, Direction, Direction, Coord); 
	void debugPrint(char *);
	ChildRectangle * firstChild(void);
	ChildRectangle * lastChild(void);
	ChildRectangle * nextChild(ChildRectangle *child, int wrap=TRUE); 
	ChildRectangle * prevChild(ChildRectangle *child, int wrap=TRUE); 
        ChildRectangle * nextLeaf(ChildRectangle *leaf);
	ChildRectangle * prevLeaf(ChildRectangle *leaf);
        ChildRectangle * firstLeafChild(ChildRectangle *child);
	ChildRectangle * lastLeafChild(ChildRectangle *child);
	ChildRectangle * nextBranch(ChildRectangle *child);
	ChildRectangle * prevBranch(ChildRectangle *child);
	void positionBeforeChild(ChildRectangle *child, Rectangle *rect);
	int positionAfterChild(ChildRectangle *child, Rectangle *rect);
	void positionFirst(Rectangle *rect);
	void deleteChild(ChildRectangle *child, int squeeze=UNSPEC, int erase=TRUE);
	void changeChildSize(Coord, Coord, Rectangle *);
	int repackChildren(ElementList *cl=NULL);
	int inOrder(Coord, Coord, Coord, Coord);
	int inOrder(ChildRectangle *a, ChildRectangle *b);
	int insertFirst(ChildRectangle *child);
	int insertLast(ChildRectangle *child);
	int insertBefore(ChildRectangle *child, ChildRectangle *child);
	int insertAfter(ChildRectangle *child, ChildRectangle *child);
	int insertBefore(ChildRectangle *child, ChildRectangle *child, int allow_overlap);
	int insertAfter(ChildRectangle *child, ChildRectangle *child, int allow_overlap);
	int insertAt(Coord, Coord, ChildRectangle *child, int allow_overlap=FALSE);
	int changeSizeTo(Coord, Coord, int redraw=TRUE);
	int shrinkToFitChildren();
	int shrinkChildToFit(ChildRectangle *c);
	int growChildToFill(ChildRectangle *p);
	int minimizeChildSize(ChildRectangle *c);
	int allocateArea(Coord, Coord, Coord, Coord, ChildRectangle *child);
	int allocateArea(ChildRectangle *rect, ChildRectangle *child);
	int freeArea(Rectangle *, ChildRectangle *child, int);
	void trimExtraSize();
	Direction getPrimaryWritingDir() { return primary_writing_direction; };
	Direction getSecondaryWritingDir() { return secondary_writing_direction; };
	Coord getSpacing();
	int isAlignedWithLineSpacing(Coord, Coord, Coord, Coord);
	int isAlignedWithLineSpacing(Rectangle *rect);
	void alignWithLineSpacing(Rectangle *rect);
	void alignWithEdge(Rectangle *, RTECursor *);
	int differentLine(ChildRectangle *child, ChildRectangle *child);
	int moveChildForward(ChildRectangle *child, Coord, Coord*, Coord*);
	int moveChildrenForward(ElementList *, Coord, Coord*, Coord*);
	int moveChildToStartOfNextLine(ChildRectangle *c);
	int positionAtStartOfNextLine(Coord *, Coord *, Coord, Coord);
	int positionAtStartOfNextLine(Rectangle *rect);
	RectangleType getType() { return PackedRectangleT; };
	int isPacked() { return TRUE; }; 
};

#endif
