#ifndef _LIST_
#define _LIST_
#include "RTEMisc.h"

// This class maintains a list of items of class T, sorted
// in the order induced by the compare() function defined
// for two items of class T.  In particular, if compare()
// gives reading order as defined by the common parent,
// the list will be in reading order.

template<class T> class OrderedList {
	T **element_array;
	int size;
  CompareFunction compare;
public:
	OrderedList(CompareFunction compare);
	~OrderedList();
	int getSize();
	int insertItem(T *);
	void removeItem(T *);
	void removeItemNumber(int);
	int copyTo(OrderedList<T> *);
	T *getItemNumber(int);
	T *OrderedList<T>::operator[](int index);
	int locationOfItem(T *);
	int contains(T *);
	int isEmpty(void) { return (size == 0); };
	int numElements(void) { return size; };
};



template<class T> OrderedList<T>::OrderedList(CompareFunction input_compare)
  : size(0),
	element_array(NULL),
	compare(input_compare)
{
}


template<class T> OrderedList<T>::~OrderedList()
{
        if (element_array) { smfree(element_array); element_array = 0; }
}

template<class T> int OrderedList<T>::copyTo(OrderedList<T> * new_list)
{
	int result = FALSE;
	const int bytes = this->size * sizeof(T *);

	if(bytes) {
		if(new_list->element_array)
			new_list->element_array = (T **) smrealloc(new_list->element_array, bytes);
		else
			new_list->element_array = (T **) smalloc(bytes);
		if(new_list->element_array) {
			memcpy(new_list->element_array, this->element_array, bytes);
			new_list->size = this->size;
			result = TRUE;
		}
	} else {
		if(new_list->element_array) {
			smfree(new_list->element_array);
			new_list->element_array = 0;
			new_list->size = 0;
			result = TRUE;
		}
	}
	return result;
}


template<class T> int OrderedList<T>::getSize()
{
        return this->size;
}

template<class T> T *OrderedList<T>::operator[](int index)
{
	if(index < 0 || index > size)
		return NULL;

	return element_array[index];
}

template<class T> int OrderedList<T>::insertItem(T *item)
{
  return arrayInsert((void ***)&element_array, item, &size, this->compare);
}

template<class T> void OrderedList<T>::removeItem(T *item)
{
	removeItemNumber(locationOfItem(item));
}

template<class T> void OrderedList<T>::removeItemNumber(int i)
{
        if(i < 0 || i >= size) halt("index out of range\n");
        listDelete((void ***)&element_array, i, &size);
}


template<class T> T *OrderedList<T>::getItemNumber(int i)
{
        return element_array[i];
}


template<class T> int OrderedList<T>::locationOfItem(T *item)
{
  int found;
  int loc = bSearch((void **)this->element_array, item, this->size, &found, this->compare );
	assert(loc >= 0 && loc < this->size);
	if(found) {
		// skip indistinguishable elements
		while(loc < this->size && this->element_array[loc] != item &&
			  compare(element_array[loc], item) != GREATER_THAN)
			++loc;
		found = (loc < this->size && this->element_array[loc] == item);
	}
	assert(!found || this->element_array[loc] == item);
        if(!found) halt("item not in list\n");
        return loc;
}

template<class T> int OrderedList<T>::contains(T *item)
{
	int found;
	int loc = bSearch((void **)this->element_array, item, this->size, &found, this->compare );	

	if(found) {
		// skip indistinguishable elements
		while(loc < this->size && this->element_array[loc] != item &&
			  compare(element_array[loc], item) != GREATER_THAN)
			++loc;
		found = (loc < this->size && this->element_array[loc] == item);
	}
	
	return found;
}	

// give names to commonly used list classes
typedef OrderedList<ChildRectangle> ElementList;

#endif
