#ifndef _RECTANGLE_
#define _RECTANGLE_
#include "VDD.h"
#include "RTEMisc.h"
#include "RTEView.h"
#include "RTESet.h"
#include "RTEList.h"

// To be completed - these apply to border of rectangle
#define DEFAULT_RECTANGLE_COLOR 0
#define DEFAULT_RECTANGLE_LINE_STYLE 0
#define DEFAULT_PRIM_READ_ORDER LeftToRight
#define DEFAULT_SCND_READ_ORDER TopToBottom
#define DEFAULT_LINE_SPACING 17

// forward declarations of class names
class ParentRectangle;
class RootElement;
class RTECursor;
class PackedRectangle;

class Rectangle {
protected:
	friend class ConstrainedRectangle;
	friend class ParentRectangle; // MAV 990113 -- don't understand why this is necessary, since we are derived
	friend class PackedRectangle; // MAV 990113 -- don't understand why this is necessary, since we are derived
	Coord x, y;	  // position of upper left corner relative to coord system
	Coord w, h;	  // width and height of rectangle
	ColorType color;  // for drawing border
	LineStyle line_style;
public:
	Rectangle();
	Rectangle(Coord, Coord);  // position
	Rectangle(Coord, Coord, Coord, Coord);  // position, w, and h
	Rectangle(SimpleInterval *, SimpleInterval *); // projections on x and y axes resp.
	virtual int moveBy(Coord dx, Coord dy, int squeeze=FALSE, int allow_overlap=FALSE);
	virtual int moveTo(Coord x, Coord y, int squeeze=FALSE, int allow_overlap=FALSE);
	void setX(Coord px) { this->x = px; }; 
	void setY(Coord py) { this->y = py; };
	void setW(Coord pw) { this->w = pw; };
	void setH(Coord ph) { this->h = ph; };
	virtual int changeSizeTo(Coord w, Coord h, int redraw=TRUE);

	void setColor(ColorType);
	void setLineStyle(LineStyle);

	Coord getX() { return x; };          
	Coord getY() { return y; };
	Coord getW() { return w; };
	Coord getH() { return h; };

	virtual Rectangle * intersection(Rectangle *rect);
	virtual Rectangle * areaUnion(Rectangle *rect);
	virtual Rectangle * areaUnion(ElementList *rl);
	int removeIntersection(Rectangle *rect, RectangleSet **rp);

	virtual ElementSet * getChildrenAtLocation(Coord, Coord) {return 0;};
	virtual ElementSet * getChildrenInRectangle(Rectangle *rect) {return 0;};
	virtual ElementList *getListOfChildrenInRectangle(Rectangle *r) { return 0; };
	int area() { return this->w * this->h; };
	int isEqualTo(Rectangle *);
	int contains(Coord, Coord);
	virtual int contains(Rectangle *rect);
	int overlaps(Rectangle *rect);
	virtual RectangleType getType() { return RectangleT; };
	virtual int isPacked() { return FALSE; };
};

class ChildRectangle 
	: public Rectangle
{
	friend class ParentRectangle;
	friend class PackedRectangle;
private:
        int selected;
protected:
	int position_valid;
	ParentRectangle *parent;
	virtual void getLeafElementsAtLocation(Coord, Coord, ElementSet *);
	virtual void getLeafElementsInRectangle(Rectangle *, ElementSet *);
	virtual void getRootOffset(Coord *pdx, Coord *pdy);
public:
	virtual Direction getPrimaryWritingDir() { return RTENone; }
	virtual Direction getSecondaryWritingDir() { return RTENone; };
	ChildRectangle();
	ChildRectangle(Coord, Coord);  // position
	ChildRectangle(Coord, Coord, Coord, Coord);  // position, w, and h
	ChildRectangle(ParentRectangle*, Coord, Coord, Coord, Coord);  // position, w, and h
	void getAbsoluteCoords(Coord *x, Coord *y);
	void getRelativeCoords(Coord ax, Coord ay, Coord *rx, Coord *ry);
	void safeDelete(int squeeze=FALSE);
	void undelete();
	virtual int positionIsValid();
	virtual ElementList *getAllChildren() { return NULL; };
	virtual ElementList *getAllValidChildren() { return NULL; };
	void setPositionValid() { position_valid = TRUE; };
	void setPositionNotValid() { position_valid = FALSE; };
	int moveBy(Coord dx, Coord dy, int squeeze=FALSE, int allow_overlap=FALSE);
	int moveTo(Coord x, Coord y, int squeeze=FALSE, int allow_overlap=FALSE);
	int changeSizeTo(Coord, Coord, int redraw=TRUE);
	int changeSizeBy(Coord, Coord, int redraw=TRUE);
	virtual int becomeChildOf(ParentRectangle *parent, int redraw=TRUE, int allow_overlap=FALSE);
	int isParentOf(ChildRectangle *child);
	ParentRectangle *getParent() { return parent; };
	virtual RootElement *getRoot();
	virtual void trimExtraSize() { };
	virtual int numChildren(void) { return 0; };
	virtual int allowOverlap(void) { return FALSE; };
	virtual int maximizeWidth(void) { return FALSE; };
	RectangleType getType() { return ChildRectangleT; };

	virtual int draw(void) { return 0; };
	virtual int drawRegion(Rectangle *rect) { return 0; };
	virtual int drawBorder(void);
	virtual int erase(void);
	virtual void requestRedraw(void) { if(this->positionIsValid()) draw(); };  
	virtual void requestRedrawRegion(Rectangle *r) { if(this->positionIsValid()) drawRegion(r); };
	virtual void requestRedrawBorder(void) { if(this->positionIsValid()) drawBorder(); };
	virtual void requestErase(void) { if(this->positionIsValid()) erase(); };

	void select(void) { if(!selected){ selected = TRUE; this->requestRedraw(); } };
	void unselect(void) { if(selected){ selected = FALSE; this->requestErase(); this->requestRedraw(); } };
	int is_selected(void) { return selected; };
};


class ParentRectangle : public ChildRectangle
{
protected:
	IntervalList * h_interval_list;
	IntervalList * v_interval_list;
	int registerAreaAllocation(Rectangle *rect, ChildRectangle *child);
	void getLeafElementsAtLocation(Coord x, Coord y, ElementSet *);
	void getLeafElementsInRectangle(Rectangle *, ElementSet *);
protected:
	ElementList *element_list; // sorted by packing order
public:
	ParentRectangle();
	ParentRectangle(Coord, Coord);  // position
	ParentRectangle(Coord, Coord, Coord, Coord);  // position, w, and h
	virtual ~ParentRectangle();
	virtual int registerChild(ChildRectangle *child);
	virtual int registerChild(ChildRectangle *child, int); // allow overlap
	virtual void deleteChild(ChildRectangle *child, int squeeze=UNSPEC, int erase=TRUE);
	ChildRectangle * getChildNumber(int i) { return this->element_list->getItemNumber(i); };
	void deleteAllChildren();
	void deleteAllValidChildren();
	virtual void positionOnTopOfChild(ChildRectangle *child, Rectangle *rect);
	int changeSizeTo(Coord, Coord, int redraw=TRUE);
	virtual int growToContain(Rectangle *rect, int shift_children=TRUE) { return this->contains(rect); };
	virtual int canGrowToContain(Rectangle *rect) { return this->contains(rect); };
	int numChildren() { return this->element_list->getSize(); };
	int isParentOf(ChildRectangle *child);
	int contains(Rectangle *rect);
	Rectangle * areaUnion(Rectangle *r);
	Rectangle * intersection(Rectangle *rect);
	ElementList * getAllChildren();
	ElementList * getAllValidChildren();
	ElementSet * getChildrenAtLocation(Coord, Coord);
	ElementSet * getChildrenInRectangle(Rectangle *rect);
	ElementList * getListOfChildrenInRectangle(Rectangle *r);
	virtual void changeChildSize(Coord, Coord, Rectangle *);
	virtual int inOrder(ChildRectangle *a, ChildRectangle *b); // sorts by pointer value unless overridden

	virtual int positionAtStartOfNextLine(Rectangle *rect) { return FALSE; };
	ElementSet *getLeafElementsAtLocation(Coord x, Coord y);
	ElementSet *getLeafElementsInRectangle(Rectangle *);
	virtual int allocateArea(Coord, Coord, Coord, Coord, ChildRectangle *child);
	virtual int allocateArea(ChildRectangle *rect, ChildRectangle *child);
	virtual int allocateArea(ChildRectangle *rect, ChildRectangle *child, ElementSet **);
	virtual int allocateArea(ChildRectangle *rect, ChildRectangle *child, ElementList **);
	virtual int freeArea(Rectangle *rect, ChildRectangle *child);
	virtual int freeArea(RectangleSet *, ChildRectangle *child);
	virtual int freeArea(ChildRectangle *r);
	int draw();  // draw all children
	int erase();
	int drawRegion(Rectangle *rect);  // draw children in rectangle
	RectangleType getType() { return ParentRectangleT; };
};
	

#endif





