
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
#include "rafport.h"
#include "str-lib.h"

#include "svpmap.h"
#include "dllist.h"

#include "KeyMap.h"

extern KeyMap *initAmerican();
extern KeyMap *initRussian();
extern KeyMap *initEnglish();
extern KeyMap *initGerman();
extern KeyMap *initFrench();
extern KeyMap *initGreek();
extern KeyMap *initMicr();
extern KeyMap *initSwedish();
extern KeyMap *initSpanish();
extern KeyMap *initItalian();
extern KeyMap *initNorw();

static stringVoidPtrMap *map = 0;

static void
kmInit()
{
	KeyMap *american;
	KeyMap *russian;
	KeyMap *english;
	KeyMap *german;
	KeyMap *french;
	KeyMap *greek;
	KeyMap *micr;
	KeyMap *swedish;
	KeyMap *spanish;
	KeyMap *italian;
	KeyMap *norw;
	
	if(map == 0) {
		map = svmapAlloc();
		svmapInit(map, 0, 100);
		american = initAmerican();
		russian = initRussian();
		english = initEnglish();
		german = initGerman();
		french = initFrench();
		greek = initGreek();
		micr = initMicr();
		swedish = initSwedish();
		spanish = initSpanish();
		italian = initItalian();
		norw = initNorw();
		 
		svmapSet(map, "American", american);
		svmapSet(map, "Russian", russian);
		svmapSet(map, "English", english);
		svmapSet(map, "German", german);
		svmapSet(map, "French", french);
		svmapSet(map, "Greek", greek);
		svmapSet(map, "Micr", micr);
		svmapSet(map, "Swedish", swedish);
		svmapSet(map, "Italian", italian);
		svmapSet(map, "Norwegian", norw);
		svmapSet(map, "Spanish", spanish);
	}
}

void
kmGetMapList(DLList *list)
{
	Ref r;
	const char *key;
	
	kmInit();
	
	for(r = svmapFirst(map); r; r = svmapNext(map, r)){
		key = svmapKey(map, r);
		dllistAppend(list, key);
	}
}

KeyMap *
kmLookup(aString map_name)
{
	kmInit();
	
	if(svmapContainsP(map, map_name.string()))
		return (KeyMap *) svmapGet(map, map_name.string());
	return 0;
}
