
/*
 * Copyright (c) 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "rafutil.h"

#include "UniEditor.h"


ueSelectionType 
UniEditor::getSelectionType(void)
{
	return this->selection_type;
}

void
UniEditor::toggleCursorMode(void)
{
  switch(cursor_mode) {
  case cmBEFORE:
    cursor_mode = cmAFTER;
    break;
  case cmAFTER:
    cursor_mode = cmBEFORE;
    break;
  default:
    cursor_mode = cmBEFORE;
  }

  this->drawRange(begin_line, begin_line);
}

int 
UniEditor::getCursor(int *line, int *column, cMode *mode)
{
	if(this->getSelectionType() == stCURSOR) {
		*line = this->begin_line;
		*column = this->begin_col;
		if(mode)
		  *mode = cursor_mode;

		return TRUE;
	} else 
		return FALSE;
}

int 
UniEditor::getSelection(int *b_line, int *b_col, 
						int *e_line, int *e_col)
{
	if(this->getSelectionType() == stSELECTION) {
		*b_line = this->begin_line;
		*b_col = this->begin_col;
		*e_line = this->end_line;
		*e_col = this->end_col;
		return TRUE;
	} else 
		return FALSE;
}


void
UniEditor::scrollIfNecessary()
{
  Widget canvas_widget = this->getWidget();
  if(XtIsRealized(canvas_widget) == FALSE)
	return;

	int x = 0;
	int y = 0;
	int ul_x, ul_y;
	int lr_x, lr_y;
	if(this->selection_type == stCURSOR) {
		this->positionToCoord(this->begin_line, this->begin_col, 
							  this->cursor_mode,
							  &x, &y);
		ul_x = x - 5;
		ul_y = y - 5;
		lr_x = x + 5;
		lr_y = this->char_height + y + 5;
		ul_x = MAX(0, ul_x);
		ul_y = MAX(0, ul_y);
		lr_x = MIN(this->canvas_width, lr_x);
		lr_y = MIN(this->canvas_height, lr_y);
		if(this->isVisible(ul_x, ul_y) == FALSE || 
		   this->isVisible(lr_x, lr_y) == FALSE) {
		  // bogus -- center different depending on directions?
		  this->centerOnCoord(ul_x, ul_y);
		}
	}
}


int
UniEditor::setCursor(int line, int column)
{
	int changed;
	int first_line;
	int last_line;

	changed = !(this->selection_type == stCURSOR && 
		    line == this->begin_line && 
		    column == this->begin_col);
	changed = changed || (column == lines.line[line]->context2base(0));


	if(changed) {
		this->selectionDrawRegion(&first_line, &last_line);
	}
		

	this->selection_type = stCURSOR;
	this->begin_line = line;
	this->begin_col = column;


	if(changed) {
		this->drawRange(first_line, last_line);
		if(first_line == -1 || (line < first_line || line > last_line))
			this->drawRange(line, line);
	}

	this->scrollIfNecessary();

	return changed;
}

int
UniEditor::setSelection(int b_line, int b_col, 
						int e_line, int e_col)
{
	int changed;
	int first_line;
	int last_line;

	changed = !(this->selection_type == stSELECTION && 
			   b_line == this->begin_line && 
			   b_col == this->begin_col &&
			   e_line == this->end_line && 
			   e_col == this->end_col);


	if(changed) {
		this->selectionDrawRegion(&first_line, &last_line);
	}


	this->selection_type = stSELECTION;
	this->begin_line = b_line;
	this->begin_col = b_col;
	this->end_line = e_line;
	this->end_col = e_col;

	if(changed) {
		if(first_line == -1) {
			// this is probably not possible since there must
			// have been a cursor selection at least
			this->drawRange(b_line, e_line);
		} else {
			if(first_line == b_line) {
				this->drawRange(b_line, b_line);
				if(last_line == -1)
				  this->drawRange(e_line, e_line);
				else
				  this->drawRange(MIN(e_line, last_line), MAX(e_line, last_line));
			} else if (last_line == e_line) {
				this->drawRange(e_line, e_line);
				this->drawRange(MIN(b_line, first_line), 
								MAX(b_line, first_line));
			} else {
				this->drawRange(b_line, e_line);
				this->drawRange(first_line, last_line);
			}
		}
	}

	this->scrollIfNecessary();
	
	return changed;
}

int
UniEditor::clearSelection(void)
{
	int first_line;
	int last_line;

	int changed = this->selection_type != stNONE;

	if(changed) {
		this->selectionDrawRegion(&first_line, &last_line);
	}

	this->selection_type = stNONE;

	if(changed) {
		this->drawRange(first_line, last_line);
	}

	return changed;
}

void
UniEditor::moveCursor(uDirection direction)
{
  Line *line = lines.line[begin_line];
  int pos = line->base2screen(begin_col);

  if(direction == dRIGHT) {
    int next = line->screen2base(pos+1);
    if(next != begin_col) {
      if(line->direction(begin_col) == dLEFT && 
		 line->direction(next) == dRIGHT && 
		 cursor_mode == cmBEFORE)
		this->toggleCursorMode();
      else if(line->direction(begin_col) == dRIGHT && 
			  cursor_mode == cmAFTER)
		this->toggleCursorMode();
      else
		this->setCursor(begin_line, next);
    }
    else if (begin_line < this->lastLine()) {
      int first_pos = lines.line[begin_line + 1]->screen2base(0);
      this->setCursor(begin_line + 1, first_pos);
	  if(begin_line < this->lastLine()){
		// bogus this can reference a non-existent line!!
		if(lines.line[begin_line+1]->direction(0) == dRIGHT)
		  this->setCursorMode(cmAFTER);
		else
		  this->setCursorMode(cmBEFORE);
	  }
    }
  }
  else {
    if(pos != 0) {		    
      int prev = line->screen2base(pos-1);
      if(line->direction(begin_col) == dRIGHT &&
		 line->direction(prev) == dLEFT &&
		 cursor_mode == cmBEFORE)
		this->toggleCursorMode();
      else if(line->direction(begin_col) == dLEFT &&
			  cursor_mode == cmAFTER)
		this->toggleCursorMode();
      else
		this->setCursor(begin_line, prev);
    }
    else if(pos == 0 && line->direction(0) == dRIGHT && 
			cursor_mode == cmBEFORE) 
      this->toggleCursorMode();
    else {
      if(line->screen2base(pos) != begin_col)
		this->setCursor(begin_line, line->screen2base(pos));	
      else if(begin_line > 0) {
		// assuming 0xFFFF will be beyond the end of the line
		int last_pos = lines.line[begin_line - 1]->screen2base(0xFFFF);
		this->setCursor(begin_line - 1, last_pos);
		this->setCursorMode(cmBEFORE);
      }
    }
  }
}
