#include "rafport.h"

#include <Xm/CutPaste.h>
#include <Xm/Form.h>
#include <Xm/DrawingA.h>
#include <Xm/PushBG.h>
#include <Xm/PushB.h>
#include <Xm/SeparatoG.h>
#include <Xm/Frame.h>
#include <Xm/RowColumn.h>
#include <Xm/PanedW.h>
#include <Xm/ScrolledW.h>

#include <X11/keysym.h>
#include <X11/keysymdef.h>
#include <X11/cursorfont.h>
#include <X11/Xlib.h>

#include <stdlib.h>
#include <stdio.h>

#include "ifc.h"

#include "ucs.h"
#include "utf.h"
#include "smemory.h"
#include "keyboard/KeyMap.h"
#include "misc/Unicode.h"
#include "dafs.h"
#include "misc/misc.h"

#include "lte/Line.h"
#include "TestEditor.h"
#include "TestCanvas.h"
#include "misc/ScrollWindow.h"

static iEntityPtr doc = NULL;
static int scroll_flag = 0, mem_check = 0;
static KeyMap *arabic, *american;
static KeyMap *km;

ifc myargs[] = {
	{(char **) &scroll_flag, 0, 0, 1, IFC_FLAG, "s","SCROLL"},
	{(char **) &mem_check, 0, 0, 9, IFC_INT, "m", 0},
	{0, 0, 0, 1, IFC_NA, 0, 0}
};


extern KeyMap *initArabic(void);
extern KeyMap *initAmerican(void);

static int 
fsize(FILE *fp)
{
	long size, current_pos = ftell(fp);
	fseek(fp, 0, 2); // SEEK_END
	size = ftell(fp);
	fseek(fp, current_pos, 0); // SEEK_SET

	return size;
}
	
void
exitCallback(Widget widget, UniEditor *uni, XmPushButtonCallbackStruct *cbs)
{
	uString text = uni->getString();
	text += (iUCode) 0;
	// must multiply by 3 because of possible utf blowup
	char *buffer = (char *) smalloc(3 * text.length() + 1);
	ucs2utf(buffer, text.string());
	fprintf(stdout, "%s", buffer);

	if(doc) {
	  i_WriteEntity("foo.dr", doc, DAFSB);
	  i_DisposeEntity(doc);
	}

	extern MemoryInfo mi;
	if(mi.memory_track_flag != sNoChecks) {
		smfree(buffer);

		delete uni;
		delete arabic;
		delete american;
		i_ExitLib();
		s_ExitLib();
	}
	exit(0);
}

void
arabicCallback(Widget widget, UniEditor *uni, XmPushButtonCallbackStruct *cbs)
{
  km = arabic;
  XmProcessTraversal(uni->getWidget(), XmTRAVERSE_CURRENT);
  XmProcessTraversal(uni->getWidget(), XmTRAVERSE_CURRENT);
}

void
americanCallback(Widget widget, UniEditor *uni, XmPushButtonCallbackStruct *cbs)
{
  km = american;
  XmProcessTraversal(uni->getWidget(), XmTRAVERSE_CURRENT);
  XmProcessTraversal(uni->getWidget(), XmTRAVERSE_CURRENT);
}

int
main(int argc, char *argv[])
{
  Widget toplevel, form, rowcol, e_button, ar_button, am_button;
  Widget frame = 0;
  XtAppContext app;
  UniEditor *uni;
  ScrollWindow *sw = 0;
  
  toplevel = XtVaAppInitialize(&app, "Demos", NULL, 0, &argc, argv, NULL, NULL);
  
  processargs(argc, argv, myargs);

  s_InitLib((sMemFlags) mem_check, 0, 0);
  i_InitLib();
  miscInit(toplevel);

  arabic = initArabic();
  american = initAmerican(); 
  km = american;


  form = XtVaCreateManagedWidget("form", xmFormWidgetClass, toplevel,
				 NULL);

  rowcol = XtVaCreateManagedWidget("rolcol", xmRowColumnWidgetClass, form,
				   XmNbottomOffset, -35, 
				   XmNbottomAttachment, XmATTACH_OPPOSITE_FORM,
				   XmNleftAttachment, XmATTACH_FORM,
				   XmNrightAttachment, XmATTACH_FORM,
				   XmNorientation, XmHORIZONTAL,
				   NULL);

  e_button = XtVaCreateManagedWidget("Exit", xmPushButtonGadgetClass, rowcol, 
				     NULL);

  ar_button = XtVaCreateManagedWidget("Arabic", xmPushButtonGadgetClass, rowcol,
				     NULL);
  
  am_button = XtVaCreateManagedWidget("English", xmPushButtonGadgetClass, rowcol,
				      NULL);

  if(scroll_flag) {

    frame = XtVaCreateManagedWidget("frame", xmScrolledWindowWidgetClass, form, 
				    XmNvisualPolicy, XmCONSTANT,
				    XmNscrollingPolicy, XmAUTOMATIC,
				    XmNscrollBarDisplayPolicy, XmSTATIC,
				    NULL);  
	sw = new ScrollWindow(frame);
  }
  else {
    frame = XtVaCreateManagedWidget("frame", xmScrolledWindowWidgetClass, form, 
				    NULL);
  }


  TestCanvas *canvas;
  canvas = new TestCanvas(0, app, frame, 500, 500, &km);
  uni = new TestEditor(canvas, &km, scroll_flag);  
  canvas->setClient(uni);

#if 0
  XtVaSetValues(uni->getWidget(),
		XmNwidth, 500,
		XmNheight, 500,
		NULL);
#endif
  XtManageChild(uni->getWidget());
  XtAddCallback(e_button, XmNactivateCallback, exitCallback, (XtPointer) uni);
  XtAddCallback(ar_button, XmNactivateCallback, arabicCallback, (XtPointer) uni);
  XtAddCallback(am_button, XmNactivateCallback, americanCallback, (XtPointer) uni);
  XmScrolledWindowSetAreas(frame, 0,0, uni->getWidget());

  XtVaSetValues(frame,
		XmNnavigationType, XmTAB_GROUP,
		XmNbottomAttachment, XmATTACH_FORM,
		XmNleftAttachment, XmATTACH_FORM,
		XmNrightAttachment, XmATTACH_FORM,
		XmNtopWidget, rowcol,
		XmNtopAttachment, XmATTACH_WIDGET,
		NULL);  
  

  int index = 1;
  while(index < argc && (!argv[index] || !argv[index][0]))
    index++;

  if(index < argc){
    if(!strrchr(argv[index], '.') || strcmp(strrchr(argv[index], '.'), ".dr")) {
      FILE *fp;
      if(!(fp = fopen(argv[index], "rt"))){
	fprintf(stderr, "can't open file %s\n", argv[index]);
	return 1;
      }
      
      long size = fsize(fp);
      char *buffer = (char *) smalloc(size+1);
      fread(buffer, size, 1, fp);
      buffer[size] = 0;
      
      iUCode *ubuffer = (iUCode *) smalloc((size+1) * sizeof(iUCode));
      utf2ucs(ubuffer, buffer);
      smfree(buffer);
      uni->setString(ubuffer);
      smfree(ubuffer);
    }
    else {
      i_ReadEntity(argv[index], &doc);
      uni->setEntity(doc);
    }
  } 
  
  XtRealizeWidget(toplevel);
#if 0
  Dimension ebh;
  XtVaGetValues(exit_button,
		XmNheight, &ebh,
		NULL);
  XtVaSetValues(exit_button,
		XmNpaneMinimum, ebh,
		XmNpaneMaximum, ebh,
		NULL);
#endif

  XmProcessTraversal(uni->getWidget(), XmTRAVERSE_CURRENT);
  XmProcessTraversal(uni->getWidget(), XmTRAVERSE_CURRENT);
  XtAppMainLoop(app);

  if(0)
    prall(NULL);
  
  return 0;
}

