
/*
 * Copyright (c) 1994, 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include "rafport.h"

#include "rafutil.h" // for MIN and MAX
#include "c-type.h"

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h> 
#include <X11/Xutil.h>
#include <Xm/Xm.h>
#include <Xm/DrawingA.h>
#include <Xm/ScrolledW.h>
#include <Xm/ScrolledWP.h>

#include "misc/XArg.h"

#include "Canvas.h"
#include "bb.h"
#include "misc/scroll.h"

#include "CanvasClient.h"

CanvasClient::CanvasClient(Canvas *input_canvas)
	: canvas(input_canvas),
	  desired_width(0), desired_height(0)
{
}

CanvasClient::~CanvasClient()
{
  if(this->activeP() && this->canvas->managedP()) {
	this->canvas->unmanage();
  }
}

Canvas *
CanvasClient::getCanvas()
{
	return this->canvas;
}

int 
CanvasClient::activeP(void)
{
  Canvas *c = this->getCanvas();
  return c && c->client == this;
}

void 
CanvasClient::redisplay(XmDrawingAreaCallbackStruct *call_data)
{
  static Region region = NULL;
  XEvent *event = (XEvent *) call_data->event;
  iLBox refresh_box;
  XRectangle rect;
	
  if (this->validP() == FALSE)
	return;
	
  if(!region)
	region = XCreateRegion();
  XtAddExposureToRegion(event, region);
	
  if(event->xexpose.count != 0)
	return;
	
  XClipBox(region, &rect);
  XDestroyRegion(region);
  region = 0;
  refresh_box.x = rect.x;
  refresh_box.y = rect.y;
  refresh_box.w = rect.width;
  refresh_box.h = rect.height;

  if(refresh_box.w)
	this->drawRegionNow(refresh_box);
}
