
#include "RTEView.h"

#include "RTECursor.h"
#include "Rectangle.h"
#include "RootElement.h"

// ----------------------------- RootElement methods --------------------------

RootElement::RootElement(InputEventHandler *ieh, VirtualDisplayDevice *vdd,
                     Coord px, Coord py, Coord pw, Coord ph,
                     Direction pwd, Direction swd, Coord sp,
	             int canMove, int canGrowUp, int canGrowDown, 
	             int canGrowRight, int canGrowLeft)
        : ConstrainedRectangle(px, py, pw, ph, canMove, canGrowUp, canGrowDown,
	                       canGrowLeft, canGrowRight, pwd, swd, sp)
{
        display_device = vdd;
        input_handler = ieh;
        primary_writing_direction = pwd;
        secondary_writing_direction = swd;
        spacing = sp;
        deleted_element_manager = new DeletedPoolManager();
        root_selection_manager = new RTESelectionManager();
        root_cursor = new RTECursor(this, this->root_selection_manager);
        root_traversal_manager = new TraversalManager();

}

static void killChildren(ChildRectangle *);

static void
killAll(ChildRectangle *rect)
{
  killChildren(rect);

  delete rect;
}

static void 
killChildren(ChildRectangle *rect)
{
  if(rect->numChildren()) {
    int i, size = rect->numChildren();
    ElementList *children = ((PackedRectangle *) rect)->getAllChildren();
    for(i = 0; i < size; i++)
      killAll(children->getItemNumber(i));
    delete children;
  }
}

RootElement::~RootElement()
{
  if(root_cursor->getRoot() == this)
    root_cursor->getParent()->deleteChild(root_cursor);

  killChildren(this);

  delete deleted_element_manager;
  delete root_cursor;
  delete root_selection_manager;
  delete root_traversal_manager;
}

int
RootElement::changeSizeTo(Coord w, Coord h, int redraw)
{
  int ret;
#ifdef MAVERICK
  debugf("RootElement changed size to %d by %d\n", w, h);
#endif
  ret = ConstrainedRectangle::changeSizeTo(w, h, redraw);

  if(this->can_grow_right)
	this->display_device->changeSizeTo(w+2*x, h+2*y);

  return ret;
}
