#include "RTEView.h"
#include "Rectangle.h"
#include "RTESet.h"
#include "smemory.h"
#include <sys/time.h>
#include <sys/timeb.h>


static void print_rect(int i, Rectangle *r)
{
	printf("Rectangle %d has x = %d, y = %d, w = %d, h = %d\n",
	        i, r->getX(), r->getY(), r->getW(), r->getH());
}

static void print_element_set(char *s, ElementSet *es)
{
	printf("element set %s:\n", s);
	int n = es->numElements();
	for(int i=0; i<n; i++) {
		Rectangle *r = es->getElementNumber(i);
		print_rect(i, r);
	}
}

#define NR  50
#define ROOT_X 0
#define ROOT_Y 0
#define ROOT_W 500
#define ROOT_H 1000
#define PX 0
#define PY 0
#define CRW 200
#define CRH 200
#define PAGE_LINE_SPACE 10
#define ROOT_LINE_SPACE 10

#define PRINT_RECTANGLES

main()
{
	s_InitLib(sTrackAll, 0, 0);

	struct timeb t1, t2;
	float et, et_total=0.0;
	InputEventHandler *ieh = NULL;
	VirtualDisplayDevice *vdd = NULL;
	ChildRectangle *cr[NR];
	ConstrainedRectangle *page;
	RootElement *root = 0;
	ElementSet *es=0;
	int i, idir;
	int canMove=FALSE, canGrowUp=TRUE, canGrowDown=TRUE, canGrowLeft=TRUE, canGrowRight=TRUE;
	
	// ----------------------------------------------------------------------

	for(idir=0; idir<8; idir++) {
		ftime(&t1);
		switch(idir) {
			case 0:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, LeftToRight,
                        				TopToBottom, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             LeftToRight, TopToBottom, PAGE_LINE_SPACE);
				break;
			case 1:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, RightToLeft,
                        				TopToBottom, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             RightToLeft, TopToBottom, PAGE_LINE_SPACE);
				break;
			case 2:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, LeftToRight,
                        				BottomToTop, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             LeftToRight, BottomToTop, PAGE_LINE_SPACE);
				break;
			case 3:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, RightToLeft,
                        				BottomToTop, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             RightToLeft, BottomToTop, PAGE_LINE_SPACE);
				break;
			case 4:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, TopToBottom,
                        				LeftToRight, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             TopToBottom, LeftToRight, PAGE_LINE_SPACE);
				break;
			case 5:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, BottomToTop,
                        				LeftToRight, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             BottomToTop, LeftToRight, PAGE_LINE_SPACE);
				break;
			case 6:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, TopToBottom,
                        				RightToLeft, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             TopToBottom, RightToLeft, PAGE_LINE_SPACE);
				break;
			case 7:
				root = new RootElement(ieh, vdd, ROOT_X, ROOT_Y,
                        				ROOT_W, ROOT_H, BottomToTop,
                        				RightToLeft, ROOT_LINE_SPACE,
                        				canMove, canGrowUp, canGrowDown,
                        				canGrowRight, canGrowLeft);
				page = new ConstrainedRectangle(0,0,CRW,CRH,
				             canMove, canGrowUp, canGrowDown,
				             canGrowLeft, canGrowRight, 
				             BottomToTop, RightToLeft, PAGE_LINE_SPACE);
				break;
		}
		ChildRectangle *cursor = (ChildRectangle *)root->getCursor();
		if(!root->insertAt(PX, PY, page)) {
			printf("root->insertFirst(page) failed\n");
			halt("program halted\n");
		}
		if(!page->insertFirst(cursor)) {
			printf("page->insertFirst(cursor) failed\n");
			halt("program halted\n");
		}

		printf("inserting rectangles for idir = %d ...\n", idir);
		for(i=0; i<NR; i++) {
			cr[i] = new ChildRectangle(0,0,5,5);
			if(!page->insertBefore(cursor, cr[i])) {
				printf("pr->insertBefore(cursor,cr[%d]) failed\n", i);
				halt("program halted\n");
			}
		}
	
		printf("checking nextChild ...\n");
		for(i=0; i<NR-1; i++) {
			if(page->nextChild(cr[i], TRUE) != cr[i+1]) {
				printf("for i = %d, children are in wrong order\n", i);
				halt("program halted\n");
			}
		}

		if(page->nextChild(cr[NR-1], TRUE) != cursor) {
			printf("cursor is not next child after cr[%d]\n", NR-1);
			halt("boff\n");
		}

		printf("checking prevChild ...\n");
		for(i=1; i<NR; i++) {
			if(page->prevChild(cr[i], TRUE) != cr[i-1]) {
				printf("for i = %d, children are in wrong order\n", i);
				halt("program halted\n");
			}
		}
		printf("series of insertAfters produced expected order\n\n");

		for(i=NR/2; i<2*NR/3; i++)
			page->deleteChild(cr[i], TRUE);
		printf("page->deleteChild() worked\n");

		#if 0
		page->deleteChild(cr[NR/2], TRUE);
		printf("page->deleteChild() worked\n");
		#endif


		ftime (&t2);
		et = (float)(t2.time - t1.time) + (float)(t2.millitm - t1.millitm)/1000.0;
		printf("elapsed CPU time: %f\n", et);
		et_total += et;
		#ifdef PRINT_RECTANGLES
			es = page->getAllChildren();
			print_element_set("Contents of packed rectangle:", es);
		#endif

		delete page;  page = 0;
		delete root;  root = 0;
		delete es;  es = 0;
		for(i=0; i<NR; i++) {
			delete cr[i]; cr[i] = 0;
		}
	}
	printf("\nTotal CPU time for all directions: %f\n", et_total);

	// ----------------------------------------------------------------------


	s_ExitLib();
}
