
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */
/*  This interprets the switches given on the command line for rafread,
    rafeng, host, and poscnt.  Since all need to cycle through the same
    mail pieces and all use most of the same switches, they all use this
    file.
	
    For the most part all the command line switches can also be set with
    environment variables.
	
	*/

#include "rafport.h"
#include "io-lib.h"
#if 0
#include "c-lib.h"
#endif

#include <stdio.h>
#include <stdlib.h>

#include "ifc.h"

extern char *getenv(const char *);

static int settype(ifc *cp, char *p)
{
	int bad;
	long t;
	float f;

	switch (cp->type) {
	case IFC_CHAR:
		t = *(char *) cp->val = *p;
		bad = t < cp->lval || t > cp->fval;
		break;
	case IFC_SHORT:
		t = *(short *) cp->val = *p;
		bad = t < cp->lval || t > cp->fval;
		break;
	case IFC_INT:
		t = *(int *) cp->val = atoi(p);
		bad = t < cp->lval || t > cp->fval;
		break;
	case IFC_HEX:
		sscanf(p,"%x",cp->val);
		t = *(int*)cp->val;
		bad = t < cp->lval || t > cp->fval;
		break;
	case IFC_LONG:
		t = *(long *) cp->val = atoi(p);
		bad = t < cp->lval || t > cp->fval;
		break;
	case IFC_FLOAT:
		f = *(float*)cp->val = atof(p);
		bad = f < (float)atof((char*)cp->lval) || f > (float)atof((char*)cp->fval);
		break;
	case IFC_STR:
		*cp->val = p;
		bad = 0;
		break;
	}
	return bad;
}

int stripext(int c,char *v[],char *ev[])
{
	int i,j,elen,which,count;
	
	which = -1;
	for (count=0,i=1;i<c;i++)
		if (v[i]) count++;
	for (j=0;*ev[j];j++) {
		elen = strlen(ev[j]);
		for (i=1;i<c;i++) {
			if (!v[i]) continue;
			if (!strcmp(&v[i][strlen(v[i]) - elen], ev[j])) {
				v[i][strlen(v[i]) - elen] = 0;
				which = j;
			}
		}
	}
	return which;
}

int processargs(int c, char *v[], ifc * ip)
{
	ifc *cp;
	int len, i,bad;
	char *p;
	cp = ip;

	bad = 0;
	while (cp->type != IFC_NA) {
		if (cp->type == IFC_CHAR)
			*(char *) cp->val = cp->init;
		else if (cp->type == IFC_SHORT)
			*(short *) cp->val = cp->init;
		else if (cp->type == IFC_LONG)
			*(long *) cp->val = cp->init;
		else if (cp->type == IFC_FLOAT)
			*(float*) cp->val = atof((char*)cp->init);
		else
			*(int *) cp->val = cp->init;
		if (cp->env && (p = getenv(cp->env)))
			bad |= settype(cp, p);
		cp++;
	}
	for (i = 1; i < c; i++) {
		if (v[i][0] == '-') {
			cp = ip;
			while (cp->type != IFC_NA) {
				len = strlen(cp->sw);
				if (!strncmp(&v[i][1], cp->sw, len) &&
					(cp->type != IFC_FLAG || !v[i][len+1]))
					break;
				cp++;
			}
			if (cp->type == IFC_NA)
				return -1;
			if (cp->type != IFC_FLAG) {
				if (v[i][1 + len])
					bad |= settype(cp, &v[i][1 + len]);
				else {
					v[i++] = 0;
					bad |= settype(cp, v[i]);
				}
			} else
				*(int *) cp->val = cp->fval;
			v[i] = 0;
		}
	}
	return bad;
}
