
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/*
// all those things that should have come in the
// C library
*/

#include "rafport.h"

#include "c-lib.h"
#include "str-lib.h"
#include "c-type.h"
#include "var-args.h"

#include "rafutil.h"


#if defined(SYS_TRUESCAN) /*|| defined(SYS_OS_SOLARIS)*/

char *
index(char *s, char c)
{
	char *p;
	for(p = s; *p && *p != c; p++)
		;
	return (*p == c) ? p : 0;
}

#endif

#if defined(SYS_TRUESCAN)
char *
strncpy(char *s1, const char *s2, size_t n)
{
	int4 i;
	char *s = s1;
	for(i = 0; i < n && *s2;)
		*s1++ = *s2++;
	*s1 = '\0';
	
	return s;
}

void
vfprintf(FILE *file, char *s, ...)
{
	printf(s);
}

#endif

/* include this for systems that don't have memmove defined */
#if (defined SYS_OS_SUNOS) /*|| (defined SYS_OS_MERC)*/
void *
memmove(void *dest, const void *src, int4 n)
{
	int4 i;
	char *d=(char *)dest;
	char *s=(char *)src;
	
	if(n<=0)
		return 0;
	if(d>s)
		if(d-s>n)
			memcpy(d,s,n);
		else
			while(n--)
				d[n]=s[n];
	else
		if(s-d>n)
			memcpy(d,s,n);
		else
			for(i=0;i<n;i++)
				d[i]=s[i];
	return d;
}

void *
memset(void *dest, int value, int4 n)
{
	int4 i;
	char *d=(char *)dest;
	
	for(i=0;i<n;i++)
		d[i]=value;

	return d;
}

#endif

void HUGEPTR *
memsetH(void HUGEPTR *ptr, int v, int4 x)
{
	int4 incr = 30000;
	int4 i;
	char HUGEPTR *bog;
	bog = (char *) ptr;

	if(x < incr)
		memset(ptr, v, (size_t) x);
	else {
		for(i = 0; i + incr < x; i += incr){
			memset(bog + i, v, (size_t) incr);
		}
		if(i + incr != x)
			memset(bog + i, v, (size_t) (x - i));
	}
	return ptr;
}

