
/*
 * Copyright (c) 1995 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

/*
// This module allows the developer to override the standard file io
// and substitute his own. The format of the functions is based on the
// interface to the stream (fopen, etc) calls, but we already have
// interfaces to the file no calls (open, etc)
//
// So by default, you get the stream library
*/

#include <stdio.h>
#include "rafport.h"
#include "str-lib.h"
#include "rafio.h"
#include "rafios.h"

#ifdef feof
static int4 feofw(FILE *stream)
{
	return feof(stream);
}
#endif

#if !defined(SYS_OS_WIN16) && !defined(MAVERICK)
#if defined(SYS_OS_SUNOS)
extern int fread();
extern int fwrite();
extern int fclose();
extern int fseek();
extern int fgetc();
extern int fputs();

#endif

IoFunctionTable stream_table = {
	(ReadFunc)fread,
	(WriteFunc)fwrite,
	(OpenFunc)fopen,
	(CloseFunc)fclose,
	(SeekFunc)fseek,

	(TellFunc)ftell,
	(GetcFunc)fgetc,
	(GetsFunc)fgets,
	(PutsFunc)fputs,
#ifdef feof
	(EofFunc)feofw,
#else
	(EofFunc)feof,
#endif
};

#else

IoFunctionTable stream_table = {
	(ReadFunc)  streamRead,
	(WriteFunc) streamWrite,
	(OpenFunc)  streamOpen,
	(CloseFunc) streamClose,
	(SeekFunc)  streamSeek,

	(TellFunc)  streamTell,
	(GetcFunc)  streamGetc,
	(GetsFunc)  streamGets,
	(PutsFunc)  streamPuts,
	(EofFunc)   streamEof,
};

#endif


static IoFunctionTable *table = &stream_table;

Dll void APIENTRY
riInit(IoFunctionTable *t)
{
	table = t;
}

Dll int4 APIENTRY
riRead(char *ptr, int4 size, int4 nitems, RafFile handle)
{
	return table->read(ptr, size, nitems, handle);
}

Dll int4 APIENTRY
riWrite(const char *ptr, int4 size, int4 nitems, RafFile handle)
{
	return table->write(ptr, size, nitems, handle);
}

Dll RafFile APIENTRY
riOpen(const char *filename, const char *type)
{
	return (RafFile) table->open(filename, type);
}

Dll int4 APIENTRY
riClose(RafFile handle)
{
	return table->close(handle);
}

Dll int4 APIENTRY
riGetc(RafFile handle)
{
	return table->getcf(handle);
}	

Dll char * APIENTRY
riGets(char *s, int4 size, RafFile handle)
{
	return table->getsclone(s, size, handle);
}

Dll int4 APIENTRY
riPuts(char *s, RafFile handle)
{
	return table->putsf(s, handle);
}

Dll int4 APIENTRY
riSeek(RafFile handle, long offset, int4 whence)
{
	return table->seek(handle, offset, whence);
}

Dll long APIENTRY
riTell(RafFile handle)
{
	return table->tell(handle);
}

Dll int4 APIENTRY
riEof(RafFile handle)
{
	return table->eof(handle);
}


