
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

//{ section ********************************************
//. 
//- Length routines
//. 
//- Function - read/write length
//*   encoded as follows - LSByte to MSByte with
//*    last byte having 0x80 set
//. 
//} *************************************************

#include "aString.h"

//
// How many bytes needed to encode string
unsigned BytesRequiredForLength(unsigned long l)
{
	if(l < 0x80)      return(1);
	if(l < 0x4000)    return(2); // note only 7 bytes of data per
	if(l < 0x200000)  return(3);
	if(l < 0x10000000)  return(4);
	return(5);
}

//
// Write l UseBytes may force the use of more Bytes than
// needed. Default says use as many as needed. 
int WriteLengthString(FILE *out,int l,int UseBytes)
{
	int i;
	unsigned char c;
	if(UseBytes) 
		assert(UseBytes >= BytesRequiredForLength(l)); // cannot use less then needed
	else
		UseBytes = BytesRequiredForLength(l);
	
	for(i = 1; i <= UseBytes; i++) {
		c = l & 0x7f;
		if(i == UseBytes) 
			c |= 0x80; // add terminal marker
		else
			l >>= 7; // next byte
		
		fwrite(&c,1,1,out);
	}
	
	return(UseBytes);
}

//
// read length byte - l is read length
// return is the number of bytes read
int ReadLengthString(FILE *in,int &l)
{
	char cx[4];
	int BytesRead = 0;
	int i;
	l = 0;
	
	fread(&cx[BytesRead],1,1,in);
	while(!(cx[BytesRead] & 0x80)) {
		fread(&cx[++BytesRead],1,1,in);
	}
	cx[BytesRead] &= 0x7f; // clear stop bit
	i = BytesRead;
	l = cx[BytesRead++];
	while(i--) {
		l <<=  7;
		l += cx[i];
	}
	return(BytesRead);
}
