  /* begin example: Hierarchy */
  /* we will make these global variables so we can use them
     in later examples */
  iEntityPtr doc;
  iEntityPtr para1;
  iEntityPtr para2;
  iEntityPtr word1, word2, word3;
  iEntityPtr glyph1_1, glyph1_2, glyph2_1, glyph2_2,
     glyph2_3;
  iEntityPtr glyph3_1, glyph3_2, glyph3_3, glyph3_4;
  
  
  void
  makeHierarchy()
  {
     /* later on we will discuss the manner in which entity
     types are defined */
     doc = i_NewEntity(iDocumentT, 0);       /* supply a
     null      parent */
     para1 = i_NewEntity(iParagraphT, doc);
     para2 = i_NewEntity(iParagraphT, doc);  /* para1 and
     para2     will stay in the order they were created
     unless the     hierarchy is rearranged */
     word1 = i_NewEntity(iWordT, para1);
       /* indentation for illustrative purposes only */
       glyph1_1 = i_NewEntity(iGlyphT, word1);
       glyph1_2 = i_NewEntity(iGlyphT, word1);
     word2 = i_NewEntity(iWordT, para1);
       glyph2_1 = i_NewEntity(iGlyphT, word2);
       glyph2_2 = i_NewEntity(iGlyphT, word2);
       glyph2_3 = i_NewEntity(iGlyphT, word2);
     word3 = i_NewEntity(iWordT, para1);
       glyph3_1 = i_NewEntity(iGlyphT, word3);
       glyph3_2 = i_NewEntity(iGlyphT, word3);
       glyph3_3 = i_NewEntity(iGlyphT, word3);
       glyph3_4 = i_NewEntity(iGlyphT, word3);
     /* this example did not show associating text or image
     with an entity */
  }
  /* end example: Hierarchy */
  /* begin example: glyph */
  void setOneGlyph() {
     i_SetGlyph(glyph1_1, 'M', 0);
  }
  /* end example: glyph */
  /* begin example: text */
  void setOneString() {
     i_SetText(para2, "The dog chased the cat.");
  }
  /* end example: text */
  /* begin example: addchar */
  void addPeriod() {
     const char *s;
     char *new_s;
     int len;
  
     s = i_GetText(para2);
     len = sizeof(s);
  
     /* we must make a copy to modify the value because we
         don't know whether there is enough space
         in the existing pointer */
     new_s = (char *) malloc(len + 2); /* 1 for NUL, 1 for character
     */
     new_s = strdup(s);
     new_s[len] = '.';
     i_SetText(para2, new_s);
  
     /* we are responsible for freeing this
         since DAFS made a copy */
     free(new_s);
  }
  /* end example: addchar */
  /* begin example: delchar */
  void deleteAChar(){
     char *s;
     int len;
  
     s = (char *) i_GetText(para2); /* defeat const */
     len = sizeof(s);
  
     s[len] = '\0'; /* we know that we will not get a memory
     error */
     /* leave out the next statement and the entity's text
     is
     still changed, but callback isn't called */
     i_SetText(para2, s);
  }
  /* end example: delchar */
  /* begin example: posset */
  void setPosset(){
     iPosSet ps[2];
     ps[0].chr = 'a';
     ps[0].conf = 50;
     ps[1].chr = 'r';
     ps[1].conf = 34;
     i_SetPosSet(glyph3_1, 2, ps);
  }
  /* end example: posset */
  /* begin example: swap */
  struct MyData {
     long a_long;
     short a_short;
  };
  
  void swapMyData(void *p)
  {
     struct MyData *my_data = (struct MyData *) p;
  
     my_data->a_long = i_SwapL(my_data->a_long);
     my_data->a_short = i_SwapS(my_data->a_short);
  }
  
  void testSwapping(){
     const void *my_data;
     iEntityPtr entity;
     long size;
     iDAFSError e;
  
     e = i_GetUserDataProp(entity, "my data", (void *) &my_data,
     &size,    swapMyData);
  }
  /* end example: swap */
  /* begin example: alloc */
  /* return an array that contains the number of black
     pixels in each horizontal line -- the horizontal sum of
     pixels */
  void calcHsmp(iImagePtr image, iBox b, short *hsmp)
  {
     char *bits;
     int2 data_width;
     iFixedPoint mag = 1L<<16;
     int i, j, k;
     int sum;
  
     i_Image2Bitmap(image, b, mag, &data_width, &bits);
     for(i = 0; i < b.h; i++) { /* loop over lines
       sum = 0;
       for(j = 0; j < data_width; j++) /* loop over bytes */
          for(k = 0; k < 8; k++) /* loop over bits */
               sum += ((1 << k) & bits[i*data_width + j]) !=
     0;
       hsmp[i] = sum;
     }
  }
  
  void allocShortCut(){
     short *hsmp;
     iEntityPtr entity;
     iDAFSError e;
     iBox b;
     #define HSMP "HSMP"
  
     b = i_GetEnclosingBox(entity);
     e = i_SetUserDataProp(entity, HSMP,0, sizeof(short)*
     b.h);
     e = (e != DAFSOK) ?
          e :
          i_GetUserDataProp(entity,HSMP,(void**)&hsmp, 0,
     0);
     if(e == DAFSOK)
       calcHsmp(i_GetImage(entity), b, hsmp);
  }
  /* end example: alloc */
  /* begin example: flag */
  void flagIt() {
     i_SetLongProp(glyph3_1, "flag", 1);
  }
  /* end example: flag */
  /* begin example: session */
  #define TEXT_DIRECTION "Text Direction"
  #define L_TO_R "left to right"
  #define R_TO_L "right to left"
  
  void illustrateSessionProps(){
     iEntityPtr english_line, arabic_line;
     char *value;
  
     i_SetSessionStringProp(iLineT, TEXT_DIRECTION, L_TO_R);
     /* in case you had any doubts about my example */
     i_SetEntityType(arabic_line, iLineT);
     i_SetEntityType(english_line, iLineT);
     i_SetStringProp(arabic_line,TEXT_DIRECTION, R_TO_L);
  
     i_GetStringProp(english_line, TEXT_DIRECTION, &value); /* will return "left to
     right" because there is no TEXT_DIRECTION property
     associated with english_line, and so it defaults to the
     value associated with the line type */
  
     i_GetStringProp(arabic_line, TEXT_DIRECTION, &value); /* will return "right to
     left" because we have overridden the type property with
     one local to the entity */
  }
  /* end example: session */
  /* begin example: find parent */
  iEntityPtr findRoot(iEntityPtr entity)
  {
     iEntityPtr parent;
     while(parent = i_GetParentEntity(entity))
       entity = parent;
     return entity;
  }
  /* end example find parent */
  /*begin example: recurse */
  void printRecurse(iEntityPtr root)
  {
     iEntityPtr child;
     void printOne(iEntityPtr entity);
  
     printOne(root); /* I will describe in the next example
     */
  
     for(child = i_GetChildEntity(root);
       child;
       child = i_GetNextEntity(child)){
          printRecurse(child);
     }
  }
  /* end example: recurse */
  /* begin example: iterate */
  void printOne(iEntityPtr en)
  {
     iDTag tag;
     short c;
     char *s;
  
     tag = i_GetEDataType(en);
     /* beware, we aren't handling all types of data */
     switch(tag){
       case iD1PosSet:
          c = i_GetGlyph(en, 0);
          printf("%c ", c);
          break;
       case iDString:
          s = i_GetText(en);
          printf("%s ", s);
     }
  }
  
  void printIterative(iEntityPtr en)
  {
     for(; en; en = i_FindEntityForward(en, 0, ALL, 0)) { 
       printOne(en);
     }
  }
  /* end example: iterate */
  /* begin example: borrow and or */
  void illustrateBorrowAndOr(){
     iEntityPtr word1a, word1b;
     iEntityPtr glyph1_1a, glyph1_1b;
     iEntityPtr glyph1_2b;
     iDAFSError e;
  
     word1a = i_NewEntity(iWordT, word1);
     word1b = i_NewEntity(iWordT, word1);
     if(word1a == 0 || word1b == 0)
       e = MallocFailError;
     /* !e is short for e == DAFSOK */
     if(!e) {
       i_SetLogicalType(word1, iOr);
       e = i_TransferEntity(glyph1_1, word1a);
     }
     if(!e) e = i_TransferEntity(glyph1_2, word1a);
     if(!e) {
       glyph1_1a = i_NewEntity(iGlyphT, word1b);
       glyph1_1b = i_NewEntity(iGlyphT, word1b);
     }
     if(!e && (glyph1_1a == 0 || glyph1_1b == 0))
       e = MallocFailError;
     if(!e) {
       i_SetGlyph(glyph1_1a, 'T',0);
       i_SetGlyph(glyph1_1b, 'h',0);
     }
     if(!e) e = i_BorrowEntity(glyph1_2, word1b,
     &glyph1_2b);
  }
  /* end example: borrow and or */
  /* begin example: read */
  void readIt() {
     iEntityPtr root;
     iDAFSError e;
     e = i_ReadEntity("foo.dr", &root);
  }
  /* end example: read */
  /* begin example: write */
  void writeIt() {
     iDAFSError e;
     e = i_WriteEntity("dogs_and_cats.dr", doc, DAFSB);
  }
  /* end example: write */
  /* begin example: area */
  void printArea(iEntityPtr entity)
  {
     void *data;
     iATag type;
     iBox box;
     iBoxListPtr box_list;
     iOrthogonListPtr ortho_list;
     iOrthogonPtr ortho;
     int i;
     int j;
  
     i_GetArea(entity, &data, &type);
     switch(type){
     case iABox:
       box = *(iBox *)data;
       printf("box: (%d, %d) %dx%d\n",
               box.x, box.y,
               box.w, box.h);
       break;
     case iABoxList:
       box_list = (iBoxListPtr) data;
       printf("box_list: enclosing box (%d, %d) to (%d,
     %d)\n",
          box_list->extents.x1, box_list->extents.y1,
          box_list->extents.x2, box_list->extents.y2);
       for(i = 0; i < box_list->numRects; i++){
          printf("\tbox %d: (%d,%d) to (%d, %d)\n",
                  i,
                 box_list->rects[i].x1, box_list->rects[i].y1,
                 box_list->rects[i].x2, box_list->rects[i].y2);
       }
       break;
     case iAOrthogonList:
       ortho_list = (iOrthogonListPtr) data;
       printf("orthogon list:\n");
       for(i = 0; i < ortho_list->num_poly; i++){
          ortho = ortho_list->poly[i];
          printf("\torthogon %d:\n", i);
          for(j = 0; j < ortho->n; j++){
               printf("\t\tpoint %d: (%x, %y)\n",
                    j,
                    ortho->point[j].x,
                    ortho->point[j].y);
          }
       break;
     }
  }
}
  /* end example: area */
  /* begin example: image */
  /* ShowImage creates an image from a bitmap,
     and shows the values of the runs
     in the image. The input bitmap looks like:
  
  11001100
  00001111
  
     The output of ShowImage is:
  0 2  4 6  8 8  -1
  4 8 -1
  */
  static void rlprintline(RlLinePtr s)
  {
      int4 i;
      printf("%lx ", (unsigned long)s);
      for (i = 0; i < s->size - 1; i++)
          printf("%d %d  ", s->run[i].up, s->run[i].dn);
      printf("%d", s->run[i].up);
      printf("\n");
  }
  
  static iDAFSError
  rlprintimage(iImagePtr sp)
  {
      int4 i;
  
      for (i = 0; i < sp->height; i++)
          rlprintline(&sp->d.im.rl[i]);
      return DAFSOK;
  }
  
  iDAFSError ShowImage(void)
  {
      char bitmap[2] = {0xcc, 0x0f};
      iDAFSError e;
      iImagePtr im;
      iBox b;
  
      b.x = 0;
      b.y = 0;
      b.w = 8;
      b.h = 2;
      im = i_BitMap2Image(b,1,bitmap);
      if (!im) return MallocFailError;
  
      rlprintimage(im);
      i_DisposeImage(im);
      return DAFSOK;
  }
  /* end example: image */
  /* begin example: separate file and image */
  static iDAFSError APIENTRY writeproc(char *tag,iImagePtr
     im)
  {
     int fd = -1;
     iBox b;
     iDAFSError e;
     int2 size;
     char *data;
     int i;
  
     fd = open(tag, O_WRONLY);
     if (fd < 0) {e = WriteError; goto error; }
     b = i_GetImageBounds(im);
     if (write(fd, &b, sizeof(iBox)) != sizeof(iBox)) {
       e = WriteError; goto error;
     }
     for (i = 0; e = i_GetDataFromImage(im, i, &data,
     &size);
          i++) {
        if (write(fd, data, size) != size) {
          e = WriteError; goto error;
       }
     }
     if (e == NoMoreData) e = DAFSOK;
  error:
     if (fd >= 0) close(fd);
     return e;
  }
  
  #define CHUNKSIZE 4096
  static iDAFSError APIENTRY readproc(char *tag,iImagePtr
     *imp)
  {
     iImagePtr im = 0;
     char *data = 0;
     iDAFSError e = DAFSOK;
     iBox b;
     int fd, quan;
  
     fd = open(tag, O_RDONLY);
     if (fd < 0) {e = WriteError; goto error; }
     if (read(fd, &b, sizeof(iBox)) != sizeof(iBox)) {
       e = ReadError; goto error;
     }
     if (!(im = i_NewImage(b.w, b.h, 300, NoComp))) {
       e = MallocFailError; goto error;
     }
     if (!(data = (char *) malloc(CHUNKSIZE))) {
       e = MallocFailError; goto error;
     }
     for (;;) {
        quan = read(fd, data, CHUNKSIZE);
        if (quan <= 0) break;
        if (e = i_AddDataToImage(im, data, size)) goto
     error;
     }
     *imp = im;
     im = 0;
  error:
      if (im) i_DisposeImage(im);
      if (data) free(data);
      return e;
  }
  
  /* WriteNReadEntityImage just creates a small all white
     image and attaches it to an entity. It then writes the
     entity, disposes of it and reads it back.
  */
  iDAFSError WriteNReadEntityImage(void)
  {
     iImagePtr im = 0;
     iEntityPtr en = 0;
     char *tag;
     iDAFSError e = DAFSOK;
  
     i_SetImageCallBack(writeproc, readproc);
  
     if (!(im = i_NewImage(32, 32, 300, NoComp))) {
       e = MallocFailError; goto error;
     }
     if (e = i_SetImageTag(im, "image1.dat"))
       /* the tag is used as the file name */
        goto error;
     if (!(en = i_NewEntity(0, 0))) {
       e = MallocFailError; goto error;
     }
       i_SetImage(en,im);
     if (e = i_WriteEntity("test.dr", en, DAFSB)) goto
     error;
     i_DisposeEntity(en);
     en = 0;
     i_DisposeImage(im);
     im = 0;
  
     if (e = i_ReadEntity("test.dr", &en)) goto error;
     i_SetImageCallBack(0, 0);
  error:
     if (im) i_DisposeImage(im);
     if (en) i_DisposeEntity(en);
     return e;
  }
  /* end example: separate file and image */
  /* begin example: callback */
  void adjustParentBounds(iEntityPtr entity)
  {
     iEntityPtr parent, child;
     iBox child_union, b;
     iEntityPtr e;
     child_union.w = 0; /* no bounds are denoted by w == 0 */
     parent = i_GetParentEntity(entity);
     for(child = i_GetChildEntity(parent); child; child =
     i_GetNextEntity(child)) {
       b = i_GetEnclosingBox(child);
       if(b.w) { /* this child has area */
          if(child_union.w == 0) /* and first one */
               child_union = b;
          else /* add to previous children's boxes */
               i_UnionBox(b, child_union, &child_union);
       }
     }
     /* will cause chain reaction, might check if unchanged
     */
     i_SetBox(parent, child_union);
  }
  
  void entityCallback(iMess m, void *user_data,
     iEntityPtr entity)
  {
     switch(m) {
     case mMove:    adjustParentBounds(entity); break;
     /* we could handle more messages if we wanted to */
     }
  }
  
  int main(int argc, char **argv)
  {
     extern void doAllExamples(); /* for internal testing */
     i_InitLib();
     /* this would probably be called right after i_InitLib
     */
     i_SetCallback(entityCallback, 0);
     doAllExamples();
     i_ExitLib();
  }
  /* end example: callback */
