
/*
 * Copyright (c) 1994 RAF Technology
 * developed for ARPA under contract MDA 904-93-C-7219
 *
 * Permission to use, copy, modify, distribute, and sell this software and 
 * its documentation for any purpose is hereby granted without fee, provided
 * that (i) the above copyright notices and this permission notice appear in
 * all copies of the software and related documentation, and (ii) the names of
 * RAF Technology or ARPA  may not be used in any advertising or
 * publicity relating to the software without the specific, prior written
 * permission of RAF Technology and ARPA.
 * 
 * THE SOFTWARE IS PROVIDED "AS-IS" AND WITHOUT WARRANTY OF ANY KIND, 
 * EXPRESS, IMPLIED OR OTHERWISE, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE.  
 * 
 * IN NO EVENT SHALL RAF TECHNOLOGY OR ARPA  BE LIABLE FOR
 * ANY SPECIAL, INCIDENTAL, INDIRECT OR CONSEQUENTIAL DAMAGES OF ANY KIND,
 * OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS,
 * WHETHER OR NOT ADVISED OF THE POSSIBILITY OF DAMAGE, AND ON ANY THEORY OF 
 * LIABILITY, ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE 
 * OF THIS SOFTWARE.
 */

#include <stdio.h>
#include "rafport.h"
#include "text.h"
#include "smemory.h"
#include "rafutil.h"

#include "TextElement.h"

extern iEntityPtr pickpath(iEntityPtr doc);

/*
// ascii.cc is a bad name for the file this is in
*/

static int
hasTextContent(iEntityPtr entity)
{
	return i_GetEDataType(entity) != iDNo && i_GetEDataType(entity) != iDConf;
}


iPackType danger_pack={pmOffsets,0,pfVert};
iPackType default_pack={pmOffsets,0,pfHoriz};

static int 
badPack(iPackType *pack)
{
	return ((pack->method == danger_pack.method) &&
			(pack->flags == danger_pack.flags) &&
			(pack->orientation == danger_pack.orientation));
}

extern "C" iDAFSError i_InitTypeProp(void);

TextElement *
newParseEntity(iEntityPtr entity, 
			   TextElement *parent, 
			   void *user_data)
{
	TextElement *me = 0;  
	iPackType *pack = 0;
	iEntityPtr child;
	int has_children;
	int has_content;
	iEntityPtr e_parent;

	e_parent = i_GetParentEntity(entity);
	if(e_parent == 0){


		// reset the packing
		i_InitTypeProp();
	}
	
	
	if((entity=pickpath(entity)) == 0) {
		return 0;
	}

	has_children = i_GetChildEntity(entity) != 0;
	has_content = hasTextContent(entity);

	if(has_children == 0 && has_content == 0)
		return 0;
	
	iDAFSError e=i_GetUserDataProp(entity,PACK_NAME,
							   (void **)&pack, 0,
							   i_SwapPackType);
	
	// if no packing instructions, ignore
	// except if root
	//
	// last one is special case of pda guys who never have
	// children -- really bogus
	if(parent == 0 && (e != DAFSOK || pack == 0) && i_GetChildEntity(entity) != 0)  {
		pack = &default_pack;
		e = DAFSOK;
	} 

	// some other guy who might have text
	if(parent && e != DAFSOK){
		iDTag dt = i_GetEDataType(entity);
		switch(dt){
		case iDNo: case iDConf: case iDShort:
			break;
		default:
			// make up a packing
			e = DAFSOK;
			pack = &default_pack;
		}
	}
	
	if(e == DAFSOK) {
		if (badPack(pack) && (has_content || e_parent == 0))
			pack = &default_pack;
		if(badPack(pack) == FALSE) {
			// special exception for the root entity
			me = new TextElement((Page *) user_data, entity,
								 i_GetEntityType(entity), parent, 
								 pack);
			if(parent)
				parent->insert(me);
			parent = me;
		} 			
	} else if(parent == 0) {
		return 0;
		//halt("No packing for root entity.\n");
	}
	
	for(child = i_GetChildEntity(entity); child; child = i_GetNextEntity(child))
		newParseEntity(child, me?me:parent, user_data);
	
	return me;
}

